<?php

require_once('/var/lib/asterisk/agi-bin/phpagi.php');

// Set up bootstrap
$restrict_mods = true; // don't load any functions for any modules
//$restrict_mods = Array('areminder' => true); // only load freepbx functions related to areminder module 
$bootstrap_settings['freepbx_auth'] = false; 
$bootstrap_settings['skip_astman'] = true; 
$bootstrap_settings['freepbx_error_handler'] = false;

if (!@include_once(getenv('FREEPBX_CONF') ? getenv('FREEPBX_CONF') : '/etc/freepbx.conf')) { 
	  include_once('/etc/asterisk/freepbx.conf'); 
}


$debug = true;


/* --- ENTRY POINT --- */

//Initialize AGI object
$gtAGI = new AGI();

if ($debug) $gtAGI->exec('DumpChan','');

//Get necessary AGI variables 

/* The call is dropped into [areminder-main] at extension ReminderID*CallID
 * for example: 1*23 where 1 is the ReminderID and 23 is the CallID
 */
//$gsAGIExten = $gtAGI->get_variable('agi_extension');
//$gaIDs = explode('*',$gsAGIExten);
//$giReminderID = $gaIDs[0]; 
//$giCallID = $gaIDs[1];

$gaReminderID = $gtAGI->get_variable('arid');
$giReminderID = $gaReminderID['data'];
$gaCallID = $gtAGI->get_variable('callid');
$giCallID = $gaCallID['data'];

// Set manager credentials

$gsASMUser = $amp_conf['AMPMGRUSER'];
$gsASMPass = $amp_conf['AMPMGRPASS'];
$gsASMServer = "localhost";

//Fetch information for this call from the DB
$gsSelectSQL = "
SELECT
 a.greetingid,
 a.greeting2id,
 a.sayname,
 a.saydate,
 a.instructionsid,
 a.csext,
 a.resched,
 a.schedext,
 a.ttsengine AS ttsengineid,
 c.name,
 c.appointment,
 t.name AS ttsengine
FROM 
 areminder AS a
JOIN areminder_calls as c 
  ON a.id = c.arid
JOIN ttsengines as t
  ON a.ttsengine = t.id
WHERE
 a.id = $giReminderID AND
 c.id = $giCallID
";

$ghSQL = $db->query($gsSelectSQL);
$ghSQL->fetchInto($gaRow,DB_FETCHMODE_ASSOC);

//Set local variables from database
$giRecordingID = (int) $gaRow['greetingid'];
$giDateRecordingID = (int) $gaRow['greeting2id'];
$gbSayName = ($gaRow['sayname']>0?true:false);
$gbSayDate = ($gaRow['saydate']>0?true:false);
$giInstructionsID = (int) $gaRow['instructionsid'];
$gsCSExtension = explode(',',$gaRow['csext']);
$gsRescheduleBehavior = $gaRow['resched'];
$gsRescheduleExtension = explode(',',$gaRow['schedext']);
$giTTSEngine = (strlen($gaRow['ttsengine'])>0?$gaRow['ttsengine']:'Flite');
$gsName = $gaRow['name'];
$gsAppointment = $gaRow['appointment'];

//Get all recordings 

$gRecordingIDs = "($giRecordingID,$giDateRecordingID,$giInstructionsID)";

$gRecordingSQL = "SELECT * FROM recordings WHERE id IN $gRecordingIDs";

$ghSQL = $db->query($gRecordingSQL);

while ($bTest = $ghSQL->fetchInto($aRow,DB_FETCHMODE_ASSOC))
{
	$gaRecordings[$aRow['id']] = $aRow['filename'];
}

$gsRecording = (isset($gaRecordings["$giRecordingID"]) ? $gaRecordings["$giRecordingID"] : 'areminder-greeting');
$gsDateRecording = (isset($gaRecordings["$giDateRecordingID"]) ? $gaRecordings["$giDateRecordingID"] : 'areminder-greeting2');
$gsInstructions = (isset($gaRecordings["$giInstructionsID"]) ? $gaRecordings["$giInstructionsID"] : 'areminder-instructions');


/* --- START OF CALL LOGIC --- */

//Answer Call 
$gtAGI->answer();

//WaitForSilence (cheap answering machine detection) 
$gtAGI->exec('waitforsilence','1100,1,60');

//Playback greeting audio file areminder-greeting or play chosen greeting recording

stream_multiple($gsRecording, '');

//Playback recepient name if enabled
if($gbSayName) 
{
	$gtAGI->stream_file('areminder-for', '', 0);
	//text to speech using $giTTSEngine the recipient's name
	$gtAGI->exec($giTTSEngine . ' "' . $gsName . '"');
}

// play appointment/second greeting
stream_multiple($gsDateRecording, '');

//Say the appointment date and time
if($gbSayDate)
{
	// say the actual time of the appointment
	fvSayAppointmentTime();
}

// Play instructions and get input recursively 
fvPlayInstructionsAndGetInput();

//and we are done
$gtAGI->stream_file('goodbye', '', 0);
exit;



/* --- END OF ENTRY POINT --- */


//Plays back options and accepts input
function fvPlayInstructionsAndGetInput($p_iIteration=0, $p_iMaxIterations=3)
{
	//Check if we have exceeded the number of loops 
	if ($p_iIteration >= $p_iMaxIterations) return; 
	$p_iIteration++;
	
	//Playback areminder-instructions and read a single digit input from 0 to 3
	$aInput = stream_multiple($GLOBALS['gsInstructions'], '0123');
	
	if ($aInput['result']==0)
	{
		$nInput = '';
		if ($GLOBALS['debug']) $GLOBALS['gtAGI']->exec('NoOp',"User entered nothing");
	}else{
		$nInput = (integer) chr($aInput['result']);
		if ($GLOBALS['debug']) $GLOBALS['gtAGI']->exec('NoOp',"User entered $nInput");
	}
	$sStatus = '';
	
	switch ($nInput)
	{
		/* Digit = 1 -- action = confirmed
		 * Update DB set status = confirmed
		 * Playback areminder-confirmed
		 */
		case "1":
			$sStatus = 'confirm';
			fvDB_SetStatus($nInput);
			//$GLOBALS['gtAGI']->stream_file('areminder-confirmed', '', 0);
	
			break;
	
		/* Digit = 2 -- action = cancelled
		 * Update DB set status = cancelled
		 * Playback aremider-cancelled
		 */
		case "2":
	 		$sStatus = 'cancel';
			fvDB_SetStatus($nInput);
			//$GLOBALS['gtAGI']->stream_file('areminder-cancelled', '', 0);
	
			break;
	
		/* Digit = 3 -- action = rescheduled
		 * Upate DB set status = rescheduled
		 * 
		 */
		case "3":

			//UpDate DB status
			fvDB_SetStatus($nInput);
		
			if($GLOBALS['gsRescheduleBehavior'] == 'TXFR')
			{
				//Playback
				$GLOBALS['gtAGI']->stream_file('areminder-transfer', '', 0);						
				
				//Transfer call to the destination
				$GLOBALS['gtAGI']->set_variable('TXCONTEXT',$GLOBALS['gsRescheduleExtension'][0]);
				$GLOBALS['gtAGI']->set_variable('TXEXTENSION',$GLOBALS['gsRescheduleExtension'][1]);
				$GLOBALS['gtAGI']->set_variable('TXPRIORITY',$GLOBALS['gsRescheduleExtension'][2]);
				exit;
			}
			elseif($GLOBALS['gsRescheduleBehavior'] == 'LOG')
			{
				$sStatus = 'reschedule';
				
				//Playback
				$GLOBALS['gtAGI']->stream_file('areminder-reschedule-callback', '', 0);
			}
			
			break;
	 
		/* Digit = 0 -- action = customer service
		 * Update db status = CSR
		 * Playback areminder-trasnfer
		 * Transfer to the selected CS dest.
		 */
		case "0":
			//Playback
			$GLOBALS['gtAGI']->stream_file('areminder-transfer', '', 0);
			
	 		//Update DB status to 4 for CSR/Operator
			fvDB_SetStatus(4);
			
			//Do the transfer
			$GLOBALS['gtAGI']->set_variable('TXCONTEXT',$GLOBALS['gsCSExtension'][0]);
			$GLOBALS['gtAGI']->set_variable('TXEXTENSION',$GLOBALS['gsCSExtension'][1]);
			$GLOBALS['gtAGI']->set_variable('TXPRIORITY',$GLOBALS['gsCSExtension'][2]);
			exit;
			
			break;
			 
		/* Digit = anything else besides this
		 * Restart
		 */
		default: 
			fvPlayInstructionsAndGetInput($p_iIteration,$p_iMaxIterations);
			
			break;
	
	}
	
	/* Inform recepient of status
	 * Playback areminder-summary
	 * Playback areminder-[status]
	 * Playback areminder-summary2
	 */
	 
	 if($sStatus != '')
	 {
		 $GLOBALS['gtAGI']->stream_file('areminder-summary', '', 0);
		 $GLOBALS['gtAGI']->stream_file('areminder-' . $sStatus, '', 0);
		 $GLOBALS['gtAGI']->stream_file('areminder-summary2', '', 0);
	  	 
	 	fvConfirmSummary();
	 }
}


//Confirms the summary and provides an option for restarting
function fvConfirmSummary($p_iIteration=0, $p_iMaxIterations=2)
{
	
	if ($p_iIteration >= $p_iMaxIterations) return; 
	$p_iIteration++;
	
	/* Confirm and provide options for starting over
	 * Playback areminder-summary-instructions & accept input
	 * Digit = 0 - transfer to csr
	 * Digit = 1 - repeat currently scheduled date
	 * Digit = 2 - reset all input and restart from begining
	 */
	 
	$aInput = $GLOBALS['gtAGI']->stream_file('areminder-summary-instructions', '012',0);
	if ($aInput['result']==0)
	{
		$nInput = '';
		if ($GLOBALS['$debug']) $GLOBALS['gtAGI']->exec('NoOp',"User entered nothing");
	}else{
		$nInput = (integer) chr($aInput['result']);
		if ($GLOBALS['$debug']) $GLOBALS['gtAGI']->exec('NoOp',"User entered $nInput");
	}
	 
	switch ($nInput)
	{
		//Transfer to CSR
		case "0":
			//Playback
			$GLOBALS['gtAGI']->stream_file('areminder-transfer', '', 0);				
			//Do the transfer		
			$GLOBALS['gtAGI']->set_variable('TXCONTEXT',$GLOBALS['gsCSExtension'][0]);
			$GLOBALS['gtAGI']->set_variable('TXEXTENSION',$GLOBALS['gsCSExtension'][1]);
			$GLOBALS['gtAGI']->set_variable('TXPRIORITY',$GLOBALS['gsCSExtension'][2]);
			exit;
		
		//Repeat date / appointment time
		case "1":
			fvSayAppointmentTime();
			fvConfirmSummary($p_iIteration,$p_iMaxIterations);
			return;
			
		//Reset input and restart
		case "2": 
			fvPlayInstructionsAndGetInput();
			return;
			
		default:
			fvConfirmSummary($p_iIteration, $p_iMaxIterations);
	}
}

//Sets the status in the DB
function fvDB_SetStatus($p_iStatus)
{
	$GLOBALS['db']->query("UPDATE `areminder_calls` SET `status` = $p_iStatus WHERE id = ".$GLOBALS['giCallID']);
}

function fvSayAppointmentTime()
{
	$GLOBALS['gtAGI']->exec('SayUnixTime', $GLOBALS['gsAppointment']);
}

function stream_multiple($p_sFileList,$p_sKeys)
{
	$aFiles = explode('&',$p_sFileList);
	$result = 0;
	foreach ($aFiles as $sFile)
	{
		$aInput = $GLOBALS['gtAGI']->stream_file($sFile, $p_sKeys, 0);
		if ($aInput['result']!=0) return $aInput;
	}
	return $aInput;
}

?>
