<?php
require_once dirname(__FILE__)."/functions.inc.php";

global $db;

/* broadcast_campaigns */
$sql = "
CREATE TABLE IF NOT EXISTS `broadcast_campaigns` (
  `id` int(11) NOT NULL auto_increment,
  `name` varchar(255) NOT NULL DEFAULT '',
  `email` varchar(255) NOT NULL,
  `email_settings` varchar(64) NOT NULL,
  `active` tinyint(4) NOT NULL,
  `time_start` time NOT NULL,
  `time_finish` time NOT NULL,
  `weekday_start` int(1) NOT NULL,
  `weekday_finish` int(1) NOT NULL,
  `date_start` date NOT NULL,
  `date_finish` date NOT NULL,
  `destination_person` varchar(50) NOT NULL,
  `destination_machine` varchar(50) NOT NULL,
  `completed` tinyint(4) NOT NULL,
  `callerid` varchar (255) NOT NULL,
  `prefix` varchar(20),
  PRIMARY KEY  (`id`)
)";

$check = $db->query($sql);
if($db->IsError($check)) {
	die_freepbx("Can not create broadcast_campaigns table\n");
}

/* removing for 2.9, dont think is is still nesesary -MB
$sql = "ALTER TABLE broadcast_campaigns CHANGE name name VARCHAR( 255 ) NOT NULL DEFAULT '' ;";
$check = $db->query($sql);
if($db->IsError($check)) {
	die_freepbx("Can not alter broadcast_campaigns table\n");
}
*/
// Version 2.7.0.1 adds configurable callerid to the campaigns
$sql = "SELECT callerid FROM broadcast_campaigns";
$check = $db->getRow($sql, DB_FETCHMODE_ASSOC);
if($db->IsError($check)) {
    // add new field
    $sql = "ALTER TABLE broadcast_campaigns ADD callerid VARCHAR(255) NOT NULL DEFAULT '';";
    $result = $db->query($sql);
    if($db->IsError($result)) { 
		die_freepbx($result->getDebugInfo()); 
	}
}

// Version 2.9 adds configurable call prefix to the campaigns
$sql = "SELECT prefix FROM broadcast_campaigns";
$check = $db->getRow($sql, DB_FETCHMODE_ASSOC);
if($db->IsError($check)) {
    // add new field
    $sql = "ALTER TABLE broadcast_campaigns ADD COLUMN prefix varchar(20) DEFAULT '';";
    $result = $db->query($sql);
    if($db->IsError($result)) { 
		die_freepbx($result->getDebugInfo()); 
	}
}

/* broadcast_campaign_groups */
$sql = "
CREATE TABLE IF NOT EXISTS `broadcast_campaign_groups` (
  `id` mediumint(9) NOT NULL auto_increment,
  `campaign_id` mediumint(9) NOT NULL,
  `group_id` mediumint(9) NOT NULL,
  PRIMARY KEY  (`id`)
)";
$check = $db->query($sql);
if($db->IsError($check)) {
	die_freepbx("Can not create broadcast_campaign_groups table\n");
}

/* broadcast_settings */
$sql = "
CREATE TABLE IF NOT EXISTS `broadcast_settings` (
  `concurrent_calls` int(11) NOT NULL default '0',
  `maximum_retry` int(11) NOT NULL default '0'
)";
$check = $db->query($sql);
if($db->IsError($check)) {
	die_freepbx("Can not create broadcast_settings table\n");
}
$sql = "
INSERT INTO `broadcast_settings` (`concurrent_calls`, `maximum_retry`) VALUES
(2, 2)";

$check = $db->query($sql);
if($db->IsError($check)) {
	die_freepbx("Can not insert into broadcast_settings table\n");
}
/* broadcast_groups */
$sql = "
CREATE TABLE IF NOT EXISTS `broadcast_groups` (
  `id` mediumint(9) NOT NULL auto_increment,
  `name` varchar(50) NOT NULL,
  PRIMARY KEY  (`id`)
)";
$check = $db->query($sql);
if($db->IsError($check)) {
	die_freepbx("Can not create broadcast_groups table\n");
}
/* broadcast_callees */
$sql = "
CREATE TABLE IF NOT EXISTS `broadcast_callees` (
  `id` mediumint(9) NOT NULL auto_increment,
  `name` varchar(40) NOT NULL default '',
  `phonenum` varchar(20) NOT NULL default '',
  `group_id` int(11) NOT NULL default '0',
  PRIMARY KEY  (`id`),
  KEY `phonenum` (`phonenum`),
  KEY `name` (`name`),
  INDEX broadcast_callees_group_id (group_id)
)";
$check = $db->query($sql);
if($db->IsError($check)) {
	die_freepbx("Can not create broadcast_callees table\n");
}
/* broadcast_log */
$sql = "
CREATE TABLE IF NOT EXISTS `broadcast_log` (
  `id` mediumint(9) NOT NULL auto_increment,
  `last_updated` datetime NOT NULL,
  `callid` varchar(32) NOT NULL,
  `campaign_id` mediumint(9) NOT NULL,
  `callee` mediumint(9) NOT NULL,
  `initiated_dial` tinyint(4) NOT NULL,
  `answer` tinyint(4) NOT NULL,
  `delivered_to` varchar(32) NOT NULL,
  PRIMARY KEY  (`id`),
  INDEX broadcast_log_callee (callee),
  INDEX broadcast_log_campaign_id (campaign_id)
)";
$check = $db->query($sql);
if($db->IsError($check)) {
	die_freepbx("Can not create broadcast_log table\n");
}

//ALTER and add some indexes
$sql = "SHOW KEYS FROM `broadcast_callees` WHERE Key_name= 'broadcast_callees_group_id'";
$check = $db->getOne($sql);
if (empty($check)) {
	$sql = "ALTER TABLE broadcast_callees ADD INDEX broadcast_callees_group_id (group_id)";
	$result = $db->query($sql);
	if($db->IsError($result)) { die_freepbx($result->getDebugInfo()); }
}

$sql = "SHOW KEYS FROM `broadcast_log` WHERE Key_name= 'broadcast_log_campaign_id'";
$check = $db->getOne($sql);
if (empty($check)) { 
	$sql = "ALTER TABLE broadcast_log ADD INDEX broadcast_log_campaign_id (campaign_id)";
        $result = $db->query($sql);
        if($db->IsError($result)) { die_freepbx($result->getDebugInfo()); }
}

$sql = "SHOW KEYS FROM `broadcast_log` WHERE Key_name= 'broadcast_log_campaign_id'";
$check = $db->getOne($sql);
if (empty($check)) {
        $sql = "ALTER TABLE broadcast_log ADD INDEX broadcast_log_campaign_id (campaign_id)";
        $result = $db->query($sql);
        if($db->IsError($result)) { die_freepbx($result->getDebugInfo()); }
}

// Version 2.5.0.9 adds configurable from-email address
$sql = "SELECT from_email FROM broadcast_settings";
$check = $db->getRow($sql, DB_FETCHMODE_ASSOC);
if($db->IsError($check)) {
        // add new field
    $sql = "ALTER TABLE broadcast_settings ADD from_email VARCHAR(255) NOT NULL DEFAULT '';";
    $result = $db->query($sql);
    if($db->IsError($result)) { die_freepbx($result->getDebugInfo()); }
}

?>
