<?php
// vim: set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed');}

if (zend_loader_file_encoded()) {
    $calllimit_lic_info   = zend_loader_file_licensed();
    $calllimit_lic_exp        = isset($calllimit_lic_info['outcalllimit_exp'])?preg_replace('/-/','',$calllimit_lic_info['outcalllimit_exp']):'00000000';
    if ($calllimit_lic_exp > date('Ymd')) {
        $calllimit_licensed = true;
    } else {
        $calllimit_licensed = false;
    }
} else {
    $calllimit_licensed = true;
    global $load_license;
    global $calllimit_lic_info;
    $calllimit_licensed = true;
    $calllimit_lic_info['deploy_type'] = 'OSS';
    $load_license = true;
}

function calllimit_hookGet_config($engine) {
    global $ext, $calllimit_licensed;

    if ($calllimit_licensed) {
        switch($engine) {
            case 'asterisk':
                $usage = calllimit_get_usage('routing');
                if (is_array($usage) && !empty($usage)) {

                    $calllimits = calllimit_get_all();

                    foreach($usage as $route) {
                        $c = 'outrt-'.$route['foreign_id'];
                        $patterns = core_routing_getroutepatternsbyid($route['foreign_id']);
                        foreach ($patterns as $pattern) {
                            $formatPattern = core_routing_formatpattern($pattern);
                            $e = $formatPattern['dial_pattern'];
                            $d = 1;
                            $ext->splice($c, $e, $d++, new ext_setvar('CALLLIMIT_ROUTE',$route['foreign_id']));
                            $ext->splice($c, $e, $d++, new ext_setvar('CALLLIMIT_ID',$route['calllimit_id']));
                            $ext->splice($c, $e, $d++, new ext_setvar('CALLLIMIT_DEST', '${EXTEN}'));
                        }
                    }
                    $context="macro-dialout-trunk";
                    $ext->splice($context, 's', 9 ,new ext_agi('calllimit.agi'));
                }
                break;
        }
    }
}

function calllimit_getdest($id) {
    return array('ext-calllimit,'.$id.',1');
}

function calllimit_hookProcess_core($viewing_itemid, $request) {
    global $calllimit_licensed;

    if ($calllimit_licensed) {
        switch($request['display']) {
            case 'routing':
                $action = (isset($request['action']))?$request['action']:null;
                $route_id = $viewing_itemid;

                if (isset($request['Submit']) ) {
                    $action = (isset($action))?$action:'editroute';
                }

                if (!$action && isset($_SESSION['calllimitAddRoute']) && $_SESSION['calllimitAddRoute'] != '') {
                    calllimit_adjustroute($route_id,'delayed_insert_route',$_SESSION['calllimitAddRoute']);
                    unset($_SESSION['calllimitAddRoute']);
                } elseif ($action){
                    calllimit_adjustroute($route_id,$action,$request['calllimit']);
                }
                break;
        }
    }
}

function calllimit_adjustroute($route_id, $action, $routecalllimit = '') {
    global $db;
    $dispname = 'routing';
    $route_id = $db->escapeSimple($route_id);
    $routecalllimit = $db->escapeSimple($routecalllimit);

    switch($action) {
        case 'delroute':
            sql('DELETE FROM calllimit_usage WHERE foreign_id = '.q($route_id)." AND dispname = '$dispname'");
            break;
        case 'addroute':
            if($routecalllimit != '') {
                $_SESSION['calllimitAddRoute'] = $routecalllimit;
            }
            break;
        case 'delayed_insert_route':
            sql("INSERT INTO calllimit_usage (calllimit_id, dispname, foreign_id) VALUES ($routecalllimit, '$dispname', '$route_id')");
            break;
        case 'editroute':
            if ($routecalllimit != '') {
                sql("REPLACE INTO calllimit_usage (calllimit_id, dispname, foreign_id) VALUES ($routecalllimit, '$dispname', '$route_id')");
            } else {
                sql('DELETE FROM calllimit_usage WHERE foreign_id ='.q($route_id)." AND dispname = '$dispname'");
            }
            break;
    }
}

function calllimit_get_usage($display='') {
    $sql = 'SELECT * FROM `calllimit_usage`';
    if (isset($dispname) && !empty($dispname)) {
        $sql .= " WHERE `dispname` = '$dispname'";
    }
    return sql($sql,'getALL', DB_FETCHMODE_ASSOC);
}

function calllimit_get_all() {
	global $db;

	$res = sql('SELECT DISTINCT(`value`) from calllimit where `key` like "%\_id" ORDER BY `value`', "getAll", DB_FETCHMODE_ORDERED);
	foreach ($res as $id) {
		$retarr[$id[0]] = calllimit_get($id[0]);
	}

	if (!is_array($retarr))
		return array();

	return $retarr;
}

function calllimit_get($id = 1) {
    global $db;
    $id = $db->escapeSimple($id);

    $p = $db->prepare('SELECT `value` FROM `calllimit` WHERE `key` = ?');

	// All the keys we want to get.
	$values = array( "id", "name", "recording", "recordingid", "periodtype_1", "periodtype_2", "periodtype_3",
		"periodlength_1", "periodlength_2","periodlength_3", "qty_1", "qty_2", "qty_3");

	// Go through all the keys, build array!
	foreach ($values as $key) {
		$res = $db->execute($p, $id."_".$key);
		$row = $res->fetchRow();
		$results[$key]=$row[0];
    }

    if (!is_array($results)) {
        $results = array();
    }

    return $results;
}

function calllimit_put($post) {
    global $db;

    if (empty($post['id'])) {
        // This is an Add. Grab the next ID
        $max = sql('SELECT MAX(CAST(`value` AS SIGNED)) from `calllimit` WHERE `key` like "%\_id"', "getRow");
        if (empty($max[0])) {
            // There are no rules in the database. Start from 1.
            $id = 1;
        } else {
            $id = $max[0]+1;
        }
    } else {
        // It's an edit.
        $id = $post['id'];
    }

    $id = $db->escapeSimple($id);

    // Delete any rules that may be there already.
    sql("DELETE FROM `calllimit` WHERE `key` LIKE '".$id."\_%'");

    $p = $db->prepare('INSERT INTO `calllimit` VALUES (?, ?)');

    $db->execute($p, array($id."_id", $id));
    $db->execute($p, array($id."_name", $post['name']));
    $db->execute($p, array($id."_recordingid", $post['recording']));
    $db->execute($p, array($id."_recording", recordings_get_file($post['recording'])));
    foreach(array(1,2,3) as $c) {
        foreach(array("periodtype_$c", "periodlength_$c", "qty_$c") as $item) {
            $db->execute($p, array($id."_".$item, $post[$item]));
        }
    }
}

function calllimit_del($id) {
    global $db;
    $id = $db->escapeSimple($id);
    sql("DELETE FROM `calllimit` WHERE `key` LIKE '".$id."\_%'");
}

function calllimit_hook_core($viewing_itemid, $target_menuid) {
    global $db, $calllimit_licensed;

    if ($calllimit_licensed) {
        switch($target_menuid) {
            case 'routing':
                //get available calllimits
                $calllimit = calllimit_get_all();
                if ($viewing_itemid == '') {
                    $selected_calllimit = '';
                } else {
                    if (isset($_SESSION['calllimitAddRoute']) && $_SESSION['calllimitAddRoute'] != '') {
                        $selected_calllimit = $_SESSION['calllimitAddRoute'];
                    } else {
                        $selected_calllimit = $db->getOne("SELECT calllimit_id FROM calllimit_usage WHERE dispname='routing' AND foreign_id='".$db->escapeSimple($viewing_itemid)."'");
                        if (DB::IsError($selected_calllimit)) {
                            die_freepbx($selected_calllimit->getMessage());
                        }
                    }
                }

                $hookhtml = '
                    <tr>
                    <td><a href="#" class="info">'._("Call Limit").'<span>'._('Call limit limits the number of calls that can be made using this route.').'</span></a>:</td>
                    <td>
                    <select name="calllimit">
                    <option value=>'._('None').'</option>
                    ';
                if (is_array($calllimit)) {
                    foreach($calllimit as $item) {
                        $selected = ($item['id'] == $selected_calllimit)?'SELECTED':'';
                        $hookhtml .= "<option value={$item['id']} ".$selected.">{$item['name']}</option>";
                    }
                }
                $hookhtml .= '
                    </select>
                    </td>
                    </tr>
                    ';
                return $hookhtml;
                break;
            default:
                return false;
                break;
        }
    }
}

function calllimit_sysadmin_getinfo() {
    return array(
        "name" => "Outbound Call Limit", // Required. Descriptive name.
        "expires" => true,         // Optional. Defaults to false (never expires)
        "keyname" => "outcalllimit_exp", // Optional. Only checked if 'expires' is true
    );
}
