<?php
if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed');}
//	License for all code of this FreePBX module can be found in the license file inside the module directory
//	Copyright 2013 Schmooze Com Inc.
//

if (zend_loader_file_encoded()) {
	$lic_info	= zend_loader_file_licensed();
	$lic_exp	= isset($lic_info['confpro_exp'])
				? preg_replace('/-/', '', $lic_info['confpro_exp'])
				: 00000000;
	if ($lic_exp > date('Ymd')) {
		$confpro = true;
	} else {
		$confpro = false;
	}
} else {
	$confpro = true;
}
//render destinations in drop downs
function conferencespro_destinations() {
	global $db, $confpro;
	if (!$confpro) {
		return false;
	}
	$count = count(conferencespro_rooms_get());

	//report errors
	if ($db->isError($count)) {
		dbug($count->getUserInfo());
	}

	if(!$count) { //return nothing if there are no conferences that need to be part of the ivr
		$extens = null;
	} elseif($count > 0) { //only if there conferences that need to be part of the ivr
		$extens[] = array('destination' => 'conferencepro-ivr,s,1', 'description' => 'Conference List IVR', 'category' => _('Conferences'));
	}

	return $extens;
}

//Add Conference List IVR to the destination registry so that it doesn't show a bad dest
function conferencespro_getdestinfo($dest) {
	global $db;

	if (substr(trim($dest),0,strlen('conferencepro-ivr,')) == 'conferencepro-ivr,') {
		$count = count(conferencespro_rooms_get());

		if ($db->isError($count)) {
			dbug($count->getUserInfo());
		}

		if (!$count) {
			return array();
		} else {
			return array('description' => _("Conference List IVR"),
				'edit_url' => 'config.php?display=conferencepro',
			);
		}
	} else {
		return false;
	}
}

//generate dialplan
function conferencespro_get_config($engine){
	global $version, $ext, $amp_conf, $core_conf, $db, $confpro;
	if (!$confpro) {
		return false;
	}
	$ivr = conferencespro_rooms_get();
	$set = conferencespro_settings_get();

	if(count($ivr) == 0) {
		return; //exit if there are no conferences that need to be in this ivr
	}

	//get longest room number to use in responsetimeout() below
	$rlength = 1;
	foreach ($ivr as $i) {
		$rlength = (strlen($i['room']) > $rlength) ? strlen($i['room']) : $rlength;
	}
	switch($engine) {
		case "asterisk":
			$ctx				= 'conferencepro-ivr';
			$announcement_id	= (isset($set['recording']) ? $set['recording']	: '');
			$timeout_id 		= '';
			$invalid_id			= '';
			$loops				= 2;

			$ext->add($ctx, 'h', '', new ext_hangup(''));
			if ($announcement_id) {
				$announcement_msg = recordings_get_file($announcement_id);
				$ext->add($ctx, 's', '', new ext_setvar('MSG', "$announcement_msg"));
			} else {
				$ext->add($ctx, 's', '', new ext_setvar('MSG', ""));
			}
			$ext->add($ctx, 's', '', new ext_setvar('LOOPCOUNT', 0));

			$ext->add($ctx, 's', '', new ext_gotoif('$["${CDR(disposition)}" = "ANSWERED"]','begin'));
			$ext->add($ctx, 's', '', new ext_answer(''));
			$ext->add($ctx, 's', '', new ext_wait('1'));
			$ext->add($ctx, 's', 'begin', new ext_digittimeout(3));
			$ext->add($ctx, 's', '', new ext_responsetimeout($rlength));


			$ext->add($ctx, 's', '', new ext_execif('$["${MSG}" != ""]','Background','${MSG}'));
			$ext->add($ctx, 's', '', new ext_waitexten());
			$ext->add($ctx, 'hang', '', new ext_playback('vm-goodbye'));
			$ext->add($ctx, 'hang', '', new ext_hangup(''));

			$default_t=true;

			// Actually add the IVR commands now.
			$dests = $ivr;
			$timeout=false;
			$invalid=false;
			$addloop=false;
			if (!empty($dests)) {
				foreach($dests as $dest) {
					$dest['selection']	= isset($dest['selection']) && $dest['selection'] ? $dest['selection'] : '';
					$dest['dest']		= isset($dest['dest']) && $dest['dest'] ? $dest['dest'] : '';
					if ($dest['selection'] == 't' && empty($details['alt_timeout'])) {
					 	$timeout=true;
					} elseif ($dest['selection'] == 'i' && empty($details['alt_invalid'])) {
					 	$invalid=true;
					} elseif (($dest['selection'] == 't' && !empty($details['alt_timeout']))) {
					 	$timeout=true;
						if ($timeout_id) {
							$timeout_msg = recordings_get_file($timeout_id);
							$ext->add($ctx, $dest['selection'], '', new ext_setvar('MSG',"$timeout_msg"));
						}
						$ext->add($ctx, $dest['selection'], '', new ext_setvar('LOOPCOUNT','$[${LOOPCOUNT} + 1]'));
						$ext->add($ctx, $dest['selection'], '', new ext_gotoif('$[${LOOPCOUNT} <= '.$loops.']','s,begin'));
					} elseif (($dest['selection'] == 'i' && !empty($details['alt_invalid']))) {
					 	$invalid=true;
						$ext->add($ctx, $dest['selection'], '', new ext_setvar('LOOPCOUNT','$[${LOOPCOUNT} + 1]'));

						if ($invalid_id) {
							$invalid_msg = recordings_get_file($invalid_id);
							$ext->add($ctx, $dest['selection'], '', new ext_setvar('MSG',"$invalid_msg"));
						} else {
							$ext->add($ctx, $dest['selection'], '', new ext_execif('$[${LOOPCOUNT} <= '.$loops.']','Playback','invalid'));
						}
						$ext->add($ctx, $dest['selection'], '', new ext_gotoif('$[${LOOPCOUNT} <= '.$loops.']','s,begin'));
					}


					// if the goto goes loops back to this ivr, then don't go to the begining or it will break the return to previous ivr info
					//
					$dest_context = trim(strtok($dest['dest'],",|"));
					if ($dest_context == $ctx) {
						$dest['dest'] = $ctx.',s,begin';
					}
					$ext->add($ctx, $dest['room'],'', new ext_goto(1, $dest['room'], 'ext-meetme'));
				}
			}

			// Apply invalid if required
			if (!$invalid) {
				if ($invalid_id) {
					$invalid_msg = recordings_get_file($invalid_id);
					$ext->add($ctx, 'i', '', new ext_setvar('MSG',"$invalid_msg"));
				} else {
					$ext->add($ctx, 'i', '', new ext_playback('invalid'));
				}
				$ext->add($ctx, 'i', '', new ext_goto('loop,1'));
				$addloop=true;
			}
			if (!$timeout) {
				if ($timeout_id) {
					$timeout_msg = recordings_get_file($timeout_id);
					$ext->add($ctx, 't', '', new ext_setvar('MSG',"$timeout_msg"));
				}
				$ext->add($ctx, 't', '', new ext_goto('loop,1'));
				$addloop=true;
			}
			if ($addloop) {
				$ext->add($ctx, 'loop', '', new ext_setvar('LOOPCOUNT','$[${LOOPCOUNT} + 1]'));
				$ext->add($ctx, 'loop', '', new ext_gotoif('$[${LOOPCOUNT} > '.$loops.']','hang,1'));
				$ext->add($ctx, 'loop', '', new ext_goto($ctx.',s,begin'));

			}
		break;
	}
}

//render options in conference page
function conferencespro_hook_conferences($viewing_itemid, $target_menuid) {
	global $confpro;
	if (!$confpro) {
		return false;
	}
	$room	= isset($_REQUEST['extdisplay']) ? $_REQUEST['extdisplay']	: '';
 	$set	= conferencespro_rooms_get($room);

 	//Build display
	$html = '';
	$html .= '<table>';
		$html .= '<thead></thead>';
		$html .= '<tfoot></tfoot>';
		$html .= '<tbody>';
			$html .= '<tr><td colspan="2"><h5>' . _('Pro Options') . '<hr></h5></td></tr>';
			$html .= '<tr>';
				$html .= '<td>';
					$html .= '<a href="#" class="info">'
							. _("Manageable by user")
							. '<span>'
							. _('Allows this conference to be managed by the specified user from the Recordings Interface')
							. '</span></a>';
				$html .= '</td>';
				$html .= '<td>';
					$html .= '<select name="prouser">';
					$html .= '<option value="">' . _('none') . '</option>';
					$users = core_users_list();
					foreach ($users as $user) {
						$html .= '<option value="' . $user[0] . '" '
								. ($set['user'] == $user[0]? ' selected ' : '')
								.'>' . $user[1] . ' (' . $user[0] . ')' . '</option>';
					}
					$html .= '</select>';
				$html .= '</td>';
			$html .= '</tr>';

			$html .= '<tr>';
				$html .= '<td>';
					$html .= '<a href="#" class="info">'
							. _("Add to conference IVR")
							. '<span>'
							. _("Allows this conference to be included in the IVR context")
							. '</span></a>';
				$html .= '</td>';
				$html .= '<td>';
					$html .= '<input type="radio" name="proivr" value="false" '
							. (($set['ivr'] == 'false' || $set['ivr'] == '') ? ' checked ' : '')
							.'/> No';
					$html .= '<input type="radio" name="proivr" value="true" '
							. ($set['ivr'] == 'true' ? ' checked ' : '')
							. '/> Yes';
				$html .= '</td>';
			$html .= '</tr>';
		$html .= '</tbody>';
	$html .= '</table>';


  	return $html;
}

//prosses options from conference page
function conferencespro_hookProcess_conferences($viewing_itemid, $request) {
	$action			= isset($_REQUEST['action'])		? $_REQUEST['action']		: '';
	$room			= isset($_REQUEST['account'])		? $_REQUEST['account']	: '';
	$user			= isset($_REQUEST['prouser'])		? $_REQUEST['prouser']		: '';
	$ivr			= isset($_REQUEST['proivr'])		? $_REQUEST['proivr']		: '';
	$orig_account	= isset($_REQUEST['orig_account'])	? $_REQUEST['orig_account']	: '';

	switch ($action) {
		case 'add':
			//$room wont be set if this is a new conference
			$room = isset($_REQUEST['account'])? $_REQUEST['account'] : '';
		case 'edit':
		if ($orig_account && $orig_account != $account) {
			conferencespro_rooms_del($orig_account);
		}
			conferencespro_rooms_set($room, $user, $ivr);
			break;
		case 'delete':
			$room = (isset($_REQUEST['account']) && $_REQUEST['account'] != '')?$_REQUEST['account'] : $_REQUEST['extdisplay'];
			conferencespro_rooms_del($room);
			break;
	}
}

//get general settings
function conferencespro_settings_get() {
	global $db;
	$sql = 'SELECT * FROM conferencespro';
	$set = $db->getAll($sql, DB_FETCHMODE_ASSOC);

	if(count($set) > 0) {
		foreach( $set as $s) {
			$ret[$s['setting']] = $s['value'];
		}
		return $ret;
	} else {
		return array();
	}
}

//delete a specific setting - not currently used
function conferencespro_settings_del() {

}

//set a specific setting
function conferencespro_settings_set($set) {
	global $db;
	foreach ($set as $s => $val) {
		$sql = 'REPLACE INTO conferencespro  (setting, value) VALUES (?, ?)';
		$ins = $db->query($sql, array($s, $val));

		if($db->isError($ins)) {
			dbug($ins->getDebugInfo());
		}
	}
	needreload();

}
//delete the pro settings for a conference
function conferencespro_rooms_del($room) {
	global $db;

	$sql = 'DELETE FROM conferencespro_rooms WHERE room = ?';
	$inst = $db->query($sql, array($room));
	if ($db->isError($inst)) {
		dbug($db->getUserInfo());
	}
}

//get a list of conferences
function conferencespro_rooms_get($room='', $ivr_include = '') {
	global $db;

	if ($room) {
		$sql = 'SELECT * FROM conferencespro_rooms WHERE room = ?';
			$set = $db->getAll($sql, array($room), DB_FETCHMODE_ASSOC);
	} else if (!isset($rooms) && $ivr_include) {
		$sql = 'SELECT * FROM conferencespro_rooms WHERE ivr = ?';
                $set = $db->getAll($sql, array($ivr_include), DB_FETCHMODE_ASSOC);
	} else {
		$sql = 'SELECT * FROM conferencespro_rooms';
		$set = $db->getAll($sql, DB_FETCHMODE_ASSOC);
	}

		if ($room) {
			if (is_array($set) && count($set) > 0) {
				return $set[0] ;
			}  else {
			//dont return nothing - return blank instead
			return array('room' => $room, 'user' => '', 'ivr' => '');
			}
		} elseif (is_array($set) && count($set) > 0) {
			return $set;
		} else {
			return array();
		}

}

function conferencespro_rooms_set($room, $user, $ivr) {
	global $db;

	$sql = 'REPLACE INTO conferencespro_rooms SET room = ?, user = ?, ivr = ?';
	$inst = $db->query($sql, array($room, $user, $ivr));
	if ($db->isError($inst)) {
		dbug($db->getUserInfo());
	}
}

function conferencespro_sysadmin_getinfo() {
    return array(
        "name" => "Conference Pro", // Required. Descriptive name.
        "expires" => true,         // Optional. Defaults to false (never expires)
        "keyname" => "confpro_exp", // Optional. Only checked if 'expires' is true
    );
}
