<?php
// vim: set ai ts=4 sw=4 ft=php:

class Cos extends DB_Helper {

	private static $obj = false;

	private $cosfunc = "app-check-classofservce";
	private $haser = null;

	public function install() { }
	public function uninstall() { }
	public function backup() { }
	public function restore($backup) { }
	public function doTests($db) { }

	public static function create() {
		if (!self::$obj) {
			self::$obj = new Cos();
		}
		return self::$obj;
	}

	public function __construct() {
		self::$obj = $this;
		parent::__construct();
	}

	public static function isLicenced() {
		if (zend_loader_file_encoded()) {
			$lic_info = zend_loader_file_licensed();
			$lic_exp  = isset($lic_info['classofservice_exp']) ? preg_replace('/-/','',$lic_info['classofservice_exp']) : 00000000;
			if ($lic_exp > date('Ymd'))
				return true;
			// else
			return false;
		} else {
			return true;
		}
	}

	public static function isLicensed() {
		return self::isLicenced();
	}

	// We want to do dialplan stuff.
	public static function myDialplanHooks() {
		// 900 will be run last, just before extensions_additional.conf is written.
		return 900;
	}

	// We want to mangle the user and extensions page, which are part of core.
	public static function myGuiHooks() {
		return array("core");
	}

	// Which also means we need to catch POST's from those pages.
	public static function myConfigPageInits() {
		return array("extensions", "users");
	}

	public function doGuiHook(&$cc) {
		if ($_REQUEST['display'] == "extensions" || $_REQUEST['display'] == "users") {
			if (isset($_REQUEST['tech_hardware']) || $_REQUEST['extdisplay']) {
				$this->addInfoToPage($cc);
			}
		} 
		return;
	}

	// Manage $_REQUEST here.
	public function doConfigPageInit($pagename) { 
		if ($pagename == "cos") {
			if (isset($_REQUEST['action'])) {
				switch ($_REQUEST['action']) {
				case 'addnew':
					$this->addNewCOS();
					break;
				case 'change':
					$this->editCOS();
				break;
				}
			}
		} elseif ($pagename == "extensions" || $pagename == "users") {
			// Catch the POST.
			$allCos = $this->getAllCoS();
			foreach ($allCos as $id => $name) {
				if (isset($_REQUEST["cos_$id"]) && $_REQUEST["cos_$id"] == "cos_$id=assigned") {
					$this->addUserToCoS($_REQUEST['extension'], $name);
				} else {
					$this->removeUserFromCoS($_REQUEST['extension'], $name);
				}
			}
		}
	}

	public function doDialplanHook(&$ext, $engine, $priority) {
		// Called at priority 900, so we're normally last
		//
		// Generate our database lookup tables
		$this->delById('lookup-fc');
		$fcs = $this->getAllFCs();
		foreach ($fcs as $fc) {
			$this->setConfig($fc['code'], $fc['name'], 'lookup-fc');
		}

		// Look through every section of extensions
		foreach ($ext->_exts as $k => $v) {
			// And hook into it if we need to.
			$this->addHookIfRequired($k, $v, $ext);
		}

		// Now add our hook to check the permission
		$this->addCOSFunction($ext);

	}

	private function addHookIfRequired($k, $v, &$ext) {
		include "checksections.php";
	}

	private function addCOSFunction(&$ext) {
		// Add the COS Check to Dialplan.
		$app = $this->cosfunc;
		$ext->add($app, 's', '', new ext_noop('Starting COS Check'));
		$ext->add($app, 's', '', new ext_agi('cos.agi'));
		$ext->add($app, 's', '', new ext_return());

		// This is where we send 'not permitted' things. 
		$ext->add($app, 'denied', '', new ext_noop('I need to figure out a failthrough option'));
		$ext->add($app, 'denied', '', new ext_playback('im-sorry&access-denied'));
		$ext->add($app, 'denied', '', new ext_wait(1));
		$ext->add($app, 'denied', '', new ext_hangup());

		// Splice into Macro-User-CallerID.
		$ext->add('macro-user-callerid', 's', '', new ext_gosubif('$["${COS_TYPE}" != "" & "${COS_DISABLED}" != "TRUE"]', $this->cosfunc.",s,1"));
	}

	public function getAllFCs() {
		$fcs = featurecodes_getAllFeaturesDetailed();
		foreach ($fcs as $fc) {
			// In Call codes we can't do.
			if (strpos($fc['featuredescription'], "In-Call")===0 || !$fc['featureenabled']) {
				continue;
			}

			// Can't block logon or logoff
			if ($fc['featurename'] == "userlogoff" || $fc['featurename'] == "userlogon") {
				continue;
			}

			if ($fc['customcode']) {
				$code = $fc['customcode'];
			} else {
				$code = $fc['defaultcode'];
			}

			$tmparr[$code] = array("name" => $fc['featurename'], "desc" => $fc['featuredescription'], "code" => $code);
		}

		// Return it in order.
		ksort($tmparr);
		foreach ($tmparr as $fc) {
			$name = $fc['name'];
			$desc = $fc['desc'];
			$code = $fc['code'];
			$retarr[$name] =  array("name" => $name, "desc" => $desc, "code" => $code);
		}

		return $retarr;
	}

	public function getAllCOS() {
		// This is from DB_Helper
		$cos = $this->getAllids();
		if (!$cos) {
			return array();
		} // else
		$retarr = array();
		foreach ($cos as $c) {
			if (strpos($c, 'lookup') !== 0) {
				$retarr[] = $c;
			}
		}
		return $retarr;
	}

	private function addNewCOS() {
		// Explicitly been told to add a new COS
		// TODO: Check if it exists

		// Need a reload after this.
		needreload();

		if (!isset($_REQUEST['cosname']) || empty($_REQUEST['cosname'])) {
			$_REQUEST['cosname'] = 'Unnamed';
		}

		// Remove any 'nasty' chars.
		$_REQUEST['cosname'] = str_replace(array('"', '<', '>'), array('', '', ''), $_REQUEST['cosname']);

		// Santize
		$_REQUEST['cosname'] = htmlentities($_REQUEST['cosname']);

		// Feature Codes
		$fcs = $this->getAllFCs();
		$fcallow = array();
		foreach ($_REQUEST['fc_allow'] as $fc) {
			if (isset($fcs[$fc])) {
				$fcallow[$fcs[$fc]['name']] = true;
			}
		}
		$this->setConfig('fcallow', $fcallow, $_REQUEST['cosname']);

		// Routes

		// Users
		$cosusers = array();
		if (isset($_REQUEST['users_allow'])) {
			$users = $_REQUEST['users_allow'];
			foreach ($users as $user) {
				$cosusers[$user] = true;
			}
		}
		$this->setConfig('members', $cosusers, $_REQUEST['cosname']);

		// Redo the above code for each of the following...
		$items[] = array("rg_allow", "rgallow");
		$items[] = array("conf_allow", "confallow");
		$items[] = array("routes_allow", "routesallow");
		$items[] = array("queues_allow", "queuesallow");
		$items[] = array("paging_allow", "pagingallow");
		$items[] = array("vmblast_allow", "vmblastallow");

		foreach ($items as $i) {
			$tmparr = array();
			if (isset($_REQUEST[$i[0]])) {
				$allow = $_REQUEST[$i[0]];
				foreach ($allow as $a) {
					$tmparr[$a] = true;
				}
			}
			$this->setConfig($i[1], $tmparr, $_REQUEST['cosname']);
			unset($tmparr);
		}

		// Set these so we go straight to the edit page of the new
		// CoS we've created.
		$_REQUEST['cos'] = $_REQUEST['cosname'];
		$_REQUEST['action'] = 'edit';
	}

	private function addUserToCoS($user, $cos) {
		$users = $this->getConfig('members', $cos);
		$users[$user] = true;
		$this->setConfig('members', $users, $cos);
	}

	private function removeUserFromCoS($user, $cos) {
		$users = $this->getConfig('members', $cos);
		unset($users[$user]);
		$this->setConfig('members', $users, $cos);
	}

	private function editCOS() {
		// Edit. Well. Delete and then recreate.

		needreload();
		if (!isset($_REQUEST['cosname']) || empty($_REQUEST['cosname'])) {
			$_REQUEST['cosname'] = 'Unnamed';
		}
		$this->delById(htmlentities($_REQUEST['cos']));
		$this->delById(htmlentities($_REQUEST['cosname']));

		if (isset($_REQUEST['Delete'])) {
			unset($_REQUEST['cos']);
			unset($_REQUEST['cosname']);
		} else {
			// Display the correct CoS.
			$_REQUEST['cos'] = $_REQUEST['cosname'];
			$this->addNewCOS();
		}
	}

	public function addInfoToPage(&$cc) {
		$section = "Class of Service";
		// Add the info for this user to the page. We're on 'extensions' or 'users'.
		$cc->addoptlistitem('cosyn', 'assigned', _('Assigned'));
		$cc->addoptlistitem('cosyn', 'not', _('Not Assigned'));
		$assigned = $this->getCosForUser($_REQUEST['extdisplay']);
		foreach ($this->getAllCOS() as $id => $cos) {
			if (in_array($cos, $assigned)) {
				$cc->addguielem($section, new gui_radio("cos_$id", $cc->getoptlist('cosyn'), "assigned", $cos, "" , null));
			} else {
				$cc->addguielem($section, new gui_radio("cos_$id", $cc->getoptlist('cosyn'), "not", $cos, "" , null));
			}
		}
	}

	public function getApp($exten = null) {
		if ($exten === null) {
			throw new Exception("WTF. Y U Give me null?");
		}

		$app = $this->getConfig($exten, 'lookup-fc');
		if ($app) {
			return $app;
		}

		// We're here because we don't know what has been given to us. Lets try removing any leading or trailing _/.'s
		if (preg_match("/^_(.+)\.$/", $exten, $res)) {
			$app = $this->getConfig($res[1], 'lookup-fc');
			if ($app) {
				return $app;
			}
		}

		// Well. I got nothing.
		//print "Unknown feature code $exten\n";
		return false;
	}

	// Everything below here is called by the AGI
	public function checkCoS(&$agi) {
		// Called by the AGI script, to validate a Class of Service rule.
		$cidnum = $agi->request['agi_callerid'];
		if (!isset($cidnum) || !is_numeric($cidnum)) {
			$agi->verbose("CallerID not Parseable - received $cidnum - exiting with DENY",1);
			$agi->exec_goto("macro-trunk-dial", "barred", "1");
			exit;
		}
		$type = $this->getAGIvar($agi, 'COS_TYPE');
		if ($type == 'FC') {
			$this->checkCosFC($agi);
		} elseif ($type == 'QUEUE') {
			$this->checkCosQueue($agi);
		} elseif ($type == 'RINGGROUP') {
			$this->checkRingGroup($agi);
		} elseif ($type == 'CONFERENCE') {
			$this->checkConference($agi);
		} elseif ($type == 'PAGING') {
			$this->checkPaging($agi);
		} elseif ($type == 'VMBLAST') {
			$this->checkVmBlast($agi);
		} else {
			$agi->verbose("Don't know what COS type '$type' is.",1);
			$agi->exec_goto("macro-trunk-dial", "barred", "1");
		}
	}

	public function checkCosFC(&$agi) {
		$this->checkCosApp('fcallow', $agi);
	}

	public function checkCosQueue(&$agi) {
		$this->checkCosApp("queuesallow", $agi);
	}

	public function checkRingGroup(&$agi) {
		$this->checkCosApp("rgallow", $agi);
	}

	public function checkConference(&$agi) {
		$this->checkCosApp("confallow", $agi);
	}

	public function checkPaging(&$agi) {
		$this->checkCosApp("pagingallow", $agi);
	}

	public function checkVmBlast(&$agi) {
		$this->checkCosApp("vmblastallow", $agi);
	}

	private function checkCosApp($name, &$agi) {
		// Check to see if user is allowed to call queue.
		$dest = $this->getAGIvar($agi, 'COS_DEST');
		$cid = $this->getAGIvar($agi, 'AMPUSER');
		foreach ($this->getCoSforUser($cid) as $cos) {
			$agi->verbose("CoS $cos found for user $cid.", 3);
			$appallow = $this->getConfig($name, $cos);
			if ($appallow === null) {
				$agi->verbose("Bad things happened. Don't know about $cos variable $name", 0);
				exit;
			}

			if (!isset($appallow[$dest])) {
				$agi->verbose("$cid is a member of $cos, and $cos is not allowed to use $dest", 1);
				$agi->exec_goto($this->cosfunc, 'denied', 1);
				exit;
			} 
		}
		$agi->verbose("$cid granted access to $dest", 3);
		$agi->setvar("COS_TYPE", "");
		return true;
	}

	public function getCoSforUser($cid) {
		// Find the Class of Services that the User is linked to.
		$allCos = $this->getAllCOS();
		$cos = array();
		foreach ($allCos as $c) {
			$users = $this->getConfig('members', $c);
			if (isset($users[$cid])) {
				$cos[] = $c;
			}
		}
		return $cos;
	}

	private function getAGIvar(&$agi, $val) {
		$r = $agi->get_variable($val);
		if ($r['result'] == 1) {
			$result = $r['data'];
			return $result;
		}
		return '';
	}

	public function hasExtensionroutes() {
		if (function_exists('extensionroutes_islicenced')) {
			return extensionroutes_islicenced();
		} // else
		return false;
	}

	/**
	 * Hook for Extensionroutes
	 *
	 * Return an array along the lines of array('exten' => '1-2-3', 'exten' => '2-3-4');
	 */
	public function extensionroutes_list() {
		// Grab all our CoS Routes
		$allCos = $this->getAllCoS();
		foreach ($allCos as $c) {
			$cos[$c]['routesallow'] = $this->getConfig('routesallow', $c);
			$cos[$c]['users'] = $this->getConfig('members', $c);
		}

		// Build the 'allroutes' string.
		$allroutesArr = array();
		foreach (core_routing_list() as $r) {
			$allroutesArr[] = $r['route_id'];
		}
		sort($allroutesArr);
		$allroutes = implode('-', $allroutesArr);

		$usersarr = array();

		// Check each user against every CoS
		foreach (core_users_list() as $extArr) {
			$ext = (string)$extArr[0];
			$founduser = false;
			foreach ($cos as $c) {
				if (isset($c['users'][$ext])) {
					$founduser = true;
					foreach (array_keys($c['routesallow']) as $r) {
						$usersarr[$ext][$r] = true;
					}
				}
			}
			if (!$founduser) {
				$retarr[$ext] = $allroutes;
			}
		}

		// Now go through our created array, and hand it to extensionroutes in the
		// format it wants.
		foreach (array_keys($usersarr) as $ext) {
			$tmparr = array_keys($usersarr[$ext]);
			sort($tmparr);
			$retarr[$ext] = implode('-', $tmparr);
		}
		return $retarr;
	}
}
