<?php
// vim: set ai ts=4 sw=4 ft=php:

// If it's an inbound route, disable CoS checks.
if (strpos($k, 'ext-did-') === 0) {
	foreach ($v as $exten => $entries) {
		$exten = trim($exten);
		$ext->splice($k, $exten, '0', new ext_set('COS_DISABLED', 'TRUE'));
	}
	return;
}

$addMacro = false;
switch ($k) {
case 'app-pickup':
	// This one is a bit different, and hard to match automatically.
	foreach ($v as $exten => $entries) {
		$exten = trim($exten);
		$ext->splice($k, $exten, '0', new ext_set('COS_TYPE', 'FC'));
		$ext->splice($k, $exten, '0', new ext_set('COS_DEST', 'pickup'));
	}
	return;
case 'ext-intercom-users':
	// We need to check intercom-prefix.
	
case 'app-echo-test':
case 'app-dialvm':
case 'app-speakingclock':
case 'ext-test':
case 'app-blacklist':
case 'timeconditions-toggles':
case 'app-daynight-toggle':
	// These above don't currently call macro-user-callerid. Make them.
	$addMacro = true;
	// Note - no break.
case 'app-speakextennum':
case 'app-calltrace-perform':
case 'app-speeddial':
case 'app-pbdirectory':
case 'app-fmf-toggle':
case 'app-cf-busy-off':
case 'app-cf-busy-off-any':
case 'app-cf-busy-on':
case 'app-cf-off':
case 'app-cf-off-any':
case 'app-cf-on':
case 'app-cf-unavailable-off':
case 'app-cf-unavailable-on':
case 'app-cf-toggle':
case 'app-paging':
case 'app-page-stream':
case 'app-recordings':
case 'app-hotelwakeup':
case 'app-vmmain':
case 'app-dnd-off':
case 'app-dnd-on':
case 'app-dnd-toggle':
case 'app-campon-cancel':
case 'app-campon-request':
case 'app-campon-toggle':
case 'app-dictate-record':
case 'app-dictate-send':
case 'app-callwaiting-cwoff':
case 'app-callwaiting-cwon':
case 'app-zapbarge':
case 'app-chanspy':
case 'ext-intercom':
	// For all of those above, add the channel variables to the top.
	foreach ($v as $exten => $entries) {
		$exten = trim($exten);
		$app = $this->getApp($exten);
		if ($app) {
			if ($addMacro) {
				$ext->splice($k, $exten, '0', new ext_macro('user-callerid'));
			}
			$ext->splice($k, $exten, '0', new ext_set('COS_TYPE', 'FC'));
			$ext->splice($k, $exten, '0', new ext_set('COS_DEST', $app));
		}
	}
	return;
case 'ext-queues':
	// Go through all the queues we know of, and if they're in a class of service,
	// add a check.
	
	// Start by getting all CoS's...
    $allCos = $this->getAllCOS();
	// ...and all Queues (Note: Module is enabled, otherwise we wouldn't be here)
	$rawallqueues = queues_list();

	// Convert allqueues into something a little bit easier to manage
	foreach ($rawallqueues as $q) {
		$allqueues[$q[0]] = true;
	}

	// An array to hold potential queues
	$qlist = array();

	// Now, go through each CoS, looking for queues.
	foreach ($allCos as $cos) {
		// Get the queue details for this CoS
		$cosq = $this->getConfig('queuesallow', $cos);

		// Copy our allqueues..
		$tmparr = $allqueues;

		// Remove anything that's in allow
		foreach (array_keys($cosq) as $c) {
			unset($tmparr[$c]);
		}

		// Anything left is in the 'deny' list
		foreach (array_keys($tmparr) as $qnum) {
			$qlist[$qnum] = true;
		}
	}

	// At this point, qlist should contain a list of queues that need a check added to them
	foreach (array_keys($qlist) as $qnum) {
		$ext->splice($k, $qnum, '0', new ext_set('COS_TYPE', 'QUEUE'));
		$ext->splice($k, $qnum, '0', new ext_set('COS_DEST', $qnum));
	}

	return;

case 'ext-group':
	// Ring Groups. We're duplicating the above code.
    $allCos = $this->getAllCOS();
	$rawAllRingGrp = ringgroups_list();

	foreach ($rawAllRingGrp as $rg) {
		$allRgs[$rg[0]] = true;
	}

	$rglist = array();
	foreach ($allRgs as $rg) {
		$cosrg = $this->getConfig('rgallow', $cos);
		$tmparr = $allRgs;

		foreach (array_keys($cosrg) as $c) {
			unset($tmparr[$c]);
		}

		// Anything left is in the 'deny' list
		foreach (array_keys($tmparr) as $rgnum) {
			$rglist[$rgnum] = true;
		}
	}

	foreach (array_keys($rglist) as $rgnum) {
		$ext->splice($k, $rgnum, '0', new ext_set('COS_TYPE', 'RINGGROUP'));
		$ext->splice($k, $rgnum, '0', new ext_set('COS_DEST', $rgnum));
	}
	return;

case 'ext-meetme':
	// Conferences
    $allCos = $this->getAllCOS();
	$rawAll = conferences_list();

	foreach ($rawAll as $entry) {
		$all[$entry[0]] = true;
	}

	$mylist = array();
	foreach ($allCos as $cos) {
		$allowed = $this->getConfig('confallow', $cos);
		$tmparr = $all;

		foreach (array_keys($allowed) as $a) {
			unset($tmparr[$a]);
		}

		// Anything left is in the 'deny' list
		foreach (array_keys($tmparr) as $denyid) {
			$mylist[$denyid] = true;
		}
	}

	foreach (array_keys($mylist) as $item) {
		$ext->splice($k, $item, '0', new ext_set('COS_TYPE', 'CONFERENCE'));
		$ext->splice($k, $item, '0', new ext_set('COS_DEST', $item));
	}
	return;

case 'app-pagegroups':
	$allCos = $this->getAllCOS();
	$rawAll = paging_list();

	foreach ($rawAll as $entry) {
		$all[$entry[0]] = true;
	}

	$mylist = array();
	foreach ($allCos as $cos) {
		$allowed = $this->getConfig('pagingallow', $cos);
		$tmparr = $all;

		foreach (array_keys($allowed) as $a) {
			unset($tmparr[$a]);
		}

		// Anything left is in the 'deny' list
		foreach (array_keys($tmparr) as $denyid) {
			$mylist[$denyid] = true;
		}
	}

	foreach (array_keys($mylist) as $item) {
		$ext->splice($k, $item, '0', new ext_set('COS_TYPE', 'PAGING'));
		$ext->splice($k, $item, '0', new ext_set('COS_DEST', $item));
	}
	return;

case 'vmblast-grp':
	$allCos = $this->getAllCOS();
	$rawAll = vmblast_list();

	foreach ($rawAll as $entry) {
		$all[$entry[0]] = true;
	}

	$mylist = array();
	foreach ($allCos as $cos) {
		$allowed = $this->getConfig('vmblastallow', $cos);
		$tmparr = $all;

		foreach (array_keys($allowed) as $a) {
			unset($tmparr[$a]);
		}

		// Anything left is in the 'deny' list
		foreach (array_keys($tmparr) as $denyid) {
			$mylist[$denyid] = true;
		}
	}

	foreach (array_keys($mylist) as $item) {
		$ext->splice($k, $item, '0', new ext_set('COS_TYPE', 'VMBLAST'));
		$ext->splice($k, $item, '0', new ext_set('COS_DEST', $item));
	}
	return;

default:
	// Probably.. do something here.
	return;
}

