<?php /* $Id */
if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed'); }

function extensionroutes_islicenced() {
	if (zend_loader_file_encoded()) {
		$lic_info	= zend_loader_file_licensed();
		$lic_exp        = isset($lic_info['extroute_exp'])?preg_replace('/-/','',$lic_info['extroute_exp']):'00000000';
		if ($lic_exp > date('Ymd')) {
			return true;
		} else {
			return false;
		}
	} else {
		return true;
	}
}

/**
 * Hook into Configuration Generator
 * @author Philippe Lindheimer
 */
function extensionroutes_hookGet_config($engine) {
	global $ext;
	global $version;
	global $db, $astman;

	if (!extensionroutes_islicenced()) {
		return;
	}

	$ast_lt_16 = version_compare($version, '1.6', 'lt');

	switch($engine) {
		case "asterisk":

		$context = 'outbound-restricted-routes';
		$exten = '_X.';

		$ext->add($context, $exten, '', new ext_macro('user-callerid', 'LIMIT','EXTERNAL'));
		$ext->add($context, $exten, '', new ext_set('ROUTEUSER','${IF($["${EXTTOCALL}"!=""]?${EXTTOCALL}:${AMPUSER})}'));
		$ext->add($context, $exten, '',
			new ext_gotoif('$[${LEN(${ROUTEUSER})}>0 & ${DB_EXISTS(AMPUSER/${ROUTEUSER}/outroute)}]', '${DB(AMPUSER/${ROUTEUSER}/outroute)},${EXTEN},2', 'outbound-allroutes,${EXTEN},2'));

		$ext->spliceInclude('from-internal-additional', 'outbound-allroutes', '', $context, 'extensionroutes');

		$tmpcoreroutes = core_routing_list();
		if (!is_array($tmpcoreroutes)) {
			// There are no routes.
			return;
		}

		// Tidy up the core routes slightly..
		foreach ($tmpcoreroutes as $r) {
			$id = $r['route_id'];
			$name = $r['name'];
			$time_group_id = $r['time_group_id'];
			$coreroutes[$id] = array("id" => $id, "name" => $name, "time_group_id" => $time_group_id);
		}

		$delim = $ast_lt_16 ? '|' : ',';

		// get the list of extension routes in the form of:
		// [exten] => [id1-id2-id3]
		// where ids are in ascending order
		if (function_exists('cos_islicenced') && cos_islicenced()) {
			$exten_routes = Cos::create()->extensionroutes_list();
		} else {
			$exten_routes = extensionroutes_list();
		}
		$route_hash = array();

		$extens = core_users_list();
		foreach ($extens as $extenArr) {
			$exten = (string) $extenArr[0];

			if (isset($exten_routes[$exten])) {
				$rn = "restrictedroute-".$exten_routes[$exten];
			} else {
				$rn = "restrictedroute-0";
			}
			// Set the required AMPUSER object
			if ($astman) {
				$astman->database_put("AMPUSER",trim($exten)."/outroute",$rn);
			}

			// Have we already made this?
			if (isset($route_hash[$rn])) {
				continue;
			}

			// Well, now we're going to.
			$route_hash[$rn] = true;

			// Which route numbers are we adding?
			$routes = explode('-',$rn);
			array_shift($routes);

			// Ensure routes are included in the correct order.
			$routes = reorder_routes($routes);

			$ext->add($rn, 'null', '', new ext_noop('Unused Variable'));

			foreach ($routes as $route) {
				// 0 means 'nothing'
				if ($route == 0) {
					continue;
				}
				$outrt = "outrt-$route";
				$comment = $coreroutes[$route]['name'];

				if (function_exists('timeconditions_timegroups_get_times') && $coreroutes[$route]['time_group_id'] !== null) {
					$times = timeconditions_timegroups_get_times($coreroutes[$route]['time_group_id'],true);
					if (is_array($times) && count($times)) {
						foreach ($times as $time) {
							$ext->addInclude($rn,$outrt.$delim.$time[1],$comment);
						}
					} else {
						$ext->addInclude($rn,$outrt,$comment);
					}
				} else {
					$ext->addInclude($rn,$outrt,$comment);
				}
			}
		}
		break;
	}
}

/**
 * Get the list of routes
 * @author Philippe Lindheimer
 */
function extensionroutes_list() {
	global $db;
	$rh = array();

	// [exten] => [id1-id2-id3]

	$extens = core_users_list();
	if(is_array($extens)) {
		foreach($extens as $exten) {
			$routes = extensionroutes_get_routes($exten[0]);
			if (!empty($routes)) {
				// Cast exten to a string, to ensure leading
				// zeros don't get dropped.
				$rh[(string) $exten[0]] = implode('-',$routes);
			}
		}
	}
	return $rh;
}

/**
 * Get Routes allowed for said extension
 * @author Philippe Lindheimer
 * @param	int	$exten	The FreePBX Extension Number
 */
function extensionroutes_get_routes($exten) {
	global $db;

	$sql = "SELECT `route_id` FROM `extensionroutes` WHERE `exten` = '$exten' ORDER BY `route_id` ASC";
	$routes = $db->getCol($sql);
	if(DB::IsError($routes)) {
		die_freepbx($routes->getDebugInfo());
	}
	return $routes;
}

/**
 * Get Extensions for said Route
 * @author Philippe Lindheimer
 * @param	int	$route	The FreePBX Route ID
 */
function extensionroutes_get_extens($route) {
	global $db;

	$sql = "SELECT `exten` FROM `extensionroutes` WHERE `route_id` = '$route' ORDER BY `exten` ASC";
	$extens = $db->getCol($sql);
	if(DB::IsError($extens)) {
                die_freepbx($extens->getDebugInfo());
        }
	return $extens;
}

/**
 * Add/Edit/Delete Extension Routes
 * @author Philippe Lindheimer
 * @param	int	$route_id	The FreePBX Route ID
 * @param	string	$action	The action from freepbx
 * @param	array 	$extension_list	An array of allowed extensions
 */
function extensionroutes_adjustroute($route_id,$action,$extension_list='') {
  global $db;
  $dispname = 'routing';
  $route_id = $db->escapeSimple($route_id);

  switch ($action) {
  case 'delroute':
		extensionroutes_delete_route($route_id);
    break;
  case 'addroute':
    if (!empty($extension_list)) {
      // we don't have the route_id yet, it hasn't been inserted yet :(, put it in the session
      // and when returned it will be available on the redirect_standard
      $_SESSION['extensionroutesAddRoute'] = $extension_list;
    }
    break;
  case 'editroute';
		extensionroutes_edit_route($route_id, $extension_list);
    break;
  case 'copyroute': //copy route because it's pretty much the same thing as add
  case 'delayed_insert_route';
		extensionroutes_add_route($route_id, $extension_list);
    break;
  }
}

/**
 * Delete Extension Routes
 * @author Philippe Lindheimer
 * @param	int	$route_id	The FreePBX Route ID
 */
function extensionroutes_delete_route($route_id) {
	sql('DELETE FROM `extensionroutes` WHERE `route_id` = '.q($route_id));
}

/**
 * Add Extension Routes
 * @author Philippe Lindheimer
 * @param	int	$route_id	The FreePBX Route ID
 * @param	array 	$extension_list	An array of allowed extensions
 */
function extensionroutes_add_route($route_id, $extension_list) {
	global $db;

	if (!empty($extension_list)) {
		$compiled = $db->prepare("INSERT INTO `extensionroutes` (`exten`, `route_id`) VALUES (?, " . q($route_id) . ")");
		$result = $db->executeMultiple($compiled,$extension_list);
		if(DB::IsError($result)) {
			die_freepbx($result->getMessage()."failed adding/editing extensions to given route");
		}
	}
}

/**
 * Edit Extension Routes
 * @author Philippe Lindheimer
 * @param	int	$route_id	The FreePBX Route ID
 * @param	array 	$extension_list	An array of allowed extensions
 */
function extensionroutes_edit_route($route_id, $extension_list) {
	extensionroutes_delete_route($route_id);
	extensionroutes_add_route($route_id, $extension_list);
}

/**
 * Delete Extension(user) from all Routes
 * @author Philippe Lindheimer
 * @param	int	$user	The FreePBX User ID
 */
function extensionroutes_delete_user($user) {
	sql('DELETE FROM `extensionroutes` WHERE `exten` = '.q($user));
}

/**
 * Add Extension(user) to Routes
 * @author Philippe Lindheimer
 * @param	int	$user	The FreePBX User ID
 * @param	array 	$route_list	The list of allowed routes
 */
function extensionroutes_add_user($user, $route_list) {
	global $db;

	if (!empty($route_list)) {
		$compiled = $db->prepare("INSERT INTO `extensionroutes` (`exten`, `route_id`) VALUES (" . q($user) . ", ?)");
		$result = $db->executeMultiple($compiled,$route_list);
		if(DB::IsError($result)) {
			die_freepbx($result->getMessage()."failed adding/editing routes to given user");
		}
	}
}

/**
 * Ext Extension(user) to Routes
 * @author Philippe Lindheimer
 * @param	int	$user	The FreePBX User ID
 * @param	array 	$route_list	The list of allowed routes
 */
function extensionroutes_edit_user($user, $route_list) {
	extensionroutes_delete_user($user);
	extensionroutes_add_user($user, $route_list);
}

// provide hook for routing
function extensionroutes_hook_core($viewing_itemid, $target_menuid) {

	if (extensionroutes_islicenced()) {
		switch ($target_menuid) {
			case 'routing':
				if (extensionroutes_checkforcos()) {
					return "<tr><td>Class Of Service is enabled</td><td><a href='?display=cos'>Please use CoS to manage your routes.</a></td></tr>\n";
				}
				$extens = core_users_list();
				if ($viewing_itemid != "") {
					// If we are coming back from an add, we can end up with an id but have not yet added
					// the new list so use what's in the session which is what is gonna be added anyhow
					//
					if (!$_REQUEST['action'] && !empty($_SESSION['extensionroutesAddRoute'])) {
						$extens_route = $_SESSION['extensionroutesAddRoute'];
					} else {
						// If we are coming back from a duplicate, we can end up with an id but have not yet added
						// the new list so use what's in the session which is what is gonna be added anyhow
						//
						$extens_route = (isset($_SESSION['or']) && !empty($_SESSION['or'])) ? $_SESSION['or'] : extensionroutes_get_extens($viewing_itemid);
					}

				} else {
					$extens_route = array();
					foreach ($extens as $ext) {
						$extens_route[] = $ext[0];
					}
				}

				$hookhtml = '';
				if (is_array($extens)) {
					$hookhtml .= "<script>
						var head = document.getElementsByTagName('head')[0];
						var extensionroutesScript = document.createElement('script');
						extensionroutesScript.type = 'text/javascript';
						extensionroutesScript.src = 'modules/extensionroutes/assets/js/extensionroutes.js';
						head.appendChild(extensionroutesScript);
					</script>";

					//here
					foreach ($extens_route as $extR) {
	        				$hookhtml .= '<input type="hidden" name="extensionroutes_list[]" value="' . $extR . '">';
					}

					$extensions_allowed = '<ul id="extensions_allowed" class="sortable extensions ui-widget-content">';
					$extensions_blocked = '<ul id="extensions_blocked" class="sortable extensions ui-widget-content">';

					foreach ($extens as $ext) {
						switch($viewing_itemid) {
							case FALSE:
								$extensions_allowed .= '<li data-extension="' . $ext[0] . '">'
                                     	                                        . '<a href="#">'
                                     	                                        . '<span class="dragable"></span>'
                                     	                                        . '<b> ' . $ext[0]
                                     	                                        . ' (' . $ext[1] . ')</b>'
                               	                                        	. '</a>'
                        	                                		. '</li>';
								break;
							default:
								if (in_array($ext[0], $extens_route) === FALSE) {
									$extensions_blocked .= '<li data-extension="' . $ext[0] . '">'
        			                	        		         . '<a href="#">'
                			        	        		         . '<span class="dragable"></span>'
                			        	        		         . '<b> ' . $ext[0]
                        		        	        		         . ' (' . $ext[1] . ')</b>'
                        		        	                		 . '</a>'
                        		        	                		 . '</li>';
								} else {
									$extensions_allowed .= '<li data-extension="' . $ext[0] . '">'
                        	                                        		. '<a href="#">'
                        	                                        		. '<span class="dragable"></span>'
                        	                                        		. '<b> ' . $ext[0]
                        	                                        		. ' (' . $ext[1] . ')</b>'
                        	                                        		. '</a>'
                        	                                        		. '</li>';
								}
								break;
						}
					}

					$extensions_allowed .= '</ul>';
					$extensions_blocked .= '</ul>';

					$hookhtml .= '<tr class="guielToggle extensionRouteHook" data-toggle_class="ExtensionRoutes">
						        <td><span class="guielToggleBut">+  </span>Extension Routing<span class="help">?<span>Optional: Select extensions that are able to use this route.</span></span>:</td>
						     </tr>
						     <tr>
						     </tr>';
					$hookhtml .= '<tr class="ExtensionRoutes extensionRouteHook">
							<td> ' . fpbx_label(_('Allowed Extensions'), _('Drag extension from the Blocked Extensions list to allow them to use this route')) . '</td>
							<td> ' . _('Blocked Extensions') . ' </td>
						      </tr>';
					$hookhtml .= '<tr class="ExtensionRoutes extensionRouteHook" style="vertical-align:top">
							<td> ' . $extensions_allowed . ' </td>
							<td> ' . $extensions_blocked . ' </td>
						     </tr>';
				}

				return $hookhtml;
			break;
			default:
				return FALSE;
			break;

		}
	} else {
		return extensionroutes_routespage();
	}
}


function extensionroutes_hookProcess_core($viewing_itemid, $request) {
	switch ($request['display']) {
		case 'routing':
			$action = (isset($request['action']))?$request['action']:null;
			$route_id = $viewing_itemid;
			if (isset($request['Submit']) ) {
				$action = (isset($action))?$action:'editroute';
			}

			//$action wont be set on redirect so of a duplicate route so we have to store this in session storage for now
			if(isset($_REQUEST['copyroute'])) {
				$ri = isset($_REQUEST['extdisplay']) && isset($_REQUEST['copyroute']) ? $_REQUEST['extdisplay'] : '';
				$_SESSION['or'] = extensionroutes_get_extens($ri); //store in or for Original Routes :-)
			}

			// $action won't be set on the redirect but extensionroutesAddRoute will be in the session
			//
			if (!$action && !empty($_SESSION['extensionroutesAddRoute'])) {
				extensionroutes_adjustroute($route_id,'delayed_insert_route',$_SESSION['extensionroutesAddRoute']);
				unset($_SESSION['extensionroutesAddRoute']);
			} elseif(!$action && !empty($route_id) && !empty($_SESSION['or'])) {
				//We are here because we are in the 2nd stage of a duplicate outbound routes session.
				extensionroutes_adjustroute($route_id,'copyroute',$_SESSION['or']);
				//Unset session storage for cleanup!
				unset($_SESSION['or']);
			} elseif ($action){
				extensionroutes_adjustroute($route_id,$action,$request['extensionroutes_list']);
			}
			break;
		}
}

// This is called before the page is actually displayed, so we can use addguielem(). draws hook on the extensions/users page
function extensionroutes_configpageload() {
	if (extensionroutes_checkforcos()) {
		return;
	}
	global $currentcomponent;
	global $display;

	if (extensionroutes_islicenced()) {
		$extdisplay = isset($_REQUEST['extdisplay'])?$_REQUEST['extdisplay']:null;
		if ($display == 'extensions' || $display == 'users'){
			$routes = core_routing_list();
			if ($extdisplay != '') {
				$used_routes = extensionroutes_get_routes($extdisplay);
			}
			$section=_('Allowed Routes');
			$currentcomponent->addguielem($section, new gui_checkbox('checkextenroutes', false, 'Check/Uncheck All Routes', 'By checking this box it will select all extension routes, unchecking this box will uncheck all extension routes.', '1', '<br/><br/>'));

			foreach($routes as $route){
				$status = $extdisplay != '' ? in_array($route['route_id'], $used_routes) : true;
				$currentcomponent->addguielem($section, new gui_checkbox( 'extenroute|'.$route['route_id'],
											$status, $route['name'],
											_('If checked this route is accessible to this user/extension'),
											'true','','')
											);
			}
			echo "<script>document.addEventListener('DOMContentLoaded', function(ev) {
				$(\"#checkextenroutes:checkbox\").click(function() {
					$(\"input[id\^='extenroute|']\").prop('checked', $(\"#checkextenroutes:checkbox\").prop('checked'));
				});
			});</script>";
		}
	}
}

//process received arguments
function extensionroutes_configprocess() {
	$extdisplay = isset($_REQUEST['extdisplay'])?$_REQUEST['extdisplay']:null;
	if ($extdisplay == '' && !empty($_REQUEST['extension'])) {
		$extdisplay = $_REQUEST['extension'];
	}
	$display = isset($_REQUEST['display'])?$_REQUEST['display']:null;
	$action = isset($_REQUEST['action'])?$_REQUEST['action']:null;

	if ($display == 'extensions' || $display == 'users') {
		switch ($action) {
		case 'edit':
		case 'add':
			$routes=core_routing_list();
			$allowed_routes = array();
			foreach($routes as $route){
				if (isset($_REQUEST['extenroute|'.$route['route_id']]) && $_REQUEST['extenroute|'.$route['route_id']] == 'true') {
					$allowed_routes[] = $route['route_id'];
				}
			}
			extensionroutes_edit_user($extdisplay, $allowed_routes);
			break;
		case 'del':
			extensionroutes_delete_user($extdisplay);
			break;
		}
	}
}

function extensionroutes_configpageinit($pagename) {
	if (extensionroutes_checkforcos()) {
		return;
	}
	if (!extensionroutes_islicenced()) {
		extensionroutes_userpage();
	}
	global $currentcomponent;
	$display = isset($_REQUEST['display']) ? $_REQUEST['display'] : '';
	$extdisplay = isset($_REQUEST['extdisplay']) ? $_REQUEST['extdisplay'] : '';
	if ($extdisplay == '' && !empty($_REQUEST['extension'])) {
		$extdisplay = $_REQUEST['extension'];
	}
	// On a 'new' user, 'tech_hardware' is set, and there's no extension.
	if ($display == 'users' || ($display == 'extensions' && (isset($_REQUEST['tech_hardware']) || $extdisplay != ''))) {
		$currentcomponent->addprocessfunc('extensionroutes_configpageload', 1);
		$currentcomponent->addprocessfunc('extensionroutes_configprocess', 1);
	}
}

function extensionroutes_applyhooks() {
	if (extensionroutes_checkforcos() || !extensionroutes_islicenced()) {
		return;
	}
	global $currentcomponent;
	// Add the 'process' function - this gets called when the page is loaded, to hook into
	// displaying stuff on the page.
	$currentcomponent->addguifunc('extensionroutes_configpageload');
}

function extensionroutes_checkforcos() {
	if (!function_exists('cos_islicenced')) {
		return false;
	}
	return cos_islicenced();
}

/**
 * Return the routes ordered in the way they're specified in outbound routes.
 */

function reorder_routes($routes) {
	// This returns the array in the correct order
	$core = core_routing_list();

	// Reprocess into a hash so we're not searching through
	// the array for each route, and can do a fast lookup.
	foreach ($routes as $r) {
		$tmproutes[$r] = true;
	}

	// Create a new blank array.
	$newroutes = array();

	foreach ($core as $r) {
		if (isset($tmproutes[$r['route_id']])) {
			$newroutes[] = $r['route_id'];
		}
	}

	return $newroutes;
}

