<?php
$restrict_mods = true;
if (!@include_once(getenv('FREEPBX_CONF') ? getenv('FREEPBX_CONF') : '/etc/freepbx.conf')) {
	include_once('/etc/asterisk/freepbx.conf');
}

class FaxproSend{
	var $files;
	var $fax;
	var $coversheet;
	var $user;
	var $org;
	var $to;
	var $start_time;

	var $doCoverPage;
	var $destContext;
	var $is_forward;
	var $astman;
	var $amp_conf;

	//set up object; set all variables
	function __construct($user, $num, $is_local = false, $is_forward = false){
		global $amp_conf, $astman;
		ignore_user_abort(true);//Dont quit if the user browses away
		//get the amp_conf array
		$this->start_time			= time();
		$this->amp_conf 			= $amp_conf;
		$this->astman 				= $astman;
		$this->is_forward			= $is_forward;
		$this->is_local				= $is_local;
		$this->to['num']			= preg_replace('/[^\d]/', '', $num);
		$this->user['ampuser']		= $user;
		$this->user['outdir']		= faxpro_user_dir_struct($user, 'out');
		$this->tmpdir				= $this->amp_conf['ASTSPOOLDIR'] . '/tmp/fax-'
									. time() . rand(100, 999);
		$this->fax['file']			= '';//file to send
		//ensure dir exists
		mkdir($this->tmpdir, 0755, true);

		//set employee related variables
		$this->__user();

		//set organization related variable
		$this->__org();

		//use orginazation stationid if the employee doesnt have one set
		if (!$this->user['faxnum']){
			$this->user['faxnum']	= $this->org['faxnum'];
		}
		$this->fax['id']			= time() . rand(100,999);

		//get the license info
		$this->license				= sysadmin_get_license();

		//hardcoding for now, should be dynamicly set some day
		$this->filePaperSize		= 'Letter';
	}


	//called by  __construct(); sets sender (Emp=Employe) details
	function __user(){
		$set = fax_get_user($this->user['ampuser']);
		$this->user['faxnum'] = isset($set['faxstationid']) ? $set['faxstationid'] : '';
		$this->user['faxemail'] = isset($set['faxemail']) ? $set['faxemail'] : '';
		$this->user['emailresults'] = isset($set['emailresults']) ? $set['emailresults'] : '';
		$this->user['header'] = isset($set['faxheader']) ? $set['faxheader'] : '';
		$this->user['retries'] = isset($set['faxretries']) ? $set['faxretries'] : '';


		$defset = fax_get_settings();
		if ($this->user['faxnum'] == '') {
			$this->user['faxnum'] = isset($defset['localstationid']) ? $defset['localstationid'] : '';
		}
		if ($this->user['emailresults'] == '') {
			$this->user['emailresults'] = isset($defset['emailresults']) ? $defset['emailresults'] : '';
		}
		if ($this->user['header'] == '') {
			$this->user['header'] = isset($defset['headerinfo']) ? $defset['headerinfo'] : '';
		}
		if ($this->user['retries'] == '') {
			$this->user['retries'] = isset($defset['retries']) ? $defset['retries'] : '0';
		}
	}

	/**
	 * Add a file to the fax
	 * @param string - path to file
	 */
	function add_file($file) {
		if (!file_exists($file) || !is_file($file)) {
			return false;
		}
		//file type
		$type = strtolower(pathinfo($file, PATHINFO_EXTENSION));

		//create a temp name for this file
		$newfile = md5(rand(100,999999) . time());

		//copy file to our working dir
		$newfile = $this->tmpdir . '/' . $newfile . '.' . $type;
		copy($file,  $newfile);

		//convert file to tif, or delete it if we cant
		switch ($type) {
			case 'pdf':
				$newfile = fax_file_convert('pdf2tif', $newfile);
				break;
			case 'tif':
			case 'tiff':
				//nothing to do here
				break;
			case 'ps':
				$newfile = fax_file_convert('ps2tif', $newfile);
				break;
			default:
				//delete all other files, we dont know how to handle them
				unlink($newfile);
				break;
		}

		//add file to our list of files
		$this->files[] = $newfile;
	}

	/**
	 * Add a cover sheet to the fax
	 *
	 * @param string - name of receving party
	 * @param string - coverpage message
	 * @param string - name of sender
	 * @param string - phone number of sender
	 * @param string - email address of sender
	 */
	function add_coversheet($toname = '', $msg = '',  $sender = '', $tel = '', $email = '') {
		$this->to['name']		= $toname;
		$this->to['msg']		= $msg;
		$this->user['name']		= $sender;
		$this->user['tel']		= $tel;
		$this->user['email']	= $email;

		$this->doCoverPage		= true;

	}

	//called by  __construct(); sets senders' organaztion's details
	function __org(){
		$set					= fax_get_settings();
		$logo					= $this->amp_conf['ASTSPOOLDIR'] . '/faxpro/images/companylogo.jpg';
		$this->org['logo']		= file_exists($logo) 				? $logo 					: '';
		$this->org['name']		= isset($set['companyname']) 		? $set['companyname']		: '';
		$this->org['addr1']		= isset($set['companyaddress1'])	? $set['companyaddress1']	: '';
		$this->org['addr2']		= isset($set['companyaddress2'])	? $set['companyaddress2']	: '';
		$this->org['addr3']		= isset($set['companyaddress3'])	? $set['companyaddress3']	: '';
		$this->org['tel']		= isset($set['companyphone'])		? $set['companyphone']		: '';
		$this->org['www']		= isset($set['companywebsite'])		? $set['companywebsite']	: '';
		$this->org['faxnum']	= isset($set['localstationid'])		? $set['localstationid']	: '';
		$this->org['footer']	= isset($set['companyfooter'])		? $set['companyfooter']		: '';
		$this->fax['prefix']	= isset($set['prefix'])				? $set['prefix']			: '';
	}

	//genertate fax cover page
	//requires paramiters to be passed in, dose NOT pull from the calsses global scope
	function _generate_coverpage($user, $org, $to, $license, $pages, $stream = false){
		$pdfpath = dirname(__FILE__) . '/pdf';
		require($pdfpath . '/class.ezpdf.php');
		$pdf = new Cezpdf('LETTER');
		$pdf->ezSetMargins(60, 30, 60, 60);
		$pdf->selectFont($pdfpath . '/fonts/Helvetica.afm');

		//from orginzataion
		$y = 737;
		if ($org['logo']) {
			$pdf->addJpegFromFile($org['logo'], 60, $y -= 82, 250);
		}
		if ($org['name']) {
			$pdf->addText(60, $y -= 35, 20, '<b>' . $org['name'] . '</b>');
		}
		if ($org['addr1']) {
			$pdf->addText(60, $y -= 15, 15, $org['addr1']);
		}
		if ($org['addr2']) {
			$pdf->addText(60, $y -= 15, 15, $org['addr2']);
		}
		if ($org['addr3']) {
			$pdf->addText(60, $y -= 15, 15, $org['addr3']);
		}
		if ($org['tel']) {
			$pdf->addText(60, $y -= 15, 15, $org['tel']);
		}
		if ($org['www']) {
			$pdf->addText(60, $y -= 15, 15, $org['www']);
		}

		//from person
		$y = 440;
		$pdf->rectangle(60, $y, 495, 75);
		$pdf->filledRectangle(60, $y + 52, 66, 23);
		$pdf->setColor(255,255,255);
		$pdf->addText(70, $y + 59, 15, '<i><b>FROM:</b></i>');
		$pdf->setColor(0,0,0);

		$y = 500;
		if ($user['name']) {
			$pdf->addText(136, $y, 15, '<b>' . $user['name'] . '</b>'); $y -= 17;
		}
		if ($user['faxnum']) {
				$pdf->addJpegFromFile($pdfpath .'/fax.jpg', 136, $y, 15);
				$pdf->addText(156, $y += 2, 15, $user['faxnum']);
		}
		if ($user['tel']) {
			$pdf->addJpegFromFile($pdfpath . '/phone.jpg', 136, $y -= 17, 15);
			$pdf->addText(156, $y += 2, 15, $user['tel']);
		}
		if ($user['email']) {
			$pdf->addJpegFromFile($pdfpath . '/email.jpg', 136, $y -= 18, 15);
			$pdf->addText(156,$y+=3,15,$user['email']);
		}

		// data/pages/FAX
		$data = array(array('key' => '<i><b>Date:</b></i>', 'value' => date('Y-M-d')),
					array('key' => '<i><b>Page:</b></i>', 'value' => '1 of ' . $pages ));
		$settings = array('showLines'	=> 0,
						'fontSize'		=> 15,
						'xPos'			=> 'right',
						'xOrientation'	=> 'left',
						'shaded'		=> 0,
						'showHeadings'	=> 0
					);
		$pdf->ezTable($data, '', '', $settings);
		$pdf->ezText('<i><b>FAX</b></i>',70,array('justification' => 'right'));

		//for
		$y = 430;
		$pdf->rectangle(60,$y-52,495,42);
		$pdf->filledRectangle(60,$y-33,52,23);
		$pdf->setColor(255,255,255);
		$pdf->addText(68,$y-27,15,'<i><b>FOR:</b></i>');
		$pdf->setColor(0,0,0);
		if ($to['name']) {
			$pdf->addText(120,$y-=25,15,'<b>'.$to['name'].'</b>');
		}
		$pdf->addJpegFromFile($pdfpath . '/fax.jpg', 120, $y -= 25, 15);
		$pdf->addText(140,$y+=2,15,$to['num']);


		//Message
		$y = 360;
		$pdf->rectangle(60,$y-180,495,180);
		$pdf->filledRectangle(60,$y-23,95,23);
		$pdf->setColor(255,255,255);
		$pdf->addText(68,$y-16,15,'<i><b>MESSAGE:</b></i>');
		$pdf->setColor(0,0,0);
		$pdf->ezSetY($y-25);
		$pdf->ezSetMargins(80,30,80,60);
		$pdf->ezText(substr($to['msg'],0,1340),10);
		$pdf->ezSetMargins(60,30,60,60);

		//Footer
		/*
		 * schmooze logo - removing for production use
		 * $pdf->addJpegFromFile($amp_conf['AMPWEBROOT'].'/admin/modules/faxpro/pdf/pbxact.jpg',150,82,90);
		 * $pdf->addJpegFromFile($amp_conf['AMPWEBROOT'].'/admin/modules/faxpro/pdf/schmooze.jpg',325,82,180);
		 * $pdf->addText(130,75,8,'<i>The turnkey, enterprise-class, IP PBX</i>');
		 * $pdf->ezSetY(60);
		 * $pdf->ezText('<i>'.chr(169).' '.date('Y').' Schmooze Communications, Ltd. '.chr(149).' www.schmoozecom.com</i>',8,array('justification' => 'center'));
		*/
		$y=175;
		$pdf->ezSetY($y - 20);
		$pdf->ezText(substr($org['footer'], 0, 500), 10);
		$pdf->ezSetY(60);

		if ($stream) {
			$pdf->ezStream(); //return a file to the browser
		} else {
			return $pdf->ezOutput();
		}

	}


	/**
	 * Add, concatenate and prepare files
	 * Then copy a pdf to the users directory, and return a tif to be sent out
	 */
	function _prepare_files() {
		//fail if we dont have any files
		if (!isset($this->files)) {
			return false;
		}

		$this->fax['file'] = $this->amp_conf['ASTSPOOLDIR'] . '/tmp/fax-' . time() . rand(100, 999) . '.tif';

		//concatenate files
		exec(fpbx_which('tiffcp') . ' '
				. implode(' ', $this->files) . ' '
				. $this->fax['file']);


		//get number of pages - used in coverpage and to update db status
		$this->fax['pages'] = fax_tiffinfo($this->fax['file'], 'Page Number');

		//add a coversheet if requested
		if ($this->doCoverPage) {
			$this->fax['pages']++;
			$coverPagePath = $this->tmpdir . '/' . md5(rand(100,999999).time()) . '.pdf';
			file_put_contents($coverPagePath,
						$this->_generate_coverpage($this->user,
							$this->org,
							$this->to,
							$this->license,
							$this->fax['pages'])
			);

			$coverPagePath = fax_file_convert('pdf2tif', $coverPagePath);

			//concatenate coversheet & fax
			system(fpbx_which('tiffcp') . ' ' . $coverPagePath . ' ' . $this->fax['file'] . ' ' . $this->fax['file'] . '.tmp');
			rename($this->fax['file'] . '.tmp', $this->fax['file']);
		}

		//copy a pdf of the file to the users dir
		$pdf = $this->user['outdir'] . '/' . pathinfo($this->fax['file'], PATHINFO_FILENAME) . '.pdf';
		$this->fax['pdffile'] = fax_file_convert('tif2pdf', $this->fax['file'], $pdf, true);

		return $this->fax['file'];

	}

	//call asterisk to send fax
	function send_fax($verbose  = true){
		if (version_compare($this->amp_conf['ASTVERSION'] , '1.6', 'ge')) {
			$seperator = ',';
		} else {
			$seperator = '|';
		}

		if ($verbose){
			echo 'Gathering information...' . PHP_EOL;
		}

		//ensure that we have a proper license
		if (zend_loader_file_encoded()) {
			$lic_exp	= preg_replace('/-/','',$this->license['xfax_exp']);
			if ($lic_exp < date('Ymd')) {
				$this->report('Invalid Faxpro License');
				if ($verbose){
					echo _('Invalid Faxpro License') . PHP_EOL;
				}
				return false;
			}
		}

		//
		$this->destContext	= $this->is_local ? 'ext-fax' : 'from-internal';

		$this->fax['file'] = $this->_prepare_files();

		//ensure that we have all the data needed
		if (!$this->to['num']) {
			$this->report('Missing destination number.');
			if ($verbose) {
				echo _('Missing destination number.') . PHP_EOL;
			}
			return false;
		}

		if (!$this->fax['file']) {
			$this->report('Could not find file to send');
			if ($verbose) {
				echo _('Could not find file to send') . PHP_EOL;
			}
			return false;
		}

		if (!$this->astman) {
			$this->report('Could not connect to asterisk');
			if ($verbose) {
				echo _('Could not connect to asterisk.') . PHP_EOL;
			}
			return false;
		}



		$faxstats = $this->faxstats();

		//update db with a status of 'no answer'. This is the default update, we will add a new update when we have fresh data
		$this->report('Failure: No Answer');

		$retry = true;
		$success = false;
		$tries = 0;
		while ($retry) {
			$det = faxpro_get_fax_details($this->fax['id']);

			switch ($det['status']) {
			case "Fax successfully queued":
			case "In Progress":
				sleep(45);
				break;
			case "OK":
				$success = true;
				$retry = false;
				break;
			default:
				if ($tries > $this->user['retries']) {
					$retry = false;
					break;
				}

				if ($tries > 0) {
					$this->report('Retrying (' . $tries . ' of ' . $this->user['retries'] . ')...');
				}

				$tries++;

				//The number we should be using in the originate should change based off
				//	of if it's a local call or not, as we don't need a prefix locally.
				$number_to_dial = $this->is_local ? $this->to['num'] : $this->fax['prefix'].$this->to['num'];

				//send fax
				$o = $this->astman->Originate(
					array(
						'Channel' => 'Local/' . $number_to_dial . '@' . $this->destContext,
						'Exten' => 's',
						'Context' => 'send-fax',
						'Priority' => '1',
						'CallerID' => $this->user['faxnum'],
						'Variable' => array(
							'file' => $this->fax['file'],
							'TO' => $this->to['num'] . $seperator,
							'AMPUSER' => $this->user['ampuser'] . $seperator,
							'FAXOPT(headerinfo)' => 'FROM:'
								. $this->user['header']
								. (($this->user['faxnum'])
									? ' FAX:' . $this->user['faxnum']
									: ''),
							'faxid' => $this->fax['id'],
							'FAXOPT(localstationid)' => $this->user['faxnum'],
							'DIR' => ($this->is_forward ? 'forward' : 'out')
						)
					)
				);

				if ($o['Message'] == 'Originate successfully queued') {
					if ($verbose) {
						echo 'Fax successfully queued' . PHP_EOL;
					}
					$this->report('Fax successfully queued');
				} else {
					if ($verbose) {
						echo 'Unknown error (' .$o['Message'] . ')' . PHP_EOL;
					}
					$this->report('Unknown error (' .$o['Message'] . ')');
				}
			}
		}

		$sendemail = false;
		switch ($this->user['emailresults']) {
		case '':
		case 'never':
			break;
		case 'always':
			$sendemail = true;
			break;
		case 'success':
			if ($success) {
				$sendemail = true;
			}
			break;
		case 'failure':
			if (!$success) {
				$sendemail = true;
			}
			break;
		}

		if ($sendemail && isset($this->user['faxemail'])) {
			$cmd = $this->amp_conf['ASTVARLIBDIR'] . '/bin/fax2mail.php';
			$cmd.= ' --direction outgoing';
			$cmd.= ' --to "' . $this->user['faxemail'] . '"';
			$cmd.= ' --dest "' . $this->to['num'] . '"';
			$cmd.= ' --file ' . $this->fax['file'];
			$cmd.= ' --exten ' . $this->user['ampuser'];
			$cmd.= ' --status "' . $det['status'] . '"';
			exec($cmd);
		}

		//remove temp dir
		rrmdir($this->tmpdir);

		return true;
	}


	//update db with fax details before sending
	function report($status){
		$data = array(
				'faxid'		=> $this->fax['id'],
				'user' 		=> $this->user['ampuser'],
				'dir'		=> ($this->is_forward ? 'forward' : 'out'),
				'date'		=> $this->start_time,
				'callid'	=> $this->user['ampuser'],
				'dest'		=> $this->to['num'],
				'status'	=> $status,
				'pages'		=> $this->fax['pages'],
				'file'		=> $this->fax['pdffile'],
				'new'		=> 'no',
				'finished'	=> false
		);
		faxpro_put_fax_details($data);
	}

	function faxstats(){
		$cmd = $this->astman->send_request('Command', array('Command' => 'fax show stats'));
		foreach(explode("\n",$cmd['data']) as $data){
			$d = explode(':',$data);
			$ret[trim($d['0'])] = isset($d['1']) ? trim($d['1']) : null;
		}
		return $ret;
	}

//end class
}
?>
