<?php
/**
 * This is the User Control Panel Object.
 *
 * Copyright (C) 2013 Schmooze Com, INC
 * Copyright (C) 2013 Andrew Nagy <andrew.nagy@schmoozecom.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package   FreePBX UCP BMO
 * @author   Andrew Nagy <andrew.nagy@schmoozecom.com>
 * @license   AGPL v3
 */
namespace UCP\Modules;
use \UCP\Modules as Modules;

class Faxpro extends Modules{
	protected $module = 'Faxpro';
	private $data = array();
	private $faxes = array();
	private $fuser = false;

	function __construct($Modules) {
		$this->Modules = $Modules;
		$this->faxpro = $this->UCP->FreePBX->Faxpro;
		$user = $this->UCP->User->getUser();
		$this->fuser = $this->faxpro->getUserDetails($user['default_extension']);
		$this->type = $this->UCP->FreePBX->Fax->faxDetect();
		$this->settings = $this->faxpro->getSettings();
		$this->usagePercentage = number_format((100/$this->settings['maxpages']) * $this->faxpro->getUserUsage($user['default_extension']), 0);
		if($this->fuser['faxenabled'] == 'true') {
			$faxes = $this->faxpro->getUserStorageInfo($user['default_extension']);
			foreach($faxes as $fax) {
				if($fax['dir'] == 'out' || $fax['dir'] == 'forward') {
					//outbound
					if(empty($fax['finished'])) {
						$this->faxes['out']['out'][] = $fax;
					} else if($fax['status'] == 'OK') {
						$this->faxes['out']['sent'][] = $fax;
					} else {
						$this->faxes['out']['failed'][] = $fax;
					}
				} elseif($fax['dir'] == 'in') {
					//inbound
					if($fax['new'] == 'yes') {
						$this->faxes['in']['new'][] = $fax;
					} else {
						$this->faxes['in']['archive'][] = $fax;
					}
				}
			}
		}
	}

	function getDisplay() {
		$html = '';
		$displayvars = array();
		$html .= $this->loadLESS();
		$html .= $this->load_view(__DIR__.'/views/header.php',$displayvars);


		$displayvars['folders'] = array(
			array(
				'folder' => 'new',
				'name' => _('Inbox'),
				'count' => $this->getNewFaxes()
			),
			array(
				'folder' => 'archive',
				'name' => _('Archived'),
				'count' => count($this->getAllFaxesByType('archive'))
			),
			array(
				'folder' => 'out',
				'name' => _('Outgoing'),
				'count' => count($this->getAllFaxesByType('out'))
			),
			array(
				'folder' => 'sent',
				'name' => _('Sent'),
				'count' => count($this->getAllFaxesByType('sent'))
			),
			array(
				'folder' => 'failed',
				'name' => _('Failed'),
				'count' => count($this->getAllFaxesByType('failed'))
			)
		);

		$view = !empty($_REQUEST['view']) ? $_REQUEST['view'] : '';

		$displayvars['maxaction'] = $this->settings['maxaction'];
		$displayvars['usage'] = $this->usagePercentage;
		$displayvars['user'] = $this->fuser;
		switch($view) {
			case "send":
				$mainDisplay= $this->load_view(__DIR__.'/views/send.php',$displayvars);
				$displayvars['activeList'] = 'send';
			break;
			case "settings":
				$displayvars['faxattachformats'] = array(
					"pdf" => "PDF",
					"tiff" => "TIFF",
					"both" => _("both")
				);
				$displayvars['retries'] = array(
					0 => 0,
					1 => 1,
					2 => 2,
					3 => 3
				);
				$displayvars['emailresults'] = array(
					"never" => _("Never"),
					"always" => _("Always"),
					"success" => _("On Success"),
					"failure" => _("On Failure")
				);
				$mainDisplay= $this->load_view(__DIR__.'/views/settings.php',$displayvars);
				$displayvars['activeList'] = 'settings';
			break;
			case "folder":
			default:
				$reqFolder = !empty($_REQUEST['folder']) ? $_REQUEST['folder'] : 'new';
				$displayvars['folder'] = $reqFolder;
				$displayvars['faxes'] = $this->getAllFaxesByType($reqFolder);
				$mainDisplay = $this->load_view(__DIR__.'/views/mailbox.php',$displayvars);
				$displayvars['activeList'] = $reqFolder;
			break;
		}

		$html .= $this->load_view(__DIR__.'/views/nav.php',$displayvars);
		$html .= $mainDisplay;
		$html .= $this->load_view(__DIR__.'/views/footer.php',$displayvars);
		return $html;
	}

    function poll() {
		if(empty($this->fuser)) {
			return array("status" => false);
		}

		$folders = array(
			array(
				'folder' => 'new',
				'count' => $this->getNewFaxes()
			),
			array(
				'folder' => 'archive',
				'count' => count($this->getAllFaxesByType('archive'))
			),
			array(
				'folder' => 'out',
				'count' => count($this->getAllFaxesByType('out'))
			),
			array(
				'folder' => 'sent',
				'count' => count($this->getAllFaxesByType('sent'))
			),
			array(
				'folder' => 'failed',
				'count' => count($this->getAllFaxesByType('failed'))
			)
		);
		$ofaxes = array();
		foreach($this->getAllFaxesByType('out') as $fax) {
			$ofaxes[$fax['faxid']] = $fax;
		}

		$nfaxes = array();
		foreach($this->getAllFaxesByType('new') as $fax) {
			$fax['time'] = date('h:m:sa',$fax['date']);
			$fax['date'] = date('Y-m-d',$fax['date']);
			$nfaxes[$fax['faxid']] = $fax;
		}
		return array("status" => true, "new" => $this->getNewFaxes(), "activeFaxes" => $ofaxes, "newFaxes" => $nfaxes, "folderCounts" => $folders, "usage" => $this->usagePercentage);
	}

	/**
	 * Determine what commands are allowed
	 *
	 * Used by Ajax Class to determine what commands are allowed by this class
	 *
	 * @param string $command The command something is trying to perform
	 * @param string $settings The Settings being passed through $_POST or $_PUT
	 * @return bool True if pass
	 */
	function ajaxRequest($command, $settings) {
		if(empty($this->fuser)) {
			return false;
		}
		switch($command) {
			case 'view':
			case 'download':
			case 'upload':
			case 'save':
			case 'sendfax':
			case 'deleteattachment':
			case 'deletefax':
			case 'forward':
				return true;
			default:
				return false;
			break;
		}
	}

	/**
	 * The Handler for all ajax events releated to this class
	 *
	 * Used by Ajax Class to process commands
	 *
	 * @return mixed Output if success, otherwise false will generate a 500 error serverside
	 */
	function ajaxHandler() {
		$return = array("status" => false, "message" => "");
		switch($_REQUEST['command']) {
			case 'forward':
				$user = $this->UCP->User->getUser();
				$fax = $this->faxpro->getFaxDetails($_POST['id']);
				$local = ($_POST['local'] == 'true') ? true : false;
				$this->faxpro->sendBackgroundFax($user['default_extension'], $_POST['dest'], array($fax['file']), array(),$local,true);
				return array("status" => true);
			break;
			case 'deletefax':
				$this->faxpro->deleteFax($_POST['id']);
				return array("status" => true);
			break;
			case 'sendfax':
				$user = $this->UCP->User->getUser();
				$coversheet = array();
				if($_POST['coversheet'] == 'true') {
					$coversheet = array(
						"name" => $_POST['rname'],
						"msg" => $_POST['msg'],
						"sender" => $_POST['faxcovername'],
						"tel" => $_POST['faxcovertel'],
						"email" => $_POST['faxcoveremail']
					);
				}
				$this->faxpro->sendBackgroundFax($user['default_extension'], $_POST['destination'], $_POST['files'], $coversheet);
				return array("status" => true);
			break;
			case "deleteattachment":
				$out = $this->faxpro->deleteAttachment($_REQUEST['name']);
				if(!$out) {
					return array("status" => false, "message" => _("Unable to delete file"));
				} else {
					return array("status" => true);
				}
				break;
			case "upload":
				foreach ($_FILES["newfax"]["error"] as $key => $error) {
					if ($error == UPLOAD_ERR_OK) {
						$extension = pathinfo($_FILES["newfax"]["name"][$key], PATHINFO_EXTENSION);
						if($extension == 'pdf') {
							$tmp_name = $_FILES["newfax"]["tmp_name"][$key];
							$dname = $_FILES["newfax"]["name"][$key];
							$id = time();
							$name = pathinfo($_FILES["newfax"]["name"][$key],PATHINFO_FILENAME) . '-' . $id . '.' . $extension;
							if(!file_exists(__DIR__."/tmp")) {
								if(!mkdir(__DIR__."/tmp")) {
									return array("status" => false, "message" => _("Cant Create Temp Directory"));
								}
							}
							move_uploaded_file($tmp_name, __DIR__."/tmp/$name");
							$contents = file_get_contents(__DIR__."/tmp/$name");
							unlink(__DIR__."/tmp/$name");
							$user = $this->UCP->User->getUser();
							$this->faxpro->saveAttachment($user['default_extension'],$id,$name,$contents);
							return array("status" => true, "filename" => $dname, "localfilename" => $name, "id" => $id);
						} else {
							return array("status" => false, "message" => _("unsupported file format"));
							break;
						}
					}
				}
				return array("status" => false, _("Can Not Find Uploaded Files"));
			case 'save':
				$user = $this->UCP->User->getUser();
				$faxstore = ($_POST['faxlocalstore'] == 'true') ? 'true' : '';
				$emailresults = empty($_POST['emailresults']) ? '' : $_POST['emailresults'];
				$faxretries = ($_POST['faxretries'] == 'null') ? '' : $_POST['faxretries'];
				$this->faxpro->putUserDetails($user['default_extension'],$faxstore, $_POST['faxstationid'],$_POST['faxheader'],$_POST['faxcovername'],$_POST['faxcovertel'],$_POST['faxcoveremail'],$_POST['faxattachformat'],$emailresults,$faxretries);
				return array("status" => true);
			default:
				return false;
			break;
		}
		return $return;
	}

	/**
	 * The Handler for quiet events
	 *
	 * Used by Ajax Class to process commands in which custom processing is needed
	 *
	 * @return mixed Output if success, otherwise false will generate a 500 error serverside
	 */
	function ajaxCustomHandler() {
		switch($_REQUEST['command']) {
			case "view":
			case "download":
				$user = $this->UCP->User->getUser();
				$download = ($_REQUEST['command'] == 'download') ? true : false;
				$this->readRemoteFile($_REQUEST['faxid'],$user['default_extension'],$download);
				return true;
			break;
			default:
				return false;
			break;
		}
		return false;
	}

	public function getAllFaxesByType($type) {
		if(isset($this->faxes['in'][$type])) {
			return !empty($this->faxes['in'][$type]) ? $this->faxes['in'][$type] : array();
		} elseif(isset($this->faxes['out'][$type])) {
			return !empty($this->faxes['out'][$type]) ? $this->faxes['out'][$type] : array();
		}
		return array();
	}

	public function getNewFaxes() {
		return !empty($this->faxes['in']['new']) ? count($this->faxes['in']['new']) : 0;
	}

	public function getMenuItems() {
		if(empty($this->fuser)) {
			return false;
		}
		$menu = array(
			"rawname" => "faxpro",
			"name" => "Fax",
			"badge" => $this->getNewFaxes()
		);
		return $menu;
	}

	private function readRemoteFile($faxid,$ext,$download=false) {
		if(empty($this->fuser)) {
			return false;
		}
		$fax = $this->faxpro->getUserStorageInfo($ext,$faxid);
		$fax = !empty($fax) && isset($fax[0]) ? $fax[0] : false;
		if(!empty($fax) && $fax['file_exists'] && !empty($fax['length'])) {
			header("Cache-Control: no-cache, must-revalidate"); // HTTP/1.1
			header("Expires: Sat, 26 Jul 1997 05:00:00 GMT"); // Date in the past
			$size   = $fax['length']; // File size
			$length = $size;           // Content length
			$start  = 0;               // Start byte
			$end    = $size - 1;       // End byte
			$format = pathinfo($fax['file'],PATHINFO_EXTENSION);
			$filename = pathinfo($fax['file'],PATHINFO_FILENAME) . '.' . $format;

			header('Content-Description: File Transfer');
			header("Content-Transfer-Encoding: binary");
			$ct = null;
			switch($format) {
				case "pdf":
					$ct = "application/pdf";
				break;
			}
			header('Content-Type: '.$ct);
			header("Accept-Ranges: 0-".$size);
			if (isset($_SERVER['HTTP_RANGE'])) {
				$c_start = $start;
				$c_end   = $end;

				list(, $range) = explode('=', $_SERVER['HTTP_RANGE'], 2);
				if (strpos($range, ',') !== false) {
					header('HTTP/1.1 416 Requested Range Not Satisfiable');
					header("Content-Range: bytes $start-$end/$size");
					exit;
				}
				if ($range == '-') {
					$c_start = $size - substr($range, 1);
				}else{
					$range  = explode('-', $range);
					$c_start = $range[0];
					$c_end   = (isset($range[1]) && is_numeric($range[1])) ? $range[1] : $size;
				}
				$c_end = ($c_end > $end) ? $end : $c_end;
				if ($c_start > $c_end || $c_start > $size - 1 || $c_end >= $size) {
					header('HTTP/1.1 416 Requested Range Not Satisfiable');
					header("Content-Range: bytes $start-$end/$size");
					exit;
				}
				$start  = $c_start;
				$end    = $c_end;
				$length = $end - $start + 1;
				header('HTTP/1.1 206 Partial Content');
			} else {
				header("HTTP/1.1 200 OK");
			}

			header("Content-Range: bytes $start-$end/$size");
			header('Content-length: ' . $size);
			$dtype = ($download) ? 'attachment;' : '';
			header('Content-Disposition: '.$dtype.'filename="' . $filename.'"');
			$buffer = 1024 * 8;
			$wstart = $start;
			ob_end_clean();
			ob_start();
			while(true) {
				$content = $this->faxpro->readFaxBinaryByMessageIDExtension($faxid,$ext,$wstart,$buffer);
				if(!$content) {
					break;
				}
				echo $content;
				ob_flush();
				flush();
				$wstart = $wstart + $buffer;
				set_time_limit(0);
			}
			exit;
		} else {
			header("HTTP/1.0 404 Not Found");
			echo _("File Not Found");
			exit;
		}
	}
}
