<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

class Alerts {

	public function __construct($ha = null) {
		if (!is_object($ha))
			throw new Exception("I need a FreepbxHA Object");
		$this->ha = $ha;
	}

	public function getExtern($resource = null) {
		if ($resource === null) 
			throw new Exception("No resource given to me");

		try { 
			$ret = $this->ha->cman->getResource($resource); 
		} catch (Exception $e) {
			// Resource doesn't exist
			return array('dest' => " ");
		}
		if (!isset($ret['attributes']['extra_options']))
			throw new Exception("Unknown configuration of $resource");

		if (!preg_match('/-e (.+)"$/', $ret['attributes']['extra_options'], $out))
			throw new Exception("Unable to parse configuration of $resource");

		return array('dest' => $out[1]);
	}

	public function checkLocal() {
		if (!file_exists("/usr/local/asterisk/clustermon.sh"))
			$this->ha->incron->updateFiles();
		$mystat = stat("/usr/local/asterisk/clustermon.sh");

		// Always check it's executable, as if it's not it will crash the machine.
		if ($mystat['mode'] != 33261)
			return false;

		return $mystat['size'];
	}

	public function checkRemote() {
		$stat = $this->ha->ssh->runOnOtherNode("stat -t /usr/local/asterisk/clustermon.sh");
		if ($stat['ret'] != 0) {
			$this->ha->ssh->copyToOtherNode("/usr/local/asterisk/clustermon.sh");
			$stat = $this->ha->ssh->runOnOtherNode("stat -t /usr/local/asterisk/clustermon.sh");
		}

		$otherstat = explode(' ', $stat['result'][0]);
		if ($otherstat[3] != "81ed") {
			return false;
		}
		return $otherstat[1];
	}

	private function removeResource($res) {
		return $this->ha->cman->pcsCommand("resource delete $res");
	}

	public function removeSMTP() {
		return $this->removeResource("ClusterMon-SMTP");
	}

	public function updateSMTP($addr) {
		$addr = trim($addr);
		if (preg_match('/ /', $addr))
			throw new Exception("Please remove all spaces between email addresses (use commas)");

		$this->removeSMTP();
		return $this->ha->cman->pcsCommand("resource create ClusterMon-SMTP --clone ClusterMon user=root extra_options='-W -E /usr/local/asterisk/clustermon.sh -e $addr'");
	}

	public function removeSNMP() {
		return $this->removeResource("ClusterMon-SNMP");
	}

	public function updateSNMP($addr) {
		$addr = trim($addr);
		if (!filter_var($addr, FILTER_VALIDATE_IP))
			throw new Exception("Please enter a single IPv4 or IPv6 Address");

		$this->removeSNMP();
		return $this->ha->cman->pcsCommand("resource create ClusterMon-SNMP --clone ClusterMon user=root extra_options='-W -E /usr/local/asterisk/clustermon.sh -e $addr'");
	}
}
