<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

class checks {

	// These are the links that should exist on both machines.
	public $links = array(
		// Asterisk
		"/var/log/asterisk" => "/drbd/asterisk/log",
		"/var/spool/asterisk" => "/drbd/asterisk/spool",
		"/var/lib/asterisk" => "/drbd/asterisk/lib",
		"/opt/isymphony" => "/drbd/asterisk/isymphony",
		"/etc/asterisk" => "/drbd/asterisk/etcasterisk",
		// Apache
		"/var/lib/php/session" => "/drbd/httpd/session",
		"/etc/httpd" => "/drbd/httpd/etc",
		"/var/www" => "/drbd/httpd/www",
		"/tftpboot" => "/drbd/httpd/tftpboot",
		// MySQL
		"/var/lib/mysql" => "/drbd/mysql",
	);

	public function __construct() {
		$this->ha = new freepbxha;
	}
	public function getChecks() {
		$r = array(
			"checknodeint" => "Check connectivity (internal)",
			"sshkeys" => "Check SSH keys",
			"checkmykey" => "Verify this node's SSH Key",
			"sshto" => "Check SSH <strong><i>to</i></strong> other node",
			"checkotherkey" => "Verify other node's SSH Key",
			"sshfrom" => "Check SSH <strong><i>from</i></strong> other node",
			"checknodeext" => "Check connectivity (external)",
			"checkuids" => "Checking Asterisk UIDs",
			"sizes" => "Validate LVM Sizes",
			"mysymlinks" => "Checking DRBD Symlinks on this node",
			"othersymlinks" => "Checking DRBD Symlinks on other node",
		);
		return $r;
	}

	public function checknodeint() {
		if (!$this->ha->net->ping($this->ha->getConfig("PRIMARY_intip"))) {
			return array("status" => "warning", "message" => "Unable to ping Primary Internal IP (".$ha->getConfig("PRIMARY_intip").")");
		}
		if (!$this->ha->net->ping($this->ha->getConfig("SECONDARY_intip"))) {
			return array("status" => "warning", "message" => "Unable to ping Secondary Internal IP (".$ha->getConfig("SECONDARY_intip").")");
		}
		return array("status" => "ok");
	}

	public function sshkeys() {
		$pkey = $this->ha->getConfig("PRIMARY_publickey");
		$skey = $this->ha->getConfig("SECONDARY_publickey");
		if (!$pkey) {
			return array("status" => "warning", "message" => "Primary SSH Key empty");
		}
		if (!$skey) {
			return array("status" => "warning", "message" => "Secondary SSH Key empty");
		}
		if ($pkey === $skey) {
			return array("status" => "warning", "message" => "Primary and Secondary SSH keys are the same");
		}
		return array("status" => "ok");
	}

	public function checkmykey() {
		$iam = $this->ha->whichMachineIsThis();
		$mykey = $this->ha->getConfig($iam."_publickey");
		$fskey = $this->ha->ssh->getSSHKey();
		if ($mykey != $fskey) {
			$this->ha->setConfig($iam."_publickey", $fskey);
			return array("status" => "warning", "message" => "My SSH Key was wrong - Fixed.");
		}
		return array("status" => "ok");
	}

	public function sshto() {
		$res = $this->ha->ssh->runOnOtherNode("echo thisworked");
		if ($res['ret'] == 0 && $res['result'][0] == "thisworked") {
			return array("status" => "ok");
		}
		if ($res['ret'] == 255) {
			return array("status" => "warning", "message" => "Unable to SSH - authorized_keys error on other node?");
		}

		return array("status" => "failed", "message" => "Unknown error - result is: ".json_encode($res));
	}

	public function checkotherkey() {
		$res = $this->ha->ssh->runOnOtherNode("cat ".$this->ha->ssh->pub);
		if ($res['ret'] != 0) {
			return array("status" => "failed", "message" => "Unknown error - result is: ".json_encode($res));
		}
		$realkey = $res['result'][0];
		$dbkey = $this->ha->getConfig($this->ha->whichMachineIsOther()."_publickey");
		if ($dbkey != $realkey) {
			$this->ha->setConfig($this->ha->whichMachineIsOther()."_publickey", $realkey);
			$this->ha->ssh->injectSSHKey($this->getConfig('PRIMARY_publickey'));
			$this->ha->ssh->injectSSHKey($this->getConfig('SECONDARY_publickey'));
			return array("status" => "warning", "message" => "Other SSH Key was wrong - Fixed.");
		}
		return array("status" => "ok");
	}


	public function sshfrom() {
		$iam = $this->ha->whichMachineIsThis();
		if ($iam == "PRIMARY") {
			$me = "freepbx-a";
		} else {
			$me = "freepbx-b";
		}

		$ssh = "ssh -q -o ConnectTimeout=1 -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -o PreferredAuthentications=publickey -i ".$this->ha->ssh->priv." asterisk@$me ";
		$res = $this->ha->ssh->runOnOtherNode("$ssh hostname");
		if ($res['ret'] == 0 && $res['result'][0] == $me) {
			return array("status" => "ok");
		}
		return array("status" => "warning", "message" => "Unable to SSH from other node back to me. ".json_encode($res));
	}

	public function checknodeext() {
		if (!$this->ha->net->ping("8.8.8.8")) {
			return array("status" => "warning", "message" => "Unable to ping Google's DNS Servers (8.8.8.8).");
		} else {
			return array("status" => "ok");
		}
	}

	public function checkuids() {
		$localuid = $this->ha->passwd->getAsteriskUID();
		$knownuid = $this->ha->getConfig("asterisk_uid");
		if (!$knownuid) {
			$knownuid = $localuid;
			$this->ha->setConfig("asterisk_uid", $knownuid);
		}

		if ($knownuid != $localuid) {
			$this->ha->setConfig("asterisk_uid", $localuid);
			return array("status" => "Found UID Mismatch Error, fixed. Rerun tests to continue.");
		}

		$res = $this->ha->ssh->runOnOtherNode("grep ^asterisk /etc/passwd | cut -d: -f3");
		if ($res['ret'] != 0 || empty($res['result'])) {
			return array("status" => "warning", "message" => "Unable to check");
		}

		$remoteuid = trim($res['result'][0]);
		if ($remoteuid != $localuid) {
			return array("status" => "error", "message" => "Severe Error: UID's don't match. Contact support.");
		}
		return array("status" => "ok");
	}

	public function sizes() {
		$lvdisplay = $this->ha->incron->lvdisplay();
		if ($lvdisplay['ret'] != 0) {
			return array("status" => "failed", "message" => "Unable to validate LVM configuration - ".json_encode($res));
		}

		$allvgs = $lvdisplay['result'];

		$vgname = $this->ha->lvm->getVGName();
		$services = $this->ha->getServices();

		$fixed = "Fixed: ";
		foreach ($services as $s) {
			if (!isset($allvgs[$s])) {
				print_r($allvgs);
				return array("status" => "failed", "message" => "Service $s doesn't have a volume!");
			}

			$extents = $allvgs[$s]['extents'];

			if (!$this->ha->getConfig("EXTENTS_$s")) {
				$this->ha->setConfig("EXTENTS_$s", $extents);
			}

			if ($this->ha->getConfig("EXTENTS_$s") != $extents) {
				$fixed .= "EXTENTS_$s, ";
				$this->ha->setConfig("EXTENTS_$s", $extents);
			}
		}

		if (substr($fixed, -2) == ": ") {
			return array("status" => "ok");
		} else {
			return array("status" => "warning", "message" => substr($fixed, 0, -2));
		}
	}

	public function mysymlinks() {

		foreach ($this->links as $src => $dst) {
			if (!is_link($src)) {
				$fixcommands  = "<hr /><h3>Local configuration Error</h3><div>The machine that's currently running Apache has ";
				$fixcommands .= "configuration errors. Please run the following command (as root) on the current node.<br />\n";
				$fixcommands .= "<pre>mv $src ${src}.fix && ln -s $dst $src</pre>After you have done that, please re-run checks.</div>";
				return array("status" => "$src is not a link.", "setdata" => array("localcommands" => $fixcommands));
			}
		}
		return array("status" => "ok");
	}

	public function othersymlinks() {

		$fixcommands = "<hr><h3>Remote configuration error</h3><div>Copy and paste these commands (as root) on the other node to fix these problems. \n";
		$fixcommands .= "To avoid problems, ensure that the node is in Standby before proceeding.<br />\n<pre>\n";
		$errors = false;

		foreach ($this->links as $src => $dst) {
			$res = $this->ha->ssh->runOnOtherNode("/usr/bin/stat -t $src");
			if ($res['ret'] == 1) {
				$errors = true;
				$fixcommands .= "ln -s $dst $src\n";
				continue;
			}
			$res = $this->ha->ssh->runOnOtherNode("readlink $src");
			if($res['ret'] != 0) {
				$errors = true;
				$fixcommands .= "mv $src $src.fix && ln -s $dst $src\n";
				continue;
			}
			if ($res['result'][0] != $dst) {
				$errors = true;
				$fixcommands .= "rm -f $src && ln -s $dst $src\n";
			}
		}
		if (!$errors) {
			return array("status" => "ok");
		}

		return array("status" => "failed", "message" => "Errors", "setdata" => array("commands" => $fixcommands."\n</pre></div>"));
        }
}
