<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

// Only required when moving apache around, as file locations change.
if (class_exists('CMAN')) {
	return;
} else {
	/**
	 * @backupGlobals disabled
	 */
	class CMAN {

		public $conffile = "/etc/cluster/cluster.conf";
		private $constraints = "unset";
		private $pcsVers = "unset";

		public function __construct($ha = null) {
		/* if (!function_exists('freepbxha_licensed') || !freepbxha_licensed())
		throw new Exception("License violation in ".__FILE__); */
			if ($ha == null) {
				$this->ha = new freepbxHA;
			} else {
				$this->ha = $ha;
			}
		}

		public function tearDown() {

			// Drop out of any cluster that exists;
			exec("/usr/sbin/cman_tool leave force 2>&1");

			// Stop pacemaker and cman
			$this->initCMAN('stop');
			$this->initPacemaker('stop');

			// Delete the config files
			@unlink($this->conffile);
			exec("rm -rf /var/lib/pacemaker");


		}
		// /////////////////////////////////// //
		// These are called by a non-root user //
		// /////////////////////////////////// //
		public function configCMAN() {

			// This does $this->root_configCMAN(); as root.
			$res = $this->ha->incron->regenCMAN();
			if ($res['ret'] == 0) 
				return true;
			return false;
		}

		public function configPacemaker() {

			$res = $this->ha->incron->regenPacemaker();
			if ($res['ret'] == 0) 
				return true;
			return false;
		}

		public function getPCSStatus() {

			$res = $this->ha->incron->getPCSStatus();
			if ($res['result'] == 0) 
				return $res['ret'];
			return array();
		}

		public function pcsCommand($cmd) {

			return $this->ha->incron->pcsCommand($cmd);
		}

		// ////////////////////////////////// //
		//  These are run as root via incron. //
		// ////////////////////////////////// //
		public function root_configCMAN() {
			// Create CMAN cluster config!
			if ($this->ccs("--createcluster freepbx-ha") != 0)
				throw new Exception('Unable to create cluster');

			$this->ccs("--addfencedev pcmk agent=fence_pcmk");
			foreach (array("freepbx-a", "freepbx-b") as $host) {
				$this->ccs("--addnode $host");
				$this->ccs("--addmethod pcmk-redirect $host");
				$this->ccs("--addfenceinst pcmk $host pcmk-redirect port=$host");
			}

			$cmanconf = parse_ini_file("/etc/sysconfig/cman");
			if (!isset($cmanconf['CMAN_QUORUM_TIMEOUT'])) {
				$fh = fopen("/etc/sysconfig/cman", "a");
				fwrite($fh, "CMAN_QUORUM_TIMEOUT=0\n");
				fclose($fh);
			}

			return array("result" => "", "ret" => 0);
		}

		public function root_getPCSStatus() {
			if (!file_exists("/usr/sbin/pcs"))
				throw new Exception("pcs command doesn't exist");

			exec("/usr/sbin/crm_mon -1 -f 2>&1", $ret, $result);

			return array("result" => $result, "ret" => $ret);
		}

		public function root_configPacemaker() {
			// Load up our defaults
			$this->pcs("property set no-quorum-policy=ignore"); // We can't have a quorum
			$this->pcs("resource defaults migration-threshold=2"); // 2 failures? You're history.
			$this->pcs("resource defaults resource-stickiness=100"); // Don't move things around unless asked.
			$this->pcs("property set stonith-enabled=false"); // STONITH will be extra
			$this->pcs("property set default-action-timeout=30");

			return array("result" => "", "ret" => 0);
		}

		public function root_pcsCommand($cmd) {

			$fh = fopen ("/tmp/pcslog", "a");
			fwrite($fh, "/usr/sbin/pcs $cmd:");
			exec("/usr/sbin/pcs $cmd", $ret, $result);
			exec("/usr/sbin/cibadmin -Ss"); // Force sync before returning
			fwrite($fh, json_encode(array("ret" => $ret, "result" => $result))."\n");
			fclose($fh);
			return array("result" => $result, "ret" => $ret);
		}

		// Private functions, coz I'm lazy.
		private function ccs($cmd) {
			// Just run a CCS command, return $ret
			if (!file_exists("/usr/sbin/ccs"))
				return false;
			exec("/usr/sbin/ccs -i -f ".$this->conffile." $cmd", $ret, $res);
			return $res;
		}
		private function pcs($cmd) {
			// Just run a PCS command, return $ret
			if (!file_exists("/usr/sbin/pcs"))
				return false;
			exec("/usr/sbin/pcs $cmd", $ret, $res);
			return $res;
		}

		public function root_pcsUpdate() {
			// Update from HA version 0 to 1:
			//  Add the proper colocation set that can't be done with PCS.
			$z = `/usr/sbin/cibadmin --query`;

			$cib = new SimpleXMLElement($z);

			$foundit = false;
			foreach ($cib->configuration->constraints->rsc_colocation as $rs) {
				if ($rs{'id'} == "freepbx-colo") {
					// We don't need to add it, it's already here.
					$foundit = true;
				}
			}

			if (!$foundit) {
				$cib{'epoch'}++;
				$fpbx = $cib->configuration->constraints->addChild('rsc_colocation');
				$fpbx->addAttribute('id', 'freepbx-colo');
				$fpbx->addAttribute('score', 'INFINITY');

				$rs = $fpbx->addChild('resource_set');
				$rs->addAttribute('id', 'colo-freepbx-0');
				$asterisk = $rs->addChild('resource_ref');
				$asterisk->addAttribute('id', 'asterisk');
				$httpd = $rs->addChild('resource_ref');
				$httpd->addAttribute('id', 'httpd');

				$rs = $fpbx->addChild('resource_set');
				$rs->addAttribute('id', 'colo-freepbx-1');
				$rs->addAttribute('role', 'Master');
				$ast_drbd = $rs->addChild('resource_ref');
				$ast_drbd->addAttribute('id', 'ms-asterisk');
				$httpd_drbd = $rs->addChild('resource_ref');
				$httpd_drbd->addAttribute('id', 'ms-httpd');

				$fds = array( 0 => array("pipe", "r"), 1 => array("pipe", "w"), 2 => array("file", "/tmp/cibr-output.txt", "a") );
				$proc = proc_open("/usr/sbin/cibadmin --replace -p -s", $fds, $pipes);
				if ($proc === false)
					throw new Exception("Unable to run cibadmin for some reason");

				fwrite($pipes[0], $cib->asXML());
				fclose($pipes[0]);

				// Now, wait to make sure we've got nothing back.
				if (!stream_select($r = array($pipes[1]) , $tmp = null, $tmp = null, 2)) {
					throw new Exception("cibadmin locked up and didn't respond");
				}

				$out = stream_get_contents($pipes[1]);
				if (strlen($out) != 0)
					return array("result" => "cibadmin error: $out", "ret" => "99");
			}

			// Force a sync to all nodes.
			exec("/usr/sbin/cibadmin -S");
			// Update our version
			$this->ha->setConfig('version', 1);

			return array("result" => "", "ret" => 0);
		}

		// ///////////////////////////////// //
		// No further root-related commands. //
		// ///////////////////////////////// //

		public function getConfFile() {
			return $this->conffile;
		}

		public function startCluster() {
			if ($this->startCMAN())
				return $this->startPacemaker();
		}

		public function stoptCluster() {
			$this->startCMAN();
			return $this->startPacemaker();
		}

		public function startCMAN() {
			return $this->initCMAN('start');
		}

		public function stopCMAN() {
			return $this->initCMAN('stop');
		}

		public function startPacemaker() {
			return $this->initPacemaker('start');
		}

		public function stopPacemaker() {
			return $this->initPacemaker('stop');
		}

		private function initCMAN($cmd) {

			$res = $this->ha->incron->manageService('cman', $cmd);
			if ($res['ret'] == 0) 
				return true;

			return false;
		}

		private function initPacemaker($cmd) {

			$res = $this->ha->incron->manageService('pacemaker', $cmd);
			if ($res['ret'] == 0) 
				return true;

			return false;
		}

		public function isNodeOnline($nodename) {
			$status = $this->getPCSStatus();

			if ($status == false)
				return false;

			foreach ($status as $line) {
				if (preg_match('/^Online: \[ (.+) \]/', $line, $res)) {
					if (preg_match("/".$nodename."/", $res[1]))
						return true;
				}
			}
			return false;
		}

		public function isThisNodeOnline($wait = false) {

			$me = $this->ha->whichMachineIsThis();

			if ($me == "PRIMARY") {
				$hostname = "freepbx-a";
			} else {
				$hostname = "freepbx-b";
			} 

			if ($wait == false) 
				return $this->isNodeOnline($hostname);

			return $this->waitForNode($hostname);
		}

		private function waitForNode($hostname) {
			// Wait for up to 60 seconds for a node to come online.
			// Check every 1/2 a second.

			for ($delay = 0; $delay < 120; $delay++) {
				if ($this->isNodeOnline($hostname))
					return true;
				usleep(500000);
			}
			return false;
		}

		public function createAllServices() {

			$services = $this->ha->getServices();
			$res = "";
			foreach ($services as $service)
				$this->createIPService("${service}_ip", $this->ha->net->getServiceIP($service));
			return true;
		}

		public function createIPService($service, $ip, $cidr = 24) {
			$this->ha->setConfig($service, $ip);
			//$res = $this->pcsCommand("resource create $service ocf:heartbeat:IPaddr2 ip=$ip cidr_netmask=$cidr op monitor interval=30s");
			$res = $this->pcsCommand("resource create $service ocf:heartbeat:IPaddr2 ip=$ip op monitor interval=30s");
			if ($res['result'] == 0) 
				return true;
			throw new Exception("Unable to create service $service with ip $ip");
		}

		public function linkDrbd($service) {
			$id = $this->ha->getConfig($service."_id");

			$fn = "/tmp/drbd.${service}.cfg";
			$this->pcsCommand("cluster cib $fn");
			$this->pcsCommand("-f $fn resource create drbd_$service ocf:linbit:drbd drbd_resource=$service op monitor interval=60s");
			$this->pcsCommand("-f $fn resource master ms-${service} drbd_$service master-max=1 master-node-max=1 clone-max=2 clone-node-max=1 notify=true");
			$this->pcsCommand("-f $fn resource create ${service}_fs Filesystem device='/dev/drbd$id' directory='/drbd/$service' fstype='ext4'");
			// If you update these constraints, make sure you update checkConstraints too.
			$this->pcsCommand("-f $fn constraint colocation add ${service}_fs ms-${service} INFINITY with-rsc-role=Master");
			$this->pcsCommand("-f $fn constraint order promote ms-${service} then start ${service}_fs score=INFINITY");

			if ($this->getPcsVersion() == "0.9.26") {
				$this->pcsCommand("cluster push cib $fn");
			} else {
				$this->pcsCommand("cluster cib-push $fn");
			}

			$complete = false;
			$count = 0;
			while (!$complete) {
				$count++;
				$check = $this->checkDRBDConstraints($service);
				if ($check['status'] == "ok")
					$complete = true;
				if ($count > 10) 
					throw new Exception("Unable to validate DRBD service $service");
				usleep(50000); // Sleep for a bit.
			}
			return true;

		}

		public function getAllStatus() {

			$pcs = $this->getPCSStatus();

			// Ok, we have some stuff. Lets find our service.
			$resourcegroup = null;
			$failedactions = false;
			$insummary = false;
			foreach ($pcs as $line) {
				// $retarr['raw'][] = $line;

				// Ignore blank lines.
				if (trim($line) == "")
					continue;

				// Collect failed actions.
				if (preg_match("/^Failed actions/", $line)) {
					$failedactions = true;
					continue;
				}
				if ($failedactions) {
					$retarr['failed'][] = trim($line);
					continue;
				}

				if (preg_match("/Resource Group: (.+)/", $line, $res)) {
					$resourcegroup = $res[1];
					continue;
				}

				if (preg_match("/Clone Set: (.+) \[(.+)\]/", $line, $res)) {
					$cloneset = $res[2];
					continue;
				}

				if (preg_match("/Master.Slave Set: (.+) \[(.+)\]/", $line, $res)) {
					$ms = $res[2];
					continue;
				}

				if (preg_match("/(Stopped|Masters|Slaves): \[ (.+) \]/", $line, $res)) {
					$retarr['ms'][$ms][$res[1]] = $res[2];
					continue;
				}

				if (preg_match("/Migration summary:/", $line, $res)) {
					$insummary = true;
				}

				if (preg_match("/^Online: \[ (.+) \]/", $line, $res)) {
					$retarr['online'] = explode(" ", $res[1]);
					foreach($retarr['online'] as $n) {
						$retarr['allnodes'][] = $n;
						$retarr[$n."_status"] = "online";
					}
					continue;
				}
				if (preg_match("/^OFFLINE: \[ (.+) \]/", $line, $res)) {
					$retarr['offline'] = explode(" ", $res[1]);
					foreach($retarr['offline'] as $n) {
						$retarr['allnodes'][] = $n;
						$retarr[$n."_status"] = "offline";
					}
					continue;
				}
				if (preg_match("/Node (.+): standby/", $line, $res)) {
					$retarr['standby'] = explode(" ", $res[1]);
					$retarr['allnodes'][] = $res[1];
					$retarr[$res[1]."_status"] = "standby";
					continue;
				}
				if (preg_match("/^Current DC: (.+) -.+/", $line, $res)) {
					$retarr['dc'] = $res[1];
					continue;
				}

				if (preg_match("/^Version: (.+)$/", $line, $res)) {
					$retarr['version'] = $res[1];
					continue;
				}

				if (preg_match("/\b(.+)\b.+\(stonith:(.+)\):\s+(.+)$/", $line, $res)) {
					$retarr['stonith'][$res[1]] = $res[2].",".$res[3];
					continue;
				}

				if (preg_match("/(Started|Stopped): \[ (.+) \]/", $line, $res)) {
					// Clone set?
					if (isset($cloneset)) {
						$retarr['cloneset'][$cloneset][$res[1]] = explode(" ", $res[2]);
					}
				}

				if ($insummary && preg_match("/\* Node (.+):/", $line, $res)) {
					$currentsum = $res[1];
					continue;
				}

				if ($insummary && preg_match("/^\s\s\s(.+): migration-threshold=(.+) fail-count=(.+) last-failure=(.+)$/", $line, $res)) {
					$retarr['failures'][$currentsum][$res[1]] = array(
						"migration-threshold" => $res[2],
						"fail-count" => $res[3],
						"last-failure" => $res[4],
					);
				}

				if (!preg_match("/\s(\w+)\s+\(.+:\s+(.+)/", $line, $res)) {
					// $line didn't match the format of a resource. Ignore it.
					continue; 
				}
				if ($resourcegroup) {
					$retarr[$res[1]] = $res[2];
					$retarr['rg'][$resourcegroup][] = $res[1];
				} else {
					$retarr[$res[1]] = $res[2];
				}
			}

			if (!isset($retarr['offline']))
				$retarr['offline'] = array();

			if ($failedactions) 
				$retarr['errors'] = $this->describeErrors($retarr);

			return $retarr;
		}

		public function getStatus($service, $servicecache = null) {
			// Returns either the machine it's on, or, false if the service isn't running.
			// Takes either an array, or a string.
			// Returns an array, or a string. 

			// ServiceCache is used when being recusrive, so it doesn't
			// need to do a 'pcs status' a million times.
			if ($servicecache == null) 
				$allserv = $this->getAllStatus();
			else 
				$allserv = $servicecache;

			if (is_array($service)) {
				// Handed an array. Recursion is nice.
				foreach ($service as $s)
					$retarr[$s] = $this->getStatus($s, $allserv);
				return $retarr;
			} else { 
				if (isset($allserv[$service])) {
					if (preg_match('/(\S+)\s+(\S+)/', $allserv[$service], $out)) {
						return $out[2];
					} else {
						return "Stopped";
					}
				}
				throw new Exception("Unable to find service $service");
			}
		}

		public function waitForService($service, $loops = 60) {
			// We're going to try $loops times, waiting for the service to start.
			// This isn't 60 seconds. Slower machines may take longer, so we'll
			// be generous.

			$count = 0;
			while ($count < 60) {
				try { // getStatus will throw an exception if service doesn't exist.
					$result = $this->getStatus($service);
					if ($result != 'Stopped' && $result != '')
						return true;
					$count++;
					sleep(1);
				} catch(Exception $e) {
					$count++;
					sleep(1);
				}
			}
			return false;
		}

		public function linkServiceToCluster($service) {
			switch ($service) {
			case "httpd":
				$sn = "httpd_service";
				$ocf = "apache";
				$params = "configfile='/etc/httpd/conf/httpd.conf' httpd='/usr/sbin/httpd' op monitor interval=30s";
				$group = "add httpd httpd_fs httpd_ip httpd_service";
				break;
			case "mysql":
				$sn = "mysql_service";
				$ocf = "mysql";
				$params = "binary='/usr/bin/mysqld_safe' client_binary='/usr/bin/mysql' datadir='/drbd/mysql' test_table='asterisk.freepbxha'";
				$params .= " test_user='".$this->ha->conf->getConf('AMPDBUSER')."' test_passwd='".$this->ha->conf->getConf('AMPDBPASS')."' enable_creation=1 op monitor interval=30s";
				$group = "add mysql mysql_fs mysql_ip mysql_service";
				break;
			case "asterisk":
				$sn = "asterisk_service";
				$ocf = "freepbx";
				$params = "binary='/usr/sbin/asterisk' canary_binary='/usr/sbin/astcanary' op monitor interval=30s";
				$constraints[] = "colocation add floating_ip asterisk_ip INFINITY";
				$group = "add asterisk asterisk_fs asterisk_ip asterisk_service";
				break;
			case "clustermon":
				$sn = "clustermon_service";
				$ocf = "ClusterMon";
				$params = "user=root update=30 extra_options=-E /drbd/clustermon/clustermon.sh";
				$group = "add clustermon clustermon_fs clustermon_ip clustermon_service";
				break;
			default:
				throw new Exception("Unknown service, can't make cluster service $service");
				break;
			}

			$status = $this->getAllStatus();
			if (isset($status[$sn]))
				return "Service already exists";

			$fn = "/tmp/$service.cluster.conf";

			$this->pcsCommand("cluster cib $fn");

			if (isset($ocf)) {
				$this->pcsCommand("-f $fn --force resource create $sn $ocf $params");

				if (isset($constraints)) {
					foreach ($constraints as $c) 
						$this->pcsCommand("-f $fn constraint $c");
				}
			}

			if (isset($group)) 
				$this->pcsCommand("-f $fn resource group $group");

			if ($this->getPcsVersion() == "0.9.26") {
				$this->pcsCommand("cluster push cib $fn");
			} else {
				$this->pcsCommand("cluster cib-push $fn");
			}
		}

		public function toggleNode($nodename) {

			if ($this->isNodeOnline($nodename)) {
				$this->pcsCommand("cluster standby $nodename || crm_standby -v on -N $nodename");
				return "$nodename has been set to Standby";
			} else {
				$this->pcsCommand("cluster unstandby $nodename || crm_standby -D -N $nodename");
				return "$nodename has been set to Online";
			}
		}

		public function getConstraints($full = false) {

			if ($full) {
				$c = $this->pcsCommand('constraint list --full');
			} else {
				$c = $this->pcsCommand('constraint list');
			}
			if ($c['result'] != 0) {
				return array(
					"Location" => array(),
					"Ordering" => array(),
					"Colocation" => array(),
				);
			}

			$cgroup = "Unknown";
			foreach ($c['ret'] as $l) {
				if (preg_match("/(.+) Constraints:$/", $l, $match)) {
					$cgroup = $match[1];
					continue;
				}
				$retarr[$cgroup][] = trim($l);
			}

			return $retarr;
		}

		public function checkDRBDConstraints($srv) {
			$c = $this->getConstraints();
			if (!in_array("${srv}_fs with ms-${srv} (with-rsc-role:Master)", $c['Colocation']))
				return (array("status" => "warning", "message" => "${srv}_fs not found in Colocation - ".json_encode($c)));
			if (!in_array("promote ms-${srv} then start ${srv}_fs", $c['Ordering']))
				return (array("status" => "warning", "message" => "ms-${srv} not found in Ordering - ".json_encode($c)));

			return array("status" => "ok");
		}

		public function describeErrors($stat) {
			if (!is_array($stat))
				return false;

			$founderrors = false;
			foreach ($stat['failed'] as $l) {
				// pcs older than 0.90 looks like this:
				// asterisk_service_monitor_30000 (node=freepbx-a, call=503, rc=7, status=complete): not running
				if (preg_match('/(\w+)\s\(node=(\w+-.),.+: (.+)/', $l, $m)) {
					$founderrors = true;
					$r = explode('_', $m[1]);
					array_pop($r); array_pop($r);
					$rsc = join("_", $r);
					$tmp['desc'] = "The resource ".$rsc." on ".$m[2]." was found to be ".$m[3];
					$tmp['rsc'] = $rsc;
					$tmp['node'] = $m[2];
					$tmp['failcount'] = $this->crm_failcount($tmp['rsc'], $tmp['node'], $stat);
					$tmp['failtime'] = "Unknown";
					$retarr[] = $tmp;
				// pcs 0.90 and higher looks like this:
				// isymphony_service_monitor_30000 on freepbx-b 'not running' (7): call=372, status=complete, last-rc-change='Tue Nov 19 12:07:05 2013', queued=22ms, exec=0ms
				} elseif (preg_match('/([\w-]+) on (\w+-.) \'(.+)\'.+last-rc-change=\'(.+)\'/', $l, $m)) {
					$founderrors = true;
					$r = explode('_', $m[1]);
					array_pop($r); array_pop($r);
					$rsc = join("_", $r);
					$tmp['rsc'] = $rsc;
					$tmp['node'] = $m[2];
					$tmp['failcount'] = $this->crm_failcount($tmp['rsc'], $tmp['node'], $stat);
					$tmp['failtime'] = strtotime($m[4]);
					$tmp['desc'] = "<span class='date'>".strftime("%F %T", $tmp['failtime']).":</span> ".$rsc." on ".$m[2]." was ".$m[3];
					$retarr[] = $tmp;
				}

			}
			if ($founderrors)
				return $retarr;

			return false;
		}

		public function crm_resource($cmd) {
			return $this->ha->incron->crm_resource($cmd);
		}

		public function crm_failcount($rsc, $node, $stat = null) {

			if (is_array($stat) && isset($stat['failures'][$node][$rsc])) {
				$threshhold = $stat['failures'][$node][$rsc]['migration-threshold'];
				$failcount['result'][] = "value=".$stat['failures'][$node][$rsc]['fail-count'];
			} else {
				$failcount = $this->ha->incron->crm_failcount($rsc, $node);
				if ($this->getPcsVersion() == "0.9.26") {
					$def = $this->pcsCommand("resource rsc defaults");
				} else {
					$def = $this->pcsCommand("resource defaults");
				}
				$threshhold = 1;

				foreach ($def['ret'] as $r => $l) {
					if (preg_match("/migration-threshold: (.+)/", $l, $out))
						$threshhold = $out[1];
				}
			}

			if (isset($failcount['result'][0]) && preg_match("/value=(.+)/", $failcount['result'][0], $out)) {
				if ($out[1] == 'INFINITY') {
					return array("status" => "error", "message" => "$rsc has been deemed to be unstartable by the cluster.");
				} else if ($out[1] >= $threshhold) {
					return array("status" => "error", "message" => "$rsc has failed ".$out[1]." times and will not start on $node");
				} else {
					return array("status" => "warning", "message" => "$rsc can fail ".($threshhold - $out[1])." more time(s) until it is blocked on $node");
				}
			} else {
				return array("status" => "wft", "message" => "No idea. What is ".$failcount['result'][0]);
			}
		}

		public function cleanConstraints() {
			// Removes any duplicate constraints that may have been added by multiple runs
			// of the installer.
			$constraints = $this->getConstraints(true);
			foreach ($constraints as $group) {
				foreach ($group as $item) {
					if (preg_match("/\(id:(.+-(?:INFINITY|mandatory)-\d)\)/", $item, $out)) {
						// Found a duplicate - the IDs should never match, because, it's
						// the same constraint.
						$this->pcsCommand('constraint remove '.$out[1]);
					}
				}
			}
		}

		public function getPcsVersion() {

			if ($this->pcsVers != "unset") 
				return $this->pcsVers;

			if ($this->ha->packages->packageIsInstalled("pcs-0.9.26")) {
				$this->pcsVers = "0.9.26";
			} elseif ($this->ha->packages->packageIsInstalled("pcs-0.9.90")) {
				$this->pcsVers = "0.9.90";
			} else {
				throw new Exception("Unknown PCS version installed! Contact support!");
			}
			return $this->pcsVers;
		}

		public function getPcsProperty($var = null) {
			$tmpout = $this->pcsCommand("property");
			$res = $tmpout['ret'];
			$retarr = array();
			foreach ($res as $line) {
				// If the first chararcter ISN'T a space, ignore.
				if ($line[0] != ' ')
					continue;
				$tmparr = explode(":", $line);
				$retarr[trim($tmparr[0])] = trim($tmparr[1]);
			}
			if (isset($var)) {
				if (isset($retarr[$var])) {
					return $retarr[$var];
				} else {
					return false;
				}
			}
			return $retarr;
		}

		public function getResource($res = null) {
			if ($res == null)
				throw new Exception ("No resource given to getResource");

			$r = $this->pcsCommand("resource show $res");
			if ($r['result'] == 1)
				throw new Exception ("Unknown resource $res");

			foreach ($r['ret'] as $line) {
				if (preg_match("/Resource: (.+) \(class=(\w+) provider=(\w+) type=(\w+)\)/", $line, $out)) {
					$ret['name'] = $out[1];
					$ret['class'] = $out[2];
					$ret['provider'] = $out[3];
					$ret['type'] = $out[4];
					continue;
				}

				if (preg_match("/(Attributes|Meta Attrs): (.+)/", $line, $attrib)) {
					if ($attrib[1] == "Attributes") {
						$key = "attributes";
					} else {
						$key = "meta";
					}
					// This isn't line noise. Honest.
					preg_match_all("/([\w-_]+)=((?:[\w\/_]+|(?:\".+?\")))/", $attrib[2], $out);
					// How many matches?
					$r = count($out[0]);
					for ($i = 0; $i <  $r; $i++) 
						$ret[$key][$out[1][$i]] = $out[2][$i];
					continue;
				}
			}

			return $ret;
		}

	}
}
