<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

Class drbd {

	public function __construct() {
		/* if (!function_exists('freepbxha_licensed') || !freepbxha_licensed())
			throw new Exception("License violation in ".__FILE__); */
	}

	public function startService() {
		require_once('incron.php');
		$i = new incron;

		$ret = $i->manageService('drbd', 'start');
		$ret = $i->manageService('drbd', 'on');
	}

	public function checkModuleLoaded() {
		if (file_exists("/proc/drbd"))
			return true;
		// else
		return false;
	}

	public function unloadModule() {
		exec("rmmod drbd 2>&1", $result, $ret);
		if ($ret != 0) 
			return false;
		// else
		return true;
	}

	public function loadModule() {
		exec("modprobe drbd 2>&1", $result, $ret);
		if ($ret != 0)
			return false;
		// else
		return true;
	}

	public function createConfigs() {
		include_once 'functions.inc.php';
		include_once 'incron.php';
		$ha = new freepbxHA;
		$i = new incron;

		$services = $ha->getServices();

		foreach ($services as $service) {
			$conf = $this->createConfig($ha->getConfig($service."_id"), $service);
			$i->writedrbd($service, $conf);
		}

		// DRBD global conf.
		if ($ha->whichMachineIsThis() == "PRIMARY") {
			$timeout = 5;
		} else {
			$timeout = 30;
		}	
		$c = "global { usage-count no; }\ncommon {\n";
		$c .= "\tstartup { wfc-timeout $timeout; degr-wfc-timeout 1; outdated-wfc-timeout 1; }\n}\n";
		$i->writedrbd("global_common.conf", $c);

		return true;
	}
	public function createConfig($id, $name) {
		include_once 'lvm.php';
		$lvm = new LVM;
		$ha = new freepbxHA;
		$vgname = $lvm->getVGName();
		$prim = $ha->getConfig('PRIMARY_intip');
		$sec = $ha->getConfig('SECONDARY_intip');
		$c = "resource $name {\n\tdevice /dev/drbd$id;\n\tmeta-disk internal;\n";
		$c .= "\ton freepbx-a {\n\t\tdisk /dev/mapper/$vgname-drbd_$name;\n\t\taddress $prim:400$id;\n\t}\n";
		$c .= "\ton freepbx-b {\n\t\tdisk /dev/mapper/$vgname-drbd_$name;\n\t\taddress $sec:400$id;\n\t}";
		$c .= "\tnet {\n\t\tafter-sb-0pri discard-zero-changes;\n\t\tafter-sb-1pri discard-secondary;\n\t\tafter-sb-2pri call-pri-lost-after-sb;\n\t}\n";
		$c .= "\thandlers {\n\t\tpri-lost-after-sb \"/sbin/reboot -f\";\n\t}\n";
		$c .= "\tdisk {\n\t\tresync-rate 100M;\n\t}\n";
		$c .= "}\n";
		return $c;
	}

	public function enableVolumes() {
		$ha = new freepbxHA;
		$services = $ha->getServices();

		foreach ($services as $service) 
			$res = $this->enableVolume($service);

		return true;
	}

	public function enableVolume($service) {
		// Has this volume already be activated?
		$r = $this->drbdadm("dump-md $service 2>&1");
		if ($r['ret'] == 0)
			return true;

		$this->drbdadm("detach $service 2>&1");
		return $this->drbdadm("create-md $service 2>&1");
	}

	public function drbdadm($cmd) {
		include_once 'incron.php';
		$i = new incron;
		return $i->drbdadm($cmd);
	}

	public function forceActivateVolumes() {
		include_once 'functions.inc.php';
		$ha = new freepbxHA;
		$services = $ha->getServices();

		foreach ($services as $service) 
			$res = $this->activateDRBDVolume($service);

		# $this->drbdadm("drbdadm primary --force all"); //No.
		return true;
	}

	public function activateDRBDVolume($service) {

		$this->drbdadm("adjust $service");
		$this->drbdadm("primary --force $service");
	}

	public function resyncMax() {
		$this->drbdadm("disk-options --resync-rate=110M all");
	}

	public function getRawDrbdInfo($thisnode = true) {
		if ($thisnode)
			return (file("/proc/drbd"));

		include_once 'ssh.php';
		$s = new SSH;
		$ssh = $s->runOnOtherNode('cat /proc/drbd');
		if ($ssh['ret'] == 0)
			return ($ssh['result']);

		// Other node is down.
		return false;
	}

	public function getDrbdInfo($node = true) {
		include_once 'functions.inc.php';
		include_once 'hosts.php';
		$ha = new freepbxha;
		$hosts = new Hosts;

		if ($node === true || $node == $hosts->getHostname()) {
			// This node
			$stat = $this->getRawDrbdInfo(true);
		} else { // Other node!
			$stat = $this->getRawDrbdInfo(false);
		}

		if ($stat === false) {
			$stat[] = " 1: cs:NodeDown ro:Unknown/Unknown ds:DUnknown/DUnknown C r----s";
			$stat[] = " 2: cs:NodeDown ro:Unknown/Unknown ds:DUnknown/DUnknown C r----s";
			$stat[] = " 3: cs:NodeDown ro:Unknown/Unknown ds:DUnknown/DUnknown C r----s";
			$stat[] = " 4: cs:NodeDown ro:Unknown/Unknown ds:DUnknown/DUnknown C r----s";
			$stat[] = " 5: cs:NodeDown ro:Unknown/Unknown ds:DUnknown/DUnknown C r----s";
			$stat[] = " 6: cs:NodeDown ro:Unknown/Unknown ds:DUnknown/DUnknown C r----s";
			$stat[] = " 7: cs:NodeDown ro:Unknown/Unknown ds:DUnknown/DUnknown C r----s";
			$stat[] = " 8: cs:NodeDown ro:Unknown/Unknown ds:DUnknown/DUnknown C r----s";
		}

		$lastsrv = null;
		foreach ($stat as $line) {
			if (preg_match('/^ (\d): cs:(.+) ro:(.+) ds:(.+) (:?.) (.)(.)(.)(.)(.)(.)/', $line, $res)) {
				$service = $ha->getConfig("service_id_".$res[1]);
				if ($service) {
					$lastsrv = $service;
					$retarr[$service]['connstate'] = $res[2];
					$retarr[$service]['roles'] = $res[3];
					$retarr[$service]['diskstate'] = $res[4];
					$retarr[$service]['blockedio'] = $res[9];
				}
			}
			if (preg_match('/\[.+sync.+[KMG]$/', $line))
				$retarr[$lastsrv]['syncpercent'] = trim($line);

			if (preg_match('/finish.+sec$/', $line))
				$retarr[$lastsrv]['syncfinish'] = trim($line);
		}


		return $retarr;
	}
}


