<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

class Fencing {

	public function __construct($ha = null) {
		if ($ha == null)
			throw new Exception("Fencing needs to be handed a freepbxHA object");

		$this->ha = $ha;
	}

	public function getAllFencing() {
		$retarr['freepbx-a'] = $this->getFencing('freepbx-a');
		$retarr['freepbx-b'] = $this->getFencing('freepbx-b');
		return $retarr;
	}

	public function getFencing($node = null) {
		if ($node == null) 
			throw new Exception("Wasn't given a node name to get fencing details from");

		$retarr = json_decode($this->ha->getConfig("fencing-".$node."-params"), true);
		$retarr['name'] = $node;
		$retarr['type'] = $this->ha->getConfig("fencing-".$node."-type");

		return $retarr;
	}

	public function updateFencing($req) {
		if (!isset($req['fencingparams']))
			throw new Exception("No parameters given to updateFencing");

		$type = $req['fencingparams']['type'];

		if ($type == 'None') {
			return $this->removeFencing($req['node']);
		} else if ($type == 'VMware') { // VMware
			return $this->updateFencingVMware($req);
		} else if ($type == 'ILO') { // ILO
			return $this->updateFencingILO($req);
		} else if ($type == 'IPMI') { // IPMI (Most useless comments ever, I think)
			return $this->updateFencingIPMI($req);
		} else if ($type == 'DRAC') { // OpenStack. Wait. No. DRAC.
			return $this->updateFencingDRAC($req);
		} else {
			throw new Exception("Don't know the type $type.");
		}
	}

	public function updateFencingVMware($req) {
		// Make sure that we can run 'fence_vmware_helper' on both nodes.
		exec("/usr/sbin/fence_vmware_helper --help", $out, $retcode);
		if ($retcode == 1) 
			throw new Exception("Unable to run fence_vmware on local node - VMware API not installed?");

		$other = $this->ha->ssh->runOnOtherNode("/usr/sbin/fence_vmware_helper --help");
		if ($other['ret'] == 1) 
			throw new Exception("Unable to run fence_vmware on other node - VMware API not installed?");

		if (!isset($req['fencingparams']['serverip'])) 
			throw new Exception("No Server IP Supplied");
		$attr['ipaddr'] = escapeshellarg($req['fencingparams']['serverip']);

		if (!isset($req['fencingparams']['login'])) 
			throw new Exception("No VMware Username Supplied");
		$attr['login'] = escapeshellarg($req['fencingparams']['login']);

		if (!isset($req['fencingparams']['passwd'])) 
			throw new Exception("No VMware Password Supplied");
		$attr['passwd'] = escapeshellarg($req['fencingparams']['passwd']);

		if (!isset($req['fencingparams']['vmware_guestname']))
			throw new Exception("No VMware Guest Name Supplied");

		$attr['pcmk_host_map'] = $req['node'].":".escapeshellarg($req['fencingparams']['vmware_guestname']);

		$check = $this->checkStonith("fence_vmware", $attr, escapeshellarg($req['fencingparams']['vmware_guestname']));

		if ($check == 0) {
			$this->createStonith($req['node'], 'fence_vmware', $attr);
			$this->ha->setConfig("fencing-".$req['node']."-type", 'VMware');
			$this->ha->setConfig("fencing-".$req['node']."-params", json_encode($req['fencingparams']));
			return array("status" => "ok", "data" => "Stonith Created");
		} else if ($check == 1) {
			throw new Exception("Unknown VMware Guest name Supplied");
		} else if ($check == 2) {
			throw new Exception("Node is reported as OFF! (Wrong node name?)");
		} else {
			throw new Exception("Unknown Error from fence_vmware - $check");
		}
	}

	public function updateFencingILO($req) {
		if (!isset($req['fencingparams']['serverip'])) throw new Exception("No Server IP Supplied");
		$attr['ipaddr'] = escapeshellarg($req['fencingparams']['serverip']);

		if (!isset($req['fencingparams']['login'])) throw new Exception("No ILO Username Supplied");
		$attr['login'] = escapeshellarg($req['fencingparams']['login']);

		if (!isset($req['fencingparams']['passwd'])) throw new Exception("No ILO Password Supplied");
		$attr['passwd'] = escapeshellarg($req['fencingparams']['passwd']);

		$attr['pcmk_host_map'] = $req['node'].":0";

		$check = $this->checkStonith("fence_ilo3", $attr);

		if ($check == 0) {
			$this->createStonith($req['node'], 'fence_ilo3', $attr);
			$this->ha->setConfig("fencing-".$req['node']."-type", 'ILO');
			$this->ha->setConfig("fencing-".$req['node']."-params", json_encode($req['fencingparams']));
			return array("status" => "ok", "data" => "Stonith Created");
		} else if ($check == 2) {
			throw new Exception("Node is reported as OFF! (Wrong ILO Device?)");
		} else {
			throw new Exception("Unknown Error from fence_ilo3 - $check");
		}
	}

	public function updateFencingIPMI($req) {
		if (!isset($req['fencingparams']['serverip'])) throw new Exception("No Server IP Supplied");
		$attr['ipaddr'] = escapeshellarg($req['fencingparams']['serverip']);

		if (!isset($req['fencingparams']['login'])) throw new Exception("No IPMI Username Supplied");
		$attr['login'] = escapeshellarg($req['fencingparams']['login']);

		if (!isset($req['fencingparams']['passwd'])) throw new Exception("No IPMI Password Supplied");
		$attr['passwd'] = escapeshellarg($req['fencingparams']['passwd']);

		$attr['pcmk_host_list'] = $req['node'];

		$check = $this->checkStonith("fence_ipmi", $attr);

		if ($check == 0) {
			$this->createStonith($req['node'], 'fence_ipmi', $attr);
			$this->ha->setConfig("fencing-".$req['node']."-type", 'IPMI');
			$this->ha->setConfig("fencing-".$req['node']."-params", json_encode($req['fencingparams']));
			return array("status" => "ok", "data" => "Stonith Created");
		} else if ($check == 2) {
			throw new Exception("Node is reported as OFF! (Wrong IPMI Device?)");
		} else {
			throw new Exception("Unknown Error from fence_ipmi - $check");
		}
	}

	public function updateFencingDRAC($req) {
		if (!isset($req['fencingparams']['serverip'])) throw new Exception("No Server IP Supplied");
		$attr['ipaddr'] = escapeshellarg($req['fencingparams']['serverip']);

		if (!isset($req['fencingparams']['login'])) throw new Exception("No DRAC Username Supplied");
		$attr['login'] = escapeshellarg($req['fencingparams']['login']);

		if (!isset($req['fencingparams']['passwd'])) throw new Exception("No DRAC Password Supplied");
		$attr['passwd'] = escapeshellarg($req['fencingparams']['passwd']);

		$attr['pcmk_host_list'] = $req['node'];

		$attr['extraoptions'] = "-x";

		$check = $this->checkStonith("fence_drac5", $attr);

		$attr['extraoptions'] = "secure";

		if ($check == 0) {
			$this->createStonith($req['node'], 'fence_drac5', $attr);
			$this->ha->setConfig("fencing-".$req['node']."-type", 'DRAC');
			$this->ha->setConfig("fencing-".$req['node']."-params", json_encode($req['fencingparams']));
			return array("status" => "ok", "data" => "Stonith Created");
		} else if ($check == 2) {
			throw new Exception("Node is reported as OFF! (Wrong IPMI Device?)");
		} else {
			throw new Exception("Unknown Error from fence_ipmi - $check");
		}
	}

	public function checkStonith($agent, $attrs, $port = null) {
		if (!isset($attrs['extraoptions']))
			$attrs['extraoptions'] == "";

		if (!$port) {
			exec("$agent -ostatus ".$attrs['extraoptions']." -a".$attrs['ipaddr']." -l".$attrs['login']." -p".$attrs['passwd'], $result, $ret);
		} else {
			exec("$agent -ostatus ".$attrs['extraoptions']." -a".$attrs['ipaddr']." -l".$attrs['login']." -p".$attrs['passwd']." -n$port", $result, $ret);
		}
		return $ret;
	}

	private function getStonithName($node) {
		if ($node == "freepbx-a") {
			$res = "fence_a";
		} else if ($node == "freepbx-b") {
			$res = "fence_b";
		} else {
			throw new Exception("Seriously, WTF is node $node meant to be?");
		}
		return $res;
	}

	private function createStonith($node, $agent, $attrs) {
		$res = $this->getStonithName($node);
		$this->ha->cman->pcsCommand("stonith delete $res");
		$cmd = "stonith create $res $agent ";
		foreach ($attrs as $attr=>$val) {
			if ($attr == "extraoptions") {
				$cmd .= "$val ";
			} else {
				$cmd .= "$attr=$val ";
			}
		}
		$this->ha->cman->pcsCommand($cmd);
		return true;
	}

	private function removeFencing($node) {
		$res = $this->getStonithName($node);
		$this->ha->cman->pcsCommand("stonith delete $res");
		$this->ha->setConfig("fencing-".$node."-type");
		$this->ha->setConfig("fencing-".$node."-params");
		return $this->ha->cman->pcsCommand("property set stonith-enabled=false");
	}

	public function doFence($req) {
		if (!isset($req['node']) || !isset($req['fencecmd']))
			throw new Exception("Invalid parameters to doFence");

		$node = $req['node'];
		$do = $req['fencecmd'];

		$params = $this->getFencing($node);

		if ($params['type'] == 'IPMI') {
			if ($do == "reboot") {
				$do = "power reset";
			} else {
				$do = "power $do";
			}
		}

		$f = "do".$params['type']."Fence";
		if (!method_exists($this, $f))
			throw new Exception ("Unable to call $f");

		return $this->$f($do, $params);
	}

	public function getFencingStatus() {
		if ($this->ha->cman->getPcsProperty('stonith-enabled') === 'false') {
			// Can we turn it on?
			$conf = $this->ha->cman->getAllStatus();
			if (isset($conf['stonith']['fence_a']) && isset($conf['stonith']['fence_b'])) {
				$this->ha->cman->pcsCommand("property set stonith-enabled=true");
				return "enabled";
			}
			if (!isset($conf['stonith']['fence_a']) && !isset($conf['stonith']['fence_b']))
				return "<strong>not enabled <i>(Both nodes unfenced)</i></strong>";
			if (!isset($conf['stonith']['fence_a']))
				return "<strong>not enabled <i>(freepbx-a unfenced)</i></strong>";
			if (!isset($conf['stonith']['fence_b']))
				return "<strong>not enabled <i>(freepbx-b unfenced)</i></strong>";
		}

		return "enabled";
	}

	public function vmwareListGuests($req) {
		if (!isset($req['serverip']))
			return array("status" => "error", "message" => "No Server IP");

		if (!$this->ha->net->ping($req['serverip']))
			return array("status" => "error", "message" => "No ping.");

		if (!isset($req['login']) || !isset($req['passwd']))
			return array("status" => "error", "message" => "Invalid User/Password");

		// OK, lets try!
		// fence_vmware_helper --server '192.168.15.9' --username 'root' --password 'vmware' --operation on --vmname=HA-b
		$ip = escapeshellarg($req['serverip']);
		$u = escapeshellarg($req['login']);
		$p = escapeshellarg($req['passwd']);

		$cmd = "/usr/sbin/fence_vmware_helper --server $ip --username $u --password $p --operation list 2>&1";

		exec($cmd, $result, $ret);

		if ($ret != 0) {
			$output = implode("<br />\n", $result);
			return array("status" => "error", "message" => "Unable to run fence_vmware_helper - Is the VMware API Installed? <br />\nCommand: $cmd<br />\nOutput: $output<br />\n");
		}

		// We get a list that is basically machinename:morestuff.
		$vms = array();
		foreach ($result as $line) {
			$vmarray = explode(':', $line);
			$vms[] = $vmarray[0];
		}

		if (empty($vms))
			return array("status" => "error", "message" => "Connected OK, but no VM's Visible");

		return array("status" => "ok", "data" => $vms);
	}

	public function getFenceStatus($node = null) {
		$fencing = $this->getFencing($node);
		return $this->checkFencing($fencing);
	}

	public function checkFencing($f) {
		switch ($f['type']) {
		case 'VMware':
			return $this->getVMwareFenceStatus($f);
		case 'IPMI':
			return $this->getIPMIFenceStatus($f);
		case 'ILO':
			return $this->getILOFenceStatus($f);
		case 'DRAC':
			return $this->getDRACFenceStatus($f);
		default:
			return ("No fencing available for ".$f['type']);
		}
	}

	public function getVMwareFenceStatus($fencing) {
		return $this->doVMwareFence("status", $fencing);
	}

	public function getIPMIFenceStatus($fencing) {
		return $this->doIPMIFence("power status", $fencing);
	}

	public function getILOFenceStatus($fencing) {
		return $this->doILOFence("status", $fencing);
	}

	public function getDRACFenceStatus($fencing) {
		return $this->doDRACFence("status", $fencing);
	}

	private function doVMwareFence($cmd, $fencing) {
		$e = "/usr/sbin/fence_vmware -o$cmd -a".$fencing['serverip']." -l".$fencing['login']." -p".$fencing['passwd']." -n".$fencing['vmware_guestname']." 2>&1";
		exec($e, $out, $ret);
		return array("out" => $out, "ret" => $ret, "exec" => $e);
	}

	private function doIPMIFence($cmd, $fencing) {
		$e = "/usr/bin/ipmitool -H".$fencing['serverip']." -U".$fencing['login']." -P".$fencing['passwd']." $cmd 2>&1";
		exec($e, $out, $ret);
		return array("out" => $out, "ret" => $ret, "exec" => $e);
	}

	private function doILOFence($cmd, $fencing) {
		$e = "/usr/sbin/fence_ilo3 -a".$fencing['serverip']." -l".$fencing['login']." -p".$fencing['passwd']." -o$cmd 2>&1";
		exec($e, $out, $ret);
		return array("out" => $out, "ret" => $ret, "exec" => $e);
	}

	private function doDRACFence($cmd, $fencing) {
		$e = "/usr/sbin/fence_drac5 -x -a".$fencing['serverip']." -l".$fencing['login']." -p".$fencing['passwd']." -o$cmd 2>&1";
		exec($e, $out, $ret);
		return array("out" => $out, "ret" => $ret, "exec" => $e);
	}
}
