<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

class freepbxHA {

	private $dbname = "freepbxha";

	public function __construct() {
		// Stop spurious warnings about leaking FDs
		putenv("LVM_SUPPRESS_FD_WARNINGS=1");
		/* if (!function_exists('freepbxha_licensed') || !freepbxha_licensed())
		throw new Exception("License violation in ".__FILE__); */
		date_default_timezone_set(@date_default_timezone_get()); // FIXME: This needs to be fixed in FreePBX Distro.
		touch("/etc/schmooze/pbx-ha");
	}

	public function __get($var) {
		switch ($var) {
		case "database":
			include_once 'mysql.php';
			$this->database = new MySQL();
			return $this->database;
		case "update":
			include_once 'update.php';
			$this->update = new Update_HA($this);
			return $this->update;
		case "hosts":
			include_once 'hosts.php';
			$this->hosts = new Hosts();
			return $this->hosts;
		case "net":
			include_once 'network.php';
			$this->net = new Network();
			return $this->net;
		case "lvm":
			include_once 'lvm.php';
			$this->lvm = new LVM();
			return $this->lvm;
		case "cman":
			include_once 'cman.php';
			$this->cman = new CMAN($this);
			return $this->cman;
		case "drbd":
			include_once 'drbd.php';
			$this->drbd = new DRBD();
			return $this->drbd;
		case "ssh":
			include_once 'ssh.php';
			$this->ssh = new SSH($this);
			return $this->ssh;
		case "incron":
			include_once 'incron.php';
			$this->incron = new incron($this);
			return $this->incron;
		case "fencing":
			include_once 'fencing.php';
			$this->fencing = new Fencing($this);
			return $this->fencing;
		case "rseries":
			include_once 'rseries.php';
			$this->rseries = new R_Series($this);
			return $this->rseries;
		case "alerts":
			include_once 'alerts.php';
			$this->alerts = new Alerts($this);
			return $this->alerts;
		case "moveservices":
			include_once 'moveservices.php';
			$this->moveservices = new Move();
			return $this->moveservices;
		case "filemgr":
			include_once 'filemgr.php';
			$this->filemgr = new FileMgr();
			return $this->filemgr;
		case "passwd":
			include_once 'passwd.php';
			$this->passwd = new passwd;
			return $this->passwd;
		case "conf":
			include_once 'fpbxconf.php';
			$this->conf = new Conf();
			return $this->conf;
		case "packages":
			include_once 'packages.php';
			$this->packages = new Packages();
			return $this->packages;
		}
	}

	public function isEnabled() {
		// Return true or false if freepbx_ha is enabled
		return false;
	}

	public function whichMachineIsThis() {
		// Return 'PRIMARY' or 'SECONDARY' if we can figure it out,
		// if we can't, just assume we're PRIMARY.

		// First, check my hostname.
		$hn = $this->hosts->getHostname();

		if ($hn === 'freepbx-a') {
			return 'PRIMARY';
		} elseif ($hn === 'freepbx-b') {
			return 'SECONDARY';
		} else {
			// The hostname doesn't say.
			// We can look through our network interfaces to
			// get an idea.
			$ints = $this->net->discoverInterfaces();
			$primip = $this->getConfig('PRIMARY_intip');
			$secip = $this->getConfig('SECONDARY_intip');
			foreach ($ints as $int) {
				foreach ($int as $ip) {
					if ($ip[0] == $primip) return 'PRIMARY';
					if ($ip[0] == $secip) return 'SECONDARY';
				}
			}
		}
		return 'PRIMARY'; // Don't know? It's the primary.
	}

	public function whichMachineIsOther() {

		$thismachine = $this->whichMachineIsThis();

		if ($thismachine == "PRIMARY")
			return "SECONDARY";

		return "PRIMARY";
	}

	public function getConfig($var) {

		// Does our table exist?
		if (!$this->doesDBexist())
			$this->setupDefaultDB();

		return $this->database->getConfig($var);
	}

	public function setConfig($key, $val = null) {

		// Does our table exist?
		if (!$this->doesDBexist())
			$this->setupDefaultDB();

		return $this->database->setConfig($key, $val);
	}

	private function setupDefaultDB() {
		// Create the table
		$this->createDBTable();
		// Generate default config
		$defaults = $this->generateConfig();
		// Load it into the database
		foreach ($defaults as $key => $val)
			$this->setConfig($key, $val);

		$count = 1;
		$services = $this->getServices();
		foreach ($services as $service) {
			$this->setConfig($service."_id", $count);
			$this->setConfig("service_id_".$count, $service);
			$count++;
		}
	}

	public function doesDBexist() {
		// Check to see if we can load from the database.
		// This isn't very portable. Blech.
		$sql = "SELECT count(`key`) FROM ".$this->dbname;
		try {
			$result = $this->database->getAll($sql);
			return true;
		} catch (Exception $e) {
			return false;
		}
	}


	public function createDBTable() {

		// Does it already exist?
		if ($this->doesDBexist())
			return true;

		// Lets make some stuff. Really basic Key/Value storage, indexed on the key.
		$sql = 'CREATE TABLE `'.$this->dbname.'` ( `key` CHAR(64) NOT NULL , `value` VARCHAR(1024), PRIMARY KEY (`key`))';
		$this->dbQuery($sql);
		return $this->doesDBexist();
	}

	public function generateConfig() {
		// There is no config, we need to generate one.

		// Required Packages
		$def['PACKAGES'] = "drbd kmod-drbd pacemaker-1.1.10 cman ccs resource-agents pcs-0.9.90 resource-agent-freepbx";

		// Define the services to be HA'ed
		// 'spare' is there for any other service they want to make HA (eg, dhcpd, ldap-proxy, tftp, whatever)
		$def['SERVICES'] = "asterisk mysql httpd spare";

		// LVM Percentages (and, also minimum size in GB) of total space
		$def['SIZE_asterisk'] = 40;
		$def['SIZE_mysql'] = 20;
		$def['SIZE_httpd'] = 20;
		$def['SIZE_spare'] = 15;  // Not equal to 100, leave at least 1% spare, otherwise you may not be able to allocate it all.

		$def['DESCR_asterisk'] = "The Asterisk service";
		$def['DESCR_mysql'] = "MySQL Database";
		$def['DESCR_httpd'] = "Apache Web Server";
		$def['DESCR_spare'] = "Currently Spare";

		return $def;
	}

	public function destroyHA() {

		// Delete all LVs
		$this->lvm->tearDown();

		// Destroy cman
		$this->cman->tearDown();

		// Nuke IP addresses
		$this->net->tearDown();

		// Delete everything from FreePBX that mentions HA.
		$this->dbQuery('DROP TABLE IF EXISTS `freepbxha`');
		return false;
	}

	public function getServices() {
		// return an array of services
		return explode(" ", $this->getConfig('SERVICES'));
	}

	public function dbQuery($sql) {

		$result = $this->database->query($sql);

		return $result;
	}

	public function getSuggestedConf() {
		// This generates everything we want
		$extint = $this->getConfig($this->whichMachineIsThis()."_extint");
		if (isset($extint)) {
			$ret['externalint'] = $extint;
		} else {
			$ret['externalint'] = $this->net->getDefault();
		}
		$ret['suggestedip'] = $this->net->getSuggestedIP();
		if (!$this->getConfig('internalnet')) {
			$ret['internalnet'] = $this->net->generateRFC1918()."0/24";
			$this->setConfig('internalnet', $ret['internalnet']);
		} else {
			$ret['internalnet'] = $this->getConfig('internalnet');
		}
		$services = $this->getServices();
		$intstart = 10;
		foreach ($services as $s) {
			$ret['services'][$s]['IP'] = $this->net->getServiceIP($s);
			$ret['services'][$s]['SIZE'] = floor($this->lvm->getVolumeExtents($this->lvm->getVGName(), $s)/256);
			$ret['services'][$s]['DESCR'] = $this->getConfig("DESCR_".$s);
		}
		$ret['interfaces'] = $this->net->discoverInterfaces();
		$ret['internalint'] = $this->net->getInternalInterface();

		return $ret;
	}

	public function newMachineAdded() {
		// Called when a new machine is added to the cluster.
		// It would have - itself - installed the correct SSH
		// keys. We just need to do it on this node, too.
		$this->drbd->resyncMax();

		$this->ssh->injectSSHKey($this->getConfig('PRIMARY_publickey'));
		$this->ssh->injectSSHKey($this->getConfig('SECONDARY_publickey'));
		$check = sha1($this->getConfig('PRIMARY_publickey').$this->getConfig('SECONDARY_publickey'));

		// Only mark as done if BOTH machines have updated to the correct keys.
		if ($this->getConfig($this->whichMachineIsOther()."_updated") == $check) {
			$this->setConfig($this->whichMachineIsThis()."_updated", null);
			$this->setConfig($this->whichMachineIsOther()."_updated", null);
		} else {
			$this->setConfig($this->whichMachineIsThis()."_updated", $check);
		}
	}
}
