<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

include_once 'network.php';

class Hosts {

	public $hf = "/etc/hosts";

	public function __construct() {
		/* if (!function_exists('freepbxha_licensed') || !freepbxha_licensed())
			throw new Exception("License violation in ".__FILE__); */
		if (defined('INSIDE_TESTS')) {
			$this->hf = "/tmp/hosts";
		}
	}

	public function setHostname($hostname = null) {

		// Are you retarded?
		if ($hostname == null) 
			return false;

		// Don't change localhost.
		if ($hostname == "localhost")
			return false;

		$net = new Network;
		$ip = $net->getValidInterface();
		
		include_once 'incron.php';
		$incron = new incron;
		return $incron->changeHostname($hostname);
	}

	public function getHostname() {
		// Return the current hostname
		exec("/bin/hostname", $output, $ret);
		if ($ret != 0)
			return false;
		return $output[0];
	}

	public function getOtherHostname() {
		// Return the hostname of the other machine.
		include_once 'functions.inc.php';
		$ha = new freepbxha;

		if ($ha->whichMachineIsThis() == 'PRIMARY')
			return 'freepbx-b';

		return 'freepbx-a';
	}

	// Public access to incron function
	public function removeHost($hostname) {
		include_once 'incron.php';
		$i = new incron;

		return $i->removeHost($hostname);
	}

	// Root access from incron.
	public function root_removeHost($hostname) {
		// Delete this host from /etc/hosts
		$hosts = file($this->hf);

		$newhosts = array();
		$foundhost = false;

		foreach ($hosts as $line) {

			// Remove blank space, \n's etc.
			$line = ltrim(rtrim($line));

			// If the line starts with #, continue on.
			if (preg_match("/^#/", $line)) {
				$newhosts[] = $line."\n";
				continue;
			}

			// It's going to be along the lines of [address][whitespace][(hostname)[whitespace](hostname)[whitespace)...]
			// Lets break it in half first.
			if (preg_match("/^(\d*+\.\d*+\.\d*+\.\d*+)\s*+(.+)$/", $line, $result)) {
				// It looks valid. It's got at least two things in it.
				if (preg_match("/\b${hostname}\b/", $result[2])) {
					// It also has the hostname we're looking for in it. 
					$foundhost = true;
					$newline = preg_replace("/\b${hostname}\b/", "", $result[2]);
					// If there's anything left, add the remains back. 
					if (rtrim(ltrim($newline)) != "") 
						$newhosts[] = $result[1]."\t\t".$newline."\n";
				} else {
					$newhosts[] = $line."\n";
				}
			} else {
				// I don't know WHAT this line is, but leave it in there anyway.
				$newhosts[] = $line."\n";
			}
		}

		if ($foundhost == false) 
			return array("ret" => 5, "result" => "");

		if (empty($newhosts))
			throw new Exception('Hosts file would be empty');

		rename($this->hf, $this->hf.".bak");
		return array("ret" => (file_put_contents($this->hf, $newhosts) != false), "result" => "");
	}

	// Public access
	public function addHost($hostname, $ip) {
		include_once 'incron.php';
		$i = new incron;

		return $i->addHost($hostname, $ip);
	}

	// Root access from incron
	public function root_addHost($hostname, $ip) {
		// Add this hosts to /etc/hosts
		// 
		// Do not try to validate anything that requires a database call
		// as this may be called when the node is in an interminate state 
		// of joining the cluster.
		//
		// Santiy check IP address.

		if (!filter_var($ip, FILTER_VALIDATE_IP)) 
			throw new Exception("Invalid IP supplied - $ip");

		$hosts = file($this->hf);

		$newhosts = array();
		$foundip = false;

		// If that IP address is already in hosts, stick it on the end.
		// If that IP isn't in hosts, add a new line.
		foreach ($hosts as $line) {

			// Remove trailing and leading space, \n's etc.
			$line = rtrim(ltrim($line));

			// If the line starts with #, continue on.
			if (preg_match("/^#/", $line)) {
				$newhosts[] = $line."\n";
				continue;
			}

			// It's going to be along the lines of [address][whitespace][(hostname)[whitespace](hostname)[whitespace)...]
			// Lets break it in half first.
			if (preg_match('/^([0-9\.a-f:]+)\s+(.+)$/', $line, $result)) {
				// It looks valid. It's got at least two things in it.
				// Does this line have this hostname on it already? If so, we don't want it there.
				$line = preg_replace("/\s*+${hostname}\b/", "", $line);
				if ($ip == $result[1]) {
					// Ahha. This IP allready exists.
					$foundip = true;
					$newhosts[] = $line."\t".$hostname."\n";
				} else {
					if ( ltrim(rtrim($line)) != ltrim(rtrim($result[1])) )
						$newhosts[] = $line."\n";
				}
			} else {
				// I don't know WHAT this line is, but leave it in there anyway.
				// If the line isn't empty, put it back. Sometimes there's a reason
				// for an IP without a host, but for the moment we'lla assume it's broken.
				if (!preg_match("/^([0-9\.a-f:]+)$/", $line)) 
					$newhosts[] = $line."\n";
			}
		}

		if (!$foundip) 
			$newhosts[] = $ip."\t\t".$hostname."\n";

		if (empty($newhosts))
			throw new Exception('Hosts file would be empty');

		rename($this->hf, $this->hf.".bak");
		return array("ret" => (file_put_contents($this->hf, $newhosts) !== false), "result" => "") ;
	}

	public function addMainHosts() {
		include_once 'functions.inc.php';
                $ha = new freepbxHA;

                $network = $ha->getConfig('internalnet');
                if (substr($network, -4) != "0/24")
                        throw new Exception('internal net is not a /24 - '.$network);
                $primip = substr($network, 0, -4)."1";
                $secip = substr($network, 0, -4)."2";

                include_once 'hosts.php';
                $h = new Hosts;

                $h->addHost("freepbx-a", $primip);
                $h->addHost("freepbx-b", $secip);
                return array("status" => "ok");
        }

	public function addAllHosts() {
		include_once 'functions.inc.php';
		$ha = new freepbxha;
		include_once 'network.php';
		$net = new Network;

		$services = $ha->getServices();

		foreach ($services as $service) {
			$this->addHost($service, $net->getServiceIP($service));
		}

		$this->addMainHosts();

	}

	public function setMyHostname() {
                // Update the hostname to be 'freepbx-a' or 'freepbx-b'
                $ha = new freepbxHA;
                $me = $ha->whichMachineIsThis();

                if ($me == 'PRIMARY') {
                        $r = $this->setHostname('freepbx-a');
			if ($r['ret'] != 1) 
				return $r;
                        if ($this->getHostname() == 'freepbx-a')
                                return array("status" => "ok");
                } elseif ($me == 'SECONDARY') {
                        $r = $this->setHostname('freepbx-b');
			if ($r['ret'] != 1) 
				return $r;
                        if ($this->getHostname() == 'freepbx-b')
                                return array("status" => "ok");
                }
                return array("status" => "Unable to determine which machine this is");
        }

}

