<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:
// incron commands

// Only required when moving apache around, as file locations change.
if (class_exists('incron')) {
	return;
} else {
	class incron {

		// Need to figure out a better way of doing this.
		private $hadir = "/var/www/html/admin/modules/freepbx_ha/functions.inc";

		private $netconf = "/etc/sysconfig/network-scripts/";
		private $trigger = "/usr/local/asterisk/ha_trigger"; 
		private $fn = "/tmp/incron.ha";
		private $drbddir = "/etc/drbd.d";
		private $allowedservices = array("pacemaker", "cman", "drbd", "mysqld", "asterisk", "iSymphonyServer", "httpd");

		private $outdir = "/tmp/";
		public $delay = 500;  // Loop this many times, waiting for a command to complete. (Loop delay = .5 secs)


		private $commands = array(
			0  => "__fullTest",
			1  => "__vgDisplay",
			2  => "__lvDisplay",
			3  => "__lvCreate",
			4  => "__addIP",
			5  => "__rmInt",
			6  => "__ifchange",
			7  => "__regenCMAN",
			8  => "__manageService",
			9  => "__getPCSStatus",
			10 => "__changeHostName",
			11 => "__regenPacemaker",
			12 => "__pcsCommand",
			13 => "__addHost",
			14 => "__removeHost",
			15 => "__writedrbd",
			16 => "__drbdadm",
			17 => "__formatVolume",
			18 => "__arping",
			19 => "__updatefpbxconf",
			20 => "__moveApache",
			21 => "__apacheConf",
			22 => "__updateMySQLConf",
			23 => "__moveService",
			24 => "__updateFiles",
			25 => "__checkHomes",
			26 => "__crm_resource",
			27 => "__crm_failcount",
			28 => "__pcsUpdate",
			29 => "__rctl",
		);

		public function __construct($ha = null) {
			if (!class_exists('freepbxHA')) 
				include $this->hadir.'/functions.inc.php';

			if (defined('INSIDE_TESTS') || posix_getuid() == 0) {
				$this->trigger="/tmp/trigger";
			}

			if ($ha == null) {
				$this->ha = new freepbxHA;
			} else {
				$this->ha = $ha;
			}

		}

		public function doTrig() {
			if (defined('INSIDE_TESTS') || posix_getuid() == 0) {
				$this->rootTrigger();
			} else {
				//  Crash and burn if incrond doesn't know about us, and we can't fix it.
				$fh = fopen("/var/spool/incron/root", "r");
				$foundit = false;
				while ($line = fgets($fh)) {
					if (preg_match('/^(.+) IN_CLOSE_WRITE .usr.sbin.sysadmin_ha$/', $line, $res)) {
						if ($res[1] == $this->trigger)
							$foundit = true;
					}
				}
				if (!$foundit) 
					throw new Exception('/var/spool/incron/root is not setup correctly.');

				if (!file_exists("/usr/sbin/sysadmin_ha"))
					throw new Exception('/usr/sbin/sysadmin_ha trigger does not exist');

				if (!file_exists($this->trigger)) {
					// Incron wasn't able to read this file when it started. 
					// Create it, and then tell incron to restart.
					fclose(fopen($this->trigger, "w")); // Create our trigger file.
					chmod($this->trigger, 0777); // Let everyone write to it.
					fclose(fopen("/var/spool/asterisk/sysadmin/incron_restart", "w")); // Tell incron to restart.
					sleep(1); // Wait a second for incron to reload.
				}
				// Actually trigger the root-run process.
				fclose(fopen($this->trigger, "w"));
			} 
		}

		public function rootTrigger() {
			if (!file_exists($this->fn)) 
				return;
			// There may be multiple commands in here.
			$lines = file($this->fn);
			unlink($this->fn);
			foreach ($lines as $cmd) {
				$cmd = trim($cmd);
				$this->doLog("rootTrigger each - $cmd");
				$arr = json_decode($cmd, true);
				$uid = end($arr);
				if (!$this->isUniqid($uid)) {
					$this->doLog("rootTrigger - Invalid uid - $uid");
					exit;
				}

				if (isset($this->commands[$arr[0]])) {
					$f = $this->commands[$arr[0]];
					if (method_exists($this, $f)) {
						// Take off the command
						array_shift($arr);

						// Run it
						try {
							$this->doLog("rootTrigger - Doing $f(".json_encode($arr).")");
							$result = $this->$f($arr);
							if (!isset($result['ret']))
								$result = array(
									"status" => "systemerror", 
									"result" => "Correct array was not returned from function $f(".json_encode($arr).") - ".json_encode($result),
									"ret" => 99);
							$this->dumpOutput($result, end($arr));
						} catch (Exception $e) {
							$result = array(
								"status" => "systemerror", 
								"result" => "System Error: ".$e->getMessage(),
								"ret" => 99);
							$this->dumpOutput($result, end($arr));
						}
					} else {
						$result = array(
							"status" => "systemerror", 
							"result" => "System Error: $f is unimplemented",
							"ret" => 99);
						$this->dumpOutput($result, end($arr));
					}
				}
			}
		}

		private function isUniqid($uid) {
			if (preg_match('/^[0-9a-f]+$/', $uid))
				return true;
		}

		private function dumpOutput($res, $uid) {
			$this->doLog("Output $uid - ".json_encode($res));
			file_put_contents($this->outdir.$uid, json_encode($res));
			chmod($this->outdir.$uid, 0700);
			chown($this->outdir.$uid, "asterisk");
		}

		public function getOutput($uid) {
			$counter = 0;
			$exists = false;
			while ( $counter++ <= $this->delay ) {
				if (file_exists($this->outdir.$uid)) {
					$exists = true;
					$res = json_decode(file_get_contents($this->outdir.$uid), true);
					if ($res !== false) {
						if (isset($res['ret'])) {
							unlink($this->outdir.$uid);
							return $res;
						} 
					}
				}
				// Wait 1/2 a second before trying again.
				usleep(500000);
			}
			if ($exists) {
				throw new Exception("Unable to process $uid");
			} else {
				throw new Exception("Never received trigger $uid");
			}
		}

		private function genTrig() {
			$args = func_get_args();
			$uid = uniqid();
			$args[] = $uid;
			file_put_contents($this->fn, json_encode($args)."\n", FILE_APPEND);
			$this->doLog("genTrig - ".json_encode($args));
			$this->doTrig();
			return $this->getOutput($uid);
		}

		private function doLog($str) {
			if (!file_exists("/tmp/incron.log")) 
				return true;

			// Don't spam errors if we can't write to it.
			if ($fh = @fopen("/tmp/incron.log", "a")) {
				fwrite($fh, posix_getpid().": $str\n");
				fclose($fh);
			}
		}


		/********************
		 * Tests start here *
		 ********************/

		public function testIncron() {
			return $this->genTrig(0);
		}

		private function __fullTest() {
			return array("result" => "Incron test suceeded", "ret" => 0);
		}

		public function vgDisplay() {
			return $this->genTrig(1);
		}

		private function __vgDisplay() {
			// Return the amount of unusued LVM space in Gigabytes.
			putenv("LVM_SUPPRESS_FD_WARNINGS=1");
			exec('/sbin/vgdisplay -c 2>&1', $results, $ret);
			if (trim($results[0]) == "No volume groups found") {
				return array("result" => "", "ret" => 99);
			}
			foreach ($results as $r) {
				$arr = explode(':', trim($r));
				// vg_ha:r/w:772:-1:0:2:2:-1:0:1:1:209199104:4096:51074:12256:38818:zsWk8A-fO91-n9Ze-GaQ1-9DJ7-yyPT-27coPc
				$name = $arr[0];
				$extsize = $arr[12];
				$exts = $arr[13];
				$alloc = $arr[14];
				$free = $arr[15];
				$retarr[] = array( 'name' => $name, 'extsize' => $extsize, 'exts' => $exts,
					'alloc' => $alloc, 'free' => $free);
			}
			return array("result" => $retarr, "ret" => $ret);
		}

		public function lvDisplay($vgname = null, $lvname = null) {
			// Return information about all lvs, or, a specific one.
			if ($vgname === null && $lvname === null) {
				return $this->genTrig(2);
			} else {
				return $this->genTrig(2, $vgname, $lvname);
			}
		}

		private function __lvDisplay($inarr) {
			if (count($inarr) == 3) {
				// We have parameters. 
				list($vgname, $lvname, $uid) = $inarr;
				// Sanity check.
				if (preg_match('/^[a-zA-Z0-9_]+$/', $vgname) && preg_match('/^[a-zA-Z0-9_]+$/', $lvname)) {
					putenv("LVM_SUPPRESS_FD_WARNINGS=1");
					exec("/sbin/lvdisplay -c /dev/$vgname/$lvname 2>&1", $result, $ret);
					if ($ret != 0) {
						return array("result" => $result, "ret" => $ret);
					}

					// Volume exists.
					$raw = explode(':', trim($result[0]));
					$retarr['extents'] = $raw[7];
					exec("/sbin/lvdisplay -C --noheadings --nosuffix --units g /dev/$vgname/$lvname 2>&1", $result, $ret);
					$raw = explode(' ', trim($result[1]));
					$retarr['size_in_g'] = $raw[3];
					exec("/sbin/vgdisplay -c $vgname 2>&1", $result, $ret);
					$raw = explode(':', trim($result[2]));
					$retarr['extentsize'] = $raw[12];
					return array("result" => $retarr, "ret" => 0);
				} else {
					// Failed sanity check
					return false;
				}
			} else {
				// No params
				putenv("LVM_SUPPRESS_FD_WARNINGS=1");
				exec("/sbin/lvdisplay -c 2>&1", $result, $ret);
				if ($ret != 0) {
					return array("result" => $result, "ret" => $ret);
				}
				if (trim($result[0]) == "No volume groups found") {
					return array("result" => $result[0], "ret" => 99);
				}
				foreach ($result as $r) {
					$arr = explode(':', trim($r));
					if (preg_match('!/dev/vg_ha/drbd_(.+)!', $arr[0], $match)) {
						$retarr['service'] = $match[1];
						$retarr['vgname'] = $arr[1];
						$retarr['size_in_g'] = round($arr[6]/2097152, 2);  // 1024*2048 (sector size)
						$retarr['extents'] = $arr[7];
						$out[$match[1]] = $retarr;
					}
				}
				$out['raw'] = $result;
				return array("result" => $out, "ret" => 0);
			}
		}

		public function lvCreate($vgname, $name, $size, $byextents = false) {
			return $this->genTrig(3, $vgname, $name, $size, $byextents);
		}

		private function __lvCreate($inarr) {
			if (count($inarr) == 5) {
				list ($vgname, $name, $size, $byextents, $uid) = $inarr;
				// Sanity check
				if (preg_match('/^[a-zA-Z0-9_]+$/', $vgname) && preg_match('/^[a-zA-Z0-9_]+$/', $name) && is_numeric($size)) {
					putenv("LVM_SUPPRESS_FD_WARNINGS=1");
					if ($byextents) {
						exec("/sbin/lvcreate $vgname -n $name -l${size} 2>&1", $results, $ret);
					} else {
						exec("/sbin/lvcreate $vgname -n $name -L${size}G 2>&1", $results, $ret);
					}
					return array("result" => $results, "ret" => $ret);
				} else {
					return array("result" => "System Error: invalid params provided ($vgname, $name, $size)", "ret" => 99);
				}
			}
		}

		public function addIP($ip, $class, $int) {
			return $this->genTrig(4, $ip, $class, $int);
		}

		public function rmInt($int) {
			return $this->genTrig(5, $int);
		}

		public function __rmInt($inarr) {
			$int = $inarr[0];

			if ($this->ha->getConfig("PRIMARY_int") != $int && $this->ha->getConfig("SECONDARY_int") != $int)
				throw new Exception("Trying to remove $int that I don't know about");

			$res = $this->__ifchange(array("down", $int, "1"));
			if ($res['ret'] == 0) {
				unlink($this->netconf.$int);
				`ifconfig $int 0.0.0.0`;
				return $res;
			} else {
				return array("result" => "error", "ret" => 5);
			}
		}

		private function __addIP($inarr) {
			if (count($inarr) == 4) {
				list ($ip, $class, $int) = $inarr;
				if (!filter_var($ip, FILTER_VALIDATE_IP))
					throw new Exception("Invalid IP - $ip");

				// Make sure the interface exists.
				if (!$this->ha->net->checkValidInterface($int))
					throw new Exception("Unknown interface - $int");

				$netmask = $this->ha->net->getNetmaskFromCIDR($class);
				if (!filter_var($netmask, FILTER_VALIDATE_IP))
					throw new Exception("Invalid CIDR - $class");

				$filename = "error";
				$f = "error";
				$result = "";

				// Are there any IPs on that device?
				if (!isset($knownints[$int][0])) {
					// There's not. We're creaing a new one.
					// Is there a script for this somewhere? Would be better.
					$result = $int;
					$filename = "ifcfg-$int";
					$f = "DEVICE=$int\nIPADDR=$ip\nNETMASK=$netmask\nBOOTPROTO=static\nIPV6INIT=no\nNETWORKING_IPV6=no\nONBOOT=yes\nNM_CONTROLLED=no\n";
				} else {
					// Find the first unused virtual interface.
					$v = 0;
					for ( ; $v < 255; $v++) {
						$used = false;
						foreach ($knownints[$int] as $find) {
							if ($find[1] == "$int:$v")
								$used = true;
						}
						if ($used == false) 
							break;
					}
					if ($v > 254)
						throw new Exception("Can't find valid alias");

					// OK, our alias shall be $v. Lets create it.
					$result = "${int}:${v}";
					$filename = "ifcfg-${int}:${v}";
					$f = "DEVICE=${int}:${v}\nIPADDR=$ip\nNETMASK=$netmask\nBOOTPROTO=static\nIPV6INIT=no\nNETWORKING_IPV6=no\nONBOOT=yes\nNM_CONTROLLED=no\n";
				}
				if ($filename == "error" || $f == "error")
					throw new Exception("Unable to create interface $ip, $class, $int");

				// So, we're here, with stuff. Hit it.
				$ret = file_put_contents($this->netconf.$filename, $f);

				if ($ret == false) 
					throw new Exception("Unable to write to file $filename");

				return array("result" => $result, "ret" => 0);
			}
			return array("result" => "", "ret" => 5);
		}

		public function ifup($int) {
			return $this->genTrig(6, "up", $int);
		}

		public function ifdown($int) {
			return $this->genTrig(6, "down", $int);
		}

		private function __ifchange($inarr) {
			if (count($inarr) != 3)
				return array("result" => "", "ret" => 5);

			$cmd = $inarr[0];
			$int = $inarr[1];

			if (!$this->ha->net->checkValidInterface($int))
				throw new Exception("Trying to start $int that isn't an interface");

			if ($cmd == "up") {
				exec("/sbin/ifup $int 2>&1", $results, $ret);
			} elseif ($cmd == "down") {
				exec("/sbin/ifdown $int 2>&1", $results, $ret);
				// Delete the interface
				@unlink("/etc/sysconfig/network-scripts/ifcfg-$int");
			}
			return array("result" => $results, "ret" => $ret);
		}

		public function regenCMAN() {
			return $this->genTrig(7);
		}

		private function __regenCMAN($inarr) {
			// No params.
			if ($this->ha->cman->root_configCMAN())
				return array("result" => "", "ret" => 0);

			return array("result" => "", "ret" => 5);
		}

		public function manageService($cmd, $service) {
			return $this->genTrig(8, $cmd, $service);
		}

		private function __manageService($inarr) {
			$service = $inarr[0];
			$cmd = $inarr[1];
			// Sanity check
			$allowedcommands = array("start", "stop", "reload", "restart", "on", "off");
			if (in_array($cmd, $allowedcommands) && in_array($service, $this->allowedservices)) {
				if ($cmd == "on") {
					exec("/sbin/chkconfig $service on", $results, $ret);
					return array("result" => $results, "ret" => $ret);
				} elseif ($cmd == "off") {
					exec("/sbin/chkconfig $service off", $results, $ret);
					return array("result" => $results, "ret" => $ret);
				} else {
					exec("/etc/init.d/$service $cmd 2>&1", $results, $ret);
				}
			}  else {
				throw new Exception("Not allowed to do $cmd to $service");
			}
			return array("result" => $results, "ret" => $ret);
		}

		public function changeHostname($hostname) {
			return $this->genTrig(10, $hostname);
		}

		private function __changeHostname($inarr) {
			$newhostname = $inarr[0];
			// Sanity check
			if ("'$newhostname'" != escapeshellarg($newhostname))
				throw new Exception("Invalid hostname - x${newhostname}x");

			exec("/bin/hostname $newhostname 2>&1", $results, $ret);
			// Don't forget to update /etc/sysconfig/network!
			$network = parse_ini_file("/etc/sysconfig/network", false, INI_SCANNER_RAW);
			$network['HOSTNAME'] = $newhostname;
			// Write the config file back
			$newconf[] = "# Updated by FreePBX HiAv";
			foreach ($network as $k => $v) {
				$newconf[] = $k."=".$v;
			}
			$newconf[] = "# End Config\n";
			file_put_contents("/etc/sysconfig/network", join("\n", $newconf));

			// Now, test it.
			$test = parse_ini_file("/etc/sysconfig/network");
			if ("x".$test['HOSTNAME'] != "x".$newhostname)
				return array("result" => "Unable to set hostname. Check returned ".$test['HOSTNAME']." instead of ".$newhostname, "ret" => 98);

			return array("result" => $results, "ret" => true);
		}

		public function getPCSStatus() {
			return $this->genTrig(9);
		}

		private function __getPCSStatus() {
			// No params.
			return $this->ha->cman->root_getPCSStatus();
		}

		public function regenPacemaker() {
			return $this->genTrig(11);
		}

		private function __regenPacemaker() {
			// No params.
			return $this->ha->cman->root_configPacemaker();
		}

		public function pcsCommand($cmd) {
			// This is only called internally, don't need so much sanity checking
			return $this->genTrig(12, $cmd);
		}

		private function __pcsCommand($inarr) {
			return $this->ha->cman->root_pcsCommand($inarr[0]);
		}

		public function addHost($host, $ip) {
			return $this->genTrig(13, $host, $ip);
		}

		private function __addHost($inarr) {
			return $this->ha->hosts->root_addHost($inarr[0], $inarr[1]);
		}

		public function removeHost($host) {
			return $this->genTrig(14, $host);
		}

		private function __removeHost($inarr) {
			return $this->ha->hosts->root_removeHost($inarr[0]);
		}

		public function writedrbd($service, $conf) {
			return $this->genTrig(15, $service, $conf);
		}

		private function __writedrbd($inarr) {
			list($service, $conf) = $inarr;
			if ($service == "global_common.conf") {
				$filename = "/etc/drbd.d/global_common.conf";
			} else {
				$filename = $this->drbddir."/".$service.".res";
				if (dirname($filename) != $this->drbddir)
					throw new Exception('Service name did not pass sanity checks');
				if (!file_exists("/drbd"))
					mkdir("/drbd");
				if (!file_exists("/drbd/$service"))
					mkdir("/drbd/$service");
			}

			if (file_put_contents($filename, $conf) !== false)
				return array("result" => "", "ret" => 0);

			return array("result" => "failed", "ret" => 5);
		}

		public function drbdadm($cmd) {
			return $this->genTrig(16, $cmd);
		}

		private function __drbdadm($inarr) {
			$cmd = $inarr[0];
			if ("'$cmd'" != escapeshellarg($cmd))
				throw new Exception("Invalid command - x${cmd}x");
			exec("yes yes|/sbin/drbdadm $cmd 2>&1", $results, $ret);
			return array("result" => $results, "ret" => $ret);
		}

		public function formatDRBDVolume($id, $service) {
			return $this->genTrig(17, $id, $service);
		}

		private function __formatVolume($inarr) {
			$id = $inarr[0];
			$service = $inarr[1];
			if (!is_numeric($id))
				throw new Exception("Invalid service id - x${id}x");
			$mntpoint = "/drbd/$service";
			exec("/sbin/mkfs.ext4 /dev/drbd${id} -E lazy_itable_init -M $mntpoint 2>&1", $results, $ret);
			return array("result" => $results, "ret" => $ret);
		}

		public function arping($int, $ip) {
			return $this->genTrig(18, $int, $ip);
		}

		private function __arping($inarr) {
			list($int, $ip) = $inarr;

			// Make sure the interface exists.
			$knownints = $this->ha->net->discoverInterfaces();

			if (!isset($knownints[$int]))
				throw new Exception("Unknown interface - $int");

			if (!filter_var($ip, FILTER_VALIDATE_IP))
				throw new Exception("Invalid IP - $ip");

			exec("/sbin/ifconfig $int up"); // Ensure the interface is up
			exec("/usr/sbin/arping -c1 -D -I $int $ip", $results, $ret);
			return array("result" => $results, "ret" => $ret);
		}

		public function updateFpbxConf($arr) {
			return $this->genTrig(19, $arr);
		}

		private function __updatefpbxconf($inarr) {
			$newconf = $inarr[0];
			$uid = $inarr[1];

			$conf = file_get_contents("/etc/freepbx.conf");
			foreach ($newconf as $k => $v) {
				$conf = preg_replace("/(amp_conf\['$k.+=)(.+)$/m", "$1 '$v';", $conf);
			}
			$ret = file_put_contents("/etc/freepbx.conf", $conf);
			if ($ret == false) 
				throw new Exception("Unable to write to freepbx.conf");

			return array("result" => $conf, "ret" => $ret);
		}

		public function moveApache($rsync = false) {
			if ($rsync) {
				return $this->genTrig(20, "");
			} else {
				return $this->genTrig(20, "--secondary");
			}
		}

		private function __moveApache($inarr) {
			exec(dirname(__FILE__)."/../moveapache.php ".$inarr[0]." >> /tmp/apachemove.log 2>&1", $results, $ret);
			return array("result" => $results, "ret" => $ret);
		}

		public function apacheConf() {
			return $this->genTrig(21);
		}

		private function __apacheConf() {
			// Generate /etc/httpd/conf.d/status.conf
			$conf = "<Location /server-status>\n\tSetHandler server-status\n\tOrder deny,allow\n\tDeny from all\n\tAllow from localhost\n</Location>";
			$ret = file_put_contents("/etc/httpd/conf.d/status.conf", $conf);
			if ($ret == false) 
				throw new Exception("Unable to write to /etc/httpd/conf.d/status.conf");
			return array("result" => "", "ret" => 0);
		}

		public function updateMySQLConf($conf) {
			return $this->genTrig(22, $conf);
		}

		private function __updateMySQLConf($arr) {
			$conf = implode("\n", $arr[0]);
			$ret = file_put_contents("/etc/my.cnf", $conf);
			if ($ret == false) 
				throw new Exception("Unable to write to /etc/my.cnf");
			return array("result" => "", "ret" => 0);
		}

		public function moveService($service, $rsync) {
			return $this->genTrig(23, $service, $rsync);
		}

		private function __moveService($arr) {
			return $this->ha->moveservices->root_moveService($arr[0], $arr[1]);
		}

		public function updateFiles() {
			return $this->genTrig(24);
		}

		private function __updateFiles() {
			return array("result" => $this->ha->filemgr->getCopied(), "ret" => 0);
		}

		public function checkHomes() {
			return $this->genTrig(25);
		}

		private function __checkHomes() {
			return array("result" => "", "ret" => $this->ha->passwd->updateHome('asterisk', '/home/asterisk'));
		}

		public function crm_resource($cmd) {
			return $this->genTrig(26, $cmd);
		}

		private function __crm_resource($arr) {
			$cmd = $arr[0];
			exec("/usr/sbin/crm_resource $cmd 2>&1", $results, $ret);
			return array("result" => $results, "ret" => $ret);
		}

		public function crm_failcount($rsc, $node) {
			return $this->genTrig(27, $rsc, $node);
		}

		private function __crm_failcount($arr) {
			$rsc = $arr[0];
			$node= $arr[1]; 
			exec("crm_failcount -r $rsc --node=$node 2>&1", $results, $ret);
			return array("result" => $results, "ret" => $ret);
		}

		public function pcsUpdate() {
			return $this->genTrig(28);
		}

		private function __pcsUpdate() {
			return $this->ha->cman->root_pcsUpdate();
		}

		public function rctl($str) {
			return $this->genTrig(29, $str);
		}

		private function __rctl($arr) {
			$cmd = $arr[0];
			exec("/sbin/rctl $cmd 2>&1", $results, $ret);
			return array("result" => $results, "ret" => $ret);
		}
	}
}
