<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

class JoinCluster {

	public function __construct() {
		/* if (!function_exists('freepbxha_licensed') || !freepbxha_licensed())
		throw new Exception("License violation in ".__FILE__); */
	}

	public function getJoinStuff() {
		$r = array(
			"checkother" => array("Making sure the other machine is reachable", "If this machine has many interfaces, this may take some time"),
			"checkmyip" => array("Validate internal address", "If this machine has many interfaces, this may take some time"),
			"findinternal" => array("Discover 'internal' interface", "If this machine has many interfaces, this may take some time"),
			"assignip" => "Create internal IP Address",
			"startip" => "Activate internal IP Address",
			"checkotherping" => "Final Connectivity check",
			"checkdb" => "Attempting to connect to MySQL on other node",
			"checkdbcontents" => "Checking that the other node is configured.",
			"checklvmspace" => "Check for available disk space",
		);

		return $r;
	}

	public function doJoin() {
		$r = array(
			"linkhosts" => "Linking to Cluster Database",
			"checkdblocal" => "Checking Datatabase Consistency",
			"changehostname" => "Change Hostname",
			"addhosts" => "Add all hosts entries",
			"addsshkey" => "Adding SSH key from other node",
			"gensshkey" => "Generating SSH Key for this node",
			"mysqlconfig" => "Updating MySQL Configuration",
			"stopmysql" => "Shut down and disable local MySQL",
			"stopasterisk" => "Shut down and disable local Asterisk",
			"createlvm" => "Create LVM Volumes",
			"createdrbd" => "Create DRBD Configuration",
			"activatedrbd" => "Prepare DRBD Volumes",
			"startdrbd" => "Start DRBD Service",
			"maxdrbd" => "Set temporary full-bandwidth resync",
			"cmanconf" => "Generate Cluster Configuration",
			"moveservices" => array("Finish Machine Configuration", "Waiting for local Apache to error..."),
		);

		return $r;
	}

	public function checkBase64($str) {
		// Ajax call. If it's sane, it'll return the values.
		$decoded = base64_decode(ltrim(rtrim($str)));

		if ($decoded === false) 
			return array('status' => 'invalid');

		$arr = json_decode($decoded, true);
		return array('status' => 'decoded', 'ret' => $arr);
	}

	private function isReachable($ip, $int = null) {
		// Return true if you can find this ip on any interface

		include_once 'incron.php';
		$incron = new incron;

		include_once 'network.php';
		$net = new Network;

		if ($int == null) {
			$ints = $net->discoverInterfaces(true);  // Ain't nobody got time for lo

			foreach ($ints as $int => $val ) {
				$i = $incron->arping($int, $ip);
				if ($i['ret'] == 1)
					return true;
			}
			return false;
		} else {
			$i = $incron->arping($int, $ip);
			if ($i['ret'] == 1)
				return true;
			return false;
		}
	}

	public function checkother($arr) {
		// Ensure we can find the other node

		if ($this->isReachable($arr['otherip']))
			return array('status' => 'ok');

		return array('status' => 'Unable to find '.$arr['otherip'].' on any interface');
	}

	public function checkmyip($arr) {
		// Make sure that the machine I'm becoming doesn't already exist.
		if (!isset($arr['otherip']))
			throw new Exception("IP Address not given");

		include_once 'network.php';
		$net = new Network;

		$myip = $net->getOtherIp($arr['otherip']);

		if ($this->isReachable($myip))
			return array('status' => "Something on this network already is using the address $myip");

		return array('status' => "ok", "setdata" => array("myip" => $myip));
	}

	public function findinternal($arr) {
		include_once 'network.php';
		$net = new Network;

		// Lets start with the easiest.  Can we guess what interface it's on?
		$try = $net->getInternalInterface();

		if ($this->isReachable($arr['otherip'], $try)) {
			// Well, that was easy. 
			return array('status' => 'ok', 'setdata' => array('myint' => $try));
		}

		// Poot. OK. Lets loop through the interfaces. We found it before.
		$ints = $net->discoverInterfaces(true);  // Ain't nobody got time for lo
		foreach ($ints as $int => $val ) {
			if ($this->isReachable($arr['otherip'], $int))
				return array('status' => 'ok', 'setdata' => array('myint' => $int));
		}

		return array('status' => "Wait. Now I can't find it? Something's gone crazy",);
	}

	public function assignip($arr) {
		include_once "network.php";
		$net = new Network;

		$res = $net->addIp($arr['myip'], 24, $arr['myint']);
		if ($res['ret'] == 0) {
			return array("status" => "ok", 'setdata' => array('myint' => $res['result']));
		}
		return array("status" => "Unable to create network interface - ".$res['result']);
	}

	public function startip($arr) {
		include_once "network.php";
		$net = new Network;

		if ($net->ifup($arr['myint']))
			return array("status" => "ok");

		return array("status" => "failed");
	}

	public function checkotherping($arr) {
		include_once "network.php";
		$net = new Network;

		if ($net->ping($arr['otherip']))
			return array("status" => "ok");

		return array("status" => "failed");
	}

	public function checkdb($arr) {
		include_once 'mysql.php';

		$dsn = "mysql:host=".$arr['otherip'].";dbname=asterisk";
		try {
			$otherhost = new MySQL($dsn, 'freepbxuser', $arr['pass']);
			return array("status" => "ok");
		} catch(PDOException $e) {
			return array("status" => $e->getMessage());
		}
	}

	public function checkdbcontents($arr) {
		include_once 'mysql.php';
		$ha = new freepbxha;

		$dsn = "mysql:host=".$arr['otherip'].";dbname=asterisk";
		try {
			$otherhost = new MySQL($dsn, 'freepbxuser', $arr['pass']);
		} catch(PDOException $e) {
			// If this fails, it worked 500 msec ago!!!?
			return array("status" => $e->getMessage());
		}

		$q = $otherhost->query('SELECT `value` FROM `freepbxha` WHERE `key`="installstate"');
		$res = $q->fetch();
		if (!isset($res[0]) || !($res[0] == "status" || $res[0] == "complete" || $res[0] == "newmachine")) {
			return array("status" => "Cluster is not configured on other node");
		}

		$q = $otherhost->query('SELECT `value` FROM `freepbxha` WHERE `key`="asterisk_uid"');
		$res = $q->fetch();
		$pw = $ha->passwd;
		if (!isset($res[0]) || $res[0] != $pw->getAsteriskUID()) {
			return array("status" => "Asterisk UID Mismatch. Run 'Verify' on other node.");
		}

		$q = $otherhost->query("SELECT `value` FROM `freepbxha` WHERE `key` LIKE 'EXTENTS_%'");
		$validExtents = true;
		$allSameSize = true;
		$numrows = 0;
		$lastval = false;
		$requiredExtents = 0;
		while ($r = $q->fetch()) {
			if ($lastval == false) {
				$lastval = $r[0];
			}

			// Version 1 had all the extents set to the same size.
			if ($r[0] != $lastval) {
				$allSameSize = false;
			}

			$numrows++;
			if (!is_numeric($r[0]) || $r[0] == 0) {
				$validExtents = false;
			} else {
				$requiredExtents = $requiredExtents + $r[0];
			}
		}
		if (!$validExtents || $numrows < 3 || $allSameSize) {
			return array("status" => "Update module and run 'Verify Cluster Configuration' on other node.");
		}
		$q = $otherhost->query('SELECT `value` FROM `freepbxha` WHERE `key`="freepbxha-version"');
		$res = $q->fetch();
		if (!isset($res[0]) || ($res[0] != 2)) {
			return array("status" => "Version Mismatch. Cluster is version ".$res[0].", should be 2.");
		}
		return array("status" => "ok", "setdata" => array("requiredextents" => $requiredExtents));
	}

	public function checklvmspace($arr) {
		include_once 'lvm.php';
		$lvm = new LVM;
		$avail = $lvm->getAvailableLVMExtents();
		if (isset($arr['requiredextents']) && $arr['requiredextents'] <= $avail) {
			return array("status" => "ok");
		}
		return array("status" => "Insufficient disk space (Run 'Verify' on other node)");
	}

	// DoJoin stuff starts here.

	public function linkhosts($arr) {
		include_once 'mysql.php';
		include_once 'fpbxconf.php';

		$dsn = "mysql:host=".$arr['otherip'].";dbname=asterisk";
		try {
			$otherhost = new MySQL($dsn, 'freepbxuser', $arr['pass']);
		} catch(PDOException $e) {
			// If this fails, it worked 500 msec ago!!!?
			return array("status" => $e->getMessage());
		}

		$q = $otherhost->query('SELECT `value` FROM `freepbxha` WHERE `key`="mysql_ip"');
		$res = $q->fetch();

		$conf = new Conf;
		$oldpass = $conf->getConf('AMPDBPASS'); // Keep it before it changes.
		$conf->updateFpbxConf($arr['pass'], $res[0]);
		return array("status" => "ok", "setdata" => array("oldpassword" => $oldpass));
	}

	public function checkdblocal($arr) {
		include_once 'functions.inc.php';

		$ha = new freepbxha;
		if ($ha->getConfig($ha->whichMachineIsOther()."_publickey") != "") 
			return array("status" => "ok", "setdata" => array("floatingip" => $ha->getConfig('floatingip')));

		return array("status" => "Unable to get ".$ha->whichMachineIsOther()."_publickey");
	}

	public function createlvm() {
		include_once 'lvm.php';

		$lvm = new LVM;
		if ($lvm->createVolumes())
			return array("status" => "ok");

		return array("status" => "Unable to create LVM Volumes");
	}

	public function createdrbd() {
		include_once 'drbd.php';

		$drbd = new drbd;

		if ($drbd->createConfigs())
			return array("status" => "ok");

		return array("status" => "Unable to create DRBD Configuration");
	}

	public function changehostname() {
		include_once 'hosts.php';
		$hosts = new Hosts;

		return ($hosts->setMyHostname());
	}

	public function activatedrbd() {
		include_once 'drbd.php';
		$drbd = new drbd;
		if ($drbd->enableVolumes())
			return array("status" => "ok");
		return array("status" => "failed");
	}

	public function startdrbd() {
		include_once 'drbd.php';
		$drbd = new DRBD;
		$out = $drbd->startService();
		if ($out['res'] == 0)
			return array("status" => "ok");
		return array("status" => "failed");
	}

	public function maxdrbd() {
		include_once 'drbd.php';
		$drbd = new DRBD;
		$drbd->resyncMax();
		return array("status" => "ok");
	}

	public function cmanconf() {
		include_once 'cman.php';
		$cman = new cman;
		$cman->configCMAN();
		return array("status" => "ok");
	}

	public function addhosts() {
		include_once 'hosts.php';
		$h = new Hosts;

		$h->addAllHosts();
		return array("status" => "ok");
	}

	public function stopmysql() {
		include_once 'incron.php';
		$i = new incron;
		$i->manageService('mysqld', 'stop');
		$i->manageService('mysqld', 'off');
		return array("status" => "ok");
	}

	public function stopasterisk() {
		include_once 'incron.php';
		$i = new incron;
		$i->manageService('asterisk', 'stop');
		$i->manageService('asterisk', 'off');
		$i->manageService('iSymphonyServer', 'stop');
		$i->manageService('iSymphonyServer', 'off');
		return array("status" => "ok");
	}

	public function moveservices() {
		include_once 'functions.inc.php';
		include_once 'moveservices.php';
		include_once 'incron.php';
		$i = new incron;
		$ha = new freepbxha;
		$ha->newMachineAdded();
		$ha->setConfig('installstate', 'newmachine'); 
		$move = new Move(false); // Don't rsync

		$services = $ha->getServices();
		foreach ($services as $service) {
			if ($service != "httpd") {
				$move->moveService($service);
			}
		}
		$i->manageService('httpd', 'off');
		$i->manageService('pacemaker', 'on');
		$i->manageService('cman', 'on');
		$i->moveApache(false);
		return array("status" => "ok");
	}

	public function addsshkey() {
		include_once 'functions.inc.php';
		include_once 'ssh.php';

		$ha = new freepbxha;
		$ssh = new ssh;

		$otherkey = $ha->getConfig($ha->whichMachineIsOther()."_publickey");
		if ($otherkey != null) {
			$ssh->injectSSHKey($otherkey);
			return array("status" => "ok");
		} else {
			return array("status" => "Unable to retrieve ".$ha->whichMachineIsOther()."_publickey");
		}
	}

	public function gensshkey() {
		include_once 'functions.inc.php';
		include_once 'ssh.php';

		$ha = new freepbxha;
		$ssh = new ssh;

		$mykey = $ssh->getSSHkey();
		$ha->setConfig($ha->whichMachineIsThis()."_publickey", $mykey);
		return array("status" => "ok");
	}

	public function mysqlconfig() {
		include_once 'mysql.php';

		$mysql = new MySQL; // This is a PDO Object
		$mysql->updateConfig();
		return array("status" => "ok");
	}
}
