<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

class LVM {

	public $lvstatus;
	private $ha;
	private $incron;

	public function __construct() {
		/* if (!function_exists('freepbxha_licensed') || !freepbxha_licensed())
			throw new Exception("License violation in ".__FILE__); */

		include_once 'incron.php';
		$this->incron = new incron;
	}

	public function tearDown() {
		include_once 'functions.inc.php';
		// Deletes all LVM volumes
		$ha = new freepbxHA;

		// Delete all LVs
		$services = $ha->getConfig('SERVICES');
		$lvs = explode(" ", $services);

		$vgname = $this->getVGName();

		foreach ($lvs as $lv) {
			$this->deleteVolume($vgname, "drbd_$lv");
		}
	}

	public function checkForLVM() {
		// Find out if this machine is running LVM
		$ret = $this->incron->lvDisplay(); 

		if ($ret['ret'] == 99)
			return false;

		return true;
	}

	public function checkDisks() {
		// Check to see if the required volumes already exist

		// Set this to false if we're missing a volume.
		$return = true;

		$services = $this->getServices();
		$vgname = $this->getVGName();

		foreach ($services as $service) {
			if (!$this->getExistingVolumeSize($vgname, $service))
				$return = false;
		}
		return $return;
	}

	public function createVolumes() {
		// Set up LVM disks. This is complex. Sorry.

		// First, do the disks already exist?
		if ($this->checkDisks()) 
			return true;

		// They don't. Do we have space to create them?
		if (!$this->checkForLVMSpace()) 
			return false;

		$vgname = $this->getVGName();
		$services = $this->getServices();

		// Figure out how big the volumes should be.
		$size = array();
		foreach ($services as $service) {
			if (!$this->getExistingVolumeSize($vgname, $service)) {
				$size[$service] = $this->getVolumeExtents($vgname, $service);
			}
		}

		// Now create them!
		foreach ($size as $name => $s) {
			$this->createVolume($vgname, "drbd_$name", $s);
		}
		return true;
	}

	public function getVolumeExtents($vgname, $service) {

		// If the service exists, return the size of it. 
		// If it doesn't, return what it should be.
		$res = $this->getExistingVolumeExtents($vgname, $service);
		if ($res == false) {
			// Do we have it in the database?
			$db = $this->getConfig("EXTENTS_$service");
			if ($db)
				return $db;

			//Lets figure out the sizes.
			$avail = $this->getAvailableLVMExtents();

			$percentage = $avail/100;
			$size = ceil($this->getConfig("SIZE_$service")*$percentage);
			return $size;
		} else {
			return $res;
		}
	}

	public function createVolume($vgname, $name, $extents) {
		// Create an LVM Volume called $name, consisting of $extents extents.

		$arr = $this->incron->lvCreate($vgname, $name, $extents, true);
		if ($arr['ret'] != 0)
			return false;
		// else
		return true;
	}

	public function deleteVolume($vgname, $name) {
		// Destroy an LVM volume. Probably only used in testing?
		// Be careful with this.
		exec("/sbin/lvremove -f /dev/$vgname/$name 2>&1", $void, $ret); // Only care about the return code
		if ($ret == 5) 
			return false;
		// else
		return true;
	}

	public function getExistingVolumeSize($vgname, $name) {
		// Retun the amount of size for this volume group, or false if it doesn't
		// exist

		$arr = $this->incron->lvdisplay($vgname, $name);
		if ($arr['ret'] == 5)
			return false;

		return $arr['result']['size_in_g'];
	}

	public function getExistingVolumeExtents($vgname, $name) {
		// Retun the number of extents of this volume group, or false if it doesn't
		// exist

		$arr = $this->incron->lvdisplay($vgname, $name);
		if ($arr['ret'] == 5)
			return false;

		return $arr['result']['extents'];
	}

	public function getAvailableLVMExtents() {
		// Return the number of free LVM Extents
		$result = $this->incron->vgDisplay();

		if (count($result['result']) != 1) {
			throw new Exception('Only 1 active logical volume supported, '.count($result).' found.');
		}

		return $result['result'][0]['free'];
	}

	public function getVGName() {
		// return the Volume Group name
		$result = $this->incron->vgDisplay();
		return $result['result'][0]['name'];
	} 

	public function getServices() {
		// return an array of services
		return explode(" ", $this->getConfig('SERVICES'));
	}

	public function checkForLVMSpace() {
		// OK. Do we have enough LVM space to create the required volumes?

		// How many services have we defined?
		$services = $this->getServices();

		// And grab all our existing volumes
		$alllvs = $this->incron->lvdisplay();

		$extneeded = 0;

		foreach ($services as $service) {
			if (!isset($alllvs['result'][$service])) {
				$extents = $this->getConfig("EXTENTS_$service");
				if (!$extents) {
					$size = $this->getConfig("SIZE_$service");
					if ($size == false) {
						throw new Exception("Service '$service' defined in config without a SIZE param");
					}
					$extents = ceil($size * 256);
				}
				$extneeded = $extneeded + $extents;
			}
		}

		$avail = $this->getAvailableLVMExtents();

		if ($avail < $extneeded)
			return false;
		// else
		return true;
	}

	public function getConfig($var) {
		include_once 'functions.inc.php';
		// Don't create a new object if not needed
		if (!method_exists($this->ha, "getConfig"))
			$this->ha = new freepbxHA;

		return $this->ha->getConfig($var);
	}

}
