<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

class Move {

	public $rsync;

	public function __construct($move = true) {
		/* if (!function_exists('freepbxha_licensed') || !freepbxha_licensed())
			throw new Exception("License violation in ".__FILE__); */

		$this->rsync = $move;
	}

	public function moveService($service) {
		include_once 'incron.php';
		$i = new incron;
		return ($i->moveService($service, $this->rsync));
	}

	public function root_moveService($service, $rsync = true) {

		$this->rsync = $rsync;

		switch ($service) {
		case "asterisk":
			return $this->moveAsterisk();
			break;
		case "mysql":
			return $this->moveMySQL();
			break;
		case "httpd":
			return $this->moveHttpd();
			break;
		case "spare":
			return array("result" => "Unused Service", "ret" => 0);
		default:
			throw new Exception("Unable to move unknown service $service");
		}
	}

	// All of these are root-run processes. Be cautious.

	private function createLink($src, $dst) {
		// Rsync $src to $dst
		// Move $src to backup.$src.YYYYMMDD (or available)
		// Create symbolic link at $src pointing to $dst

		$log = fopen("/tmp/createLink.log", "a");
		fwrite($log, "createLink called with $src and $dst\n");
		if (is_link($src)) {
			fwrite($log, "$src already a symlink\n");
			fclose($log);
			return array("result" => "$src is already a symlink", "ret" => 0);
		}

		if ($this->rsync) {
			fwrite($log, "Rsyncing $src to $dst\n");
			fclose($log);
			// Make sure dst exists
			@mkdir($dst); // Supress warnings.
			$s = stat($src);
			chown($dst, $s['uid']);
			chgrp($dst, $s['gid']);
			chmod($dst, $s['mode']);
			exec("rsync -av $src/ $dst 2>&1 >> /tmp/createLink.log", $result, $ret);
		} else {
			fwrite($log, "Only linking, not rsyncing\n");
			fclose($log);
			$result = "Manual Linking";
			$ret = 0;
		}

		$newdir = strftime(dirname($src)."/backup.".basename($src)."%Y%m%d");

		if (file_exists($newdir)) {
			// Lets just try adding numbers until it works. 
			$start = 0;
			while (file_exists($newdir.".".$start)) {
				$start++;
			}
			$newdir = $newdir.".".$start;
		}
		rename($src, $newdir);

		symlink($dst, $src);

		// This bit me.  When you move the directories that incron
		// uses for triggers, incron stops working. WHAT A SURPRISE.
		// Sigh.
		if ($src == "/var/spool/asterisk") {
			exec("/etc/init.d/incrond restart");
			sleep(1);
			fclose(fopen("/usr/local/asterisk/ha_trigger", "w"));
		}

		return array("result" => $result, "ret" => $ret);
	}

	private function checkFilesystem($service) {
		if ($this->rsync) {
			//  Ensure that $service_fs is running, and we've got /drbd/$service mounted. 
			include_once 'cman.php';
			include_once 'hosts.php';
			$c = new cman;

			$hosts = new Hosts;
			$myhostname = $hosts->getHostname();
			$status = $c->getStatus($service."_fs"); // Throws an exception if service doesn't exist

			if ($myhostname != $status) 
				throw new Exception("$service Filesystem is not running on this node. Reported as: $status");

			return true;
		} else {
			// We're just creating links.
			return true;
		}
	}

	private function moveMySQL() {
		//  If /var/lib/mysql is already a symlink, return true.
		if (is_link("/var/lib/mysql"))
			return array("result" => "MySQL already linked", "ret" => 0);

		$this->checkFilesystem('mysql');

		//  Link it
		return $this->createLink("/var/lib/mysql", "/drbd/mysql"); 
	}

	private function moveAsterisk() {

		$this->moveLicenses();

		// If /etc/asterisk is already a symlink, return true.
		if (is_link("/etc/asterisk"))
			return array("result" => "Asterisk already linked", "ret" => 0);

		$this->checkFilesystem('asterisk');

		$astdirs = array (
			"/var/log/asterisk" => "log", 
			"/var/spool/asterisk" => "spool",
			"/var/lib/asterisk" => "lib",
			"/opt/isymphony" => "isymphony", 
			"/etc/asterisk" => "etcasterisk", 
		);
		foreach ($astdirs as $src => $dst) {
			$retarr = $this->createLink($src, "/drbd/asterisk/$dst");
		}

		// This only returns the last. FIXME
		return $retarr;
	}

	private function moveHttpd() {

		$this->createLink("/tftpboot", "/drbd/httpd/tftpboot");

		// If /var/www is already a symlink, return true.
		if (is_link("/var/www"))
			return array("result" => "Apache already linked", "ret" => 0);

		// Fix retarded symlinks
		$logs = realpath('/etc/httpd/logs');
		unlink('/etc/httpd/logs');
		symlink($logs, '/etc/httpd/logs');
		$modules = realpath('/etc/httpd/modules');
		unlink('/etc/httpd/modules');
		symlink($modules, '/etc/httpd/modules');
		$run = realpath('/etc/httpd/run');
		unlink('/etc/httpd/run');
		symlink($run, '/etc/httpd/run');

		$this->checkFilesystem('httpd');

		$this->createLink("/var/lib/php/session", "/drbd/httpd/session");
		$this->createLink("/etc/httpd", "/drbd/httpd/etc");
		return $this->createLink("/var/www", "/drbd/httpd/www");
	}

	public function moveLicenses() {
		// It's almost impossible to have a file name collision. So
		// we can just rsync files across from both machines.

		$src = '/var/lib/asterisk/licenses';
		$dst = '/usr/local/asterisk/licenses';

		@mkdir($dst, 0755, true); // Supress warnings.
		chown($dst, 'asterisk');
		chgrp($dst, 'asterisk');
		chmod($dst, 0755);

		exec("rsync -av $src/ $dst");
		@rename($src, '/var/lib/oldlicenses');
		if ($this->rsync) { // Running this on the current live system
			symlink($dst, $src);
		}
	}
}
