<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

class MySQL extends PDO {

	private $user;
	private $pselect;
	private $pinsert;
	private $table = "freepbxha";

	public function __construct($dsn = null, $username = null, $password = null, $params = null) {

		/* if (!function_exists('freepbxha_licensed') || !freepbxha_licensed())
		   throw new Exception("License violation in ".__FILE__); */

		// If we're not given any parameters, grab them from /etc/freepbx.conf, or, 
		// any other configuration file that we're given. 

		include_once 'fpbxconf.php';
		$conf = new Conf;

		if ($dsn === null) {
			// Make our DSN string
			$dsn = "mysql:host=".$conf->getConf('AMPDBHOST').";dbname=".$conf->getConf('AMPDBNAME');
		}

		if ($username === null) {
			$username = $conf->getConf('AMPDBUSER');
		}

		if ($password === null) {
			$password = $conf->getConf('AMPDBPASS');
		}

		$this->user = $username; 

		if ($params === null) {
			parent::__construct($dsn, $username, $password);
		} else {
			parent::__construct($dsn, $username, $password, $params);
		}
		$this->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

		$this->pselect = $this->prepare('SELECT `value` FROM `'.$this->table.'` where `key`=:key');
		$this->pinsert = $this->prepare('INSERT INTO `'.$this->table.'` (`key`, `value`) VALUES ( :key, :val )');
	}

	public function setConfig($key, $val = null) {
		$this->query("DELETE FROM `".$this->table."` where `key`='$key'");
		if ($val != null) {
			$this->pinsert->execute(array("key" => $key, "val" => $val));
		}
		if ($this->getConfig($key) == $val)
			return true;

		return false;
	}
	public function getConfig($var) {
		$this->pselect->execute(array("key" => $var));
		$results = $this->pselect->fetch();
		if (empty($results)) {
			$this->pselect->closeCursor();
			return null;
		}
		$ret = $results['value'];
		$this->pselect->closeCursor();
		return $ret;
	}

	public function getAll($sql) {
		$q = $this->query($sql);
		if (!$q) 
			return array();

		return $q->fetchAll();
	}

	public function addgrants() {
		// Grant permission on all the tables to connections coming from
		// ANY of the internal service IPs.
		include_once 'fpbxconf.php';

		if ($this->user != "root")
			throw new Exception("Can only add grants as MySQL Root User");

		include_once 'functions.inc.php';
		$ha = new freepbxha;

		include_once 'network.php';
		$net = new Network;

		$services = $ha->getServices();

		$fpbxconf= new Conf;
		$conf = $fpbxconf->parse_fpbx_conf();

		$ast = $this->prepare("GRANT ALL ON `".$conf['AMPDBNAME']."`.* TO ".$conf['AMPDBUSER']."@:host IDENTIFIED BY '".$conf['AMPDBPASS']."'");
		$cdr = $this->prepare("GRANT ALL ON asteriskcdrdb.* TO ".$conf['AMPDBUSER']."@:host IDENTIFIED BY '".$conf['AMPDBPASS']."'");

		foreach ($services as $service) {
			$ast->execute(array(":host" => $service));
			$cdr->execute(array(":host" => $service));
			$ast->execute(array(":host" => $net->getServiceIP($service)));
			$cdr->execute(array(":host" => $net->getServiceIP($service)));
		}

		$ast->execute(array(":host" => "freepbx-a"));
		$cdr->execute(array(":host" => "freepbx-a"));
		$ast->execute(array(":host" => "freepbx-b"));
		$cdr->execute(array(":host" => "freepbx-b"));
		$ast->execute(array(":host" => $ha->getConfig("PRIMARY_intip")));
		$cdr->execute(array(":host" => $ha->getConfig("PRIMARY_intip")));
		$ast->execute(array(":host" => $ha->getConfig("SECONDARY_intip")));
		$cdr->execute(array(":host" => $ha->getConfig("SECONDARY_intip")));
	}

	public function updateConfig() {
		// Update /etc/my.cnf
		$conf[] ="[mysql]";
		$conf[] ="port                           = 3306";
		$conf[] ="socket                         = /var/lib/mysql/mysql.sock";
		$conf[] ="[mysqld]";
		$conf[] ="user                           = mysql";
		$conf[] ="default-storage-engine         = MyISAM";
		$conf[] ="socket                         = /var/lib/mysql/mysql.sock";
		$conf[] ="pid-file                       = /var/lib/mysql/mysql.pid";
		$conf[] ="key-buffer-size                = 1344M";
		$conf[] ="myisam-recover                 = FORCE,BACKUP";
		$conf[] ="max-allowed-packet             = 16M";
		$conf[] ="max-connect-errors             = 1000000";
		$conf[] ="datadir                        = /var/lib/mysql/";
		$conf[] ="log-bin                        = /var/lib/mysql/mysql-bin";
		$conf[] ="expire-logs-days               = 14";
		$conf[] ="sync-binlog                    = 0";
		$conf[] ="tmp-table-size                 = 32M";
		$conf[] ="max-heap-table-size            = 32M";
		$conf[] ="query-cache-type               = 1";
		$conf[] ="query-cache-size               = 1048576";
		$conf[] ="max-connections                = 500";
		$conf[] ="thread-cache-size              = 50";
		$conf[] ="open-files-limit               = 65535";
		$conf[] ="table-definition-cache         = 1024";
		$conf[] ="table-open-cache               = 64";
		$conf[] ="innodb-flush-method            = O_DIRECT";
		$conf[] ="innodb-log-files-in-group      = 2";
		$conf[] ="innodb-log-file-size           = 32M";
		$conf[] ="innodb-flush-log-at-trx-commit = 1";
		$conf[] ="innodb-file-per-table          = 1";
		$conf[] ="innodb-buffer-pool-size        = 8M";
		$conf[] ="log-error                      = /var/lib/mysql/mysql-error.log";
		$conf[] ="log-queries-not-using-indexes  = 0";
		$conf[] ="slow-query-log                 = 0";
		$conf[] ="slow-query-log-file            = /var/lib/mysql/mysql-slow.log";
		$conf[] ="";

		include_once 'incron.php';
		$i = new incron;
		return ($i->updateMySQLConf($conf));
	}

}
