<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

class Network {

	public function __construct() {
		/* if (!function_exists('freepbxha_licensed') || !freepbxha_licensed())
			throw new Exception("License violation in ".__FILE__); */
	}

	public function tearDown() {
		include_once 'functions.inc.php';
		$ha = new freepbxHA;
		$services = $ha->getServices();
		foreach ($services as $service) {
			if ($ha->getConfig($service."_ip")) {
				exec('/sbin/ip addr del '.$ha->getConfig($service."_ip")."/24 dev ".$ha->getConfig('PRIMARY_int')." 2>/dev/null");
				exec('/sbin/ip addr del '.$ha->getConfig($service."_ip")."/24 dev ".$ha->getConfig('SECONDARY_int')." 2>/dev/null");
			}
		}
		if ($ha->getConfig("floatingip")) {
			exec('/sbin/ip addr del '.$ha->getConfig("floatingip")."/24 dev ".$ha->getConfig('PRIMARY_int')." 2>/dev/null");
			exec('/sbin/ip addr del '.$ha->getConfig("floatingip")."/24 dev ".$ha->getConfig('SECONDARY_int')." 2>/dev/null");
		}
	}

	public function discoverInterfaces($ignorelo = false) {
		// Return an array:
		// ("phys" => array( "logical", "ip.add.re.ss") , "phys" => array( "logical", "ip.dd.re.ss"), ...)
		exec("/sbin/ip -o addr", $result, $ret);

		if ($ret != 0)
			throw new Exception('ip -o addr failed somehow.');

		foreach ($result as $line) {
			$vals = preg_split("/\s+/", $line);

			if ($ignorelo && ($vals[1] == "lo" || $vals[1] == "lo:")) 
				continue;

			// We only care about ipv4 (inet) lines, or definition lines
			if ($vals[2] != "inet" && $vals[3] != "mtu")
				continue;

			if (preg_match("/(.+?)(?:@.+)?:$/", $vals[1], $res)) { // Matches vlans, which are eth0.100@eth0
				// It's a network definition.
				// This won't clobber an exsiting one, as it always comes
				// before the IP addresses.
				$interfaces[$res[1]] = array();
				continue;
			}
			if ($vals[4] == "scope" && $vals[5] == "host") {
				$int = 6;
			} else {
				$int = 8;
			}

			// Strip netmask off the end of the IP address
			$ret = preg_match("/(\d*+.\d*+.\d*+.\d*+)\/(\d*+)/", $vals[3], $ip);

			$interfaces[$vals[1]][] = array($ip[1], $vals[$int], $ip[2]);

		}

		if (isset($interfaces)) 
			return $interfaces;

		// Somehow, 'ip -o addr' didn't even return lo.
		return false;
	}

	public function generateMulticast() {
		// Return a random multicast address.
		// Refer http://www.cisco.com/en/US/tech/tk828/technologies_white_paper09186a00802d4643.shtml
		return "239.".mt_rand(0,127).".".mt_rand(0,254).".".mt_rand(0,254);
	}

	public function generateRFC1918() {
		// Return a random RFC1918 network
		return "172.".mt_rand(16,31).".".mt_rand(0,254).".";
	}

	public function checkValidInterface($int) {
		// Returns true or false if an interface exists.
		// Ignores vlans (.\d*+) and aliases (:\d*+)

		if (preg_match('/(.+)[:\.]\d*+/', $int, $res))
			$int = $res[1];

		$ints = $this->discoverInterfaces();
		foreach ($ints as $hw => $intarr) {
			if ($hw == $int) {
				return true;
			}
		}
		return false;
	}

	public function getValidInterface() {
		// Return array(name, ip) of an interface that's not lo.
		$ints =  $this->discoverInterfaces();

		foreach ($ints as $hw => $intarr) {
			if ($hw == "lo")
				continue; // Don't want no loopback.
			// Check all IPS in the current interface
			foreach ($intarr as $interface) {
				if ($this->ping($interface[0])) 
					return array($hw, $interface[0]);
			}
		}
		return false;
	}

	public function ping($ip, $int = null) {
		// Return true if it's answering pings, false otherwise.

		if ($int == null) {
			exec("/bin/ping -w1 -c1 -q $ip 2>&1", $result, $ret);
		} else {
			exec("/bin/ping -I$int -w1 -c1 -q $ip 2>&1", $result, $ret);
		}

		if ($ret == 0)
			return true;
		// else
		return false;
	}

	public function testMulticast($mcast) {

		// Checks to make sure that this is a valid multicast address.
		if (preg_match('/^239\.(:?(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){2}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)/', $mcast) == 0) 
			return false;
		// else
		return true;
	}

	public function testRFC1918($ip) {
		// Checks to make sure this is a valid RFC1918 address.

		// Thanks to GameGamer43 for the NO_PRIV_RANGE tip!
		return !filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE); 

		// There was a mass of small and twisty regexps here. 

	}

	public function getDefault() {
		// returns interface the default route is on
		exec("/sbin/route -n", $result, $ret);
		if ($ret != 0) 
			throw new Exception('Unable to run route');
		foreach ($result as $line) {
			$exploded = preg_split('/\s+/', $line);
			if ($exploded[0] == "0.0.0.0") {
				return $exploded[7];
			}
		}
		// No default route. I hate you.
		return false;
	}

	public function getInternalInterface() {
		// returns a different interface to getDefault, or if the machine
		// doesn't HAVE any other interfaces, returns getDefault.

		include_once 'functions.inc.php';
		$ha = new freepbxHA;
		$int = $ha->getConfig($ha->whichMachineIsThis()."_int");
		if (isset($int))
			return $int;

		$def = $this->getDefault();
		$ints = $this->discoverInterfaces(true); // don't want lo

		// First, try to find a bonded interface that isn't my
		// default interface
		foreach ($ints as $k => $v) {
			if (strpos($k, "bond") === 0 && $k !== $def)
				return $k;
		}

		// Try again, this time for every interface
		foreach ($ints as $k => $v) {
			if ($k !== $def) 
				return $k;
		}
		return $def; // Sigh.
	}

	public function getSuggestedIP() {
		include_once 'functions.inc.php';
		$ha = new freepbxHA;

		$ip = $ha->getConfig('floatingip');
		if ($ip) {
			return $ip;
		}
		// We don't know of one. Lets suggest something.
		$ints = $this->discoverInterfaces();
		$defnetwork = $ints[$this->getDefault()];
		foreach ($defnetwork as $arr) {
        		if (!empty($arr[0])) {
                		return rtrim($arr[0], "0..9");
			}
		}
		// Hmm. Couldn't find anything.
		return "192.168.22.22";
        }

	public function getNetmaskFromCIDR($c) {
		// Return a string netmask matching the CIDR
		$netmask = -1 << (32 - (int)$c);
		return long2ip($netmask);
	}

	public function getCIDRFromNetmask($m) {
		// Return a CIDR representation of the netmask
		$cidr = 32-log((ip2long("255.255.255.255") ^ ip2long($m))+1,2);
		return $cidr;
	}

	public function ipIsIn($ip, $network, $mask) {
		// return true if $ip is inside $network $netmask

		// I need a CIDR mask AND a bitwise mask
		if (strlen($mask) >= 3) {
			$netmask = ip2long($mask);
			$cidr = $this->getCIDRFromNetmask($mask);
		} else {
			$cidr = $mask;
			$netmask = ip2long($this->getNetmaskFromCIDR($mask));
		}

		$nnet = ip2long($network);
		$nip = ip2long($ip);

		// Now un-derp the network, based on the netmask. Just shift it back and foward the
		// required number of bits.
		$nnet = $nnet >> (32 - $cidr);
		$nnet = $nnet << (32 - $cidr);

		$res = $nip & $netmask;

		return ($res == $nnet);
	}

	public function checkRoute($r) {
		// Return true if this machine knows about this route

		// Is this a sensbile route?
		$ret = preg_match("/(\d*+.\d*+.\d*+.\d*+)\/(\d*+)/", $r, $range);
		if ($ret != 1) 
			throw new Exception('checkRoute failed sanity check');

		$route = $range[1];
		$net = $range[2];

		exec("/sbin/route -n", $result, $ret);

		if ($ret != 0) 
			throw new Exception('Unable to run route');

		$found = false;
		foreach ($result as $line) {
			$exploded = preg_split('/\s+/', $line);
			if ($exploded[2] == "0.0.0.0") 
				continue;
			if (!filter_var($exploded[2], FILTER_VALIDATE_IP)) {
				continue;
			}
			if ($this->ipIsIn($route, $exploded[0], $exploded[2])) {
				return true;
			}
		}

		return false;
	}

	public function getTopIP($net) {
		// Return the top IP of the current network
		if (preg_match("/(\d*+.\d*+.\d*+.\d*+)\/24/", $net, $out)) {
			$ip = preg_replace("/^(\d*+\.\d*+\.\d*+\.)(\d*+)/", '${1}254', $out[1]);
			return $ip;
		} else {
			throw new Exception("Only /24 networks supported.");
		}
	}

	public function getBottomIP($net) {
		// Return the top IP of the current network
		if (preg_match("/(\d*+.\d*+.\d*+.\d*+)\/24/", $net, $out)) {
			$ip = preg_replace("/^(\d*+\.\d*+\.\d*+\.)(\d*+)/", '${1}1', $out[1]);
			return $ip;
		} else {
			throw new Exception("Only /24 networks supported.");
		}
	}

	public function topIpCheck($net) {
		// Ajax call. Ping the top of the network provided, return
		// ajax-ified response

		if ($this->ping($this->getTopIP($net))) {
			return array('status' => 'exists');
		} else {
			return array('status' => 'noanswer');
		}
	}

	public function bottomIpCheck($net) {
		// Ajax call. Ping the bottom of the network provided, return
		// true if there's no response.
		if ($this->ping($this->getBottomIP($net))) {
			return array('status' => 'exists');
		} else {
			return array('status' => 'noanswer');
		}
	}

	public function addIP($ip, $class, $int) {

		$ha = new freepbxHA;
		if (!$ha->net->checkValidInterface($int))
			throw new Exception("Unknown interface - $int");

		// If this machine ALREADY has this IP address:
		$ints = $this->discoverInterfaces(); 
		foreach ($ints as $i => $v) {
			foreach ($v as $iparr) {
				if ($iparr[0] == $ip) {
					// Woah. This machine already has this IP. Am I being told to do something
					// that I've already done?
					if ($int == $i) {
						return array('result' => $iparr[1], 'ret' => 0);
					}
					// OK, maybe I've been given aliases. Strip off
					// any aliases, and check again.
					$baseint = preg_replace("/:.+/", "", $int);
					$foundint = preg_replace("/:.+/", "", $i);
					if ($baseint == $foundint) {
						return array('result' => $iparr[1], 'ret' => 0);
					}

					// Right. It's on a different interface. Should I delete that interface
					// and add it to the correct one? I think no. 
					throw new Exception("IP Address $ip already exists on ".$iparr[1].", can't add it");
				}
			}
		}

		include_once 'incron.php';
		$incron = new incron;

		$class = 24;
		return $incron->addIP($ip, $class, $int);
	}

	public function ifup($int) {
		include_once 'incron.php';
		$incron = new incron;

		$res = $incron->ifup($int);
		if ($res['ret'] == 0) {
			return true;
		} else {
			return false;
		}
	}

	public function ifdown($int) {
		include_once 'incron.php';
		$incron = new incron;

		$res = $incron->ifdown($int);
		if ($res['ret'] == 0) {
			return true;
		} else {
			return false;
		}
	}

	public function getServiceIP($service) {
		include_once 'functions.inc.php';
		$ha = new freepbxHA;

		$services = $ha->getServices();

		// Do we already know about it?
		$existing = $ha->getConfig($service."_ip");
		if ($existing != "") 
			return $existing;

		$internalnet = $ha->getConfig('internalnet');

		if (substr($internalnet, -4) != "0/24")
			throw new Exception('internal net is not a /24');

		$myprefix = substr($internalnet, 0, -4);

		$index = 10;
		foreach ($services as $s) {
			if ($s == $service) {
				$ha->setConfig($service."_ip", $myprefix.$index);
				return $myprefix.$index;
			}
			$index++;
		}

		// Didn't find the service.
		return false;
	}

	public function getAllInterfaceConfigs() {
		// Get the configuration for all the interfaces.
		$files = glob("/etc/sysconfig/network-scripts/ifcfg-*");
		if ($files === false) 
			throw new Exception("Unable to locate network configuration files. Permissions error?");

		foreach ($files as $int) {
			$i = preg_replace('!/etc/sysconfig/network-scripts/ifcfg-!', '', $int);
			if ($i === "lo")
				continue;
			$retarr[$i] = $this->getInterfaceConfig($i);
		}
		return $retarr;
	}

	public function getInterfaceConfig($int) {
		if (!is_readable("/etc/sysconfig/network-scripts/ifcfg-$int"))
			throw new Exception("Unable to read interface file $int");
		$conf = parse_ini_file("/etc/sysconfig/network-scripts/ifcfg-$int");
		return $conf;
	}

	public function getOtherIp($ip) {
		// Return the IP address of the other node.
		if (preg_match("/(.+)\/(\d*+)$/", $ip, $res)) {
			// We have a subnet. Repeat. WE HAVE A SUBNET.
			$ip = $res[1];
		}
		if (substr($ip, -2) == ".1") 
			return substr($ip, 0, -1)."2";

		if (substr($ip, -2) == ".2") 
			return substr($ip, 0, -1)."1";

		throw new Exception("I'm seriously confused about $ip");
	}

	public function checkLocalIP($ip) {
		if (!filter_var($ip, FILTER_VALIDATE_IP)) {
			return false;
		}

		# Note. Using backticks instead of exec here, as it will occasionally
		# throw a FPE. See https://bugzilla.redhat.com/show_bug.cgi?id=1045200
		$result = `ip route get $ip 2>&1 | head -1`;
		# Local net
		#  ip route get 10.46.81.5
		#  10.46.81.5 dev eth0  src 10.46.81.1
		# Remote net
		#  ip route get 10.46.82.5
		#  10.46.82.5 via 122.99.122.1 dev eth1  src 122.99.122.101
		$field = explode(" ", trim($result));
		return (isset($field[1]) && $field[1] == "dev");
	}
}
