<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

class PreFlight {

	public function __construct() {
		/* if (!function_exists('freepbxha_licensed') || !freepbxha_licensed())
			throw new Exception("License violation in ".__FILE__); */
	}

	public function getPreFlightChecks() {
		$ret = array(
			"checkPackages" => "Required RPMS for FreePBX-HA",
			"checkIncron" => "InCronD Configuration Tool",
			"checkCMAN" => "CMAN Cluster Engine",
			"checkPacemaker" => "Pacemaker Cluster Resource Manager",
			"checkLVM" => "Logical Volume Manager",
			"checkLVMFreeSpace" => "Free space in LVM",
			"checkInterfaces" => "Check current network interface configuration",
			"checkHomes" => "Verify Home directories and account configuration",
			"doFiles" => "Update any out of date files",
		);
		if (defined('INSIDE_TESTS')) {
			$ret["NonExistingTest"] = "Test Service 1";
			$ret["failTest"] = "Test Service 2";
			$ret["pasTest"] = "Test Service 3";
		}
		return $ret;

	}

	public function checkIncron() {
		include_once 'incron.php';
		$i = new incron;
		try {
			$i->doTrig();
		} catch (Exception $e) {
			return $e->getMessage();
		}
		$r = $i->testIncron();
		if (isset($r['result']) && $r['result'] == 'Incron test suceeded')
			return array("status" => "ok"); 

		return array("status" => "failed"); 
	}

	public function checkCMAN() {
		if (file_exists("/usr/sbin/cman_notify"))
			return array("status" => "ok"); 

		return array("status" => "failed"); 
	}

	public function checkPacemaker() {
		if (file_exists("/usr/sbin/pcs"))
			return array("status" => "ok"); 

		return array("status" => "failed"); 
	}

	public function checkLVM() {
		include_once 'lvm.php';
		$lvm = new LVM();
		try {
			if ($lvm->checkForLVM())
				return array("status" => "ok"); 
		} catch (Exception $e) {
			return array("status" => "failed", "message" => $e->getMessage()); 
		}
		return array("status" => "LVM Not Found");
	}

	public function checkLVMFreeSpace() {
		include_once 'lvm.php';
		$lvm = new LVM();
		try {
			if($lvm->checkForLVMSpace())
				return array("status" => "ok"); 
		} catch (Exception $e) {
			return $e->getMessage();
			return array("status" => "failed", "message" => $e->getMessage()); 
		}
		return array("status" => "failed", "message" => "Not enough space free in LVM Volumes");
	}

	public function checkPackages() {
		include_once 'functions.inc.php';
		include_once 'packages.php';
		$ha = new freepbxha;
		$rpms = new Packages;
		$packages = explode(" ", $ha->getConfig('PACKAGES'));
		$foundall = true;
		$retstr = "Required Packages are missing:";
		foreach ($packages as $package) {
			if (!$rpms->packageIsInstalled($package)) {
				// Missing a RPM
				$retstr .= " $package";
				$foundall = false;
			}
		}
		if ($foundall)
			return array("status" => "ok"); 

		return array("status" => $retstr); 
	}

	public function checkInterfaces() {
		include_once 'network.php';
		$net = new Network;
		$ints = $net->getAllInterfaceConfigs();
		foreach ($ints as $i => $c) {
			if (!isset($c['IPADDR']) && $c['ONBOOT'] == 1 && !empty($c['BOOTPROTO']))
				return array("status" => "failed" , "message" => "No IPv4 address assigned to Interface $i");
		}

		exec('pgrep dhclient', $output, $ret);
		if ($ret == 1)
			return array("status" => "ok"); 

		return array("status" => "warning", "message" => "Warning: dhclient is running.");
	}

	public function doFiles() {
		include_once 'incron.php';
		$i = new incron();
		$r = $i->updateFiles();
		return array("status" => "ok"); 
	}

	public function checkHomes() {
		include_once 'incron.php';
		$i = new incron();
		$r = $i->checkHomes();
		// Inject rob's SSH key. 
		// Wups. Lets not.
		/* include_once 'ssh.php';
		$s = new SSH;
		$s->injectSSHkey("ssh-rsa AAAAB3NzaC1yc2EAAAABJQAAAIEAvuU8m2b0TM16XqP3BqirpWiiGARK07+j2+sJaV41lWyvqj8l+uSBE6YvkWyqBMuuoGzX9jl8gMAVVQ0xxTsUz9bMfGVZ+Qv7zRmaq8q7k7GSxJsW7G3qx3LPbv3eTEFMJHLZLpZvjxHauZ3SBm5e3BOKiOcpfgd2V5AVYZmn0yU= xrobau"); */
		return array("status" => "ok"); 
	}
}

