<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

class R_Series {

	public function __construct($ha = null) {
		if (!is_object($ha)) 
			throw new Exception("I need a FreepbxHA Object");
		$this->ha = $ha;
	}

	public function getIDs() {
		$retarr = array(); // Always return an array.
		foreach (glob("/dev/rseries?") as $r) {
			$num = substr($r, -1);
			$retarr[$num] = true;
		}
		return $retarr;
	}
	public function getCount() {
		return count($this->getIDs());
	}

	public function getInfo($id = null) {
		if ($id === null || !is_numeric($id)) 
			throw new Exception("Invalid ID Given ($id)");

		$ret = $this->ha->incron->rctl("-t /dev/rseries$id --get-product-number --get-serial-number --get-port-count -pall --get-port-status");

		if ($ret['ret'] != 0)
			throw new Exception("Unknown error from rctl - ".$ret['ret']);

		foreach ($ret['result'] as $l) {
			if (preg_match('/Serial number: (.+)$/', $l, $out)) 
				$retarr['serialno'] = $out[1];
			if (preg_match('/Product number: (.+)$/', $l, $out))
				$retarr['product'] = $out[1];
			if (preg_match('/Port (\d): \d, (\w+), ([\w\/\n]+)$/', $l, $out)) {
				$retarr['ports'][$out[1]]['type'] = $out[3];
				$retarr['ports'][$out[1]]['mode'] = $out[2];
			}
		}
		return $retarr;
	}

	public function addRseries($id = null) {
		if ($id === null || !is_numeric($id)) 
			throw new Exception("Invalid ID Given ($id)");

		$status = $this->ha->cman->getAllStatus();

		$this->ha->cman->pcsCommand("resource create rseries$id ocf:Digium:rseries tty=/dev/rseries$id op monitor interval=10s");
		$this->ha->cman->pcsCommand("resource master ms-rseries$id rseries$id master-max='1' master-node-max='1' clone-max='2' clone-node-max='1' target-role='Master'");
		// Now, promote it to the same node that Asterisk is running on..
		$this->ha->cman->pcsCommand("resource move ms-rseries$id --master ".$status['ms']['drbd_asterisk']['Masters']);
		// Wait for it to finish moving
		$status = $this->ha->cman->getAllStatus();
		$count = 0;
		while ($status['ms']["rseries$id"]['Masters'] != $status['ms']['drbd_asterisk']['Masters']) {
			$count++;
			if ($count > 10)
				throw new Exception("Resource not moving");

			usleep(500000); // 1/2 a second
			$status = $this->ha->cman->getAllStatus();
		}
		// Remove the promotion constraint
		$this->ha->cman->pcsCommand("resource clear ms-rseries$id");
		// And finally, Bind it to Asterisk
		return $this->ha->cman->pcsCommand("constraint colocation add asterisk ms-rseries$id INFINITY with-rsc-role=Master");
	}

	public function delRseries($id = null) {
		if ($id === null || !is_numeric($id)) 
			throw new Exception("Invalid ID Given ($id)");

		return $this->ha->cman->pcsCommand("resource delete rseries$id");
	}

	public function switchRseries($id = null) {
		if ($id === null || !is_numeric($id)) 
			throw new Exception("Invalid ID Given ($id)");

		$ret = $this->ha->incron->rctl("-t /dev/rseries$id -pall --get-port-status");
		$type = explode(',', $ret['result'][0]);
		print_r($type);
		if (trim($type[1]) == "primary") {
			return $this->ha->incron->rctl("-t /dev/rseries$id -pall --control-request --set-port-status 1");
		} else {
			return $this->ha->incron->rctl("-t /dev/rseries$id -pall --control-request --set-port-status 0");
		}
	}
}

