<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

// Pretty much all of these need to be run by root.

class SSH {

	public $ak;
	public $pub;
	public $priv;

	public function __construct($ha = null) {

		include_once 'passwd.php';
		$p = new passwd;
		$asthome = $p->getHome('asterisk');

		$this->ak = $asthome."/.ssh/authorized_keys";
		$this->pub = $asthome."/.ssh/id_rsa.pub";
		$this->priv = $asthome."/.ssh/id_rsa";

		if ($ha === null) {
			include_once 'functions.inc.php';
			$this->ha = new freepbxHA;
		} else {
			$this->ha = $ha;
		}

	}

	public function genSSHkey($force = false) {
		// Generate a SSH key. It won't generate one if a public key already exists,
		// unless you force it.
		if (file_exists($this->pub) && $force == false)
			return false;

		// Make sure there's no old keys lying around. Silence warnings if the file
		// doesn't exist.
		@unlink($this->pub);
		@unlink($this->priv);

		@mkdir (dirname($this->priv));  // Just in case it doesn't exist.
		chmod(dirname($this->priv), 0700); // Permissions need to be sane

		exec("/usr/bin/ssh-keygen -b2048 -f ".$this->priv." -N '' -C'FreePBX-HA'", $result, $ret);
		if ($ret != 0)
			return false;
		// else
		return true;
		
	}

	public function getSSHkey() {
		// Return this machine's public key in the filesystem

		// If we don't have one, generate it.
		if (!file_exists($this->pub)) 
			$this->genSSHkey();

		// Grab it
		$key = trim(file_get_contents($this->pub));

		// If it's not a valid key, don't return it.
		if (!preg_match("/^ssh-rsa\s/", $key))
			return false;

		return $key;
	}

	public function injectSSHkey($key = null) {
		// If we're not given a key, use ours.
		if ($key == null)
			$key = $this->getSSHkey();

		// Strip off new lines, blank space, whatever.
		$key = trim($key);

		// If the key's already there, continue on happily.
		if ($this->checkSSHkey($key)) 
			return true;

		// It doesn't. Lets add it.
		@mkdir (dirname($this->ak));  // Just in case it doesn't exist.
		chmod(dirname($this->ak), 0700); // Permissions need to be sane
		$fp = fopen($this->ak, "a");
		$result = fwrite($fp, $key."\n");

		if ($result == false) 
			throw new Exception('odd fwrite error. Unhandled');
		// else
		return true;
	}

	public function removeSSHkey($key = null) {
		// Remove the key given from authorized_keys

		// If we're not given a key, use ours.
		if ($key == null)
			$key = $this->getSSHkey();

		if (!file_exists($this->ak)) 
			return false;

		$authorized = file($this->ak);
		$foundkey = false;

		$newfile = array();
		foreach ($authorized as $authed_key) {
			if ($key == $authed_key) {
				$foundkey = true;
			} else {
				$newfile[] = $key;
			}
		}
		if ($foundkey == false)	{
			// Didn't find it, return false.
			return false;
		}

		// Rename authorized_keys to authorized_keys.bak
		rename($this->ak, $this->ak.".bak");

		// If that was the only key in the file, we don't need a new one.
		if (empty($newfile)) {
			return true;
		} 

		// Create the new one.
		fopen($this->ak, "w");
		foreach ($newfile as $newkey) {
			$this->injectSSHkey($newkey);
		}

		return true;
	}

	public function checkSSHkey($key = null) {
		// Does our key already exist in authorized_keys?

		// If we're not given a key, use ours.
		if ($key == null)
			$key = $this->getSSHkey();

		if (file_exists($this->ak)) {
			$authorized = file($this->ak);
			$inthere = false;
			foreach ($authorized as $authed_key) {
				if ($key == trim($authed_key)) {
					$inthere = true;
				}
			}
			return $inthere;
		}
		// else, we don't HAVE an authorized_keys file
		return false;
	}

	public function runOnOtherNode($cmd) {
		// Run this command on the OTHER node, and
		// return the result, raw.
		$otherip = $this->ha->getConfig($this->ha->whichMachineIsOther()."_intip");

		exec("ssh -q -o ConnectTimeout=1 -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -o PreferredAuthentications=publickey -i ".$this->priv." asterisk@$otherip $cmd 2>&1", $result, $ret);

		return array("result" => $result, "ret" => $ret);
	}

	public function copyToOtherNode($file) {
		// Copy this file to the same location on the other node.

		// First, make sure it exists.
		if (!file_exists($file))
			throw new Exception("File $file doesn't exist");

		if (is_link($file))
			throw new Exception("$file is a symbolic link");

		// And the other machine is alive...
		$otherip = $this->ha->getConfig($this->ha->whichMachineIsOther()."_intip");
		if (!$this->ha->net->ping($otherip)) 
			return false;

		$this->runOnOtherNode("rm -f $file");

		exec("scp -q -o ConnectTimeout=1 -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -o PreferredAuthentications=publickey -i ".$this->priv." $file asterisk@$otherip:$file 2>&1", $result, $ret);

		return array("result" => $result, "ret" => $ret);
	}
}
