<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

include_once 'functions.inc.php';

class stage2 {

	public function __construct() {
		/* if (!function_exists('freepbxha_licensed') || !freepbxha_licensed())
			throw new Exception("License violation in ".__FILE__); */
	}

	public function getNewSetup() {
		// Return all the things we need to do.
		$r = array(
			"updatesettings" => "Updating base cluster settings",
			"assignprivip_primary" => "Assign the private IP to this host",
			"startip_primary" => "Activate the private IP",
			"checkip_primary" => "Check the private IP",
			"changehostname" => "Change the hostname of this machine",
			"addhosts" => "Update the /etc/hosts file with new hosts",
			"configcman" => "Configuring Cluster Manager",
			"startcman" => "Starting Cluster Manager",
			"startpacemaker" => "Starting Pacemaker",
			"waitfornode" => array("Waiting for Cluster to stablize", "This may take up to 60 seconds"),
			"configpacemaker" => "Configuring Pacemaker",
			"createservices" => "Creating Cluster Services",
			"assignclusterip" => "Assign the floating IP to the Hi-Av Cluster",
			"checkclusterip" => "Waiting for the floating IP address to become active",
			"addservicehosts" => "Create the /etc/host entries for the services",
			"updatemysql" => "Grant permissions to the cluster in MySQL",
		);

		return $r;
	}

	public function updatesettings() {
		$ha = new freepbxHA;
		$ha->setConfig("asterisk_uid", $ha->passwd->getAsteriskUID());
		return array("status" => "ok");
	}

	public function assignprivip_primary() {
		include_once 'network.php';
		$net = new Network;
		$ha = new freepbxHA;
		$network = $ha->getConfig("internalnet");
		$int = $ha->getConfig("PRIMARY_int");
		// This will always be /24.
		// Throw an expection if not.
		if (substr($network, -4) != "0/24")
			throw new Exception('internal net is not a /24');
		$myip = substr($network, 0, -4)."1";
		$ha->setConfig("PRIMARY_intip", $myip);
		if (!$ha->getConfig("SECONDARY_intip")) {
			// Generate the secondary one if it doesn't exist
			$otherip = substr($network, 0, -4)."2";
			$ha->setConfig("SECONDARY_intip", $otherip);
		}
		$res =$net->addIp($myip, 24, $int);
		if ($res['ret'] == 0) {
			// Update the primary interface, it may be an alias
			$ha->setConfig("PRIMARY_int", $res['result']);
			return array("status" => "ok");
		}
		return array("status" => "failed");
	}

	public function startip_primary() {
		include_once 'network.php';
		$net = new Network;
		$ha = new freepbxHA;

		if ($net->ifup($ha->getConfig("PRIMARY_int")))
			return array("status" => "ok");

		return array("status" => "failed");
	}

	public function addhosts() {
		include_once 'hosts.php';
		$h = new Hosts;

		return ($h->addMainHosts());
	}

	public function checkip_primary() {
		include_once 'network.php';
		$net = new Network;
		$ha = new freepbxHA;

		if ($net->ping($ha->getConfig("PRIMARY_intip")))
			return array("status" => "ok");

		return array("status" => "failed");
	}

	public function changehostname() {
		include_once 'hosts.php';
		$hosts = new Hosts;

		return ($hosts->setMyHostname()); 
	}

	public function configcman() {
		// Generate the cman configuration files
		include_once 'cman.php';
		$cman = new CMAN;
		if ($cman->configCMAN())
			return array("status" => "ok");
		return array("status" => "failed");
	}

	public function startcman() {
		include_once 'cman.php';
		$cman = new CMAN;
		if ($cman->startCMAN())
			return array("status" => "ok");
		return array("status" => "failed");
	}

	public function configpacemaker() {
		include_once 'cman.php';
		$cman = new CMAN;
		if ($cman->configPacemaker())
			return array("status" => "ok");
		return array("status" => "failed");
	}
		
	public function startpacemaker() {
		include_once 'cman.php';
		$cman = new CMAN;
		if ($cman->startPacemaker())
			return array("status" => "ok");
		return array("status" => "failed");
	}

	public function waitfornode() {
		include_once 'cman.php';
		$cman = new CMAN;
		if ($cman->isThisNodeOnline(true)) 
			return array("status" => "ok");
		return array("status" => "failed");
	}

	public function createservices() {
		include_once 'cman.php';
		$cman = new CMAN;
		if ($cman->createAllServices()) 
			return array("status" => "ok");
		return array("status" => "failed");
	}

	public function assignclusterip() {
		// We'll treat the 'clusterip' as the 'apache' service.
		include_once 'cman.php';
		$cman = new CMAN;
		$ha = new freepbxHA;

		// Need to figure out the netmask of this IP.  If we can't, guess 24

		// Start at 0
		$cidr = "0";

		$myip = $ha->getConfig('floatingip');
		$ints = $ha->net->discoverInterfaces();

		foreach ($ints as $int) {
			foreach ($int as $addr) {
				if ($ha->net->ipIsIn($myip, $addr[0], $addr[2])) {
					// We always prefer the 'more exact' network. If
					// a machine has a route for 10/8 via bond0, and
					// a route for 10.1.2/24 via bond1, we want to
					// use /24 when we're being asked to find 10.1.2.3
					if ($addr[2] > $cidr)
						$cidr = $addr[2];
				}
			}
		}

		if ($cidr == 0)
			$cidr = 24;

		if ($cman->createIPService("floating_ip", $ha->getConfig('floatingip'), $cidr))
			return array("status" => "ok");

		return array("status" => "failed");
	}

	public function checkclusterip() {
		include_once 'network.php';
		$net = new Network;
		$ha = new freepbxHA;

		$floatingip = $ha->getConfig('floatingip');

		for ($x = 0; $x < 5; $x++) {
			if ($net->ping($floatingip))
				return array("status" => "ok");
		}
		return array("status" => "failed");
	}

	public function addservicehosts() {
		include_once 'hosts.php';
		$hosts = new Hosts;

		$hosts->addAllHosts();
		return array("status" => "ok");
	}

	public function updatemysql() {
		include_once 'mysql.php';
		$mysql = new MySQL(null, 'root', ''); // Connect as Root.

		$mysql->addgrants();
		return array("status" => "ok");
	}
}

