<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

include_once 'functions.inc.php';

class stage3 {

	public function __construct() {
		/* if (!function_exists('freepbxha_licensed') || !freepbxha_licensed())
			throw new Exception("License violation in ".__FILE__); */
	}

	public function getNewSetup() {
		// Return all the things we need to do.

		$ha = new freepbxha;

		$r = array(
			"generatessh" => "Generate SSH Key",
			"createlvm" => "Creating LVM Volumes",
			"configdrbd" => "Generate DRBD Config",
			"startdrbd" => "Start the DRBD Service",
			"enablevolumes" => "Preparing DRBD Volumes",
			"activatevolumes" => "Activating DRBD Volumes",
			"createfilesystems" => "Formatting newly created volumes",
		);
		$services = $ha->getServices();
		foreach($services as $service) {
			$r["linkdrbd$service"] = "Associating $service with DRBD";
		}
		return array_merge($r, array(
			"waitfordrbd" =>"Waiting for Cluster Manager to start volumes",
			"checkcmancolo" => "Validate current CMAN Configuration",
			"apacheconf" => "Update Apache configuration",
			"mysqlconfig" => "Configure MySQL for HA",
			"movemysql" => "Migrate MySQL to Cluster Service",
			"waitmysql" => "Wait for MySQL to start as Cluster Service",
			"updateconf" => "Linking this FreePBX instance to the Cluster",
			"moveasterisk" => array("Migrate Asterisk to Cluster Server", "This will terminate any existing calls"),
			"waitasterisk" => "Wait for Asterisk to start as Cluster Service",
			"moveisymphony" => "Migrate Operator Panels to new start procedure",
			// "startcmon" => "Configure Cluster Monitoring Service",
		));
	}

	public function generatessh() {
		include_once 'ssh.php';
		$ssh = new SSH;
		$ha = new freepbxHA;

		$mykey = $ssh->getSSHKey();
		if (!$mykey) 
			return array("status" => "failed");

		$me = $ha->whichMachineIsThis();
		$ha->setConfig($me."_publickey", $mykey);

		return array("status" => "ok");
	}

	public function createlvm() {
		include_once 'lvm.php';
		$lvm = new LVM;
		if ($lvm->createVolumes())
			return array("status" => "ok");

		return array("status" => "failed");
	}

	public function formatvolumes() {
		include_once 'lvm.php';
		$lvm = new LVM;
		if ($lvm->formatVolumes())
			return array("status" => "ok");

		return array("status" => "failed");
	}

	public function configdrbd() {
		include_once 'drbd.php';
		$drbd = new DRBD;
		if ($drbd->createConfigs())
			return array("status" => "ok");

		return array("status" => "failed");
	}

	public function startdrbd() {
		include_once 'drbd.php';
		$drbd = new DRBD;
		$out = $drbd->startService();
		if ($out['res'] == 0)
			return array("status" => "ok");
		return array("status" => "failed");
	}

	public function enablevolumes() {
		include_once 'drbd.php';
		$drbd = new DRBD;
		if ($drbd->enableVolumes())
			return array("status" => "ok");
		return array("status" => "failed");
	}

	public function activatevolumes() {
		include_once 'drbd.php';
		$drbd = new DRBD;
		if ($drbd->forceActivateVolumes())
			return array("status" => "ok");
		return array("status" => "failed");
	}

	public function createfilesystems() {
		include_once 'incron.php';
		$ha = new freepbxHA;
		$in = new incron;
		$vgname = $ha->lvm->getVGName();
		$services = $ha->getServices();
		foreach($services as $service) {
			$res = $in->formatDRBDVolume($ha->getConfig($service."_id"), $service);
			$ha->setConfig("EXTENTS_$service", $ha->lvm->getExistingVolumeExtents($vgname, "drbd_$service"));
		}

		return array("status" => "ok");
	}

/*	public function linkdrbd() {
		include_once 'cman.php';
		$ha = new freepbxHA;
		$services = $ha->getServices();
		$cman = new CMAN;
		foreach($services as $service) {
			try {
				$cman->getStatus($service);
			} catch (Exception $e) {
				$res = $cman->linkDrbd($service);
			}
		}
		return array("status" => "ok");
} */

	public function waitfordrbd() {
		include_once 'cman.php';
		$ha = new freepbxHA;
		$services = $ha->getServices();
		foreach ($services as $s) 
			$check[] = $s."_fs";
		$cman = new CMAN;

		for ($delay = 0; $delay <= 10; $delay++) {
			$allready = true;
			try { 
				$status = $cman->getStatus($check);
				foreach ($status as $k => $v)  {
					if ($v === false)
						$allready = false;
				}
			} catch (Exception $e) { }

			if ($allready == true)
				return array("status" => "ok");

			sleep(1);
		}

		return array("status" => "Some volumes did not become ready");
	}

	public function apacheconf() {
		include_once 'incron.php';
		$i = new incron;

		$i->apacheConf();
		return array("status" => "ok");
	}

	public function mysqlconfig() {
		include_once 'mysql.php';

		$mysql = new MySQL; // This is a PDO Object
		$mysql->updateConfig();
		return array("status" => "ok");
	}

	public function movemysql() {
		include_once 'incron.php';
		include_once 'moveservices.php';
		include_once 'cman.php';
		$i = new incron;
		$m = new Move;
		$c = new cman;

		$i->manageService('mysqld', 'stop'); // Shut down MySQL
		$i->manageService('mysqld', 'off'); // Stop it from starting automatically.
		$m->moveService('mysql'); // Move Data
		$c->linkServiceToCluster('mysql'); // Configuer Cluster
		return array("status" => "ok");
	}

	public function waitmysql() {
		include_once 'cman.php';
		include_once 'mysql.php';

		$cman = new cman;


		$count = 0;
		while ($count < 60) {
			// WaitForService won't return until pacemaker thinks the service is up.
			$cman->waitForService('mysql_service');
			// Just double check. 
			try {
				$db = new MySQL; // PDO Object, will throw if unable to connect
				return array("status" => "ok");
			} catch(Exception $e) {
				$count++;
				sleep(1);
			}
		}

		return array("status" => "Unable to start MySQL. Contact Support.");
	}

	public function moveasterisk() {
		include_once 'incron.php';
		include_once 'moveservices.php';
		include_once 'cman.php';
		$i = new incron;
		$m = new Move;
		$c = new cman;

		$i->manageService('asterisk', 'stop'); // Shut down MySQL
		$i->manageService('asterisk', 'off'); // Stop it from starting automatically.
		$m->moveService('asterisk'); // Move Data
		$c->linkServiceToCluster('asterisk'); // Configure Cluster
		return array("status" => "ok");
	}

	public function waitasterisk() {
		include_once 'cman.php';
		$cman = new cman;

		if ($cman->waitForService('asterisk_service'))
			return array("status" => "ok");

		return array("status" => "Unable to start Asterisk. Contact Support.");
	}

	public function moveisymphony() {
		include_once 'incron.php';
		include_once 'cman.php';
		$i = new incron;
		$cman = new cman;

		$i->manageService('iSymphonyServer', 'stop'); // Shut down iSymphony
		$i->manageService('iSymphonyServer', 'off'); // Stop it from starting automatically.
		return array("status" => "ok");
	}

	public function startcmon() {
		// Unused
		include_once 'cman.php';

		$cman = new cman;
		$c->linkServiceToCluster('clustermon');
		return array("status" => "ok");
	}

	private function doLink($srv) {
		include_once 'cman.php';
		$cman = new CMAN;
		$cman->linkDrbd($srv);
		return $cman->checkDRBDConstraints($srv);
	}

	public function linkdrbdasterisk() { return $this->doLink('asterisk'); }
	public function linkdrbdhttpd () { return $this->doLink('httpd'); }
	public function linkdrbdmysql () { return $this->doLink('mysql'); }
	public function linkdrbdspare () { return $this->doLink('spare'); }

	public function checkcmancolo() {
		include_once 'cman.php';
		$ha = new freepbxha;
		$c = new CMAN;
		$s = $ha->getServices();
		foreach ($s as $srv) {
			$r = $c->checkDRBDConstraints($srv);
			if ($r['status'] != 'ok')
				return $r;
		}

		return array('status' => 'ok');
	}

	public function updateconf() {
		include_once 'functions.inc.php';
		$ha = new freepbxha;
                $ip = $ha->getConfig('mysql_ip');
                $conf = new Conf;
                $conf->updateFpbxConf($conf->getConf('AMPDBPASS'), $ip);
		unset($conf);
                $conf = new Conf;
                $host = $conf->getConf('AMPDBHOST'); // Keep it before it changes.
		if ($host == $ip) 
                	return array("status" => "ok");

		return array("status" => "AMPDBHOST set to $host, expected $ip");
	}

}
