<?php
@define('INSIDE_TESTS', true);
/**
 * @backupGlobals disabled
 */
class cmanTest extends PHPUnit_Framework_TestCase {

	protected $obj;

	public static function setUpBeforeClass() {
		include_once('functions.inc.php');
		include_once('hosts.php');
		include_once('network.php');
		include_once('stage2.php');

		$ha = new freepbxHA;
		$ha->destroyHA();
                $ha->getConfig('SERVICES'); // Just to create the tables
                $ha->setConfig("internalnet", "172.16.1.0/24");
                $ha->setConfig("PRIMARY_int", "eth1");
                $ha->setConfig("PRIMARY_intip", "172.16.1.1");

		$h = new Hosts;
		$h->setHostname('freepbx-a');

		$n = new Network;
		$n->getSuggestedIP();

		$s = new stage2;
		$s->assignprivip_primary();
		$s->startip_primary();
	}

	 public static function tearDownAfterClass() {
		include_once('functions.inc.php');
		$ha = new freepbxHA;
		$int = $ha->getConfig("PRIMARY_int");
		include_once('network.php');
		$n = new Network;
		$n->ifdown($int); // Deletes the interface
		$cman = new CMAN();
		$cman->tearDown();
		foreach (array(10, 11, 12, 13) as $ip) {
			// IF something's derped, it may be on either one.
			exec("ip addr del 172.16.1.$ip/24 dev eth0 2>&1");
			exec("ip addr del 172.16.1.$ip/24 dev eth1 2>&1");
		}
                $ha->destroyHA();
        }



	protected function setUp() {
		include_once('cman.php');
		$this->obj = new CMAN();
	}

	public function testConfig() {
		$this->assertFileNotExists($this->obj->getConfFile(), "corosync config file exists");
		$this->assertTrue($this->obj->configCMAN(), "Unable to generate config file");
		$this->assertFileExists($this->obj->getConfFile(), "corosync config file isn't there");
		// Check that corosync is generating a multicast address
		// $checkgen = $ha->getConfig('multicast');
		// $this->assertNotNull($checkgen, "genConf didn't generate a multicast address");
	}

	public function testEarlyStatus() {
		$this->assertEmpty($this->obj->getPCSStatus(), "pcs status returned null");
	}

	public function testStart() {
		$this->assertTrue($this->obj->startCluster(), "Cluster did not start");
	}

	public function testStatus() {
		$this->assertNotNull($this->obj->getPCSStatus(), "pcs status returned null");
	}

	public function testNodeOnline() {
		$this->assertFalse($this->obj->isThisNodeOnline(), "This node is online, and it shouldn't be");
		$this->assertTrue($this->obj->isThisNodeOnline(true), "This node never came online");
	}

	public function testPacemaker() {
		$this->assertTrue($this->obj->configPacemaker(),  "Unable to configure Pacemaker");
	}

	public function testPCScommand() {
		global $db;
		$c = $this->obj->pcsCommand("property set no-quorum-policy=ignore");
		$this->assertEmpty($c['ret'],  "pcsCommand not working");
		$c = $this->obj->pcsCommand("invalidcommand");
		$this->assertEquals(1, $c['result'], "pcsCommand not erroring");
	}

	public function testCreateIPs() {
		$this->assertTrue($this->obj->createAllServices(), "Unable to create services");
	}

	public function testIPsOnCorrectInt() {
		sleep(5); // Delay for cluster to start services.
		exec("/sbin/ip addr", $result, $ret);
		// Note leading spaces.
		$this->assertContains("    inet 172.16.1.10/24 brd 172.16.1.255 scope global secondary eth1", $result, "IP not assigned correctly");
	}





}
