<?php
@define('INSIDE_TESTS', true);

class functionsTest extends PHPUnit_Framework_TestCase {
	public static $testobj;

	public static function setUpBeforeClass() {
		global $amp_conf;
		global $db;
		//include_once('/etc/freepbx.conf');
		include_once('functions.inc.php');
		self::$testobj = new freepbxHA;
		self::$testobj->destroyHA();
	}

		
	public function testBegin() {
		$this->assertNotEmpty(self::$testobj);
	}

	public function testCheckIfConfigLoaded() {
		$this->assertNotEmpty(self::$testobj->conf->getConf('AMPDBENGINE'));
	} 

	public function testDatabaseCreationAndDeletion() {
		$this->assertFalse(self::$testobj->doesDBexist(), 'Database already exists.');
		$this->assertTrue(self::$testobj->createDBTable(), 'Unable to create table.');
		$this->assertTrue(self::$testobj->doesDBexist(), 'Table not created.');
		self::$testobj->destroyHA(); // We know it works, let it be created properly.
	}

	public function testCheckConfigGeneration() {
		$this->assertArrayHasKey('PACKAGES', self::$testobj->generateConfig(), 'No PACKAGE in Default Config');
		$this->assertTrue(self::$testobj->createDBTable(), 'Unable to create Database');
		$this->assertTrue(self::$testobj->setConfig('TESTVAR1', 'Tests'), 'Unable to Set Config Option');
		$this->assertEquals(self::$testobj->getConfig('TESTVAR1'), 'Tests', 'TESTVAR1 is not equal to Tests in config');
		$this->assertNull(self::$testobj->getConfig('UNSETVAR'), 'getConfigOption did not return null on an unknown key');
		$this->assertTrue(self::$testobj->setConfig('TESTVAR1'), 'Unable to delete config var TESTVAR1');
		$this->assertNull(self::$testobj->getConfig('TESTVAR1'), 'TESTVAR1 should have returned null');
	}

	public function testDBConfigSave() {
		$this->assertTrue(self::$testobj->setConfig('TESTVAR2', 'Tests'), 'Unable to Set Config Option');
		$this->assertEquals('Tests', self::$testobj->getConfig('TESTVAR2'), "Config was not saved to Database");
		self::$testobj->setConfig('TESTVAR2');
		$this->assertNull(self::$testobj->getConfig('TESTVAR2'), "Config was not deleted");
	}

	public function testEnabled() {
		$this->assertFalse(self::$testobj->isEnabled(), "FreePBX HA shouldn't be enabled");
	}

	/* Preflight tests are too hard at the moment */
/*	public function testTests() {
		$this->assertArrayHasKey('status', self::$testobj->checkPreFlight('false'), "checkPreFlight not returning failure array");
		$this->assertContains('Unknown Service', self::$testobj->checkPreFlight('false'), "checkPreFlight knows about an unknown service");
		$this->assertContains('Unimplemented', self::$testobj->checkPreFlight('test1'), "checkPreFlight tried to do something with an unimplemented test");
		$this->assertContains('Failed Test', self::$testobj->checkPreFlight('test2'), "test2 (fail) didn't fail");
		$this->assertContains('passed', self::$testobj->checkPreFlight('test3'), "test3 (pass) didn't pass");
		$this->assertContains('passed', self::$testobj->checkPreFlight('lvm'), "lvm test didn't pass");
	} */

	public function testGetHost() {
		self::$testobj->destroyHA();
		include_once('network.php');
		$n = new Network;
		$this->assertEquals("PRIMARY", self::$testobj->whichMachineIsThis(), "whichMachineIsThis guessed wrong with no information");
		self::$testobj->setConfig('PRIMARY_int', "eth99");
		self::$testobj->setConfig('PRIMARY_intip', "99.99.99.99");
		self::$testobj->setConfig('SECONDARY_int', "eth1");
		self::$testobj->setConfig('SECONDARY_intip', "1.1.1.1");
		$n->addIP("1.1.1.1", "24", "eth1");
		$n->ifup("eth1");
		self::$testobj->hosts->setHostname('unconfigured');
		$this->assertEquals("SECONDARY", self::$testobj->whichMachineIsThis(), "whichMachineIsThis didn't guesse correctly with only the IP address");
		self::$testobj->incron->rmInt("eth1");
		self::$testobj->destroyHA();
	}
}

