<?php
@define('INSIDE_TESTS', true);

class lvmTest extends PHPUnit_Framework_TestCase {
	public static $testobj;
	public static $ha;

	public static function setUpBeforeClass() {
		include_once('functions.inc.php');
		include_once('lvm.php');
		self::$ha = new freepbxHA;
		self::$testobj = new LVM;
		self::$ha->destroyHA();
	}

	public static function tearDownAfterClass() {
		self::$ha->destroyHA();
	}

	// Is LVM configured, and do we have enough disk space?
	public function testCheckLVM() {
		$this->assertTrue(self::$testobj->checkForLVM(), 'LVM is not running on this host');
		$vgname = self::$testobj->getVGname();
		$this->assertEquals($vgname, "vg_ha", 'Test Volume Group name is not "test"');
		$this->assertGreaterThan(0, self::$testobj->getAvailableLVMExtents(), 'No free LVM Space');
		// Make sure that testlv isn't there
		self::$testobj->deleteVolume($vgname, "testlv"); 
		$this->assertTrue(self::$testobj->createVolume($vgname, "testlv", 10), 'Unable to create test volume');
		$this->assertFalse(self::$testobj->createVolume($vgname, "testlv", 10), 'Able to create the same volume twice');
		$this->assertEquals("10", self::$testobj->getVolumeExtents($vgname, "testlv"), 'Test Volume incorrect size');
		$this->assertEquals("10", self::$testobj->getExistingVolumeExtents($vgname, "testlv"), 'Test Volume incorrect size');
		$this->assertTrue(self::$testobj->deleteVolume($vgname, "testlv"), 'Unable to remove volume');
		$this->assertFalse(self::$testobj->deleteVolume($vgname, "testlv"), 'Able to remove volume twice');
	}

	// Back to where we started, and we can now try to create the disks
	/** @depends testCheckLVM */
	public function testCheckLogic() {
		$vgname = self::$testobj->getVGname();
		$this->assertGreaterThan(0, self::$testobj->getAvailableLVMExtents(), 'No free LVM Space');
		$vgspace = self::$testobj->getAvailableLVMExtents();
		$this->assertTrue(self::$testobj->checkForLVMSpace(), 'Not enough free space');
		// Fill up the available space
		self::$testobj->createVolume($vgname, "testlv", $vgspace - 1);
		$this->assertFalse(self::$testobj->checkForLVMSpace(), 'Space calculations broken');
		self::$testobj->deleteVolume($vgname, "testlv");
		$this->assertTrue(self::$testobj->createVolumes(), 'Unable to automatically create volumes');
	}

	/** @depends testCheckLogic */
	public function testCheckVolumeSizes() {
		$vgname = self::$testobj->getVGname();
		$services = self::$testobj->getServices();
		foreach ($services as $service) {
			$expectedmin = self::$ha->getConfig("SIZE_$service");
			$this->assertGreaterThanOrEqual($expectedmin, floor(self::$testobj->getVolumeExtents($vgname, "drbd_$service")/256), "Volume drbd_$service too small");
		}
	}

	/** @depends testCheckVolumeSizes */
	public function testDestroyLVM() {
		$vgname = self::$testobj->getVGname();
		$services = self::$ha->getServices();
		$service = "drbd_".$services[0];
		$this->assertNotEmpty(self::$testobj->getExistingVolumeSize($vgname, $service), "Volume $service doesn't exist");
		self::$testobj->tearDown();
		$this->assertFalse(self::$testobj->getExistingVolumeSize($vgname, $service), "Volume $service still exists after teardown");
	}


}
