<?php
// vim: :set filetype=php tabstop=4 shiftwidth=4 autoindent smartindent:

class Update_HA {

	private $currentStatus;

	// IF YOU CHANGE THIS, ALSO UPDATE IT IN JOINCLUSTER.
	private $upgradepath = array(
		"0" => "upgradeTov1",
		"1" => "upgradeTov2",
	);

	public function __construct($obj) {
		$this->freepbx = $obj;
	}

	public function __get($var) {
		switch ($var) {
		case "currentVersion":
			$cv = $this->freepbx->getConfig("freepbxha-version");
			if (!$cv) {
				$this->currentVersion = 0;
			} else {
				$this->currentVersion = $cv;
			}
			return $this->currentVersion;
		}
	}

	public function isUpToDate() {
		if (isset($this->upgradepath[$this->currentVersion])) {
			// There's an upgrade path available
			return false;
		} else {
			return true;
		}
	}

	public function doUpgrade() {
		// Is there an upgrade available?
		if (!isset($this->upgradepath[$this->currentVersion]))
			throw new Exception ("Upgrade already completed. Your computer may be too fast!");

		// Apply any upgrades needed.
		while (isset($this->upgradepath[$this->currentVersion])) {
			$upgradeFunc = $this->upgradepath[$this->currentVersion];
			if (!method_exists($this, $upgradeFunc))
				throw new Exception ("Tried to call ".$this->upgradepath[$this->currentVersion]." but it doesn't exist");
			$this->currentStatus = $this->freepbx->cman->getAllStatus();
			$this->$upgradeFunc();
		}
	}

	private function checkPcsVersion() {
		$status = $this->currentStatus;

		if ($status['version'] != "1.1.10-1.el6_4.4-368c726" && $status['version'] != "1.1.10-14.el6-368c726")
			throw new Exception ("Pacemaker Cluster not at correct version. Both nodes need to be upgraded.<br />Current Version is ".$status['version'].", which is unknown");

		return true;
	}

	private function upgradeTov1() {

		$this->checkPcsVersion();

		$status = $this->currentStatus;

		if (isset($status['rg']['freepbx'])) {
			// Remove this resource group
			foreach ($status['rg']['freepbx'] as $res) {
				$this->freepbx->cman->pcsCommand("resource group remove freepbx $res");
			}
		}
		if (!isset($status['rg']['httpd'])) {
			// Create the Apache Group
			$this->freepbx->cman->pcsCommand("resource group add httpd httpd_fs httpd_ip httpd_service");
		}

		if (!isset($status['rg']['asterisk'])) {
			$this->freepbx->cman->pcsCommand("resource group add asterisk asterisk_fs asterisk_ip asterisk_service");
		}

		$this->freepbx->cman->pcsCommand("property set cluster-recheck-interval=5m");
		$this->freepbx->incron->pcsUpdate();
		$this->freepbx->incron->updateFiles();
		$this->freepbx->setConfig('freepbxha-version', 1);
		$this->currentVersion = 1;
		if ($this->freepbx->getConfig('installstate') == 'firstview') {
			$this->freepbx->setConfig('installstate', 'status');
		}
		return "Complete";
	}

	public function upgradeTov2() {
		$this->removeiSymphony();
		$this->movetftpboot();
		$this->changeMySqlBinlog();
		$this->freepbx->setConfig("asterisk_uid", $this->freepbx->passwd->getAsteriskUID());
		$this->currentVersion = 2;
		$this->freepbx->setConfig('freepbxha-version', 2);
		return "Complete";
	}

	private function removeiSymphony() {
		if (!file_exists("/var/lib/asterisk/bin/freepbx_engine_hook_highavail")) {
			$this->freepbx->incron->updateFiles();
		}

		$this->checkPcsVersion();

		$status = $this->currentStatus;

		if (isset($status['isymphony_service'])) {
			$this->freepbx->cman->pcsCommand("resource delete isymphony_service");
		}
	}

	private function movetftpboot() {
		if (!is_link("/tftpboot")) {
			$this->freepbx->moveservices->moveService("httpd");
		}
	}

	private function changeMySqlBinlog() {
		$ini = parse_ini_file("/etc/my.cnf");
		if (isset($ini['sync-binlog']) && $ini['sync-binlog'] == 1) {
			$this->freepbx->database->updateConfig();
		}
	}
}
