<?php
if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed'); }

/* 	Generates dialplan for "parkpro" components
	We call this with retrieve_conf
*/

/** parkpro_get_config
 * Short dialplan generation for this module
 * Long dialplan generation as well as population of conf_classes etc
 * that this module is responsible for.
 *
 * @author Philippe Lindheimer
 * @param string
 */

function parkpro_get_config($engine) {
	global $db;
	global $amp_conf;
	global $ext;  // is this the best way to pass this?
	global $asterisk_conf;
	global $core_conf;
	global $version;
	global $parkpro;
	global $pagepro;

	$pagepro = isset($pagepro) ? $pagepro : false;

	if(!$parkpro) {
		return false;
	}

	switch($engine) {
	case "asterisk":
    	$ast_ge_10 = version_compare($version,'10','ge');

		// Some contexts used throughout:
		//
		$por = 'park-orphan-routing';
		$ph  = 'park-hints';
		$pd  = 'park-dial';

		$sub_park_user = true;

		$ext->addInclude('from-internal-additional', $ph);

		$fcc = new featurecode('parking', 'parkedcall');
		$parkfetch_code = $fcc->getCodeActive();
		$fcc = new featurecode('parkpro', 'park');
		$parkput_code = $fcc->getCodeActive();
		unset($fcc);

		$noorphan = true;
		$lots = parkpro_get('all');
		foreach ($lots as $id => $lot) {

			// Need to setup featurecode.conf configuration for the parking lots:
			//
			$parkpos1	= $lot['parkpos'];
			$parkpos2	= $parkpos1 + $lot['numslots'] - 1;

			if ($lot['defaultlot'] == 'yes') {
				$default_lot = $id;
				$park_context = 'default';
				$hint_context = 'parkedcalls';
				$core_conf->addFeatureGeneral('parkext', $lot['parkext']);
				$core_conf->addFeatureGeneral('parkpos', $parkpos1."-".$parkpos2);
				$core_conf->addFeatureGeneral('context', $hint_context);
				$core_conf->addFeatureGeneral('parkext_exclusive', $lot['type'] == 'public' ? 'yes' : 'no');
				$core_conf->addFeatureGeneral('parkingtime', $lot['parkingtime']);
				$core_conf->addFeatureGeneral('comebacktoorigin', 'no'); //Set this to no as we can manage our own internal comebacktoorigin
				$core_conf->addFeatureGeneral('parkedplay', $lot['parkedplay']);
                $core_conf->addFeatureGeneral('courtesytone', 'beep');
				$core_conf->addFeatureGeneral('parkedcalltransfers', $lot['parkedcalltransfers']);
				$core_conf->addFeatureGeneral('parkedcallreparking', $lot['parkedcallreparking']);
				$core_conf->addFeatureGeneral('parkedmusicclass', $lot['parkedmusicclass']);
				$core_conf->addFeatureGeneral('findslot', $lot['findslot']);
			} else {
				$park_context = 'parkinglot_' . $lot['id'];
				$hint_context = $park_context;
				$core_conf->addFeatureGeneralSection($park_context, 'parkext', $lot['parkext']);
				$core_conf->addFeatureGeneralSection($park_context, 'parkpos', $parkpos1."-".$parkpos2);
				$core_conf->addFeatureGeneralSection($park_context, 'context', $park_context);
				$core_conf->addFeatureGeneralSection($park_context, 'parkext_exclusive', $lot['type'] == 'public' ? 'yes' : 'no');
				$core_conf->addFeatureGeneralSection($park_context, 'parkingtime', $lot['parkingtime']);
				$core_conf->addFeatureGeneralSection($park_context, 'comebacktoorigin', 'no'); //Set this to no as we can manage our own internal comebacktoorigin
				$core_conf->addFeatureGeneralSection($park_context, 'parkedplay', $lot['parkedplay']);
				$core_conf->addFeatureGeneralSection($park_context, 'parkedcalltransfers', $lot['parkedcalltransfers']);
				$core_conf->addFeatureGeneralSection($park_context, 'parkedcallreparking', $lot['parkedcallreparking']);
				$core_conf->addFeatureGeneralSection($park_context, 'parkedmusicclass', $lot['parkedmusicclass']);
				$core_conf->addFeatureGeneralSection($park_context, 'findslot', $lot['findslot']);
			}
			$ext->addInclude($ph, $hint_context, $lot['name']);
			// Save this for setting device defaults
			//
			$lots[$id]['park_context'] = $park_context;

			// Each lot needs a routing table to handle orphaned calls in the event
			// that the call were to timeout if they were routed to return to
			// originator, we route them to the ${PLOT} previously set

			if ($lot['comebacktoorigin'] == 'yes') {

				// If they haven't provided a destination then we need to make a context to
				// handle orphaned calls, we'll require destinations but this is a stop gap
				// to be nice to cusotmers and broken systems.
				//
				if (!$lot['dest']) {
					$ext->add($por, $lot['parkext'], '', new ext_noop('ERROR: No Alternate Destination Available for Orphaned Call'));
					$ext->add($por, $lot['parkext'], '', new ext_playback('sorry&an-error-has-occured'));
	                $ext->add($por, $lot['parkext'], '', new ext_hangup(''));
					$noorphan = false;
	            } else {
	                $ext->add($por, $lot['parkext'], '', new ext_goto($lot['dest']));
	            }
			}

			// Setup the specific items to do in the park-return-routing context for each lot, we will deal
			// with the per slot routing to this extension in the per slot loop below
			//
			parking_generate_sub_return_routing($lot, $pd, $parkpos1, $parkpos2);

			// Now we have to create the per-slot routing to the park-return-routing as well as hints
			// and the specific parking slots for picking up the calls since we do not use the dynamic
			// generated ParkedCall()
			//
			$pexten = $lot['parkext'];
			$hv_all = '';
			for ($slot = $parkpos1; $slot <= $parkpos2; $slot++) {

				// Route park-return-routing from slot to PARK_TARGET:
				$ext->add($ph, $slot, '', new ext_macro('parked-call',$slot . ',' . ($lot['type'] == 'public' ? $park_context : '${CHANNEL(parkinglot)}')));
				if ($parkfetch_code != '' && $lot['generatefc'] == 'yes') {
					$ext->add($ph, $parkfetch_code.$slot, '', new ext_macro('parked-call',$slot . ',' . $park_context));
				}

				if ($lot['generatehints'] == 'yes') {
					$hv = "park:$slot@$hint_context";
					$hv_all .= $hv.'&';
					$ext->addHint($ph, $slot, $hv);
					if ($parkfetch_code != '' && $lot['generatefc'] == 'yes') {
						$ext->addHint($ph, $parkfetch_code.$slot, $hv);
					}
				}
			}
			$hv_all = rtrim($hv_all,'&');
			if ($parkfetch_code != '' && $lot['generatefc'] == 'yes') {
				$ext->add($ph, $parkfetch_code.$pexten, '', new ext_macro('parked-call', ',' . $park_context));
				if ($lot['generatehints'] == 'yes') {
					$ext->addHint($ph, $parkfetch_code.$pexten, $hv_all);
				}
			}
			if ($parkput_code != '' && $lot['generatefc'] == 'yes') {
				$ext->add($ph, $parkput_code.$pexten, '', new ext_park(',,,,,' . $park_context));
				if ($lot['generatehints'] == 'yes') {
					$ext->addHint($ph, $parkput_code.$pexten, $hv_all);
				}
			}
		}

		if ($sub_park_user) {
			parking_generate_sub_park_user();
		}

		// We've now got the lots setup, let's assign each sip, iax and dahdi device their default lot
		$device_list = core_devices_list("all", 'full', true);
		$parklist = parkpro_device_get();

		foreach ($device_list as $device) {
			$thislot = isset($parklist[$device['id']]) ? $parklist[$device['id']] : $default_lot;
			switch ($device['tech']) {
			case 'sip':
				$core_conf->addSipAdditional($device['id'], 'parkinglot', $lots[$thislot]['park_context']);
			case 'iax2':
				$core_conf->addIaxAdditional($device['id'], 'parkinglot', $lots[$thislot]['park_context']);
			case 'dahdi':
				$core_conf->addDahdiAdditional($device['id'], 'parkinglot', $lots[$thislot]['park_context']);
			}

			if ($amp_conf['USEDEVSTATE'] && $parkfetch_code != '') {
				if ($device['tech'] == 'sip' || $device['tech'] == 'iax2') {
					$ext->add($ph, $parkfetch_code.$device['id'], '', new ext_macro('parked-call', ',' . $lots[$thislot]['park_context']));
					$ext->addHint($ph, $parkfetch_code.$device['id'], 'Custom:PARK'.$device['id']);
				}
			}
		}

		//End Park Pro
		//Begin Park and Announce

		$ext->addInclude('from-internal-additional', 'app-parkandannounce');
		$ext->addInclude('from-internal-additional', 'app-parkandannouncetimeout');

		foreach(parkpro_announce_get('all') as $list) {
			$lot = parkpro_get($list['park_id']);
			$page = paging_get_pagingconfig($list['page_id']);
			$pac = 'app-parkandannounce';
			$patc = 'app-parkandannouncetimeout';
			$pae = $list['announceext'];
			$pgrp = trim($page['page_group']);

			// Prepend options are parkingslot they were parked on, or the extension number or user name of the user who parked them
			switch ($list['autocidpp']) {
			case 'slot':
				$autopp = '${PARKEDAT}:';
				break;
			default:
				$autopp = '';
				break;
			}
			if(!empty($lot) && !empty($page)) {
				$parkinglot = ($lot['defaultlot'] == 'yes' ) ? 'default' : 'parkinglot_'.$lot['id'];
				$announce_template = array('silence/2'); //This should be user settable

				$ext->add($pac, $pae, '', new ext_noop('Dropping Call into Park and Announce '.$list['name']));
				$ext->add($pac, $pae, '', new ext_answer());
				$ext->add($pac, $pae, '', new ext_set('RETRYCOUNT','0'));

				if (($list['cidpp'] || $autopp != '') && $pagepro) {
					$cidpp = $list['cidpp'] . $autopp;
					$ext->splice('app-pagegroups', $pgrp, 'agi',
						new ext_execif('$[${PARKEDAT} >= '.$lot['parkpos'].' && ${PARKEDAT} < '.($lot['parkpos']+$lot['numslots']).']','Set','CALLERID(name)='.$cidpp.'${CALLERID(name)}'));
				}

				if ($list['primary_announcement_id']) {
					$parkannounceannmsg = recordings_get_file($list['primary_announcement_id']);
					$ext->add($pac, $pae, '', new ext_playback($parkannounceannmsg));
				}
				if($list['record_message'] == 'yes') {
					if ($list['record_announcement_id']) {
						$parkannouncerecann = recordings_get_file($list['record_announcement_id']);
						$ext->add($pac, $pae, '', new ext_playback($parkannouncerecann));
					}
					$ext->add($pac, $pae, '', new ext_setvar('ANNOUNCE_RECORDING_1','${ASTSPOOLDIR}/tmp/ANNOUNCE_RECORDING_1-'.$list['announceext'].'-${EPOCH}${RAND(100,999)}'));
					$ext->add($pac, $pae, '', new ext_record('${ANNOUNCE_RECORDING_1}.sln,'.$list['record_message_silence'].','.$list['record_message_length'].','));
					$ext->add($pac, $pae, '', new ext_gotoif('$["${RECORD_STATUS}" = "SILENCE"]','silence','parkcheck'));
					if ($list['failure_recording_id']) {
						$silenceannounce = recordings_get_file($list['failure_recording_id']);
						$ext->add($pac, $pae, 'silence', new ext_playback($silenceannounce));
					} else {
						$ext->add($pac, $pae, 'silence', new ext_playback('nothing-recorded'));
					}
					$aorder['pa_crm'] = '${ANNOUNCE_RECORDING_1}';
				}
				if ($list['page_announcement_id_1']) {
					$aorder['pa_pa1'] = recordings_get_file($list['page_announcement_id_1']);
				}
				if ($list['page_announcement_id_2']) {
					$aorder['pa_pa2'] = recordings_get_file($list['page_announcement_id_2']);
				}
				if ($list['page_announcement_id_3']) {
					$aorder['pa_pa3'] = recordings_get_file($list['page_announcement_id_3']);
				}

				if($list['slot_announce_enable'] == 'yes') {
					$aorder['pa_parked'] = 'PARKED';
				}

				foreach($list['announce_order'] as $order) {
					if(isset($aorder[$order]))
						$announce_template[] = $aorder[$order];
				}

				$ext->add($pac, $pae, 'parkcheck', new ext_gotoif('$["${RECORD_STATUS}" != "SILENCE"]','parkthanks', 'park'));
				if ($list['successful_recording_id']) {
					$recsuccessannounce = recordings_get_file($list['successful_recording_id']);
					$ext->add($pac, $pae, 'parkthanks', new ext_playback($recsuccessannounce));
				} else {
				    $ext->add($pac, $pae, 'parkthanks', new ext_noop('No Successful Message. Continuing'));
				}
				$ext->add($pac, $pae, 'park', new ext_setvar('PARKINGLOT',$parkinglot));
				$parktimeout = ($list['parkingtime_enable'] == 1) ? $list['parkingtime'] : $lot['parkingtime'];
				$ext->add($pac, $pae, 'park', new ext_parkandannounce(implode(':',$announce_template).','.$parktimeout.',Local/'.$pgrp.'@ext-paging,'.$patc.','.$pae.',1'));

				//timeout sections
				$ext->add($patc, $pae, '', new ext_gotoif('$[${RETRYCOUNT} < '.$list['parkingretry'].']','repark'));
				$dest = !empty($list['dest']) ? $list['dest'] : $lot['dest'];
				$ext->add($patc, $pae, '', new ext_goto($dest));
		        $ext->add($patc, $pae, '', new ext_playback('sorry&an-error-has-occured'));
		        $ext->add($patc, $pae, '', new ext_hangup(''));

				if ($list['replace_announcement_id']) {
					$parkannounceannaltmsg = recordings_get_file($list['replace_announcement_id']);
					$ext->add($patc, $pae, 'repark', new ext_playback($parkannounceannaltmsg));
				}
				$ext->add($patc, $pae, 'repark', new ext_set('RETRYCOUNT','$[${RETRYCOUNT} + 1]'));
				$ext->add($patc, $pae, 'repark', new ext_goto($pac.','.$pae.',park'));
			}
		}
		break;
	}
}
