<?php
if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed'); }

/* Parkpro APIs
 */

/** parkpro_get
 * Short get parkpro setting or all settings
 * Long pass in boolean false to get all parkinglots, otherwise
 * the id of the desired parking lot or string 'default' to get
 * the default lot
 *
 * @author Philippe Lindheimer
 * @param mixed $id
 * @return array
 */
function parkpro_get($id='default') {
	global $db;

	if ($id == 'all' || $id == '') {
		$sql = "SELECT * FROM parkplus order by defaultlot DESC, parkpos ASC";
		$res = sql($sql,'getAll',DB_FETCHMODE_ASSOC);
		foreach($res as $vq) {
			$results[$vq['id']] = $vq;
		}
	} else {
		if ($id == 'default') {
			$sql = "SELECT * FROM parkplus WHERE defaultlot = 'yes' LIMIT 1";
		} else {
			$id = $db->escapeSimple($id);
			$sql = "SELECT * FROM parkplus WHERE id = '$id'";
		}
		$results = sql($sql,'getRow',DB_FETCHMODE_ASSOC);
	}
	return $results;
}

/** parkpro_announce_get
 * Short get announce setting or all settings
 * Long pass in boolean false to get all announcements, otherwise
 * the id of the desired announcement
 *
 * @author Andrew Nagy
 * @param mixed $id
 * @return array
 */
function parkpro_announce_get($id,$by='ID') {
	global $db;
	$results = array();

	if ($id == 'all' || $id == '') {
		$sql = "SELECT * FROM parkplus_announce";
		$res = sql($sql,'getAll',DB_FETCHMODE_ASSOC);
		foreach($res as $vq) {
			$results[$vq['id']] = $vq;
			$results[$vq['id']]['announce_order'] = json_decode(stripslashes($vq['announce_order'])) ? json_decode(stripslashes($vq['announce_order'])) : array("pa_crm","pa_pa","pa_parked");
		}
	} else {
		$id = $db->escapeSimple($id);
		$sql = ($by == 'ID') ? "SELECT * FROM parkplus_announce WHERE id = '$id'" : "SELECT * FROM parkplus_announce WHERE announceext = '$id'";
		$results = sql($sql,'getRow',DB_FETCHMODE_ASSOC);
		if(!empty($results))
			$results['announce_order'] = json_decode(stripslashes($results['announce_order'])) ? json_decode(stripslashes($results['announce_order'])) : array("pa_crm","pa_pa","pa_parked");
	}
	return $results ? $results : array();
}


/** parkpro_del
 * Short deletes the record of the provided id
 * Long will delete the requested row and will die upon a db error
 * since normally deleting a row that does not exist will not result
 * in a db error
 *
 * @author Philippe Lindheimer
 * @param int $id
 * @return boolean
 */
function parkpro_del($id) {
	global $db, $amp_conf;

	$id = $db->escapeSimple($id);
	$sql = "DELETE FROM parkplus WHERE id = '$id' AND defaultlot != 'yes'";
	sql($sql);

	$sql = "DELETE FROM parkplus_device WHERE parkplus_id = '$id'";
	sql($sql);
	
	needreload();
	return true;
}

/* Parking Lot assignments for devices
 *
 */

/** parkpro_device_get
 * Short get parkinglot assignment for a device or all devices
 * Long returns the parkinglot assignment for a device
 * or 0 if none found, or for all devices if no device passed
 *
 * @author Philippe Lindheimer
 * @param mixed $id
 * @return mixed
 */
function parkpro_device_get($device='') {
	global $db;

	if ($device == '') {
		$sql = "SELECT * FROM parkplus_device";
		$res = sql($sql,'getAll',DB_FETCHMODE_ASSOC);
		foreach($res as $vq) {
			$results[$vq['device_id']] = $vq['parkplus_id'];
		}
		return $results;
	} else {
		$device = $db->escapeSimple($device);
		$sql = "SELECT parkplus_id FROM parkplus_device WHERE device_id = '$device'";
		$park_id = sql($sql, "getOne");
		return $park_id ? $park_id : 0;
	}
}

/** parkpro_device_save
 * Short save/replace the parkinglot id of a device
 * Long saves the parkinglot id of a device
 *
 * @author Philippe Lindheimer
 * @param int $device_id
 * @param int $parkinglot_id
 */
function parkpro_device_save($device_id, $parkinglot_id) {
	global $db;
	$sql = "REPLACE INTO parkplus_device (device_id, parkplus_id) VALUES (?,?)";
	$res = $db->query($sql,array($device_id, $parkinglot_id));
	if (DB::IsError($res)) {
		//TODO: log error or something
		die_freepbx($res->getMessage());
	}
}

/** parkpro_device_del
 * Short deletes the record of the provided device
 * Long delete the record for a device if the device is deleted
 *
 * @author Philippe Lindheimer
 * @param int $id
 * @return boolean
 */
function parkpro_device_del($id) {
	global $db, $amp_conf;

	$id = $db->escapeSimple($id);
	$sql = "DELETE FROM parkplus_device WHERE device_id = '$id'";
	sql($sql);
	return true;
}

/** parkpro_announce_del
 * Short deletes the Park Announcement
 * Long delete the park announcement
 *
 * @author Andrew Nagy
 * @param int $id
 * @return boolean
 */
function parkpro_announce_del($id) {
	global $db, $amp_conf;

	$id = $db->escapeSimple($id);
	$sql = "DELETE FROM parkplus_announce WHERE id = '$id'";
	sql($sql);

	return true;
}

/** parkpro_announce_save
 * Short Saves a new Park & Announce
 * Long Save a new park & announce, parameters passed as an array
 *
 * @author Andrew Nagy
 * @param array $params
 * @return mixed
 */
function parkpro_announce_save($parms=array()) {
	global $db, $amp_conf;

	if (!empty($parms['id'])) {
		$var['id'] = $db->escapeSimple($parms['id']);
	}

	$var['announceext'] = '';
	$var['name'] = '';
	$var['record_message'] = 'no';
	$var['record_announcement_id'] = null;
	$var['record_message_length'] = 5;
	$var['record_message_silence'] = 3;
	$var['successful_recording_id'] = null;
	$var['failure_recording_id'] = null;
	$var['park_id'] = null;
	$var['primary_announcement_id'] = null;
	$var['replace_announcement_id'] = null;
	$var['parkingtime_enable'] = false;
	$var['parkingtime'] = 45;
	$var['parkingretry'] = 1;
	$var['comebacktoorigin'] = 'yes';
	$var['dest'] = '';
	$var['page_id'] = null;
	$var['slot_announce_enable'] = "yes";
	$var['page_announcement_id_1'] = null;
	$var['page_announcement_id_2'] = null;
	$var['page_announcement_id_3'] = null;
	$var['cidpp'] = '';
	$var['autocidpp'] = 'none';
	$var['page_timer_extend'] = 0;
	$var['announce_order'] = json_encode(array("pa_pa1","pa_crm","pa_pa2","pa_pa3","pa_parked"));

	foreach ($var as $k => $v) {
		if (isset($parms[$k])) {
			$var[$k] = $db->escapeSimple($parms[$k]);
		}
	}
	
	$var['parkingtime_enable'] = isset($parms['parkingtime_enable']) ? 1 : 0;
	
	$fields = "announceext, name, record_message, record_announcement_id, record_message_length, record_message_silence, successful_recording_id, failure_recording_id, park_id, primary_announcement_id,
	replace_announcement_id, parkingtime_enable, parkingtime, parkingretry, comebacktoorigin, dest, page_id, slot_announce_enable, page_announcement_id_1, page_announcement_id_2, page_announcement_id_3, cidpp,
	autocidpp, page_timer_extend, announce_order";
	$holders = "?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?";

	if (empty($var['id'])) {
		$sql = "INSERT INTO parkplus_announce ($fields) VALUES ($holders)";
	} else {
		$sql = "REPLACE INTO parkplus_announce (id, $fields) VALUES (?,$holders)";
	}

	$res = $db->query($sql,$var);
	if (DB::IsError($res)) {
		$id = false;
		die_freepbx($res->getDebugInfo());
	} elseif (empty($var['id'])) {
		if(method_exists($db,'insert_id')) {
			$id = $db->insert_id();
		} else {
			$id = $amp_conf["AMPDBENGINE"] == "sqlite3" ? sqlite_last_insert_rowid($db->connection) : mysql_insert_id($db->connection);
		}
        needreload();
	} else {
		$id = $var['id'];
        needreload();
	}
	return $id;
}
