<?php
if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed'); }

/** parking_check_extensions
 * Short extension registry callback
 * Long callback that checks for conflicts in extensions that are passed to it
 * or if passed boolean true, list all the extensions generated by this module.
 *
 * @author Andrew Nagy
 * @param mixed
 * @return array
 */
function parkpro_check_extensions($exten=true) {
	$extenlist = array();
	if (is_array($exten) && empty($exten)) {
		return $extenlist;
	}

	$announcements = parkpro_announce_get('all');
	foreach ($announcements as $id => $announce) {
		$parkext 	= $announce['announceext'];

		if (!is_array($exten) || in_array($parkext, $exten)) {
			$extenlist[$parkext]['description'] = sprintf(_("Park & Announce: %s"), $announce['name']);
			$extenlist[$parkext]['status'] = _('INUSE');
			$extenlist[$parkext]['edit_url'] = 'config.php?display=pannounce&action=modify&id=' . $id;
		}

	}
	return $extenlist;
}

/** parking_recordings_usage
 * Short recordings usage callback by this module
 * Long callback to the recordings module that provides a list of URLs and descriptions
 * of those configurations that use the given recording ID, or an empty array if the ID
 * is not used by this module.
 *
 * @author Andrew Nagy
 * @param int
 * @return array
 */
function parkpro_recordings_usage($recording_id) {
	global $active_modules;

	$results1 = sql("SELECT `id`, `announceext`, `name` FROM `parkplus_announce` WHERE `primary_announcement_id` = '$recording_id'","getAll",DB_FETCHMODE_ASSOC);
	$results2 = sql("SELECT `id`, `announceext`, `name` FROM `parkplus_announce` WHERE `replace_announcement_id` = '$recording_id'","getAll",DB_FETCHMODE_ASSOC);
	$results2 = sql("SELECT `id`, `announceext`, `name` FROM `parkplus_announce` WHERE `page_announcement_id_1` = '$recording_id'","getAll",DB_FETCHMODE_ASSOC);
	$results3 = sql("SELECT `id`, `announceext`, `name` FROM `parkplus_announce` WHERE `page_announcement_id_2` = '$recording_id'","getAll",DB_FETCHMODE_ASSOC);
	$results4 = sql("SELECT `id`, `announceext`, `name` FROM `parkplus_announce` WHERE `page_announcement_id_3` = '$recording_id'","getAll",DB_FETCHMODE_ASSOC);

	if (empty($results1) && empty($results2) && empty($results3) && empty($results4)) {
		return array();
	} else {
		foreach ($results1 as $result) {
			$usage_arr[] = array(
				'url_query' => 'config.php?display=pannounce&action=modify&id='.urlencode($result['id']),
				'description' => sprintf(_("Park & Announce 'Announcement': %s (%s)"),$result['name'], $result['announceext']),
			);
		}
		foreach ($results2 as $result) {
			$usage_arr[] = array(
				'url_query' => 'config.php?display=pannounce&action=modify&id='.urlencode($result['id']),
				'description' => sprintf(_("Park & Announce 'Retry Announcement': %s (%s)"),$result['name'], $result['announceext']),
			);
		}
		foreach ($results3 as $result) {
			$usage_arr[] = array(
				'url_query' => 'config.php?display=pannounce&action=modify&id='.urlencode($result['id']),
				'description' => sprintf(_("Park & Announce 'Page Announcement 1': %s (%s)"),$result['name'], $result['announceext']),
			);
		}
		foreach ($results4 as $result) {
			$usage_arr[] = array(
				'url_query' => 'config.php?display=pannounce&action=modify&id='.urlencode($result['id']),
				'description' => sprintf(_("Park & Announce 'Page Announcement 2': %s (%s)"),$result['name'], $result['announceext']),
			);
		}
		foreach ($results5 as $result) {
			$usage_arr[] = array(
				'url_query' => 'config.php?display=pannounce&action=modify&id='.urlencode($result['id']),
				'description' => sprintf(_("Park & Announce 'Page Announcement 3': %s (%s)"),$result['name'], $result['announceext']),
			);
		}
		return $usage_arr;
	}
}

/** parking_check_destinations
 * Short destination registry callback
 * Long callback that lists all the configurations in this module that use any of the destinations
 * that are passed in which can be one destination or an array of destinations, or if boolean
 * true, it passes back all module instances that use any destination and which ones.
 *
 * @author Philippe Lindheimer
 * @param mixed
 * @return array
 */
function parkpro_check_destinations($dest=true) {
	global $active_modules;

	$destlist = array();
	if (is_array($dest) && empty($dest)) {
		return $destlist;
	}
	$sql = "SELECT id, announceext, dest, name FROM parkplus_announce ";
	if ($dest !== true) {
		$sql .= "WHERE dest in ('".implode("','",$dest)."')";
	}
	$results = sql($sql,"getAll",DB_FETCHMODE_ASSOC);

	foreach ($results as $result) {
		$thisdest = $result['dest'];
		$thisid   = $result['id'];
		if(!empty($thisdest)) {
			$destlist[] = array(
				'dest' => $thisdest,
				'description' => sprintf(_("Park & Announce: %s (%s)"),$result['name'],$result['announceext']),
				'edit_url' => 'config.php?display=pannounce&action=modify&id='.urlencode($thisid),
			);
		}
	}
	return $destlist;
}

// This is the hook for 'destinations'
function parkpro_destinations() {
	$results = parkpro_announce_get('all');
	// return an associative array with destination and description
	if (isset($results)) {
		foreach($results as $result){
            $desc = $result['name']
                ? $result['name']
                : _('Park &amp; Announce') . ' ' . $result['announceext'];
			$extens[] = array('destination' => 'app-parkandannounce,' . $result['announceext'] . ',1', 'description' => $desc, 'category' => 'Park and Announce', 'id' => 'parkandannounce');
		}
		return $extens;
	} else {
		return null;
	}
}

function parkpro_getdest($exten) {
	return array('app-parkandannounce,'.$exten.',1');
}

function parkpro_getdestinfo($dest) {
	if (substr(trim($dest),0,19) == 'app-parkandannounce') {
		$exten = explode(',',$dest);
		$exten = $exten[1];
		
		$thisexten = parkpro_announce_get($exten,'EXT');
		if (empty($thisexten)) {
			return array();
		} else {
			return array('description' => sprintf(_("Park &amp; Announce: %s"),$thisexten['name']),
			             'edit_url' => 'config.php?display=pannounce&id='.urlencode($thisexten['id']).'&action=modify',
						);
		}
	} else {
		return false;
	}
}

/** parking_change_destinations
 * Short callback to update a destination
 * Long callback given two destinations, will replace all instances
 * of the first with the second that are used in this module.
 *
 * @author Philippe Lindheimer
 * @param string
 * @param string
 */
function parkpro_change_destination($old_dest, $new_dest) {
	$sql = 'UPDATE parkplus_announce SET dest = "' . $new_dest . '" WHERE dest = "' . $old_dest . '"';
	sql($sql, "query");
}
