<?php
#
# Includes
#


#
# Formatting Functions
#



function column_format($data,$column) {
  switch (substr($column,0,3)) {
    case 'per':
      return round($data).'%';
    case 'day':
      return date("Y M d",$data);
    case 'dat':
      return date("Y M d H:i:s",$data);
    case 'tot':
    case 'avg':
    case 'min':
    case 'max':
    case 'sec':
    case 'dur':
      return format_time_duration($data);
    case 'age':
      $data = ($data == ''?'&nbsp;':$data);
      return clean_agent($data);
    case 'dis':
      return clean_reason($data);
    case 'aba':
      return ($data > 0?$data:'&nbsp;');
    case 'cid':
      $data = ($data == 'estricted'?'Restricted':$data);
      return ($data?$data:'&nbsp;');
    case 'cal':
      return ($data?$data:'0');
    default:
      return ($data?$data:'&nbsp;');
  }
}

function column_csv_format($data,$column) {
  switch (substr($column,0,3)) {
    case 'per':
      return round($data);
    case 'day':
      return date("Y-m-d",$data);
    case 'dat':
      return date("Y-m-d H:i:s",$data);
    case 'tot':
    case 'avg':
    case 'min':
    case 'max':
    case 'sec':
    case 'dur':
      return round($data);
    case 'age':
      return clean_agent($data);
    case 'dis':
      return clean_reason($data);
    case 'aba':
      return $data;
    case 'cid':
      $data = ($data == 'estricted'?'Restricted':$data);
      return ($data?$data:'');
    default:
      return ($data?$data:'');
  }
}

function column_sum_format($data,$column) {
	switch (substr($column,0,3)) {
		case 'day':
		case 'dat':
		case 'age':
                case 'aba':
		case 'dis':
                case 'que':
                case 'ori':
                case 'cid':
                case 'uni':
                case 'did':
                    return '&nbsp;&nbsp;~&nbsp;&nbsp;';
		case 'avg':
                    return column_format((array_sum($data)/count($data)),$column);
		case 'min':
                    return column_format(min($data),$column);
		case 'max':
		    return column_format(max($data),$column);
		case 'per':
                    $data = (ceil(array_sum($data)+2)>100?100:array_sum($data));
                    return column_format($data,$column);
		default:
                    return column_format(array_sum($data),$column);
  }
}

function format_sql_array($arr) { 
  $output = '';
  foreach($arr as $a) {
    $a = addslashes($a);
    $output .= "'$a', ";
  }
  return ' ('.substr($output, 0, -2).') ';
}

function format_time_duration($time=0) {
  $output = '';
  if ($time > 31556926) {
    $years = (integer) ($time / 31556926);
    $output .= " ${years}yr";
    $time = $time % 31556926;
  }
  if ($time > 7889232) {
    $months = (integer) ($time / 2629744);
    $output .= " ${months}mo";
    $time = $time % 2629744;
  }
  if ($time > 172800) {
    $days = (integer) ($time / 86400);
    $output .= " ${days}d";
    $time = $time % 86400;
  }
  if ($time > 3599) {
    $hours = (integer) ($time / 3600);
    $output .= " ${hours}h";
    $time = $time % 3600;
  }
  if ($time > 59) {
    $minutes = (integer) ($time / 60);
    $output .= " ${minutes}m";
    $time = $time % 60;
  }
  if ($time > 0) {
    $seconds = (integer) $time;
    $output .= " ${seconds}s";
  }
  if ($output == '') {
    $output = '0s';
  }
  return trim($output);
}

function clean_agent($agent='') {
  return preg_replace('/^[a-zA-Z0-9]{3,32}\/(\d{3,32}).*/',"$1",$agent);
  $agent;
}

function clean_reason($reason) {
  $reason = explode('/',$reason); 
  $reason = ($reason[0]=='ENTERQUEUE'?$reason[1]:$reason[0]);
  switch ($reason) {
    case 'COMPLETEAGENT':
      return 'Agent';
    case 'COMPLETECALLER':
      return 'Caller';
    case 'ABANDON':
      return 'Abandon';
    case 'EXITWITHTIMEOUT':
      return 'Timeout';
    case 'EXITWITHKEY':
      return 'Key Pressed';
    case 'EXITEMPTY':
      return 'Queue Empty';
	case 'TRANSFER':
	  return 'Agent';
    default:
      return $reason;
  }
}

"'LOGIN' WHERE event = 'ADDMEMBER'";

function clean_event($event) {
  switch ($event) {
    case 'ADDMEMBER':
      return 'LOGIN';
    case 'REMOVEMEMBER':
      return 'LOGOUT';
    case 'AGENTLOGIN':
      return 'LOGIN';
    case 'AGENTLOGOUT':
      return 'LOGOUT';
    case 'AGENTCALLBACKLOGIN':
      return 'LOGIN';
    case 'AGENTCALLBACKLOGOUT':
      return 'LOGOUT';
    default:
      return $event;
  }
}

function first_column_label($table_type) {
  if (strpos($table_type,'by_a')) {
    return 'Agent';
  } elseif (strpos($table_type,'by_q')) {
    return 'Queue';
  } elseif (strpos($table_type,'by_dow')) {
    return 'Day of Week';
  } elseif (strpos($table_type,'by_d')) {
    return 'Day';
  } elseif (strpos($table_type,'by_h')) {
    return 'Hour';  
  } elseif (strpos($table_type,'ervice_level')) {
    return 'Within';
  } elseif (strpos($table_type,'_cause')) {
    return 'Reason';
  } else {
    return '';
  }
}

#
# DB Functions
# 

function db_prep_int($int='') {
  if (!is_numeric($int)) {
    return 'DEFAULT';
  } else {
    return $int;
  }
}

function db_prep_char($str='') {
  if ($str == '') {
    return 'DEFAULT';
  } else {
    return "'".addslashes($str)."'";
  }
}



#
# Global Arrays
# 



$months = Array(
  "Jan" => 1, 
  "Feb" => 2, 
  "Mar" => 3, 
  "Apr" => 4, 
  "May" => 5, 
  "Jun" => 6, 
  "Jul" => 7, 
  "Aug" => 8, 
  "Sep" => 9, 
  "Oct" => 10, 
  "Nov" => 11, 
  "Dec" => 12
);



?>
