<?php


//Array of all report types
// This is the master array of all available reports and the name here
// must correspond to an array below specifying the columns.

$report_types = Array(
    'dist_by_q' => 'All Call Distribution by Queue',
    'ans_by_q' => 'Ans. Call Distribution by Queue',
    'unans_by_q' => 'Unans. Call Distribution by Queue',
    'dist_by_a' => 'Ans. Call Distribution by Agent',
    'dist_by_d' => 'All Call Distribution by Day',
    'dist_by_h' => 'All Call Distribution by Hour',
    'dist_by_dow' => 'All Call Distribution by Day of Week',
    'service_level' => 'All Service Level',
    'service_level_answered' => 'Answered Service Level',
    'service_level_unanswered' => 'Unanswered Service Level',
    'disc_cause' => 'Call Disconnection Cause',
    'call_detail' => 'Call Detail List',
    'ans_call_detail' => 'Answered Call Detail',
    'unans_call_detail' => 'Unanswered Call Detail',
    'agent_log' => 'Agent Availability Detail',
    'agent_missed_calls' => 'Agent Missed Calls Detail',
    'custom_acd_by_q' => 'Custom - ACD by Queue',
    'custom_acd_by_d' => 'Custom - ACD by Day',
);

// the following arrays named with _columns take the following format:
// Array( 0 => Long Name, 1 => Header Name, 2 => Default Value, 3 => Read Only, 4 => SQL query)

//All Call Distribution by Queue
$dist_by_q_columns = Array(
    'queue' => Array('Queue Name','Queue',1,1,"
        SELECT queuename AS queue
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'calls' => Array('Number of Calls','# Calls',1,0,"
        SELECT COUNT(*) as calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'percent_calls' => Array('Percent of Calls',"% Calls",1,0,"
        SELECT (COUNT(*)/(
          SELECT COUNT(*)
          FROM qxact_calls
          WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'tot_wait' => Array('Total Time Waiting','Tot. Wait',1,0,"
        SELECT SUM(waittime) as tot_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'avg_wait' => Array('Average Wait Time','Avg. Wait',1,0,"
        SELECT AVG(waittime) as avg_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'min_wait' => Array('Minimum Wait Time','Min. Wait',1,0,"
        SELECT MIN(waittime) as min_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'max_wait' => Array('Maximum Wait Time','Max. Wait',1,0,"
        SELECT MAX(waittime) as max_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'percent_wait' => Array('Percent of Time Waiting',"% Wait",1,0,"
        SELECT (SUM(waittime)/(
          SELECT SUM(waittime)
          FROM qxact_calls
          WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),
);

//ALL Call Distribution by Queue - COMBINED
$dist_by_q_combined_columns = $dist_by_q_columns;

//Answered Call Distribution by Queue
$ans_by_q_columns = Array(
    'queue' => Array('Queue Name','Queue',1,1,"
        SELECT queuename AS queue
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'calls' => Array('Number of Calls','# Calls',1,0,"
        SELECT COUNT(*) as calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'percent_calls' => Array('Percent of Calls',"% Calls",1,0,"
        SELECT (COUNT(*)/(
          SELECT COUNT(*)
          FROM qxact_calls
          WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'tot_wait' => Array('Total Time Waiting','Tot. Wait',1,0,"
        SELECT SUM(waittime) as tot_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'avg_wait' => Array('Average Wait Time','Avg. Wait',1,0,"
        SELECT AVG(waittime) as avg_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'min_wait' => Array('Minimum Wait Time','Min. Wait',1,0,"
        SELECT MIN(waittime) as min_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND waittime > 0
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'max_wait' => Array('Maximum Wait Time','Max. Wait',1,0,"
        SELECT MAX(waittime) as max_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'percent_wait' => Array('Percent of Time Waiting',"% Wait",1,0,"
        SELECT (SUM(waittime)/(
          SELECT SUM(waittime)
          FROM qxact_calls
          WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'tot_talk' => Array('Total Time Talking','Tot. Talk',1,0,"
        SELECT SUM(calltime) as tot_talk
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'avg_talk' => Array('Average Talk Time','Avg. Talk',1,0,"
        SELECT AVG(calltime) as avg_talk
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'min_talk' => Array('Minimum Talk Time','Min. Talk',1,0,"
        SELECT MIN(calltime) as min_talk
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND calltime > 0
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'max_talk' => Array('Maximum Talk Time','Max. Talk',1,0,"
        SELECT MAX(calltime) as max_talk
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'percent_talk' => Array('Percent of Time Talking',"% Talk",1,0,"
        SELECT (SUM(calltime)/(
          SELECT SUM(calltime)
          FROM qxact_calls
          WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_talk
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'tot_call' => Array('Total Call Duration','Tot. Call',0,0,"
        SELECT (SUM(calltime)+SUM(waittime)) as tot_call
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'avg_call' => Array('Average Call Duration','Avg. Call',0,0,"
        SELECT AVG(calltime+waittime) as avg_call
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'min_call' => Array('Minimum Call Duration','Min. Call',0,0,"
        SELECT MIN(calltime+waittime) as min_call
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND calltime > 0
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'max_call' => Array('Maximum Call Duration','Max. Call',0,0,"
        SELECT MAX(calltime+waittime) as max_call
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'percent_call' => Array('Percent of Call Duration',"% Call",0,0,"
        SELECT ((SUM(calltime)+SUM(waittime))/(
          SELECT (SUM(calltime)+SUM(waittime))
          FROM qxact_calls
          WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_call
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND agent IN ##AGENTS##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),
);


//Answered Call Distribution by Queue - COMBINED
$ans_by_q_combined_columns = $ans_by_q_columns;

//UNANS Call Distribution by Queue
$unans_by_q_columns = Array(
    'queue' => Array('Queue Name','Queue',1,1,"
        SELECT queuename AS queue
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'calls' => Array('Number of Calls','# Calls',1,0,"
        SELECT COUNT(*) as calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'percent_calls' => Array('Percent of Calls',"% Calls",1,0,"
        SELECT (COUNT(*)/(
          SELECT COUNT(*)
          FROM qxact_calls
          WHERE
          queuename IN ##QUEUES##
          AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'tot_wait' => Array('Total Time Waiting','Tot. Wait',1,0,"
        SELECT SUM(waittime) as tot_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'avg_wait' => Array('Average Wait Time','Avg. Wait',1,0,"
        SELECT AVG(waittime) as avg_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'min_wait' => Array('Minimum Wait Time','Min. Wait',1,0,"
        SELECT MIN(waittime) as min_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'max_wait' => Array('Maximum Wait Time','Max. Wait',1,0,"
        SELECT MAX(waittime) as max_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'percent_wait' => Array('Percent of Time Waiting',"% Wait",1,0,"
        SELECT (SUM(waittime)/(
          SELECT SUM(waittime)
          FROM qxact_calls
          WHERE
          queuename IN ##QUEUES##
          AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),
);

//UNANS Call Distribution by Queue - COMBINED
$unans_by_q_combined_columns = $unans_by_q_columns;

// Ans Dist by Agent
$dist_by_a_columns = Array(
    'agent' => Array('Agent Name','Agent',1,1,"
        SELECT agent AS agent
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'calls' => Array('Number of Calls','# Calls',1,0,"
        SELECT COUNT(*) as calls
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'percent_calls' => Array('Percent of Calls',"% Calls",1,0,"
        SELECT (COUNT(*)/(
          SELECT COUNT(*)
          FROM qxact_calls
          WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_calls
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'tot_wait' => Array('Total Time Waiting','Tot. Wait',1,0,"
        SELECT SUM(waittime) as tot_wait
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'avg_wait' => Array('Average Wait Time','Avg. Wait',1,0,"
        SELECT AVG(waittime) as avg_wait
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'min_wait' => Array('Minimum Wait Time','Min. Wait',1,0,"
        SELECT MIN(waittime) as min_wait
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND waittime > 0
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'max_wait' => Array('Maximum Wait Time','Max. Wait',1,0,"
        SELECT MAX(waittime) as max_wait
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'percent_wait' => Array('Percent of Time Waiting',"% Wait",1,0,"
        SELECT (SUM(waittime)/(
          SELECT SUM(waittime)
          FROM qxact_calls
          WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_wait
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'tot_talk' => Array('Total Time Talking','Tot. Talk',1,0,"
        SELECT SUM(calltime) as tot_talk
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'avg_talk' => Array('Average Talk Time','Avg. Talk',1,0,"
        SELECT AVG(calltime) as avg_talk
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'min_talk' => Array('Minimum Talk Time','Min. Talk',1,0,"
        SELECT MIN(calltime) as min_talk
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND calltime > 0
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'max_talk' => Array('Maximum Talk Time','Max. Talk',1,0,"
        SELECT MAX(calltime) as max_talk
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'percent_talk' => Array('Percent of Time Talking',"% Talk",1,0,"
        SELECT (SUM(calltime)/(
          SELECT SUM(calltime)
          FROM qxact_calls
          WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_talk
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'tot_call' => Array('Total Call Duration','Tot. Call',0,0,"
        SELECT (SUM(calltime)+SUM(waittime)) as tot_call
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'avg_call' => Array('Average Call Duration','Avg. Talk',0,0,"
        SELECT AVG(calltime+waittime) as avg_call
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'min_call' => Array('Minimum Call Duration','Min. Call',0,0,"
        SELECT MIN(calltime+waittime) as min_call
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND calltime > 0
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'max_call' => Array('Maximum Call Duration','Max. Call',0,0,"
        SELECT MAX(calltime+waittime) as max_call
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),

    'percent_call' => Array('Percent of Call Duration',"% Call",0,0,"
        SELECT ((SUM(calltime)+SUM(waittime))/(
          SELECT (SUM(calltime)+SUM(waittime))
          FROM qxact_calls
          WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_call
        FROM qxact_calls
        WHERE
          agent IN ##AGENTS##
          AND queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY agent"),
);


// Ans Dist by Agent - COMBINED
$dist_by_a_combined_columns = $dist_by_a_columns;

//All Call Distribution by Day
$dist_by_d_columns = Array(
    'day' => Array('Day','Day',1,1,"
        SELECT time as day
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'calls' => Array('Number of Calls','# Calls',1,0,"
        SELECT COUNT(*) as calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))"),

    'percent_calls' => Array('Percent of Calls',"% Calls",1,0,"
        SELECT (COUNT(*)/(
          SELECT COUNT(*)
          FROM qxact_calls
          WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

// these are not behaving as expected and will require more time to figure out ... disabling for now

//    'answered' => Array('Number of Answered Calls','# Ans.',1,0,"
//        SELECT COUNT(*) as answered
//        FROM qxact_calls
//        WHERE
//            queuename IN ##QUEUES##
//            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
//            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
//        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
//    "),
//
//    'percent_answered' => Array('Percent of Answered Calls','% Ans.',1,0,"
//        SELECT (COUNT(*)/(
//            SELECT COUNT(*)
//            FROM qxact_calls
//            WHERE
//                queuename IN ##QUEUES##
//                AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
//                AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
//            GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
//        )*100) as percent_answered
//        FROM qxact_calls
//        WHERE
//            queuename IN ##QUEUES##
//            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
//            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
//        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
//    "),
//
//    'unanswered' => Array('Number of Unanswered Calls','# Unans.',1,0,"
//        SELECT COUNT(*) as answered
//        FROM qxact_calls
//        WHERE
//            queuename IN ##QUEUES##
//            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
//            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
//        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
//    "),
//
//    'percent_unanswered' => Array('Percent of Unanswered Calls','% Unans.',1,0,"
//        SELECT (COUNT(*)/(
//            SELECT COUNT(*)
//            FROM qxact_calls
//            WHERE
//                queuename IN ##QUEUES##
//                AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
//                AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
//            GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
//        )*100) as percent_answered
//        FROM qxact_calls
//        WHERE
//            queuename IN ##QUEUES##
//            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
//            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
//        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
//    "),




    'tot_wait' => Array('Total Wait Duration','Tot. Wait',1,0,"
        SELECT SUM(waittime) as tot_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'avg_wait' => Array('Average Wait Duration','Avg. Wait',1,0,"
        SELECT AVG(waittime) as avg_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'min_wait' => Array('Minimum Wait Duration','Min. Wait',1,0,"
        SELECT MIN(waittime) as min_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND waittime > 0
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'max_wait' => Array('Maximum Wait Duration','Max. Wait',1,0,"
        SELECT MAX(waittime) as max_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'percent_wait' => Array('Percent of Wait Duration','% Wait',1,0,"
        SELECT (SUM(waittime)/(
            SELECT SUM(waittime)
            FROM qxact_calls
            WHERE
                queuename IN ##QUEUES##
                AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'tot_talk' => Array('Total Talk Duration','Tot. Talk',1,0,"
        SELECT SUM(calltime) as tot_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'avg_talk' => Array('Average Talk Duration','Avg. Talk',1,0,"
        SELECT AVG(calltime) as avg_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'min_talk' => Array('Minimum Talk Duration','Min. Talk',1,0,"
        SELECT MIN(calltime) as min_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND calltime > 0
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'max_talk' => Array('Maximum Talk Duration','Max. Talk',1,0,"
        SELECT MAX(calltime) as max_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'percent_talk' => Array('Percent of Talk Duration','% Talk',1,0,"
        SELECT (SUM(calltime)/(
            SELECT SUM(calltime)
            FROM qxact_calls
            WHERE
                queuename IN ##QUEUES##
                AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'tot_call' => Array('Total Call Duration','Tot. Call',0,0,"
        SELECT (SUM(waittime)+SUM(calltime)) as tot_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'avg_call' => Array('Average Call Duration','Avg. Call',0,0,"
        SELECT AVG(calltime+waittime) as avg_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'min_call' => Array('Minimum Call Duration','Min. Call',0,0,"
        SELECT (MIN(calltime)+MIN(waittime)) as min_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND calltime > 0
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'max_call' => Array('Maximum Call Duration','Max. Call',0,0,"
        SELECT MAX(calltime+waittime) as max_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'percent_call' => Array('Percent of Call Duration','% Call',0,0,"
        SELECT ((SUM(waittime)+SUM(calltime))/(
            SELECT (SUM(waittime)+SUM(calltime))
            FROM qxact_calls
            WHERE
                queuename IN ##QUEUES##
                AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),
);

//All Call Distribution by Day - COMBINED
$dist_by_d_combined_columns = $dist_by_d_columns;

//All Call Distribution by Hour
$dist_by_h_columns = Array(
    'hour' => Array('Hour','Hour',1,1,"
        SELECT RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5) AS hour
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'calls' => Array('Number of Calls','# Calls',1,0,"
        SELECT COUNT(*) as calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'percent_calls' => Array('Percent of Calls',"% Calls",1,0,"
        SELECT (COUNT(*)/(
          SELECT COUNT(*)
          FROM qxact_calls
          WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

// TODO when answered, percent answered, unanswered and percent unanswered are fixed add them here

    'tot_wait' => Array('Total Wait Duration','Tot. Wait',1,0,"
        SELECT SUM(waittime) as tot_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'avg_wait' => Array('Average Wait Duration','Avg. Wait',1,0,"
        SELECT AVG(waittime) as avg_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'min_wait' => Array('Minimum Wait Duration','Min. Wait',1,0,"
        SELECT MIN(waittime) as min_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'max_wait' => Array('Maximum Wait Duration','Max. Wait',1,0,"
        SELECT MAX(waittime) as max_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'percent_wait' => Array('Percent of Wait Duration','% Wait',1,0,"
        SELECT (SUM(waittime)/(
            SELECT SUM(waittime)
            FROM qxact_calls
            WHERE
                queuename IN ##QUEUES##
                AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'tot_talk' => Array('Total Talk Duration','Tot. Talk',1,0,"
        SELECT SUM(calltime) as tot_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'avg_talk' => Array('Average Talk Duration','Avg. Talk',1,0,"
        SELECT AVG(calltime) as avg_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'min_talk' => Array('Minimum Talk Duration','Min. Talk',1,0,"
        SELECT MIN(calltime) as min_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'max_talk' => Array('Maximum Talk Duration','Max. Talk',1,0,"
        SELECT MAX(calltime) as max_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'percent_talk' => Array('Percent of Talk Duration','% Talk',1,0,"
        SELECT (SUM(calltime)/(
            SELECT SUM(calltime)
            FROM qxact_calls
            WHERE
                queuename IN ##QUEUES##
                AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'tot_call' => Array('Total Call Duration','Tot. Call',0,0,"
        SELECT (SUM(waittime)+SUM(calltime)) as tot_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'avg_call' => Array('Average Call Duration','Avg. Call',0,0,"
        SELECT AVG(calltime+waittime) as avg_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'min_call' => Array('Minimum Call Duration','Min. Call',0,0,"
        SELECT (MIN(calltime)+MIN(waittime)) as min_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'max_call' => Array('Maximum Call Duration','Max. Call',0,0,"
        SELECT MAX(calltime+waittime) as max_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),

    'percent_call' => Array('Percent of Call Duration','% Call',0,0,"
        SELECT ((SUM(waittime)+SUM(calltime))/(
            SELECT (SUM(waittime)+SUM(calltime))
            FROM qxact_calls
            WHERE
                queuename IN ##QUEUES##
                AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY RIGHT(CONCAT('00',HOUR(FROM_UNIXTIME(`time`)),':00'),5)
    "),
);

//All Call Distribution by Hour - COMBINED
$dist_by_h_combined_columns = $dist_by_h_columns;

//All Call Distribution by Day of Week
$dist_by_dow_columns = Array(

    'dow' => Array('Day of Week','Day',1,1,"
        SELECT DAYNAME(FROM_UNIXTIME(`time`)) AS dow
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

// TODO when answered, percent answered, unanswered and percent unanswered are fixed add them here


    'calls' => Array('Number of Calls','# Calls',1,0,"
        SELECT COUNT(*) as calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'percent_calls' => Array('Percent of Calls',"% Calls",1,0,"
        SELECT (COUNT(*)/(
          SELECT COUNT(*)
          FROM qxact_calls
          WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),


    'tot_wait' => Array('Total Wait Duration','Tot. Wait',1,0,"
        SELECT SUM(waittime) as tot_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'avg_wait' => Array('Average Wait Duration','Avg. Wait',1,0,"
        SELECT AVG(waittime) as avg_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'min_wait' => Array('Minimum Wait Duration','Min. Wait',1,0,"
        SELECT MIN(waittime) as min_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'max_wait' => Array('Maximum Wait Duration','Max. Wait',1,0,"
        SELECT MAX(waittime) as max_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'percent_wait' => Array('Percent of Wait Duration','% Wait',1,0,"
        SELECT (SUM(waittime)/(
            SELECT SUM(waittime)
            FROM qxact_calls
            WHERE
                queuename IN ##QUEUES##
                AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_wait
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'tot_talk' => Array('Total Talk Duration','Tot. Talk',1,0,"
        SELECT SUM(calltime) as tot_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'avg_talk' => Array('Average Talk Duration','Avg. Talk',1,0,"
        SELECT AVG(calltime) as avg_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'min_talk' => Array('Minimum Talk Duration','Min. Talk',1,0,"
        SELECT MIN(calltime) as min_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'max_talk' => Array('Maximum Talk Duration','Max. Talk',1,0,"
        SELECT MAX(calltime) as max_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'percent_talk' => Array('Percent of Talk Duration','% Talk',1,0,"
        SELECT (SUM(calltime)/(
            SELECT SUM(calltime)
            FROM qxact_calls
            WHERE
                queuename IN ##QUEUES##
                AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_talk
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'tot_call' => Array('Total Call Duration','Tot. Call',0,0,"
        SELECT (SUM(waittime)+SUM(calltime)) as tot_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'avg_call' => Array('Average Call Duration','Avg. Call',0,0,"
        SELECT AVG(calltime+waittime) as avg_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'min_call' => Array('Minimum Call Duration','Min. Call',0,0,"
        SELECT (MIN(calltime)+MIN(waittime)) as min_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'max_call' => Array('Maximum Call Duration','Max. Call',0,0,"
        SELECT MAX(calltime+waittime) as max_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'percent_call' => Array('Percent of Call Duration','% Call',0,0,"
        SELECT ((SUM(waittime)+SUM(calltime))/(
            SELECT (SUM(waittime)+SUM(calltime))
            FROM qxact_calls
            WHERE
                queuename IN ##QUEUES##
                AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        )*100) AS percent_call
        FROM qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),
);

//All Call Distribution by Day of Week - COMBINED
$dist_by_dow_combined_columns = Array(

    'dow' => Array('Day of Week','Day',1,1,"
        SELECT DAYNAME(FROM_UNIXTIME(`time`)) AS dow
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
          AND qxact_calls.id IN (
            SELECT
              MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'calls' => Array('Number of Calls','# Calls',1,0,"
        SELECT COUNT(*) as calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
          AND qxact_calls.id IN (
            SELECT
              MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'percent_calls' => Array('Percent of Calls',"% Calls",1,0,"
        SELECT (COUNT(*)/(
          SELECT COUNT(*)
          FROM qxact_calls
          WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
          AND qxact_calls.id IN (
            SELECT
              MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
        )*100) AS percent_calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
          AND qxact_calls.id IN (
            SELECT
              MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'tot_wait' => Array('Total Time Waiting','Tot. Wait',1,0,"
        SELECT
          SUM(b.tot_waittime) as tot_wait
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT sum(waittime) as tot_waittime, uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'avg_wait' => Array('Average Wait Duration','Avg. Wait',1,0,"
        SELECT
          AVG(b.tot_waittime) as avg_wait
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT SUM(waittime) as tot_waittime, uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
          GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'min_wait' => Array('Minimum Wait Duration','Min. Wait',1,0,"
        SELECT
          MIN(b.tot_waittime) as min_wait
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT SUM(waittime) as tot_waittime, uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
          GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'max_wait' => Array('Maximum Wait Duration','Max. Wait',1,0,"
        SELECT
          MAX(b.tot_waittime) as max_wait
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT SUM(waittime) as tot_waittime, uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
          GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'percent_wait' => Array('Percent of Time Waiting',"% Wait",1,0,"
        SELECT
          (SUM(b.tot_waittime)/ (
            SELECT SUM(waittime)
            FROM qxact_calls
            WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
          )*100) as percent_wait
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT sum(waittime) as tot_waittime, uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'tot_talk' => Array('Total Talk Duration','Tot. Talk',1,0,"
        SELECT
          SUM(b.tot_talktime) as tot_talk
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT sum(calltime) as tot_talktime, uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'avg_talk' => Array('Average Talk Duration','Avg. Talk',1,0,"
        SELECT
          AVG(b.tot_calltime) as avg_talk
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT SUM(calltime) as tot_calltime, uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'min_talk' => Array('Minimum Talk Duration','Min. Talk',1,0,"
        SELECT
          MIN(b.tot_talktime) as min_talk
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT SUM(calltime) as tot_talktime, uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'max_talk' => Array('Maximum Talk Duration','Max. Talk',1,0,"
        SELECT
          MAX(b.tot_talktime) as max_talk
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT SUM(calltime) as tot_talktime, uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'percent_talk' => Array('Percent of Talk Duration',"% Talk",1,0,"
        SELECT
          (SUM(b.tot_calltime)/ (
            SELECT SUM(calltime)
            FROM qxact_calls
            WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
          )*100) as percent_talk
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT sum(calltime) as tot_calltime, uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'tot_call' => Array('Total Call Duration','Tot. Call',0,0,"
        SELECT
          (SUM(b.tot_talktime)+SUM(b.tot_waittime)) as tot_call
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT
            SUM(calltime) as tot_talktime,
            SUM(waittime) as tot_waittime,
            uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'avg_call' => Array('Average Call Duration','Avg. Call',0,0,"
        SELECT
          AVG((b.tot_calltime)+(b.tot_waittime)) as avg_call
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT
            SUM(calltime) AS tot_calltime,
            SUM(waittime) AS tot_waittime,
            uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'min_call' => Array('Minimum Call Duration','Min. Call',0,0,"
        SELECT
          MIN(b.tot_talktime+b.tot_waittime) as min_call
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT
            SUM(calltime) AS tot_talktime,
            SUM(waittime) AS tot_waittime,
            uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'max_call' => Array('Maximum Call Duration','Max. Call',0,0,"
        SELECT
          MAX(b.tot_talktime+b.tot_waittime) as max_call
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT
            SUM(calltime) as tot_talktime,
            SUM(waittime) as tot_waittime,
            uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),

    'percent_call' => Array('Percent of Call Duration',"% Call",0,0,"
        SELECT
          ((SUM(b.tot_calltime)+SUM(b.tot_waittime))/ (
            SELECT (SUM(calltime)+SUM(waittime))
            FROM qxact_calls
            WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
          )*100) as percent_call
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT
            SUM(calltime) as tot_calltime,
            SUM(waittime) as tot_waittime,
            uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          a.id IN (
            SELECT  MAX(id) AS id
            FROM qxact_calls
            WHERE event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%'
            GROUP BY uniqueid
          )
          AND queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY DAYOFWEEK(FROM_UNIXTIME(`time`))
    "),
);

$service_level_columns = Array(
    'seconds' => Array('Within Duration','Within',1,1,"
        SELECT
            ((ROUND(waittime/##SLAINCREMENT##)*##SLAINCREMENT##)+##SLAINCREMENT##) AS seconds
        FROM
            qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event like 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN '##FROMTIMESTAMP##' AND '##TOTIMESTAMP##'
        GROUP BY ROUND(waittime/##SLAINCREMENT##)
    "),

    'calls' => Array('Number of Calls','# Calls',1,1,"
        SELECT
            COUNT(*) AS calls
        FROM
            qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event like 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN '##FROMTIMESTAMP##' AND '##TOTIMESTAMP##'
        GROUP BY ROUND(waittime/##SLAINCREMENT##)
    "),

    'percent_calls' => Array('Percent of Calls','% Calls',1,1,"
        SELECT
            ROUND((COUNT(*)/(
                SELECT COUNT(*) AS tot_calls
                FROM
                    qxact_calls
                WHERE
                    queuename IN ##QUEUES##
                    AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event like 'COMPLETE%' OR event = 'TRANSFER')
                    AND time BETWEEN '##FROMTIMESTAMP##' AND '##TOTIMESTAMP##'
            ))*100) AS percent_calls
        FROM
            qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event like 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN '##FROMTIMESTAMP##' AND '##TOTIMESTAMP##'
        GROUP BY ROUND(waittime/##SLAINCREMENT##)
    "),
);

$service_level_answered_columns = Array(
    'seconds' => Array('Within Duration','Within',1,1,"
        SELECT
            ((ROUND(waittime/##SLAINCREMENT##)*##SLAINCREMENT##)+##SLAINCREMENT##) AS seconds
        FROM
            qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event like 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN '##FROMTIMESTAMP##' AND '##TOTIMESTAMP##'
        GROUP BY ROUND(waittime/##SLAINCREMENT##)
    "),

    'calls' => Array('Number of Calls','# Calls',1,1,"
        SELECT
            COUNT(*) AS calls
        FROM
            qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event like 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN '##FROMTIMESTAMP##' AND '##TOTIMESTAMP##'
        GROUP BY ROUND(waittime/##SLAINCREMENT##)
    "),

    'percent_calls' => Array('Percent of Calls','% Calls',1,1,"
        SELECT
            ROUND((COUNT(*)/(
                SELECT COUNT(*) AS tot_calls
                FROM
                    qxact_calls
                WHERE
                    queuename IN ##QUEUES##
                    AND agent IN ##AGENTS##
                    AND (event like 'COMPLETE%' OR event = 'TRANSFER')
                    AND time BETWEEN '##FROMTIMESTAMP##' AND '##TOTIMESTAMP##'
            ))*100) AS percent_calls
        FROM
            qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event like 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN '##FROMTIMESTAMP##' AND '##TOTIMESTAMP##'
        GROUP BY ROUND(waittime/##SLAINCREMENT##)
    "),
);

$service_level_answered_combined_columns = $service_level_answered_columns;

$service_level_unanswered_columns = Array(
    'seconds' => Array('Within Duration','Within',1,1,"
        SELECT
            ((ROUND(waittime/##SLAINCREMENT##)*##SLAINCREMENT##)+##SLAINCREMENT##) AS seconds
        FROM
            qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN '##FROMTIMESTAMP##' AND '##TOTIMESTAMP##'
        GROUP BY ROUND(waittime/##SLAINCREMENT##)
    "),

    'calls' => Array('Number of Calls','# Calls',1,1,"
        SELECT
            COUNT(*) AS calls
        FROM
            qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN '##FROMTIMESTAMP##' AND '##TOTIMESTAMP##'
        GROUP BY ROUND(waittime/##SLAINCREMENT##)
    "),

    'percent_calls' => Array('Percent of Calls','% Calls',1,1,"
        SELECT
            ROUND((COUNT(*)/(
                SELECT COUNT(*) AS tot_calls
                FROM
                    qxact_calls
                WHERE
                    queuename IN ##QUEUES##
                    AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                    AND time BETWEEN '##FROMTIMESTAMP##' AND '##TOTIMESTAMP##'
            ))*100) AS percent_calls
        FROM
            qxact_calls
        WHERE
            queuename IN ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN '##FROMTIMESTAMP##' AND '##TOTIMESTAMP##'
        GROUP BY ROUND(waittime/##SLAINCREMENT##)
    "),
);

$service_level_unanswered_combined_columns = $service_level_unanswered_columns;

//Disconnection Cause
$disc_cause_columns = Array(
    'reason' => Array('Reason','Reason',1,1,"
        SELECT
            event as reason
        FROM
             qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            AND (event LIKE 'COMPLETE%' OR event like 'EXIT%' or (event = 'ABANDON' AND agent = '') or event = 'TRANSFER')
        GROUP BY event
    "),

    'count' => Array('Number','#',1,1,"
        SELECT
            COUNT(*) as count
        FROM
             qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            AND (event LIKE 'COMPLETE%' OR event like 'EXIT%' or (event = 'ABANDON' AND agent = '') or event = 'TRANSFER')
        GROUP BY event
    "),

    'percent' => Array('Percent','%',1,1,"
        SELECT
            ROUND((COUNT(*)/(
                SELECT COUNT(*)
                FROM qxact_calls
                WHERE
                    queuename in ##QUEUES##
                    AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
                    AND (event LIKE 'COMPLETE%' OR event like 'EXIT%' or (event = 'ABANDON' AND agent = '') or event = 'TRANSFER')
            ))*100) as percent
        FROM
             qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            AND (event LIKE 'COMPLETE%' OR event like 'EXIT%' or (event = 'ABANDON' AND agent = '') or event = 'TRANSFER')
        GROUP BY event
    "),
);

//Disconnection Cause - COMBINED
$disc_cause_combined_columns = $disc_cause_columns;

//Call Detail List
$call_detail_columns = Array(
    'date' => Array('Date of Call','Date',1,1,"
        SELECT
            time AS date
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event = 'ENTERQUEUE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),

    'queue' => Array('Queue Name','Queue',1,0,"
        SELECT
            queuename AS queue
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event = 'ENTERQUEUE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'cid' => Array('Caller','Caller',1,0,"
        SELECT
            callerid AS cid
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event = 'ENTERQUEUE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'agent' => Array('Agent Name','Agent',1,0,"
        SELECT
            MAX(agent) AS agent
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event = 'ENTERQUEUE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'disconnect_cause' => Array('Disconnection Cause','Disc. Cause',1,0,"
        SELECT
            CONCAT(MAX(event),'/',MIN(event)) AS disconnect_cause
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event = 'ENTERQUEUE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'dur_wait' => Array('Wait Duration','Wait',1,0,"
        SELECT
            SUM(waittime) AS dur_wait
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event = 'ENTERQUEUE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'dur_talk' => Array('Talk Duration','Talk',1,0,"
        SELECT
            SUM(calltime) AS dur_talk
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event = 'ENTERQUEUE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'dur_call' => Array('Call Duration','Call',1,0,"
        SELECT
            (SUM(waittime)+SUM(calltime)) AS dur_call
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event = 'ENTERQUEUE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'orig_pos' => Array('Original Position','Orig. Pos.',1,0,"
        SELECT
            MAX(origpos) AS orig_pos
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event = 'ENTERQUEUE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'abandon_pos' => Array('Abandon Position','Abnd. Pos.',1,0,"
        SELECT
            MAX(position) AS abandon_pos
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event = 'ENTERQUEUE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    
    'did' => Array('Inbound DID Number','DID',0,0,"
        SELECT
            did as did
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event = 'ENTERQUEUE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
        
    'uniqueid' => Array('Unique ID','Unique ID',0,0,"
        SELECT
            uniqueid as uniqueid
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%' OR event = 'ENTERQUEUE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
);


//Call Detail List - COMBINED
$call_detail_combined_columns = $call_detail_columns;

//Answered Call Detail List
$ans_call_detail_columns = Array(
    'date' => Array('Date of Call','Date',1,1,"
        SELECT
            time AS date
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),

    'queue' => Array('Queue Name','Queue',1,0,"
        SELECT
            queuename AS queue
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'cid' => Array('Caller','Caller',1,0,"
        SELECT
            callerid AS cid
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'agent' => Array('Agent Name','Agent',1,0,"
        SELECT
            MAX(agent) AS agent
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'disconnect_cause' => Array('Disconnection Cause','Disc. Cause',1,0,"
        SELECT
            CONCAT(MAX(event),'/',MIN(event)) AS disconnect_cause
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'dur_wait' => Array('Wait Duration','Wait',1,0,"
        SELECT
            SUM(waittime) AS dur_wait
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'dur_talk' => Array('Talk Duration','Talk',1,0,"
        SELECT
            SUM(calltime) AS dur_talk
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'dur_call' => Array('Call Duration','Call',1,0,"
        SELECT
            (SUM(waittime)+SUM(calltime)) AS dur_call
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'orig_pos' => Array('Original Position','Orig. Pos.',1,0,"
        SELECT
            MAX(origpos) AS orig_pos
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),

    'did' => Array('Inbound DID Number','DID',0,0,"
        SELECT
            did as did
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    
    'uniqueid' => Array('Unique ID','Unique ID',0,0,"
        SELECT
            uniqueid as uniqueid
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
);

//Answered Call Detail List - COMBINED 
$ans_call_detail_combined_columns = $ans_call_detail_columns;

//Unanswered Call Detail List
$unans_call_detail_columns = Array(
    'date' => Array('Date of Call','Date',1,1,"
        SELECT
            time AS date
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),

    'queue' => Array('Queue Name','Queue',1,0,"
        SELECT
            queuename AS queue
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'cid' => Array('Caller','Caller',1,0,"
        SELECT
            callerid AS cid
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'disconnect_cause' => Array('Disconnection Cause','Disc. Cause',1,0,"
        SELECT
            CONCAT(MAX(event),'/',MIN(event)) AS disconnect_cause
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'dur_wait' => Array('Wait Duration','Wait',1,0,"
        SELECT
            SUM(waittime) AS dur_wait
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),

    'orig_pos' => Array('Original Position','Orig. Pos.',1,0,"
        SELECT
            MAX(origpos) AS orig_pos
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'abandon_pos' => Array('Abandon Position','Abnd. Pos.',1,0,"
        SELECT
            MAX(position) AS abandon_pos
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'did' => Array('Inbound DID Number','DID',0,0,"
        SELECT
            did as did
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
    'uniqueid' => Array('Unique ID','Unique ID',0,0,"
        SELECT
            uniqueid as uniqueid
        FROM
            qxact_calls
        WHERE
            queuename in ##QUEUES##
            AND ((event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY
            queuename,
            uniqueid
        ORDER BY time ASC
    "),
);

//Unanswered Call Detail List - COMBINED
$unans_call_detail_combined_columns = $unans_call_detail_columns;

//Agent Availability Detail
$agent_log_columns = Array(
    'date' => Array('Date','Date',1,1,"
        SELECT
            time AS date
        FROM
            qxact_agent_actions
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'LOG%' OR event LIKE '%PAUSE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        ORDER BY time ASC
    "),
    'queue' => Array('Queue Name','Queue',1,0,"
        SELECT
            queuename AS queue
        FROM
            qxact_agent_actions
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'LOG%' OR event LIKE '%PAUSE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        ORDER BY time ASC
    "),
    'agent' => Array('Agent Name','Agent',1,0,"
        SELECT
            agent
        FROM
            qxact_agent_actions
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'LOG%' OR event LIKE '%PAUSE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        ORDER BY time ASC
    "),
    'action' => Array('Agent Action','Action',1,0,"
        SELECT
            event AS action
        FROM
            qxact_agent_actions
        WHERE
            queuename in ##QUEUES##
            AND agent IN ##AGENTS##
            AND (event LIKE 'LOG%' OR event LIKE '%PAUSE')
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        ORDER BY time ASC
    "),

);

###############################################

//Agent Missed Calls List
$agent_missed_calls_columns = Array(
    'date' => Array('Date of Call','Date',1,1,"
        SELECT
            time AS date
        FROM
            qxact_agent_calls
        WHERE
            queuename in ##QUEUES##
            AND agent in ##AGENTS##
            AND event = 'RINGNOANSWER'
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        ORDER BY time ASC
    "),
    'queue' => Array('Queue Name','Queue',1,0,"
        SELECT
            queuename AS queue
        FROM
            qxact_agent_calls
        WHERE
            queuename in ##QUEUES##
            AND agent in ##AGENTS##
            AND event = 'RINGNOANSWER'
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        ORDER BY time ASC
    "),
    'cid' => Array('Caller ID','Caller',1,0,"
        SELECT
            callerid as cid
        FROM
            qxact_agent_calls a LEFT OUTER JOIN 
            qxact_calls c USING(uniqueid)
        WHERE
            a.queuename in ##QUEUES##
            AND a.agent in ##AGENTS##
            AND a.event = 'RINGNOANSWER'
            AND a.time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        ORDER BY a.time ASC
    "),
    'agent' => Array('Agent Name','Agent',1,0,"
        SELECT
            agent
        FROM
            qxact_agent_calls
        WHERE
            queuename in ##QUEUES##
            AND agent in ##AGENTS##
            AND event = 'RINGNOANSWER'
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        ORDER BY time ASC
    "),
    'ringtime' => Array('Ring Time','Ring',1,0,"
        SELECT
            CAST((ringtime/1000) AS SIGNED) as ringtime
        FROM
            qxact_agent_calls
        WHERE
            queuename in ##QUEUES##
            AND agent in ##AGENTS##
            AND event = 'RINGNOANSWER'
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        ORDER BY time ASC
    "),
    'uniqueid' => Array('Unique ID','Unique ID',0,0,"
        SELECT
            uniqueid as uniqueid
        FROM
            qxact_agent_calls
        WHERE
            queuename in ##QUEUES##
            AND agent in ##AGENTS##
            AND event = 'RINGNOANSWER'
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        ORDER BY time ASC
    "),
);

########################################






//Custom ACD by queue
$custom_acd_by_q_columns = Array(
    'queue' => Array('Queue Name','Queue',1,1,"
        SELECT queuename AS queue
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),

    'calls' => Array('Number of Calls','# Calls',1,0,"
        SELECT COUNT(*) as calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename
    "),

    'calls_answered' => Array('Answered Calls','Answered',1,0,"
        SELECT 
          data.calls_answered as calls_answered,
          queues.queuename
        FROM (
            SELECT
              COUNT(*) AS calls_answered,
              b.queuename as queuename
            FROM qxact_calls AS a
            RIGHT OUTER JOIN (
                SELECT queuename
                FROM qxact_calls 
                WHERE
                  queuename IN ##QUEUES##
                  AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                  AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
                GROUP BY queuename
            ) AS b
            USING (queuename)
            WHERE
              queuename in ##QUEUES##
              AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
              AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            GROUP BY b.queuename
        ) AS data
                RIGHT OUTER JOIN (
            SELECT queuename
                FROM qxact_calls AS a
                WHERE
                  (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                  AND queuename in ##QUEUES##
                  AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
                GROUP BY queuename
            ) AS queues
            USING (queuename)
        GROUP BY queues.queuename
    "),

    'avg_wait' => Array('Average Wait Duration','Avg. Wait',1,0,"
        SELECT
          AVG(b.tot_waittime) as avg_wait
        FROM qxact_calls AS a
        LEFT JOIN (
          SELECT SUM(waittime) as tot_waittime, uniqueid
          FROM qxact_calls
          WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          GROUP BY uniqueid
        ) AS b
        ON a.uniqueid = b.uniqueid
        WHERE
          queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename
    "),

    'percent_sla' => Array('SLA Percent','SLA %',1,0,"
        SELECT 
            ((data.tot_within_sla / data.tot_answered)*100) as percent_sla,
            queues.queuename
        FROM ( -- returns number of calls within the sla set for that queue
            SELECT COUNT(*) AS tot_within_sla,
            c.tot_answered as tot_answered,
            c.queuename as queuename
            FROM qxact_calls AS a
            LEFT OUTER JOIN (
                SELECT ba.queuename AS queuename
                FROM qxact_calls AS ba
                WHERE
                  queuename in ##QUEUES##
                  AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
                  AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                GROUP BY queuename
            ) AS b
            USING (queuename)
            LEFT JOIN
                (-- returns service levels for each queue
                    SELECT 
                      descr AS queuename,
                      data as servicelevel
                    FROM queues_config
                    RIGHT JOIN queues_details
                    ON queues_config.extension = queues_details.id
                    WHERE keyword = 'servicelevel'
                ) AS sl
            USING(queuename)
            LEFT JOIN 
                ( -- returns total answered number of calls
                    SELECT 
                        COUNT(*) AS tot_answered,
                        cb.queuename as queuename
                    FROM qxact_calls AS ca
                    RIGHT OUTER JOIN (
                        SELECT caa.queuename AS queuename
                        FROM qxact_calls AS caa
                        WHERE
                          (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                          AND queuename in ##QUEUES##
                          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
                        GROUP BY queuename
                    ) AS cb
                    USING (queuename)
                    WHERE (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
                    AND queuename in ##QUEUES##
                    AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
                    group by cb.queuename
                ) AS c 
                ON c.queuename = sl.queuename
            WHERE
              a.waittime <= sl.servicelevel
              AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
              AND c.queuename in ##QUEUES##
              AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            GROUP BY c.queuename
        ) AS data
        RIGHT OUTER JOIN (
            SELECT queuename
                FROM qxact_calls AS a
                WHERE
                  (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                  AND queuename in ##QUEUES##
                  AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
                GROUP BY queuename
            ) AS queues
            USING (queuename)
        GROUP BY queues.queuename
    "),

    'avg_talk' => Array('Average Talk Time','Avg. Talk',1,0,"
      SELECT
        data.avg_talk as avg_talk,
        queues.queuename as queuename
      FROM (
        SELECT AVG(calltime) as avg_talk,
        queuename
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename
      ) data
      RIGHT OUTER JOIN (
        SELECT queuename
            FROM qxact_calls AS a
            WHERE
              (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
              AND queuename in ##QUEUES##
              AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            GROUP BY queuename
        ) AS queues
        USING (queuename)
      GROUP BY queues.queuename
     "),
      
    'avg_ACW' => Array('After Call Wrapup','ACW',1,0,"
      SELECT 
        data.wrapuptime as avg_ACW,
        queues.queuename
      FROM (
        SELECT 
          descr AS queuename,
          data as wrapuptime
        FROM queues_config
        RIGHT JOIN queues_details
        ON queues_config.extension = queues_details.id
        WHERE keyword = 'wrapuptime'
      ) as data
      RIGHT OUTER JOIN (
        SELECT queuename
            FROM qxact_calls AS a
            WHERE
              (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
              AND queuename in ##QUEUES##
              AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            GROUP BY queuename
        ) AS queues
        USING (queuename)
      GROUP BY queues.queuename
    "),

    'calls_abandoned' => Array('Abandoned Calls','# Abnd.',1,0,"
      SELECT 
        data.calls_abandoned,
        queues.queuename
      FROM (
        SELECT queuename,
        count(*) AS calls_abandoned
        FROM qxact_calls AS a
        WHERE
          event = 'ABANDON' AND agent = ''
          AND queuename in ##QUEUES##
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename
      ) AS data
      RIGHT OUTER JOIN (
        SELECT queuename
            FROM qxact_calls AS a
            WHERE
              (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
              AND queuename in ##QUEUES##
              AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            GROUP BY queuename
        ) AS queues
        USING (queuename)
      GROUP BY queues.queuename
    "),

    'percent_abandoned' => Array('Percent Abandoned','% Abnd.',1,0,"
      SELECT 
        ((data.calls_abandoned/data.tot_calls)*100) AS percent_abandoned,
        queues.queuename
      FROM (
        SELECT a.queuename,
        count(*) AS calls_abandoned,
        b.tot_calls as tot_calls
        FROM qxact_calls AS a
        INNER JOIN (
            SELECT ba.queuename,
            count(*) AS tot_calls
            FROM qxact_calls AS ba
            WHERE
              (ba.event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
              AND ba.queuename in ##QUEUES##
              AND ba.time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            GROUP BY ba.queuename
        ) AS b
        USING (queuename)
        WHERE
          a.event = 'ABANDON' AND agent = ''
          AND a.queuename in ##QUEUES##
          AND a.time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY a.queuename
      ) AS data
      RIGHT OUTER JOIN (
        SELECT c.queuename
            FROM qxact_calls AS c
            WHERE
              (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
              AND c.queuename in ##QUEUES##
              AND c.time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            GROUP BY c.queuename
        ) AS queues
        USING (queuename)
      GROUP BY queues.queuename
    "),

    'avg_abandon_wait' => Array('Average Abandoned Wait','Avg. Abnd. Wait',1,0,"
      SELECT
        data.avg_abandon_wait,
        queues.queuename
      FROM (
        SELECT
          AVG(waittime) as avg_abandon_wait,
          queuename
        FROM qxact_calls 
        WHERE
          queuename in ##QUEUES##
          AND event = 'ABANDON' AND agent = ''
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename
      ) as data
      RIGHT OUTER JOIN (
        SELECT queuename
            FROM qxact_calls
            WHERE
              (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
              AND queuename in ##QUEUES##
              AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            GROUP BY queuename
        ) AS queues
        USING (queuename)
      GROUP BY queues.queuename
    "),

    'max_wait' => Array('Maximum Wait Time','Max. Wait',1,0,"
        SELECT MAX(waittime) as max_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY queuename"),



);

//Custom ACD by queue
$custom_acd_by_d_columns = Array(
    'day' => Array('Day','Day',1,1,"
        SELECT time AS day
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'calls' => Array('Number of Calls','# Calls',1,0,"
        SELECT COUNT(*) as calls
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'calls_answered' => Array('Answered Calls','Answered',1,0,"
        SELECT 
          data.calls_answered as calls_answered,
          days.theday
        FROM (
          SELECT 
            a.calls_answered as calls_answered,
            a.theday as theday
          FROM (
            SELECT
              COUNT(*) AS calls_answered,
              CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday
            FROM qxact_calls AS a
            WHERE
              queuename in ##QUEUES##
              AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
              AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            GROUP BY theday
          ) AS a
          RIGHT OUTER JOIN (
            SELECT 
              CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday
            FROM qxact_calls 
            WHERE
              queuename IN ##QUEUES##
              AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
              AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            GROUP BY theday
          ) AS b
          USING (theday)
        ) AS data
        RIGHT OUTER JOIN (
          SELECT 
            CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday
          FROM qxact_calls AS a
          WHERE
            (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
            AND queuename in ##QUEUES##
            AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
          GROUP BY theday
        ) AS days
        USING (theday)
        GROUP BY days.theday
    "),

    'avg_wait' => Array('Average Wait Duration','Avg. Wait',1,0,"
        SELECT
          AVG(waittime) as avg_wait
        FROM qxact_calls AS a
        WHERE
          queuename in ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
    "),

    'percent_sla' => Array('SLA Percent','SLA %',1,0,"
        SELECT
          ((within_sla/tot_answered)*100) AS percent_sla,
          theday
        FROM 
            (
                SELECT 
                  COUNT(*) AS within_sla,
                  theday
                FROM
                  (
                    SELECT 
                      CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday,
                      queuename,
                      waittime  
                    FROM qxact_calls
                    WHERE
                        (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
                        AND queuename in ##QUEUES##
                        AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
                  ) calls
                  LEFT JOIN 
                  (
                    SELECT 
                      descr AS queuename,
                      data as servicelevel
                    FROM queues_config
                    RIGHT JOIN queues_details
                    ON queues_config.extension = queues_details.id
                    WHERE keyword = 'servicelevel'
                  ) slas
                  USING (queuename)
                WHERE waittime < servicelevel
                GROUP BY theday
            ) as within
            INNER JOIN 
            (
                SELECT 
                  COUNT(*) AS tot_answered,
                  CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday  
                FROM qxact_calls 
                WHERE 
                  (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
                   AND queuename in ##QUEUES##
                   AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
                GROUP BY theday
            ) as totals 
            USING (theday)
            RIGHT OUTER JOIN 
            (
                SELECT 
                  CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday
                FROM qxact_calls AS a
                WHERE
                  (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
                  AND queuename in ##QUEUES##
                  AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
                GROUP BY theday
            ) AS days
            USING (theday)
        GROUP BY days.theday
    "),

    'avg_talk' => Array('Average Talk Time','Avg. Talk',1,0,"
      SELECT
        data.avg_talk as avg_talk,
        days.theday
      FROM (
        SELECT 
          AVG(calltime) as avg_talk,
          CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday         
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY theday
      ) data,
      (
        SELECT 
          CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday 
            FROM qxact_calls AS a
            WHERE
              (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
              AND queuename in ##QUEUES##
              AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            GROUP BY theday
      ) days
      WHERE days.theday = data.theday
      GROUP BY days.theday
     "),
      
    'avg_ACW' => Array('After Call Wrapup','Avg. ACW',1,0,"
      SELECT 
        data.wrapuptime as avg_ACW,
        days.theday
      FROM (
        SELECT 
          descr AS queuename,
          AVG(data) AS wrapuptime
        FROM queues_config
        RIGHT JOIN queues_details
        ON queues_config.extension = queues_details.id
        WHERE 
          keyword = 'wrapuptime'
          AND descr in ##QUEUES##
        GROUP BY descr
      ) as data,
      (
        SELECT 
          CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday          
        FROM qxact_calls AS a
        WHERE
          (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND queuename in ##QUEUES##
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
          GROUP BY theday
      ) AS days
      GROUP BY days.theday
    "),

    'calls_abandoned' => Array('Abandoned Calls','# Abnd.',1,0,"
      SELECT 
        data.calls_abandoned,
        days.theday
      FROM (
        SELECT
          count(*) AS calls_abandoned,
          CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday
        FROM qxact_calls AS a
        WHERE
          event = 'ABANDON' AND agent = ''
          AND queuename in ##QUEUES##
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY theday
      ) AS data
      RIGHT OUTER JOIN (
          SELECT 
            CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday
            FROM qxact_calls AS a
            WHERE
              (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
              AND queuename in ##QUEUES##
              AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
            GROUP BY theday
      ) AS days
      USING (theday)
      GROUP BY days.theday
    "),

    'percent_abandoned' => Array('Percent Abandoned','% Abnd.',1,0,"
      SELECT 
        ((a.calls_abandoned/b.tot_calls)*100) AS percent_abandoned,
        days.theday
      FROM 
      (
        SELECT 
          CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday,
          count(*) AS calls_abandoned
        FROM qxact_calls 
        WHERE
          event = 'ABANDON' AND agent = ''
          AND queuename in ##QUEUES##
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY theday
      ) a
      INNER JOIN 
      (
        SELECT 
          CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday,
          count(*) AS tot_calls
        FROM qxact_calls
        WHERE
          (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND queuename in ##QUEUES##
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY theday
      ) AS b
      USING (theday)
      RIGHT OUTER JOIN (
        SELECT 
          CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday
        FROM qxact_calls
        WHERE
          (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND queuename in ##QUEUES##
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY theday
      ) AS days
        USING (theday)
      GROUP BY days.theday
    "),

    'avg_abandon_wait' => Array('Average Abandoned Wait','Avg. Abnd. Wait',1,0,"
      SELECT
        data.avg_abandon_wait,
        days.theday
      FROM (
        SELECT
          AVG(waittime) as avg_abandon_wait,
          CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday
        FROM qxact_calls 
        WHERE
          queuename in ##QUEUES##
          AND event = 'ABANDON' AND agent = ''
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY theday
      ) as data
      RIGHT OUTER JOIN (
        SELECT 
          CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2)) AS theday
        FROM qxact_calls
        WHERE
          (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND queuename in ##QUEUES##
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY theday
      ) AS days
      USING (theday)
      GROUP BY days.theday
    "),

    'max_wait' => Array('Maximum Wait Time','Max. Wait',1,0,"
        SELECT MAX(waittime) as max_wait
        FROM qxact_calls
        WHERE
          queuename IN ##QUEUES##
          AND (event LIKE 'COMPLETE%' OR event = 'TRANSFER' OR (event = 'ABANDON' AND agent = '') OR event LIKE 'EXIT%')
          AND time BETWEEN ##FROMTIMESTAMP## AND ##TOTIMESTAMP##
        GROUP BY CONCAT(YEAR(FROM_UNIXTIME(`time`)),'-', RIGHT(CONCAT('00',MONTH(FROM_UNIXTIME(`time`))),2),'-', RIGHT(CONCAT('00',DAY(FROM_UNIXTIME(`time`))),2))
      "),



);  // end custom acd by date






?>
