<?php

function aastra_set_content_type() {
	header("Content-Type: text/xml");
}

function aastra_get_body($page_args) {
	if (count($page_args) > 0) {
		foreach ($page_args as $key => $val) {
			if ($key == "selection" || $key == "ignoreselection" || $key == "listid" || $key == "add" || $key == "delete") {
				continue;
			}

			if (is_array($val)) {
				if (isset($page_args["listid"])) {
					$list_id = $page_args["listid"];
					if (isset($page_args["delete"])) {
						$val[$list_id] = "";
					}
				}

				/* Sort before we filter, so stuff is in the right order. */
				ksort($val);

				$body[$key] = array_filter($val, "strlen");
			} else {
				$body[$key] = $val;
			}

			unset($page_args[$key]);
		}
	}

	return (count($body) > 0) ? $body : NULL;
}

function aastra_digit_keys($string) {
	$ua = $GLOBALS["user_agent"];

	$key_map = array();
	switch ($ua['brand']) {
	case 'aastra':
		$key_map = array(
			'1' => 'KeyPad1',
			'2' => 'KeyPad2',
			'3' => 'KeyPad3',
			'4' => 'KeyPad4',
			'5' => 'KeyPad5',
			'6' => 'KeyPad6',
			'7' => 'KeyPad7',
			'8' => 'KeyPad8',
			'9' => 'KeyPad9',
			'0' => 'KeyPad0',
			'*' => 'KeyPadStar',
			'#' => 'KeyPadPound',
			'X' => 'Xfer'
		);
		break;
	case 'yealink':
		$key_map = array(
			'1' => 'KEY_1',
			'2' => 'KEY_2',
			'3' => 'KEY_3',
			'4' => 'KEY_4',
			'5' => 'KEY_5',
			'6' => 'KEY_6',
			'7' => 'KEY_7',
			'8' => 'KEY_8',
			'9' => 'KEY_9',
			'0' => 'KEY_0',
			'*' => 'STAR',
			'#' => 'POUND',
			'X' => 'TRANSFER'
		);
		break;
	}

	$digits = str_split($string);

	$keys = array();
	foreach ($digits as $digit) {
		if (isset($key_map[$digit])) {
			$keys[] = $key_map[$digit];
		}
	}

	return $keys;
}

function aastra_page_action($page) {
	$ua = $GLOBALS["user_agent"];
	$xml = '';

	$entry = $page->layout[0];
	switch ($entry->type) {
	case 'transfer':
		if ($entry->dtmf) {
			$digits = $entry->dtmf . $entry->value . '#';
		} else {
			$digits = 'X' . $entry->value . 'X';
		}

		$keys = aastra_digit_keys($digits);

		$xml.= '<AastraIPPhoneExecute Beep="no" triggerDestroyOnExit="yes">';

		foreach ($keys as $key) {
			$xml.= '<ExecuteItem URI="Key:' . $key . '"/>';
		}

		$xml.= '</AastraIPPhoneExecute>';

		break;
	case 'dial':
		$xml.= '<AastraIPPhoneExecute Beep="no">';
		$xml.= '<ExecuteItem URI="Dial:' . $entry->dial . '"/>';
		if ($entry->value) {
			$uri = str_replace(trim($_SERVER['PATH_INFO'], "/"), $entry->value, $GLOBALS['server_uri']);
			if ($entry->args) {
				if (strstr($uri, "?")) {
					$uri.= "&amp;";
				} else {
					$uri.= "?";
				}
				$uri.= htmlspecialchars($entry->args);
			}
			$xml.= '<ExecuteItem URI="' . $uri . '"/>';
		}
		$xml.= '</AastraIPPhoneExecute>';

		break;
	case 'redirect':
		$xml.= '<AastraIPPhoneExecute Beep="no" triggerDestroyOnExit="yes">';

		if (isset($entry->dtmf)) {
			$keys = aastra_digit_keys($entry->dtmf);

			foreach ($keys as $key) {
				$xml.= '<ExecuteItem URI="Key:' . $key . '"/>';
			}
		}

		$uri = str_replace(trim($_SERVER['PATH_INFO'], "/"), $entry->value, $GLOBALS['server_uri']);
		if ($entry->args) {
			if (strstr($uri, "?")) {
				$uri.= "&amp;";
			} else {
				$uri.= "?";
			}
			$uri.= htmlspecialchars($entry->args);
		}
		$xml.= '<ExecuteItem URI="' . $uri . '"/>';
		$xml.= '</AastraIPPhoneExecute>';

		break;
	case 'reboot':
		$xml.= '<AastraIPPhoneExecute Beep="no" triggerDestroyOnExit="yes">';
		$xml.= '<ExecuteItem URL="Command: Reset"/>';
		$xml.= '</AastraIPPhoneExecute>';

		break;
	case 'config':
		$config = aastra_get_config($ua['mac']);
		switch($ua['brand']) {
		case 'yealink':
			$max_page_size = 10000;
			$page_overhead = strlen('<AastraIPPhoneConfiguration Beep="no"></AastraIPPhoneConfiguration>');
			$item_overhead = strlen('<Item></Item>') + strlen(' = ');
			break;
		case 'aastra':
			$max_page_size = 10000;
			$page_overhead = strlen('<AastraIPPhoneConfiguration Beep="no"></AastraIPPhoneConfiguration>');
			$item_overhead = strlen('<ConfigurationItem></ConfigurationItem>') + strlen('<Parameter></Parameter>') + strlen('<Value></Value>');
			break;
		}

		if ($entry->value == NULL) {
			$xml.= '<AastraIPPhoneExecute Beep="no" triggerDestroyOnExit="yes">';
			$pages = 1;
			$page_size = $page_overhead;
			foreach ($config as $parameter => $value) {
				$item_size = $item_overhead + strlen($parameter) + strlen($value);

				if (($page_size + $item_size) <= $max_page_size) {
					$page_size += $item_size;
				} else {
					$pages++;
					$page_size = $page_overhead + $item_size;
				}
			}

			for ($i = 1; $i <= $pages; $i++) {
				$uri = $GLOBALS['server_uri'];
				if (strstr($uri, "?")) {
					$uri.= "&amp;";
				} else {
					$uri.= "?";
				}
				$uri.= 'action=config&amp;actiondata=' . $i;

				$xml.= '<ExecuteItem URI="' . $uri . '"/>';
			}

			if (isset($config['action uri startup'])) {
				$xml.= '<ExecuteItem URI="' . $config['action uri startup'] . '"/>';
			} else if (isset($config['action uri registered'])) {
				$xml.= '<ExecuteItem URI="' . $config['action uri registered'] . '"/>';
			} else if (isset($config['action_url.setup_completed'])) {
				$xml.= '<ExecuteItem URI="' . $config['action_url.setup_completed'] . '"/>';
			} else if (isset($config['action_url.log_on'])) {
				$xml.= '<ExecuteItem URI="' . $config['action_url.log_on'] . '"/>';
			}
			$xml.= '</AastraIPPhoneExecute>';
		} else {
			$xml.= '<AastraIPPhoneConfiguration Beep="no">';

			$pages = 1;
			$page_size = $page_overhead;
			foreach ($config as $parameter => $value) {
				$item_size = $item_overhead + strlen($parameter) + strlen($value);

				if (($page_size + $item_size) <= $max_page_size) {
					$page_size += $item_size;
				} else {
					$pages++;
					$page_size = $page_overhead + $item_size;
				}

				if ($pages < $entry->value) {
					continue;
				} else if ($pages > $entry->value) {
					break;
				}

				switch($ua['brand']) {
				case 'aastra':
					$xml.= '<ConfigurationItem>';
					$xml.= '<Parameter>';
					$xml.= htmlspecialchars($parameter);
					$xml.= '</Parameter>';
					$xml.= '<Value>';
					$xml.= htmlspecialchars($value);
					$xml.= '</Value>';
					$xml.= '</ConfigurationItem>';
					break;
				case 'yealink':
					$xml.= '<Item>';
					$xml.= htmlspecialchars($parameter) . ' = ' . htmlspecialchars($value);
					$xml.= '</Item>';
					break;
				}
			}

			$xml.= '</AastraIPPhoneConfiguration>';
		}
		break;
	default:
		break;
	}

	return $xml;
}

function aastra_page_directory($page) {
	$xml = "<AastraIPPhoneDirectory style=\"none\" Beep=\"no\" destroyOnExit=\"yes\">";
	$xml.= "<Title wrap=\"yes\">" . ($page->title ? htmlspecialchars($page->title) : htmlspecialchars($page->application_display)) . "</Title>";

	foreach ($page->layout as $entry) {
		switch ($entry->type) {
		case "dial":
			$xml.= aastra_directory_dial($entry);
			break;
		}
	}

	if (aastra_device_has_softkeys()) {
		$xml.= "<SoftKey index=\"1\">";
		$xml.= "<Label>Exit</Label>";
		$xml.= "<URI>";
		if ($page->exitPath) {
			$xml.= str_replace(trim($_SERVER['PATH_INFO'], "/"), $page->exitPath, $GLOBALS['server_uri']) . "&amp;ignoreselection" . ($page->exitArgs ? "&amp;" . htmlspecialchars($page->exitArgs) : "");
		} else {
			$xml.= "SoftKey:Exit";
		}
		$xml.= "</URI>";
		$xml.= "</SoftKey>";

		$xml.= "<SoftKey index=\"2\">";
		$xml.= "<Label>Dial</Label>";
		$xml.= "<URI>SoftKey:Dial</URI>";
		$xml.= "</SoftKey>";
	}

	$xml.= "</AastraIPPhoneDirectory>";

	return $xml;
}

function aastra_page_display($page) {
	/* Show the main screen */
	$xml = "<AastraIPPhoneTextMenu style=\"none\" wrapList=\"yes\" destroyOnExit=\"yes\"";
	if ($page->exitPath) {
		$xml.= " cancelAction=\"" . str_replace(trim($_SERVER['PATH_INFO'], "/"), $page->exitPath, $GLOBALS['server_uri']) . ($page->exitArgs ? "&amp;" . htmlspecialchars($page->exitArgs) : "") . "\"";
	}
	$xml.= ">";
	$xml.= "<Title wrap=\"yes\">" . ($page->title ? htmlspecialchars($page->title) : htmlspecialchars($page->application_display)) . "</Title>";

	$icons = array();

	$need_change = false;
	$need_select = false;
	$need_dial = false;
	foreach ($page->layout as $entry) {
		if ($entry->icon && aastra_device_has_icons()) {
			$icons[] = aastra_get_named_icon($entry->icon);
			$entry->icon = count($icons);
		}

		switch ($entry->type) {
		case "label":
			$xml.= aastra_display_string($entry, false, false);
			break;
		case "text":
			$need_change = true;
			$xml.= aastra_display_string($entry, true, false);
			break;
		case "password":
			$need_change = true;
			$xml.= aastra_display_string($entry, true, true);
			break;
		case "number":
			$need_change = true;
			$xml.= aastra_display_number($entry);
			break;
		case "bool":
			$need_change = true;
			$xml.= aastra_display_bool($entry);
			break;
		case "choice":
			$need_change = true;
			$xml.= aastra_display_choice($entry);
			break;
		case "list":
			$need_change = true;
			$xml.= aastra_display_list($entry);
			break;
		case "link":
			$need_select = true;
			$xml.= aastra_display_link($entry);
			break;
		case "dial":
			$need_dial = true;
			$xml.= aastra_display_dial($entry);
			break;
		case "entry":
			$xml.= aastra_display_entry($entry);
			break;
		}
	}

	if (aastra_device_has_icons() && count($icons) > 0) {
		$xml.= "<IconList>";
		foreach ($icons as $key => $val) {
			$xml.= "<Icon index=\"" . ($key + 1) . "\">" . $val . "</Icon>";
		}
		$xml.= "</IconList>";
	}

	if (aastra_device_has_softkeys()) {
		$xml.= "<SoftKey index=\"1\">";
		$xml.= "<Label>Exit</Label>";
		$xml.= "<URI>";
		if ($page->exitPath) {
			$xml.= str_replace(trim($_SERVER['PATH_INFO'], "/"), $page->exitPath, $GLOBALS['server_uri']) . "&amp;ignoreselection" . ($page->exitArgs ? "&amp;" . htmlspecialchars($page->exitArgs) : "");
		} else {
			$xml.= "SoftKey:Exit";
		}
		$xml.= "</URI>";
		$xml.= "</SoftKey>";

		$softkey_index = 2;
		if (!aastra_device_has_onscreen_select()) {
			if ($need_change) {
				$xml.= "<SoftKey index=\"" . $softkey_index++ . "\">";
				$xml.= "<Label>Change</Label>";
				$xml.= "<URI>";
				if (aastra_device_has_softkey_uri_selection()) {
					$xml.= $GLOBALS['server_uri'];
				} else {
					$xml.= "SoftKey:Select";
				}
				$xml.= "</URI>";
				$xml.= "</SoftKey>";
			} else if ($need_select) {
				$xml.= "<SoftKey index=\"" . $softkey_index++ . "\">";
				$xml.= "<Label>Select</Label>";
				$xml.= "<URI>";
				$xml.= "SoftKey:Select";
				$xml.= "</URI>";
				$xml.= "</SoftKey>";
			}
		}

		if ($need_dial) {
			if (aastra_device_has_softkey_dial()) {
				/* Only show the Dial softkey if there is something to dial. */
				$xml.= "<SoftKey index=\"" . $softkey_index++ . "\">";
				$xml.= "<Label>Dial</Label>";
				$xml.= "<URI>";
				$xml.= "SoftKey:Dial2";
				$xml.= "</URI>";
				$xml.= "</SoftKey>";
			}
		}

		if (aastra_device_has_softkey_uri_selection()) {
			foreach ($page->action as $action) {
				$xml.= "<SoftKey index=\"" . $softkey_index++ . "\">";
				$xml.= "<Label>" . $action->display . "</Label>";
				$xml.= "<URI>";
				if ($action->link) {
					$xml.= str_replace(trim($_SERVER['PATH_INFO'], "/"), $action->link, $GLOBALS['server_uri']);
				} else {
					$xml.= $GLOBALS['server_uri'];
					$xml.= "&amp;ignoreselection&amp;action=" . $action->name;
				}
				$xml.= "</URI>";
				$xml.= "</SoftKey>";
			}
		}
	}

	$xml.= "</AastraIPPhoneTextMenu>";

	return $xml;
}

function aastra_page_auth($page) {
	$xml = "<AastraIPPhoneInputScreen destroyOnExit=\"yes\" displayMode=\"condensed\" type=\"string\" cancelAction=\"" . $GLOBALS['server_uri'] . "\">";
	$xml.= "<URL>" . $GLOBALS['server_uri'] . "</URL>";

	$xml.= "<Title wrap=\"yes\">Authentication Required</Title>";

	if (aastra_device_has_multiple_input()) {
		foreach ($page->layout as $entry) {
			$xml.= aastra_edit_number_field($entry->name, $entry->value, $entry->display, NULL);
		}
	} else {
		$parameter = "";
		$winner = NULL;

		foreach ($page->layout as $entry) {
			if ($winner == NULL && $entry->value == NULL) {
				/* First entry without a value is the winner. */
				$winner = $entry;
			} else if ($entry->value != NULL) {
				$parameter .= $entry->name . "=" . $entry->value . "&amp;";
			}
		}

		if ($winner != NULL) {
			$xml.= aastra_edit_number_field($parameter . $winner->name, NULL, $winner->display, NULL);
		}
	}

	$xml.= "</AastraIPPhoneInputScreen>";

	return $xml;
}

function aastra_get_page($page_data, $page_args) {
	if (isset($page_args["selection"]) && !isset($page_args["ignoreselection"])) {
		$selection = $page_args["selection"];

		if (isset($page_args["add"])) {
			$list_id = PHP_INT_MAX;
		} else if (isset($page_args["listid"])) {
			$list_id = $page_args["listid"];
		}

		if (isset($page_args[$selection])) {
			$edit = true;
		}
	}

	$page = json_decode($page_data);

	$screen_xml = '';

	if (count($page->error) > 0) {
		$screen_xml.= "<AastraIPPhoneTextScreen destroyOnExit=\"yes\" beep=\"yes\">";
		$screen_xml.= "<Title wrap=\"yes\">Error</Title>";
		$screen_xml.= "<Text>";
		foreach ($page->error as $error) {
			$screen_xml.= $error->display . "\n";
		}
		$screen_xml.= "</Text>";
		$screen_xml.= "</AastraIPPhoneTextScreen>";
	} else if (!$selection || ($edit && $list_id == NULL)) {
		$ua = $GLOBALS["user_agent"];

		switch ($page->type) {
		case 'action':
			$screen_xml.= aastra_page_action($page);
			break;
		case 'directory':
			if ($ua["brand"] === "yealink") {
				$screen_xml.= aastra_page_directory($page);
			} else {
				$screen_xml.= aastra_page_display($page);
			}
			break;
		case 'auth':
			$screen_xml.= aastra_page_auth($page);
			break;
		default:
			$screen_xml.= aastra_page_display($page);
			break;
		}
	} else {
		/* Show the edit field screen */
		foreach ($page->layout as $entry) {
			if ($entry->name !== $selection) {
				continue;
			}

			switch ($entry->type) {
			case "label":
				$screen_xml.= aastra_edit_string($entry, false, false);
				break;
			case "text":
				$screen_xml.= aastra_edit_string($entry, true, false);
				break;
			case "password":
				$screen_xml.= aastra_edit_string($entry, true, true);
				break;
			case "number":
				$screen_xml.= aastra_edit_number($entry);
				break;
			case "choice":
				$screen_xml.= aastra_edit_choice($entry);
				break;
			case "list":
				$screen_xml.= aastra_edit_list($entry, $edit ? NULL : $list_id);
				break;
			}
		}
	}

	header("Content-Type: text/xml");
	if ($page->refresh) {
		header("Refresh: " . $page->refresh . "; url=" . $GLOBALS['server_uri']);
	}

	return $screen_xml;
}

function aastra_display_string($entry, $editable=true, $password=false) {
	$xml = "<MenuItem" . ($entry->icon && aastra_device_has_icons() ? " icon=\"" . $entry->icon . "\"" : "") . ">";
	$xml.= "<Prompt>";
	if ($entry->iconstring && !aastra_device_has_icons()) {
		$xml.= $entry->iconstring;
	}
	if ($entry->display != NULL) {
		$xml.= htmlspecialchars($entry->display) . ": ";
	}
	$xml.= ($password ? "******" : htmlspecialchars($entry->value));
	$xml.= "</Prompt>";
	if ($editable) {
		$selection = htmlspecialchars($entry->name);
		$xml.= "<Selection>" . $selection . ($entry->args ? "&amp;" . $entry->args : "") . "</Selection>";
		$xml.= "<URI>";
		$xml.= $GLOBALS['server_uri'];
		$xml.= "&amp;selection=" . $selection . ($entry->args ? "&amp;" . $entry->args : "");
		$xml.= "</URI>";
	}
	$xml.= "</MenuItem>";

	return $xml;
}

function aastra_display_number($entry) {
	$xml = "<MenuItem" . ($entry->icon && aastra_device_has_icons() ? " icon=\"" . $entry->icon . "\"" : "") . ">";
	$xml.= "<Prompt>";
	if ($entry->iconstring && !aastra_device_has_icons()) {
		$xml.= $entry->iconstring;
	}
	$xml.= htmlspecialchars($entry->display) . ": " . htmlspecialchars($entry->value);
	$xml.= "</Prompt>";
	$selection = htmlspecialchars($entry->name);
	$xml.= "<Selection>" . $selection . ($entry->args ? "&amp;" . $entry->args : "") . "</Selection>";
	$xml.= "<URI>";
	$xml.= $GLOBALS['server_uri'];
	$xml.= "&amp;selection=" . $selection . ($entry->args ? "&amp;" . $entry->args : "");
	$xml.= "</URI>";
	$xml.= "</MenuItem>";

	return $xml;
}

function aastra_display_bool($entry) {
	$xml = "<MenuItem" . ($entry->icon && aastra_device_has_icons() ? " icon=\"" . $entry->icon . "\"" : "") . ">";
	$xml.= "<Prompt>";
	if ($entry->iconstring && !aastra_device_has_icons()) {
		$xml.= $entry->iconstring;
	}
	$xml.= htmlspecialchars($entry->display) . ": " . htmlspecialchars($entry->value ? ($entry->yes ? $entry->yes : "Yes") : ($entry->no ? $entry->no : "No"));
	$xml.= "</Prompt>";
	$selection = htmlspecialchars($entry->name) . "&amp;" . htmlspecialchars($entry->name) . "=" . (!$entry->value ? 1 : 0);
	$xml.= "<Selection>" . $selection . ($entry->args ? "&amp;" . $entry->args : "") . "</Selection>";
	$xml.= "<URI>";
	$xml.= $GLOBALS['server_uri'];
	$xml.= "&amp;selection=" . $selection . ($entry->args ? "&amp;" . $entry->args : "");
	$xml.= "</URI>";
	$xml.= "</MenuItem>";

	return $xml;
}

function aastra_display_choice($entry) {
	$xml = "<MenuItem" . ($entry->icon && aastra_device_has_icons() ? " icon=\"" . $entry->icon . "\"" : "") . ">";
	$xml.= "<Prompt>";
	if ($entry->iconstring && !aastra_device_has_icons()) {
		$xml.= $entry->iconstring;
	}
	$xml.= htmlspecialchars($entry->display);
	$xml.= "</Prompt>";
	$selection = htmlspecialchars($entry->name);
	$xml.= "<Selection>" . $selection . ($entry->args ? "&amp;" . $entry->args : "") . "</Selection>";
	$xml.= "<URI>";
	$xml.= $GLOBALS['server_uri'];
	$xml.= "&amp;selection=" . $selection . ($entry->args ? "&amp;" . $entry->args : "");
	$xml.= "</URI>";
	$xml.= "</MenuItem>";

	return $xml;
}

function aastra_display_list($entry) {
	$xml = "<MenuItem" . ($entry->icon && aastra_device_has_icons() ? " icon=\"" . $entry->icon . "\"" : "") . ">";
	$xml.= "<Prompt>";
	if ($entry->iconstring && !aastra_device_has_icons()) {
		$xml.= $entry->iconstring;
	}
	$xml.= htmlspecialchars($entry->display) . ": [" . count($entry->value) . " items]";
	$xml.= "</Prompt>";
	$selection = htmlspecialchars($entry->name);
	$xml.= "<Selection>" . $selection . ($entry->args ? "&amp;" . $entry->args : "") . "</Selection>";
	$xml.= "<URI>";
	$xml.= $GLOBALS['server_uri'];
	$xml.= "&amp;selection=" . $selection . ($entry->args ? "&amp;" . $entry->args : "");
	$xml.= "</URI>";
	$xml.= "</MenuItem>";

	return $xml;
}

function aastra_display_link($entry) {
	$xml = "<MenuItem" . ($entry->icon && aastra_device_has_icons() ? " icon=\"" . $entry->icon . "\"" : "") . ">";
	$xml.= "<Prompt>";
	if ($entry->iconstring && !aastra_device_has_icons()) {
		$xml.= $entry->iconstring;
	}
	$xml.= htmlspecialchars($entry->display);
	$xml.= "</Prompt>";
	$xml.= "<Selection>" . ($entry->args ? "&amp;" . $entry->args : "") . "</Selection>";
	$xml.= "<URI>";
	$uri = str_replace(trim($_SERVER['PATH_INFO'], "/"), $entry->value, $GLOBALS['server_uri']);
	if ($entry->args) {
		if (strstr($uri, "?")) {
			$uri.= "&amp;";
		} else {
			$uri.= "?";
		}
		$uri.= $entry->args;
	}
	$xml.= $uri;
	$xml.= "</URI>";
	if ($entry->dial) {
		$xml.= "<Dial>";
		$xml.= htmlspecialchars($entry->dial);
		$xml.= "</Dial>";
	}
	$xml.= "</MenuItem>";

	return $xml;
}

function aastra_display_dial($entry) {
	$xml = "<MenuItem" . ($entry->icon && aastra_device_has_icons() ? " icon=\"" . $entry->icon . "\"" : "") . ">";
	$xml.= "<Prompt>";
	if ($entry->iconstring && !aastra_device_has_icons()) {
		$xml.= $entry->iconstring;
	}
	$xml.= htmlspecialchars($entry->display);
	$xml.= "</Prompt>";
	$selection = htmlspecialchars($entry->name);
	$xml.= "<Selection>" . $selection . ($entry->args ? "&amp;" . $entry->args : "") . "</Selection>";
	$xml.= "<URI>";
	$xml.= "Dial:" . htmlspecialchars($entry->value);
	$xml.= "</URI>";
	$xml.= "<Dial>";
	$xml.= htmlspecialchars($entry->value);
	$xml.= "</Dial>";
	$xml.= "</MenuItem>";

	return $xml;
}

function aastra_display_entry($entry) {
	$xml = "<MenuItem" . ($entry->icon && aastra_device_has_icons() ? " icon=\"" . $entry->icon . "\"" : "") . ">";
	$xml.= "<Prompt>";
	if ($entry->iconstring && !aastra_device_has_icons()) {
		$xml.= $entry->iconstring;
	}
	$xml.= htmlspecialchars($entry->display);
	$xml.= "</Prompt>";
	$xml.= "<Selection>&amp;actiondata=" . htmlspecialchars($entry->value) . ($entry->args ? "&amp;" . $entry->args : "") . "</Selection>";
	$xml.= "</MenuItem>";

	return $xml;
}

function aastra_directory_dial($entry) {
	$xml = "<MenuItem>";
	$xml.= "<Prompt>" . htmlspecialchars($entry->display) . "</Prompt>";
	$xml.= "<URI>";
	$xml.= htmlspecialchars($entry->value);
	$xml.= "</URI>";
	$xml.= "</MenuItem>";

	return $xml;
}

function aastra_edit_string_field($parameter, $value, $prompt, $selection, $editable=true, $password=false) {
	$xml = "<InputField type=\"string\"" . (!$editable ? " editable=\"no\"" : "") . ($password ? " password=\"yes\"" : "") . ">";
	$xml.= "<Prompt>" . ($prompt ? $prompt : "") . "</Prompt>";
	$xml.= "<Parameter>" . $parameter . "</Parameter>";
	$xml.= "<Default>" . $value . "</Default>";

	$xml.= "<SoftKey index=\"1\">";
	$xml.= "<Label>Cancel</Label>";
	$xml.= "<URI>";
	$xml.= $GLOBALS['server_uri'];
	if ($selection) {
		$xml.= "&amp;selection=" . $selection;
	}
	$xml.= "</URI>";
	$xml.= "</SoftKey>";

	$xml.= "<SoftKey index=\"2\">";
	$xml.= "<Label>Done</Label>";
	$xml.= "<URI>SoftKey:Submit</URI>";
	$xml.= "</SoftKey>";

	$xml.= "<SoftKey index=\"3\">";
	$xml.= "<Label>Backspace</Label>";
	$xml.= "<URI>SoftKey:BackSpace</URI>";
	$xml.= "</SoftKey>";

	$xml.= "<SoftKey index=\"4\">";
	$xml.= "<Label>Dot \".\"</Label>";
	$xml.= "<URI>SoftKey:Dot</URI>";
	$xml.= "</SoftKey>";

	$xml.= "<SoftKey index=\"5\">";
	$xml.= "<Label>ABC&gt;</Label>";
	$xml.= "<URI>SoftKey:ChangeMode</URI>";
	$xml.= "</SoftKey>";

	$xml.= "<SoftKey index=\"6\">";
	$xml.= "<Label>NextSpace</Label>";
	$xml.= "<URI>SoftKey:NextSpace</URI>";
	$xml.= "</SoftKey>";

	$xml.= "</InputField>";

	return $xml;
}

function aastra_edit_string($entry, $editable=true, $password=false) {
	$xml = "<AastraIPPhoneInputScreen destroyOnExit=\"yes\" displayMode=\"condensed\" type=\"string\" cancelAction=\"" . $GLOBALS['server_uri'] . "\">";
	$xml.= "<URL>" . $GLOBALS['server_uri'] . "</URL>";

	$xml.= "<Title wrap=\"yes\">" . htmlspecialchars($entry->display) . "</Title>";

	$xml.= aastra_edit_string_field(htmlspecialchars($entry->name), htmlspecialchars($entry->value), NULL, NULL, $editable, $password);

	$xml.= "</AastraIPPhoneInputScreen>";

	return $xml;
}

function aastra_edit_number_field($parameter, $value, $prompt, $selection) {
	$xml = "<InputField type=\"number\">";
	$xml.= "<Prompt>" . ($prompt ? $prompt : "") . "</Prompt>";
	$xml.= "<Parameter>" . $parameter . "</Parameter>";
	$xml.= "<Default>" . $value . "</Default>";

	$xml.= "<SoftKey index=\"1\">";
	$xml.= "<Label>Cancel</Label>";
	$xml.= "<URI>";
	$xml.= $GLOBALS['server_uri'];
	if ($selection) {
		$xml.= "&amp;selection=" . $selection;
	}
	$xml.= "</URI>";
	$xml.= "</SoftKey>";

	$xml.= "<SoftKey index=\"2\">";
	$xml.= "<Label>Done</Label>";
	$xml.= "<URI>SoftKey:Submit</URI>";
	$xml.= "</SoftKey>";

	$xml.= "<SoftKey index=\"3\">";
	$xml.= "<Label>Backspace</Label>";
	$xml.= "<URI>SoftKey:BackSpace</URI>";
	$xml.= "</SoftKey>";

	$xml.= "</InputField>";

	return $xml;
}

function aastra_edit_number($entry) {
	$xml = "<AastraIPPhoneInputScreen destroyOnExit=\"yes\" displayMode=\"condensed\" type=\"string\" cancelAction=\"" . $GLOBALS['server_uri'] . "\">";
	$xml.= "<URL>" . $GLOBALS['server_uri'] . "</URL>";

	$xml.= "<Title wrap=\"yes\">";
	$xml.= htmlspecialchars($entry->display);
	if (isset($entry->min) && isset($entry->max)) {
		$xml.= " (" . $entry->min . "-" . $entry->max . ")";
	}
	$xml.= "</Title>";

	$xml.= aastra_edit_number_field(htmlspecialchars($entry->name), htmlspecialchars($entry->value), NULL, NULL);

	$xml.= "</AastraIPPhoneInputScreen>";

	return $xml;
}

function aastra_edit_choice($entry) {
	$selection = htmlspecialchars($entry->name);

	$xml = "<AastraIPPhoneTextMenu style=\"radio\" wrapList=\"yes\" destroyOnExit=\"yes\" cancelAction=\"" . $GLOBALS['server_uri'] . "\">";
	$xml.= "<Title wrap=\"yes\">" . htmlspecialchars($entry->display) . "</Title>";
	foreach ($entry->values as $key => $value) {
		$xml.= "<MenuItem>";
		$xml.= "<Prompt>" . ($entry->value == $key ? "*" : "") . htmlspecialchars($value) . "</Prompt>";
		$xml.= "<Selection>" . $selection . "&amp;" . $selection . "=" . ($key ? $key : "") . "</Selection>";
		$xml.= "<URI>";
		$xml.= $GLOBALS['server_uri'];
		$xml.= "&amp;selection=" . $selection . "&amp;" . $selection . "=" . ($key ? $key : "");
		$xml.= "</URI>";
		$xml.= "</MenuItem>";
	}
	$xml.= "</AastraIPPhoneTextMenu>";

	return $xml;
}

function aastra_edit_list($entry, $list_id) {
	$selection = htmlspecialchars($entry->name);

	if ($list_id == NULL) {
		$xml = "<AastraIPPhoneTextMenu style=\"none\" wrapList=\"yes\" destroyOnExit=\"yes\" cancelAction=\"" . $GLOBALS['server_uri'] . "\">";
		$xml.= "<Title wrap=\"yes\">" . htmlspecialchars($entry->display) . "</Title>";
		foreach ($entry->value as $key => $value) {
			$xml.= "<MenuItem>";
			$xml.= "<Prompt>" . htmlspecialchars($value) . "</Prompt>";
			$xml.= "<Selection>" . $selection . "&amp;listid=" . $key . "</Selection>";
			$xml.= "<URI>";
			$xml.= $GLOBALS['server_uri'];
			$xml.= "&amp;selection=" . $selection . "&amp;listid=" . $key;
			$xml.= "</URI>";
			$xml.= "</MenuItem>";

			$delete_params.= htmlspecialchars($entry->name) . "[" . $key . "]=" . htmlspecialchars($value) . "&amp;";
		}

		$xml.= "<MenuItem>";
		$xml.= "<Prompt>(Add New)</Prompt>";
		$xml.= "<Selection>" . $selection . "&amp;add</Selection>";
		$xml.= "<URI>";
		$xml.= $GLOBALS['server_uri'];
		$xml.= "&amp;selection=" . $selection . "&amp;add";
		$xml.= "</URI>";
		$xml.= "</MenuItem>";

		if (aastra_device_has_softkeys()) {
			$xml.= "<SoftKey index=\"1\">";
			$xml.= "<Label>Back</Label>";
			$xml.= "<URI>";
			$xml.= $GLOBALS['server_uri'];
			$xml.= "&amp;ignoreselection";
			$xml.= "</URI>";
			$xml.= "</SoftKey>";

			if (!aastra_device_has_onscreen_select()) {
				$xml.= "<SoftKey index=\"2\">";
				$xml.= "<Label>Change</Label>";
				$xml.= "<URI>";
				if (aastra_device_has_softkey_uri_selection()) {
					$xml.= $GLOBALS['server_uri'];
				} else {
					$xml.= "SoftKey:Select";
				}
				$xml.= "</URI>";
				$xml.= "</SoftKey>";
			}

			$xml.= "<SoftKey index=\"4\">";
			$xml.= "<Label>Add New</Label>";
			$xml.= "<URI>";
			$xml.= $GLOBALS['server_uri'];
			$xml.= "&amp;selection=" . $selection . "&amp;add";
			$xml.= "</URI>";
			$xml.= "</SoftKey>";

			if (aastra_device_has_softkey_uri_selection()) {
				/* No delete key for Yealink, since it doesn't pass a selection. */
				$xml.= "<SoftKey index=\"5\">";
				$xml.= "<Label>Delete</Label>";
				$xml.= "<URI>";
				$xml.= $GLOBALS['server_uri'];
				$xml.= "&amp;" . $delete_params . "delete";
				$xml.= "</URI>";
				$xml.= "</SoftKey>";
			}
		}
		$xml.= "</AastraIPPhoneTextMenu>";
	} else {
		$default_value = "";
		$default_key = PHP_INT_MAX; /* New list item */

		$parameters = "selection=" . $selection . "&amp;listid=" . $list_id . "&amp;";

		$xml = "<AastraIPPhoneInputScreen destroyOnExit=\"yes\" displayMode=\"condensed\" type=\"string\" cancelAction=\"" . $GLOBALS['server_uri'] . "\">";
		$xml.= "<URL>" . $GLOBALS['server_uri'] . "</URL>";

		$xml.= "<Title wrap=\"yes\">" . htmlspecialchars($entry->display) . "</Title>";

		foreach ($entry->value as $key => $value) {
			if ($key == $list_id) {
				$default_value = $value;
				$default_key = $key;
			} else {
				$parameters.= htmlspecialchars($entry->name) . "[" . $key . "]=" . htmlspecialchars($value) . "&amp;";
			}
		}

		$field_params = $parameters . htmlspecialchars($entry->name) . "[" . $default_key . "]";
		switch ($entry->subtype) {
		case "label":
			$xml.= aastra_edit_string_field($field_params, htmlspecialchars($default_value), NULL, $selection, false, false);
			break;
		case "text":
			$xml.= aastra_edit_string_field($field_params, htmlspecialchars($default_value), NULL, $selection, true, false);
			break;
		case "password":
			$xml.= aastra_edit_string_field($field_params, htmlspecialchars($default_value), NULL, $selection, true, true);
			break;
		case "number":
			$xml.= aastra_edit_number_field($field_params, htmlspecialchars($default_value), NULL, $selection);
			break;
		}

		$xml.= "</AastraIPPhoneInputScreen>";
	}

	return $xml;
}

function aastra_device_has_softkeys() {
	$ua = $GLOBALS["user_agent"];

	$softkeys = false;

	if ($ua["brand"] === "aastra") {
		switch ($ua["model"]) {
		case '480i':
		case '480i Cordless':
		case '55i':
		case '57i':
		case '57iCT':
		case '9480i':
		case '9480iCT':
		case '6735i':
		case '6737i':
		case '6739i':
		case '6867i':
		case '8000i':
			$softkeys = true;
			break;
		}
	} else if ($ua["brand"] === "yealink") {
		$softkeys = true;
	}

	return $softkeys;
}

/* Some devices do not need a selection softkey, as they have an on-screen key. */
function aastra_device_has_onscreen_select() {
	$ua = $GLOBALS["user_agent"];

	$onscreen = false;

	if ($ua["brand"] === "aastra") {
		switch ($ua["model"]) {
		case '6739i':
			$onscreen = true;
			break;
		}
	}

	return $onscreen;
}

/* Some devices do not properly support selection on softkeys.
 * These devices will have limited functionality, especially on menus.
 */
function aastra_device_has_softkey_uri_selection() {
	$ua = $GLOBALS["user_agent"];

	$softkeys = true;

	if ($ua["brand"] === "yealink") {
		$softkeys = false;

		switch ($ua["model"]) {
		case 'SIP-T20P': // 9.*
			$minversion = "9.72.0.35";
			break;
		case 'SIP-T22P': // 7.*
			$minversion = "7.72.0.35";
			break;
		case 'SIP-T26P': // 6.*
			$minversion = "6.72.0.35";
			break;
		case 'SIP-T28P': // 2.*
			$minversion = "2.72.0.35";
			break;
		case 'SIP-T32G': // 32.*
			break;
		case 'SIP-T38G': // 38.*
			break;
		case 'SIP-T41G': // 36.*
			$minversion = "36.72.0.15";
			break;
		case 'SIP-T42G': // 29.*
			$minversion = "29.72.0.15";
			break;
		case 'SIP-T46G': // 28.*
			$minversion = "28.72.0.15";
			break;
		case 'SIP-T48G': // 35.*
			$minversion = "35.72.0.5";
			break;
		case 'VP530P': // 23.*
			break;
		}

		if ($minversion && version_compare($ua["firmware"], $minversion, "ge")) {
			$softkeys = true;
		}
	}

	return $softkeys;
}

/* Some devices do not properly support dialing via softkeys.
 * These devices will have limited functionality.
 */
function aastra_device_has_softkey_dial() {
	$ua = $GLOBALS["user_agent"];

	$softkeys = true;

	if ($ua["brand"] === "yealink") {
		$softkeys = false;
	}

	return $softkeys;
}

function aastra_device_has_multiple_input() {
	$ua = $GLOBALS["user_agent"];

	$multiple = false;

	if ($ua["brand"] === "aastra") {
		switch($ua['model']) {
		case '55i':
		case '57i':
		case '57iCT':
		case '9480i':
		case '9480iCT':
		case '6735i':
		case '6737i':
		case '6739i':
		case '6863i':
		case '6865i':
		case '6867i':
		case '8000i':
			$multiple = true;
			break;
		}
	} else if ($ua["brand"] === "yealink") {
		$multiple = true;
	}

	return $multiple;
}

function aastra_device_has_icons() {
	$ua = $GLOBALS["user_agent"];

	$icons = false;
	if ($ua["brand"] === "aastra") {
		switch ($ua["model"]) {
		case '55i':
		case '57i':
		case '57iCT':
		case '9480i':
		case '9480iCT':
		case '6735i':
		case '6737i':
		case '6739i':
		case '6863i':
		case '6865i':
		case '6867i':
		case '8000i':
			$icons = true;
			break;
		}
	} else if ($ua["brand"] === "yealink") {
		$icons = false;
	}

	return $icons;
}

function aastra_get_named_icon($name) {
	switch (strtolower($name)) {
	case "speaker":
		$hex = "0038387CFE00281044380000";
		$image = "Icon:Speaker";
		break;
	case "muted":
		$hex = "0038387CFE00386C6C6C3800";
		$image = "Icon:Mute";
		break;
	case "office":
		$hex = "000000FEAEFAAEFE00000000";
		$image = "Icon:Office";
		break;
	case "cell":
		$hex = "000000007E565AFE00000000";
		$image = "Icon:CellPhone";
		break;
	case "home":
		$hex = "000000103E7A3E1000000000";
		$image = "Icon:Home";
		break;
	case "dial":
		$hex = "000000540054005400000000";
		$image = "Icon:PhoneDial";
		break;
	case "paused":
		$hex = "0000FE868A92A2C2FE000000";
		$image = "Icon:CircleYellow";
		break;
	case "loggedin":
		$hex = "0000FEC6AA92AAC6FE000000";
		$image = "Icon:CircleGreen";
		break;
	case "loggedout":
		$hex = "0000FE8282828282FE000000";
		$image = "Icon:CircleRed";
		break;
	case "static":
		$hex = "0000FEFEFEFEFEFEFE000000";
		$image = "Icon:CircleBlue";
		break;
	case "read":
		$hex = "00FEC6AA924A2A120A060200";
		$image = "Icon:EnvelopeOpen";
		break;
	case "unread":
		$hex = "00FEC6AA928A8A92AAC6FE00";
		$image = "Icon:Envelope";
		break;
	default:
		$hex = "000000000000000000000000";
		$image = "";
		break;
	}

	$ua = $GLOBALS["user_agent"];

	if ($ua["brand"] === "aastra") {
		switch ($ua["model"]) {
		case '6739i':
			return $image;
		}
	}

	return $hex;
}

function aastra_sync_data() {
	$xml = '<AastraIPPhoneExecute Beep="no">';
	$xml.= '<ExecuteItem URI=""/>';
	$xml.= '</AastraIPPhoneExecute>';

	return $xml;
}

function aastra_exec_uri($extension, $uri) {
	$xml = '<AastraIPPhoneExecute Beep="no">';
	$xml.= '<ExecuteItem URI="' . $uri . '"/>';
	$xml.= '</AastraIPPhoneExecute>';

	aastra_notifyxml($extension, $xml);
}

function aastra_toggle_application($extension, $application, $status, $alert = false, $message = NULL, $buttons = NULL) {
	global $astman;

	$ep_user = endpoint_api(array('ext' => $extension));

	if ($ep_user != NULL) {
		$brand = strtolower($ep_user["brand"]);
	}

	if ($buttons == NULL) {
		$buttons = endpoint_apiButtons($extension);
	}
	if ($buttons != NULL) {
		$xml = '<AastraIPPhoneExecute Beep="no">';

		foreach ($buttons as $button) {
			if (isset($button["value"]) && str_replace("REST-", "", $button["value"]) != $application) {
				continue;
			}

			switch($brand) {
			case 'yealink':
				if (preg_match('/^s(\d+)$/', $button["button"], $matches)) {
					$button_name = "MEMO" . $matches[1] . "_RED";
				} else if (preg_match('/^(\d+)$/', $button["button"], $matches)) {
					$button_name = "LINE" . $matches[1];
				}
				break;
			case 'aastra':
				$button_name = $button["button"];
				break;
			}

			$xml.= '<ExecuteItem URI="Led: ' . $button_name . '=' . ($status ? 'on' : 'off') . '"/>';
		}

		$xml.= '</AastraIPPhoneExecute>';

		aastra_notifyxml($extension, $xml);
	}

	$xml = '<AastraIPPhoneStatus' . ($alert ? ' Beep="yes"' : ' Beep="no"') . '>';
	$xml.= '<Session>' . $application . '</Session>';
	$xml.= '<Message index="0"' . ($alert ? ' type="alert" Timeout="10"' : '') . '>';
	if ($message) {
		$xml.= htmlspecialchars($message);
	} else {
		if ($status) {
			$xml.= $application . ' enabled';
		}
	}
	$xml.= '</Message>';
	$xml.= '</AastraIPPhoneStatus>';

	aastra_notifyxml($extension, $xml);
}

function aastra_notifyxml($extension, $xml) {
	global $astman;

	$astman->send_request("SIPnotify", array('Channel' => $extension, 'Variable' => array('Event' => 'aastra-xml', 'Content-Type' => 'application/xml', 'Content' => str_replace('"', '\"', $xml))));
}

function aastra_postxml($ipaddress, $xml) {
	$body = "xml=" . $xml;
	$url = "http://" . $ipaddress;

	$ch = curl_init($url);

	curl_setopt($ch, CURLOPT_HTTPHEADER, array(
		"Content-Type: text/xml"
	));
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_FAILONERROR, true);
	curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
	curl_setopt($ch, CURLOPT_POSTFIELDS, $body);

	$response = curl_exec($ch);

	if (curl_errno($ch) != CURLE_OK) {
		return NULL;
	}

	return $response;
}

function aastra_get_config($mac) {
	$ua = $GLOBALS["user_agent"];

	$raw_config = array();
	switch($ua['brand']) {
	case 'aastra':
		$aastra_config = explode("\n", file_get_contents('/tftpboot/aastra.cfg'));
		$mac_config = explode("\n", file_get_contents('/tftpboot/' . strtoupper($ua['mac']) . '.cfg'));

		$raw_config = array_merge($aastra_config, $mac_config);
		$delimiter = ":";
		$commentchar = "#";
		break;
	case 'yealink':
		$raw_config = explode("\n", file_get_contents('/tftpboot/' . strtolower($ua['mac']) . '.cfg'));
		$delimiter = "=";
		$commentchar = "#";
		break;
	}

	$config = array();
	foreach($raw_config as $line) {
		list($parameter, $value) = explode($delimiter, $line, 2);
		if (strlen($parameter) == 0 || substr($parameter, 0, 1) == $commentchar) {
			continue;
		}

		$trimchars = "'\"\r\n\t ";
		$config[trim($parameter, $trimchars)] = trim($value, $trimchars);
	}
	return $config;
}

function aastra_autoanswer() {
	return array('Alert-Info' => 'info=alert-autoanswer');
}

?>
