<?php

class RESTClient {
	function __construct($user, $token, $tokenkey) {
		/* Hash functions from the restapi module. */
		$this->api = new Api();

		if ($token && $tokenkey) {
			$this->token = $token;
			$this->tokenkey = $tokenkey;
		} else if ($user) {
			if(!function_exists('setup_userman')) {
				$token_ids = restapi_user_get_user_tokens($user);
			} else {
				$token_ids = restapi_user_get_user_tokens_by_extension($user);
			}
			if (!count($token_ids)) {
				return;
			}
			/* Just use the first one */
			$token_id = $token_ids[0];
			$token = restapi_tokens_get($token_id);
			$this->token = $token['token'];
			$this->tokenkey = $token['tokenkey'];
		}
	}

	function execute($path, $verb, $body) {
		global $db;

		$sql = "SELECT `value` FROM sysadmin_options WHERE `key` = 'restapi';";
		$port = $db->getOne($sql);
		if ($port == NULL) {
			/* REST API port wasn't found.  Try the FreePBX port. */
			$sql = "SELECT `value` FROM sysadmin_options WHERE `key` = 'acp';";
			$port = $db->getOne($sql);
			$subdir = "/restapi";

			if ($port == NULL) {
				/* Uh oh.  Try to use the port the request came in on. */
				$port = $_SERVER['SERVER_PORT'];
			}
		}

		$uri = "localhost" . (($port != '80') ? ":" . $port : "") . $subdir . "/rest.php/rest/" . $path;

		$nonce = sha1(microtime(true) . mt_rand(1000,9999999));

		$data = $this->api->auth->get_data_hash($this->token, $uri, $verb, $nonce, $body);
		$sig = $this->api->auth->get_signature($data, $this->tokenkey);

		$ch = curl_init($uri);

		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
			"Nonce: " . $nonce,
			"Signature: " . $sig,
			"Token: " . $this->token,
			"Content-Type: application/json"
		));
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_FAILONERROR, true);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $verb);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $body);

		$response = curl_exec($ch);

		if (curl_errno($ch) != CURLE_OK) {
			$code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
			return json_encode(array("error" => $code));
		}
		return $response;
	}
}

?>
