<?php

$application_name = "callforward";
$application_display = "Call Forward";

$application = new Application(
	$application_name, $application_display, array(
		new Page("main"),
		new Page("change"),
		new Page("presets")
	)
);

function callforward_daemon_init() {
	global $astman;

	$astman->add_event_handler("extensionstatus", "callforward_event_extensionstatus");
}

function callforward_event_extensionstatus($event, $data, $server, $port) {
	if (!preg_match("/^Custom:DEVCF(.*)$/", $data["Hint"], $matches)) {
		return;
	}

	$extension = $matches[1];

	dbug('Call Forward syncing: ' . $extension . ' (' . $data["Status"] . ')');
	callforward_sync($extension, $data["Status"]);
}

function callforward_sync_cb($extension) {
	global $astman;

	$fcc = new featurecode('callforward', 'cf_toggle');
	$cf_code = $fcc->getCodeActive();

	if ($cf_code == '') {
		return;
	}

	$data = $astman->send_request("ExtensionState", array("Exten" => $cf_code . $extension, "Context" => "ext-cf-hints"));

	if (!preg_match("/^Custom:DEVCF(.*)$/", $data["Hint"])) {
		return;
	}

	callforward_sync($extension, $data["Status"]);
}

function callforward_sync($extension, $status) {
	$ep_user = endpoint_api(array('ext' => $extension));

	if ($ep_user != NULL) {
		$brand = $ep_user["brand"];
	}

	switch ($brand) {
		case "Aastra":
		case "Yealink":
			aastra_toggle_application($extension, "Call Forward", $status);
			break;
	}
}

function callforward_preset_types() {
	return array(
		'home' => 'Home',
		'cell' => 'Cell',
		'office' => 'Office'
	);
}

function callforward_presets_set($user, $presets) {
	global $astman;

	foreach ($presets as $type => $number) {
		if ($number) {
			$ret = $astman->database_put('AMPUSER', $user . '/info/' . $type, $number);
		} else {
			$ret = $astman->database_del('AMPUSER', $user . '/info/' . $type);
		}
	}
}

function callforward_presets_get($user) {
	global $astman;

	$presets = array();

	foreach (callforward_preset_types() as $type => $display) {
		$ret = $astman->database_get('AMPUSER', $user . '/info/' . $type);
		if ($ret) {
			$presets[$type] = $ret;
		}
	}

	return $presets;
}

function callforward_main_get_cb($page, $user, $args) {
	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("callforward/users/" . $user, "GET", NULL));

	if (!$data) {
		return NULL;
	}

	$screen = new Screen($page);

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Call Forward is not authorized in the API module for extension " . $user . ".";
			break;
		case 404:
			$error["display"] = "Call Forward is not enabled for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$icon = "dial";

	if ($data->CF) {
		$presets = callforward_presets_get($user);
		foreach ($presets as $type => $number) {
			if ($data->CF == $number) {
				$icon = $type;
			}
		}
	}

	$screen->entry_add(array(
		"type" => "link",
		"name" => "current",
		"display" => "Call Forward: " . ($data->CF ? $data->CF : "Disabled"),
		"value" => "callforward/change",
		"icon" => $icon,
	));

	$screen->action_add(array(
		"name" => "presets",
		"link" => "callforward/presets",
		"display" => "Presets"
	));

	return json_encode($screen);
}

function callforward_change_get_cb($page, $user, $args) {
	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("callforward/users/" . $user, "GET", NULL));

	if ($data == NULL) {
		return NULL;
	}

	if (isset($data->error)) {
		$screen = new Screen($page);

		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Call Forward is not authorized in the API module for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$screen = new Screen($page);
	$screen->exitPath = "callforward/main";

	if ($data->CF) {
		$screen->entry_add(array(
			"type" => "bool",
			"name" => "enable",
			"display" => "Enabled",
			"value" => true
		));
	}

	$screen->entry_add(array(
		"type" => "text",
		"name" => "custom",
		"display" => "Enter Number",
		"value" => ($data->CF ? $data->CF : "")
	));

	$presets = callforward_presets_get($user);
	if (count($presets) > 0) {
		$types = callforward_preset_types();
		foreach ($presets as $type => $number) {
			$values[$number] = $types[$type];
		}

		$screen->entry_add(array(
			"type" => "choice",
			"name" => "preset",
			"display" => "Presets",
			"value" => ($values[$data->CF] ? $data->CF : ""),
			"values" => $values
		));
	}

	return json_encode($screen);
}

function callforward_change_set_cb($page, $user, $args, $body) {
	$rest = new RESTClient($user, NULL, NULL);
	$body = json_decode($body, true);

	$put = array();
	if (isset($body["enable"]) && !$body["enable"]) {
		$put["CF"] = "";
	} else if (isset($body["custom"])) {
		$put["CF"] = $body["custom"];
	} else if (isset($body["preset"])) {
		$put["CF"] = $body["preset"];
	}

	$data = json_decode($rest->execute("callforward/users/" . $user, "PUT", json_encode($put)));

	return NULL;
}

function callforward_presets_get_cb($page, $user, $args) {
	$presets = callforward_presets_get($user);

	$screen = new Screen($page);
	$screen->exitPath = "callforward/main";

	foreach (callforward_preset_types() as $type => $display) {
		$screen->entry_add(array(
			"type" => "text",
			"name" => $type,
			"display" => $display,
			"value" => ($presets[$type] ? $presets[$type] : "")
		));
	}

	return json_encode($screen);
}

function callforward_presets_set_cb($page, $user, $args, $body) {
	$body = json_decode($body, true);

	$presets = array();

	foreach (callforward_preset_types() as $type => $display) {
		if (isset($body[$type])) {
			$presets[$type] = $body[$type];
		}
	}

	callforward_presets_set($user, $presets);

	return NULL;
}

?>
