<?php

$application_name = "conferences";
$application_display = "Conferences";

$application = new Application(
	$application_name, $application_display, array(
		new Page("main"),
		new Page("details")
	)
);

$conferences_conferencerooms = array();
$conferences_conferencecallers = array();

function conferences_event_bridgelistrooms($event, $data, $server, $port) {
	global $conferences_conferencerooms;

	if (!isset($data["ActionID"])) {
		/* No ActionID means it isn't a response. */
		return;
	}

	$room = array();
	$room["Conference"] = $data["Conference"];
	$room["Parties"] = $data["Parties"];
	$room["Marked"] = $data["Marked"];
	$room["Locked"] = $data["Locked"];

	$conferences_conferencerooms[$data["Conference"]] = $room;
}

function conferences_event_bridgelist($event, $data, $server, $port) {
	global $conferences_conferencecallers;

	if (!isset($data["ActionID"])) {
		/* No ActionID means it isn't a response. */
		return;
	}

	$caller = array();
	$caller["Conference"] = $data["Conference"];
	if ($data["UserNumber"] != NULL) {
		$caller["UserID"] = $data["UserNumber"];
	} else {
		$caller["UserID"] = $data["Channel"];
	}
	$caller["CallerIDNum"] = $data["CallerIDNum"];
	$caller["CallerIDName"] = $data["CallerIDName"];
	$caller["MarkedUser"] = $data["MarkedUser"];
	$caller["Admin"] = $data["Admin"];
	if ($data["Muted"] == "By admin" || $data["Muted"] == "By self" || $data["Muted"] == "Yes") {
		$caller["Muted"] = true;
	} else {
		$caller["Muted"] = false;
	}

	$conferences_conferencecallers[$data["Conference"]][$caller["UserID"]] = $caller;
}

function conferences_event_bridgelistroomscomplete($event, $data, $server, $port) {
	global $astman;

	/* HACK: Force a timeout after we get this event, so that the wait_response() returns. */
	stream_set_timeout($astman->socket, 0, 1);
}

function conferences_event_bridgelistcomplete($event, $data, $server, $port) {
	global $astman;

	/* HACK: Force a timeout after we get this event, so that the wait_response() returns. */
	stream_set_timeout($astman->socket, 0, 1);
}

function conferences_main_get_cb($page, $user, $args) {
	global $astman;
	global $amp_conf;
	global $conferences_conferencerooms;

	$action = $args["action"];
	$room = $args["app_room"];

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("conferences", "GET", NULL));

	if (!$data) {
		return NULL;
	}

	if (isset($data->error)) {
		$screen = new Screen($page);
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Conferences is not authorized in the API module for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	if ($action == 'dial') {
		$screen = new Screen($page);
		$screen->type = 'action';

		$screen->entry_add(array(
			"type" => "dial",
			"value" => "conferences/details",
			"args" => "app_room=" . $room . "&action=dialing",
			"dial" => $room
		));

		return json_encode($screen);
	}

	$screen = new Screen($page);

	$astman->events("on");

	$conferences_conferencerooms = array();

	if ($amp_conf['ASTCONFAPP'] == 'app_meetme') {
		$astman->add_event_handler("meetmelistrooms", "conferences_event_bridgelistrooms");
		$astman->add_event_handler("meetmelistroomscomplete", "conferences_event_bridgelistroomscomplete");

		$response = $astman->send_request('meetmelistrooms', array('ActionID' => 'conferences' . $user));
	} else if ($amp_conf['ASTCONFAPP'] == 'app_confbridge') {
		$astman->add_event_handler("confbridgelistrooms", "conferences_event_bridgelistrooms");
		$astman->add_event_handler("confbridgelistroomscomplete", "conferences_event_bridgelistroomscomplete");

		$response = $astman->send_request('confbridgelistrooms', array('ActionID' => 'conferences' . $user));
	}

	if ($response["Response"] == "Success") {
		$astman->wait_response(true);
		stream_set_timeout($astman->socket, 30);
	}

	foreach($data as $key => $val) {
		$count = 0;

		$conference = $conferences_conferencerooms[$key];
		if ($conference && $conference["Parties"]) {
			$count = $conference["Parties"];
		}

		$screen->entry_add(array(
			"type" => "link",
			"name" => $key,
			"display" => $val->description . " (" . $count . ")",
			"value" => "conferences/details",
			"args" => "app_room=" . $key,
			"dial" => $key
		));
	}

	$screen->action_add(array(
		"name" => "dial",
		"display" => "Dial"
	));

	$screen->action_add(array(
		"name" => "refresh",
		"display" => "Refresh"
	));

	return json_encode($screen);
}

function conferences_details_get_cb($page, $user, $args) {
	global $astman;
	global $amp_conf;
	global $conferences_conferencecallers;

	$room = $args["app_room"];
	$action = $args["action"];
	$actiondata = $args["actiondata"];

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("conferences/" . $room, "GET", NULL));

	if (!$data) {
		return NULL;
	}

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Conferences is not authorized in the API module for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	if ($action == 'dialing') {
		/* Give it a second so they're listed as being in the conference. */
		sleep(1);
	}

	$screen = new Screen($page);
	$screen->exitPath = "conferences/main";

	$astman->events("on");

	$conferences_conferencecallers[$room] = array();

	if ($amp_conf['ASTCONFAPP'] == 'app_meetme') {
		$astman->add_event_handler("meetmelist", "conferences_event_bridgelist");
		$astman->add_event_handler("meetmelistcomplete", "conferences_event_bridgelistcomplete");

		$response = $astman->send_request('meetmelist', array('conference' => $room, 'ActionID' => 'conferences' . $user));
	} else if ($amp_conf['ASTCONFAPP'] == 'app_confbridge') {
		$astman->add_event_handler("confbridgelist", "conferences_event_bridgelist");
		$astman->add_event_handler("confbridgelistcomplete", "conferences_event_bridgelistcomplete");

		$response = $astman->send_request('confbridgelist', array('conference' => $room, 'ActionID' => 'conferences' . $user));

	}

	if ($response["Response"] == "Success") {
		$astman->wait_response(true);
		stream_set_timeout($astman->socket, 30);
	}

	$caller_count = 0;
	foreach($conferences_conferencecallers[$room] as $caller) {
		$skip = false;

		if ($caller["UserID"] == $actiondata) {
			switch ($action) {
				case "mute":
					if ($amp_conf['ASTCONFAPP'] == 'app_meetme') {
						if ($caller["Muted"]) {
							$response = $astman->send_request('meetmeunmute', array('meetme' => $room, 'usernum' => $actiondata));
						} else {
							$response = $astman->send_request('meetmemute', array('meetme' => $room, 'usernum' => $actiondata));
						}

						/* Toggle our mute state */
						$caller["Muted"] = !$caller["Muted"];
					} else if ($amp_conf['ASTCONFAPP'] == 'app_confbridge') {
						if ($caller["Muted"]) {
							$response = $astman->send_request('confbridgeunmute', array('conference' => $room, 'channel' => $actiondata));
						} else {
							$response = $astman->send_request('confbridgemute', array('conference' => $room, 'channel' => $actiondata));
						}

						/* Toggle our mute state */
						$caller["Muted"] = !$caller["Muted"];
					}
					break;
				case "kick":
					if ($amp_conf['ASTCONFAPP'] == 'app_meetme') {
						$response = $astman->send_request('Command', array('Command' => "meetme kick " . $room . " " . $actiondata));
					} else if ($amp_conf['ASTCONFAPP'] == 'app_confbridge') {
						$response = $astman->send_request('confbridgekick', array('conference' => $room, 'channel' => $actiondata));
					}
					/* This channel just got kicked.  Don't display it. */
					$skip = true;
					break;
				default:
					break;
			}
		}

		if (!$skip) {
			$caller_count++;
			$screen->entry_add(array(
				"type" => "entry",
				"value" => $caller["UserID"],
				"display" => $caller["CallerIDName"],
				"args" => "app_room=" . $room,
				"icon" => ($caller["Muted"] ? "muted" : NULL),
				"iconstring" => ($caller["Muted"] ? "(M) " : NULL)
			));
		}
	}

	$screen->action_add(array(
		"name" => "refresh",
		"display" => "Refresh"
	));

	if ($caller_count > 0) {
		$screen->action_add(array(
			"name" => "kick",
			"display" => "Kick"
		));

		$screen->action_add(array(
			"name" => "mute",
			"display" => "(Un)Mute"
		));
	} else {
		$screen->entry_add(array(
			"type" => "entry",
			"display" => "No callers in this conference",
			"args" => "app_room=" . $room
		));
	}

	return json_encode($screen);
}

?>
