<?php

$application_name = "daynight";
$application_display = "Call Flow";

$application = new Application(
	$application_name, $application_display, array(
		new Page("main")
	)
);

function daynight_daemon_init() {
	global $astman;

	$astman->add_event_handler("extensionstatus", "daynight_event_extensionstatus");
}

function daynight_event_extensionstatus($event, $data, $server, $port) {
	global $rest;

	$fcc = new featurecode('daynight', 'toggle-mode-all');
	$dn_code = $fcc->getCodeActive();

	if ($dn_code == '') {
		return;
	}

	if (!preg_match("/^" . preg_quote($dn_code . "*") . "(.*)$/", $data["Exten"], $matches)) {
		return;
	}

	$extension = $matches[1];

	$daynights = json_decode($rest->execute("daynight/", "GET", NULL));
	if ($daynights && !isset($daynights->error)) {
		foreach ($daynights as $daynight) {
			$fcc = new featurecode('daynight', 'toggle-mode-' . $daynight->ext);
			$code = $fcc->getCodeActive();

			if ($code == $data["Exten"]) {
				return;
			}
		}
	}

	dbug('Call Flow syncing: ' . $extension . ' (' . $data["Status"] . ')');
	daynight_sync($extension, $data["Status"]);
}

function daynight_sync_cb($extension) {
	global $astman;

	$fcc = new featurecode('daynight', 'toggle-mode-all');
	$dn_code = $fcc->getCodeActive();

	if ($dn_code == '') {
		return;
	}

	$data = $astman->send_request("ExtensionState", array("Exten" => $dn_code . "*" . $extension, "Context" => "app-daynight-toggle"));

	if (!preg_match("/^Custom:DAYNIGHT(.*)$/", $data["Hint"])) {
		return;
	}

	daynight_sync($extension, $data["Status"]);
}

function daynight_sync($extension, $status) {
	$ep_user = endpoint_api(array('ext' => $extension));

	if ($ep_user != NULL) {
		$brand = $ep_user["brand"];
	}

	switch ($brand) {
		case "Aastra":
		case "Yealink":
			aastra_toggle_application($extension, "Call Flow", $status);
			break;
	}
}

function daynight_allowed_daynights($rest, $user, $daynights) {
	$data = json_decode($rest->execute("cos/users/" . $user . "/features", "GET", NULL), true);

	if (!$data || isset($data['error'])) {
		return $daynights;
	}

	foreach ($daynights as $key => $daynight) {
		if (!isset($data['toggle-mode-' . $daynight->ext])) {
			unset($daynights[$key]);
		}
	}

	return $daynights;
}

function daynight_main_get_cb($page, $user, $args) {
	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("daynight/", "GET", NULL));

	if (!$data) {
		return NULL;
	}

	$screen = new Screen($page);

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Call Flow is not authorized in the API module for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$daynights = daynight_allowed_daynights($rest, $user, $data);
	$combined_state = false;
	foreach ($daynights as $daynight) {
		$data = json_decode($rest->execute("daynight/" . $daynight->ext, "GET", NULL));

		if ($data && !$data->error) {
			$daynight->state = ($data->state == "NIGHT" ? true : false);
		} else {
			$daynight->state = false;
		}

		$screen->entry_add(array(
			"type" => "bool",
			"name" => "state_" . $daynight->ext,
			"display" => $daynight->dest . " (" . $daynight->ext . ")",
			"value" => $daynight->state,
			"yes" => "Override",
			"no" => "Normal"
		));

		if ($daynight->state == true) {
			$combined_state = true;
		}
	}

	if (count($daynights) > 1) {
		$screen->entry_add(array(
			"type" => "bool",
			"name" => "state_all",
			"display" => "Set All",
			"value" => true,
			"yes" => "Normal",
			"no" => "Override"
		));

		$screen->entry_add(array(
			"type" => "bool",
			"name" => "state_all",
			"display" => "Set All",
			"value" => false,
			"yes" => "Normal",
			"no" => "Override"
		));
	}

	return json_encode($screen);
}

function daynight_main_set_cb($page, $user, $args, $body) {
	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("daynight/", "GET", NULL));
	$daynights = daynight_allowed_daynights($rest, $user, $data);

	$body = json_decode($body, true);

	foreach ($body as $key => $val) {
		if ($key == 'state_all') {
			if (!$daynights) {
				continue;
			}

			$body_data = array(
				'state' => ($val ? 'NIGHT' : 'DAY')
			);

			foreach ($daynights as $daynight) {
				$data = json_decode($rest->execute("daynight/" . $daynight->ext, "PUT", json_encode($body_data)));
			}
		} else if (preg_match('/^state_(\d*)$/', $key, $matches)) {
			$body_data = array(
				'state' => ($val ? 'NIGHT' : 'DAY')
			);
			$data = json_decode($rest->execute("daynight/" . $matches[1], "PUT", json_encode($body_data)));
		}
	}

	return NULL;
}

?>
