<?php

$application_name = "donotdisturb";
$application_display = "DND";

$application = new Application(
	$application_name, $application_display, array(
		new Page("main")
	)
);

function donotdisturb_daemon_init() {
	global $astman;

	$astman->add_event_handler("extensionstatus", "donotdisturb_event_extensionstatus");
}

function donotdisturb_event_extensionstatus($event, $data, $server, $port) {
	if (!preg_match("/^Custom:DEVDND(.*)$/", $data["Hint"], $matches)) {
		return;
	}

	$extension = $matches[1];

	dbug('DND syncing: ' . $extension . ' (' . $data["Status"] . ')');
	donotdisturb_sync($extension, $data["Status"]);
}

function donotdisturb_sync_cb($extension) {
	global $astman;

	$fcc = new featurecode('donotdisturb', 'dnd_toggle');
	$dnd_code = $fcc->getCodeActive();

	if ($dnd_code == '') {
		return;
	}

	$data = $astman->send_request("ExtensionState", array("Exten" => $dnd_code . $extension, "Context" => "ext-dnd-hints"));

	if (!preg_match("/^Custom:DEVDND(.*)$/", $data["Hint"])) {
		return;
	}

	donotdisturb_sync($extension, $data["Status"]);
}

function donotdisturb_sync($extension, $status) {
	$ep_user = endpoint_api(array('ext' => $extension));

	if ($ep_user != NULL) {
		$brand = $ep_user["brand"];
	}

	switch ($brand) {
		case "Aastra":
		case "Yealink":
			aastra_toggle_application($extension, "DND", $status);
			break;
	}
}

function donotdisturb_main_get_cb($page, $user, $args) {
	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("donotdisturb/users/" . $user, "GET", NULL));

	if (!$data) {
		return NULL;
	}

	$screen = new Screen($page);

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "DND is not authorized in the API module for extension " . $user . ".";
			break;
		case 404:
			$error["display"] = "DND is not enabled for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$screen->entry_add(array(
		"type" => "bool",
		"name" => "status",
		"display" => "Enabled",
		"value" => ($data->status ? true : false)
	));

	return json_encode($screen);
}

function donotdisturb_main_set_cb($page, $user, $args, $body) {
	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("donotdisturb/users/" . $user, "GET", NULL), true);

	$body = json_decode($body, true);
	if (isset($body["status"])) {
		$body["status"] = ($body["status"] ? "YES" : "");
	}

	/* Eventually, we should change this to only merge acceptable values. */
	/* But, maybe the REST API will be changed to fix that, instead. */
	$body = array_merge($data, $body);

	$data = json_decode($rest->execute("donotdisturb/users/" . $user, "PUT", json_encode($body)));

	return NULL;
}

?>
