<?php

$application_name = "endpoint";
$application_display = "Endpoint Manager";

$application = new Application(
	$application_name, $application_display, array(
		new Page("main"),
		new Page("login"),
		new Page("logout")
	)
);

function endpoint_main_get_cb($page, $user, $args) {
	$action = $args["action"];
	$actiondata = $args["actiondata"];

	switch ($action) {
	case "config":
		$screen = new Screen($page);
		$screen->type = "action";

		$screen->entry_add(array(
			"type" => "config",
			"value" => $actiondata
		));

		return json_encode($screen);
		break;
	}

	if ($user == "0000") {
		/* Dummy user, just show the login page. */
		return endpoint_login_get_cb($page, $user, $args);
	}

	$screen = new Screen($page);

	$screen->entry_add(array(
		"type" => "link",
		"value" => "endpoint/login",
		"display" => "Login"
	));

	$screen->entry_add(array(
		"type" => "link",
		"value" => "endpoint/logout",
		"display" => "Logout"
	));

	return json_encode($screen);
}

function endpoint_login_get_cb($page, $user, $args) {
	global $ctrl;

	$ua = $GLOBALS["user_agent"];

	$action = $args["action"];
	$actiondata = $args["actiondata"];

	switch ($action) {
	case "config":
		$screen = new Screen($page);
		$screen->type = "action";

		$screen->entry_add(array(
			"type" => "config",
			"value" => $actiondata
		));

		return json_encode($screen);
		break;
	}

	if (!isset($args["app_extension"]) || !isset($args["app_password"])) {
		$screen = new Screen($page);
		$screen->type = "auth";

		$screen->entry_add(array(
			"name" => "app_extension",
			"value" => (isset($args["app_extension"]) ? $args["app_extension"] : ""),
			"display" => "Extension"
		));

		$screen->entry_add(array(
			"name" => "app_password",
			"value" => (isset($args["app_password"]) ? $args["app_password"] : ""),
			"display" => "Password"
		));

		return json_encode($screen);
	}

	$settings = $ctrl->get_settings();

	$extension = $args["app_extension"];
	$password = $args["app_password"];

	$screen = new Screen($page);

	$globals = endpoint_getGlobalSettings();
	$pass_admin = $globals['admin_password'];
	$pass_novm = $globals['user_password'];

	$match = false;
	if ($pass_admin == $password) {
		$match = true;
	}

	if (!$match) {
		/* Do we need to handle admin overrides? */
		if ($settings['endpoint']['admin_override'] == '1') {
			$extensions_used = endpoint_getExtensionAssignments();
			if (isset($extensions_used[$extension]) && $extensions_used[$extension]["mac"] != "") {
				$need_override = true;
			}
		}

		$rest = new RESTClient($extension, NULL, NULL);
		$data = json_decode($rest->execute("voicemail/mailboxes/" . $extension, "GET", NULL));

		if ($data) {
			if (isset($data->error)) {
				$error = array();

				$error["reason"] = $data->error;
				switch ($error["reason"]) {
				case 401:
					$error["display"] = "Voicemail is not authorized in the API module for extension " . $extension . ".";
					break;
				}

				$screen->error_add($error);

				return json_encode($screen);
			}

			$pass_vm = $data->pwd;
		}

		if ($pass_vm != NULL && $pass_vm != "") {
			/* Voicemail is enabled and a password is set.  Check against that. */
			if ($pass_vm == $password) {
				$match = true;
			}
		} else {
			/* No voicemail.  Check against user password. */
			if ($pass_novm == $password) {
				$match = true;
			}
		}

		if ($need_override) {
			if ($match) {
				$screen->error_add(array(
					"reason" => "Override required",
					"display" => "Admin override is required for this extension.  Please enter administrator password instead."
				));

				return json_encode($screen);
			} else if ($pass_admin == $password) {
				$match = true;
			}
		}
	}

	if (!$match) {
		$screen->error_add(array(
			"reason" => "Invalid credentials",
			"display" => "Extension or password is invalid."
		));

		return json_encode($screen);
	}

	if ($user != "0000") {
		endpoint_logout($user, $ua["mac"], $ua["brand"], $ua["model"]);
	}

	$ret = endpoint_login($extension, $ua["mac"], $ua["brand"], $ua["model"], '1');

	if ($ret == "success") {
		$screen = new Screen($page);
		$screen->type = "action";

		$screen->entry_add(array(
		       	"type" => "config"
		));

		return json_encode($screen);
	} else {
		$screen = new Screen($page);

		$screen->error_add(array(
			"reason" => "Login failed",
			"display" => "Could not login to extension " . $extension . "."
		));

		return json_encode($screen);
	}
}

function endpoint_logout_get_cb($page, $user, $args) {
	global $ctrl;

	$ua = $GLOBALS["user_agent"];

	$action = $args["action"];
	$actiondata = $args["actiondata"];

	switch ($action) {
	case "config":
		$screen = new Screen($page);
		$screen->type = "action";

		$screen->entry_add(array(
			"type" => "config",
			"value" => $actiondata
		));

		return json_encode($screen);
		break;
	}

	$settings = $ctrl->get_settings();

	if ($settings['endpoint']['logout_password'] == '1') {
		if (!isset($args["app_password"])) {
			$screen = new Screen($page);
			$screen->type = "auth";

			$screen->entry_add(array(
				"name" => "app_password",
				"value" => "",
				"display" => "Password"
			));

			return json_encode($screen);
		}

		$password = $args["app_password"];

		$screen = new Screen($page);

		$globals = endpoint_getGlobalSettings();
		$pass_admin = $globals['admin_password'];
		$pass_novm = $globals['user_password'];

		$match = false;
		if ($pass_admin == $password) {
			$match = true;
		}

		if (!$match) {
			$rest = new RESTClient($user, NULL, NULL);
			$data = json_decode($rest->execute("voicemail/mailboxes/" . $user, "GET", NULL));

			if ($data) {
				if (isset($data->error)) {
					$error = array();

					$error["reason"] = $data->error;
					switch ($error["reason"]) {
					case 401:
						$error["display"] = "Voicemail is not authorized in the API module for extension " . $user . ".";
						break;
					}

					$screen->error_add($error);

					return json_encode($screen);
				}

				$pass_vm = $data->pwd;
			}

			if ($pass_vm != NULL && $pass_vm != "") {
				/* Voicemail is enabled and a password is set.  Check against that. */
				if ($pass_vm == $password) {
					$match = true;
				}
			} else {
				/* No voicemail.  Check against user password. */
				if ($pass_novm == $password) {
					$match = true;
				}
			}
		}

		if (!$match) {
			$screen->error_add(array(
				"reason" => "Invalid credentials",
				"display" => "Password is invalid."
			));

			return json_encode($screen);
		}
	}

	$ret = endpoint_logout($user, $ua["mac"], $ua["brand"], $ua["model"]);

	if ($ret == "success" || $ret == "Not mapped") {
		$screen = new Screen($page);
		$screen->type = "action";

		$screen->entry_add(array(
       			"type" => "config"
		));

		return json_encode($screen);
	} else {
		$screen = new Screen($page);

		$screen->error_add(array(
			"reason" => "Logout failed",
			"display" => "Could not logout of extension " . $user . "."
		));

		return json_encode($screen);
	}
}

?>
