<?php

$application_name = "findmefollow";
$application_display = "Follow Me";

$application = new Application(
	$application_name, $application_display, array(
		new Page("main")
	)
);

function findmefollow_daemon_init() {
	global $astman;

	$astman->add_event_handler("extensionstatus", "findmefollow_event_extensionstatus");
}

function findmefollow_event_extensionstatus($event, $data, $server, $port) {
	if (!preg_match("/^Custom:FOLLOWME(.*)$/", $data["Hint"], $matches)) {
		return;
	}

	$extension = $matches[1];

	dbug('Follow Me syncing: ' . $extension . ' (' . $data["Status"] . ')');
	findmefollow_sync($extension, $data["Status"]);
}

function findmefollow_sync_cb($extension) {
	global $astman;

	$fcc = new featurecode('findmefollow', 'fmf_toggle');
	$fmf_code = $fcc->getCodeActive();

	if ($fmf_code == '') {
		return;
	}

	$data = $astman->send_request("ExtensionState", array("Exten" => $fmf_code . $extension, "Context" => "ext-findmefollow"));

	if (!preg_match("/^Custom:FOLLOWME(.*)$/", $data["Hint"])) {
		return;
	}

	findmefollow_sync($extension, $data["Status"]);
}

function findmefollow_sync($extension, $status) {
	$ep_user = endpoint_api(array('ext' => $extension));

	if ($ep_user != NULL) {
		$brand = $ep_user["brand"];
	}

	switch ($brand) {
		case "Aastra":
		case "Yealink":
			aastra_toggle_application($extension, "Follow Me", $status);
			break;
	}
}

function findmefollow_main_get_cb($page, $user, $args) {
	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("findmefollow/users/" . $user, "GET", NULL));

	if (!$data) {
		return NULL;
	}

	$screen = new Screen($page);

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Follow Me is not authorized in the API module for extension " . $user . ".";
			break;
		case 404:
			$error["display"] = "Follow Me is not enabled for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$screen->entry_add(array(
		"type" => "bool",
		"name" => "ddial",
		"display" => "Enabled",
		"value" => (!$data->ddial ? true : false)
	));

	if (strlen($data->grplist) > 0) {
		$grplist = explode("-", $data->grplist);
	} else {
		$grplist = NULL;
	}
	$screen->entry_add(array(
		"type" => "list",
		"subtype" => "number",
		"name" => "grplist",
		"display" => "Group List",
		"value" => $grplist
	));

	$screen->entry_add(array(
		"type" => "number",
		"min" => "0",
		"max" => "60",
		"name" => "pre_ring",
		"display" => "Initial Ring Time",
		"value" => $data->pre_ring
	));

	$screen->entry_add(array(
		"type" => "number",
		"min" => "1",
		"max" => "60",
		"name" => "grptime",
		"display" => "Ring Time",
		"value" => $data->grptime
	));

	$screen->entry_add(array(
		"type" => "bool",
		"name" => "needsconf",
		"display" => "Confirm Calls",
		"value" => ($data->needsconf ? true : false)
	));

	return json_encode($screen);
}

function findmefollow_main_set_cb($page, $user, $args, $body) {
	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("findmefollow/users/" . $user, "GET", NULL), true);

	$body = json_decode($body, true);
	if (isset($body["grplist"])) {
		$body["grplist"] = implode("-", $body["grplist"]);
	}
	if (isset($body["ddial"])) {
		$body["ddial"] = (!$body["ddial"] ? "CHECKED" : "");
	}
	if (isset($body["needsconf"])) {
		$body["needsconf"] = ($body["needsconf"] ? "CHECKED" : "");
	}
	if (isset($body["pre_ring"])) {
		if ($body["pre_ring"] == '') {
			unset($body["pre_ring"]);
		} else if ($body["pre_ring"] > 60) {
			$body["pre_ring"] = 60;
		} else if ($body["pre_ring"] < 0) {
			$body["pre_ring"] = 0;
		}
	}
	if (isset($body["grptime"])) {
		if ($body["grptime"] == '') {
			unset($body["grptime"]);
		} else if ($body["grptime"] > 60) {
			$body["grptime"] = 60;
		} else if ($body["grptime"] < 1) {
			$body["grptime"] = 1;
		}
	}

	$body = array_merge($data, $body);

	$data = json_decode($rest->execute("findmefollow/users/" . $user, "PUT", json_encode($body)));

	return NULL;
}

?>
