<?php

$application_name = "parking";
$application_display = "Parking";

$application = new Application(
	$application_name, $application_display, array(
		new Page("main")
	)
);

$parking_parkedcalls = array();
$parking_lots_status = array();

function parking_daemon_init() {
	global $astman;

	$astman->add_event_handler("extensionstatus", "parking_event_extensionstatus");
}

function parking_event_extensionstatus($event, $data, $server, $port) {
	global $rest;
	global $parking_lots_status;

	if ($data["Context"] != "park-hints") {
		return;
	}

	$fcc = new featurecode('parking', 'parkedcall');
	$parking_code = $fcc->getCodeActive();

	if ($parking_code == '') {
		return;
	}

	if (preg_match("/^" . preg_quote($parking_code) . "(.*)$/", $data["Exten"])) {
		/* Ignore the feature code hints */
		return;
	}

	$parked_slot = $data["Exten"];

	$lots = parking_parkpro_get_lots($rest);

	if ($lots == NULL) {
		$lot = parking_parking_get_lot($rest);

		if ($lot == NULL) {
			return;
		}

		$lots = array();
		$lots['public'] = array();
		$lots['private'] = array();

		$lots['public'][] = $lot;
	}
	dbug('Parking lots', $lots);

	$extensions = endpoint_apiApp("REST-Parking");
	foreach($extensions as $extension=>$button) {
		$user_lots = parking_user_lots($lots, $extension);

		$found = 0;
		$status = 0;
		foreach($user_lots as $user_lot) {
			if ($parked_slot >= $user_lot->parkpos && $parked_slot < ($user_lot->parkpos + $user_lot->numslots)) {
				/* The user cares about this lot. */
				$found = 1;

				/* Store the status. */
				$parking_lots_status[$user_lot->parkext] = $data["Status"];
			}

			if (isset($parking_lots_status[$user_lot->parkext]) && $parking_lots_status[$user_lot->parkext]) {
				$status = 1;
			}
		}

		if ($found) {
			global $amp_conf;
			global $astman;

			if ($data["Status"]) {
				dbug('Parking syncing call parked: ' . $extension . ' (' . $data["Status"] . ')');
				parking_sync($extension, $status, true, "Call parked: " . $parked_slot, array($button));
			} else {
				dbug('Parking syncing: ' . $extension . ' (' . $data["Status"] . ')');
				parking_sync($extension, $status, false, NULL, array($button));
			}

			if ($amp_conf['USEDEVSTATE']) {
				$astman->set_global($amp_conf['AST_FUNC_DEVICE_STATE'] . "(Custom:PARK" . $extension . ")", ($status ? 'BUSY' : 'NOT_INUSE'));
			}
		}
	}
}

function parking_sync_cb($extension) {
	global $astman;
	global $rest;

	$fcc = new featurecode('parking', 'parkedcall');
	$parking_code = $fcc->getCodeActive();

	if ($parking_code == '') {
		return;
	}

	$lots = parking_parkpro_get_lots($rest, $extension);

	if ($lots == NULL) {
		$lot = parking_parking_get_lot($rest);

		if ($lot == NULL) {
			return;
		}

		$lots = array();
		$lots['public'] = array();
		$lots['private'] = array();

		$lots['public'][] = $lot;
	}
	$lots = parking_user_lots($lots, $extension);

	$status = 0;
	foreach($lots as $lot) {
		$data = $astman->send_request("ExtensionState", array("Exten" => $parking_code . $lot->parkext, "Context" => "park-hints"));

		if (!preg_match("/^" . preg_quote($parking_code) . "(.*)$/", $data["Exten"], $matches)) {
			continue;
		}

		/* There is at least one set, so move on. */
		if ($data["Status"]) {
			$status = 1;
			break;
		}
	}

	parking_sync($extension, $status);
}

function parking_sync($extension, $status, $alert = false, $message = NULL, $buttons) {
	$ep_user = endpoint_api(array('ext' => $extension));

	if ($ep_user != NULL) {
		$brand = $ep_user["brand"];
	}

	switch ($brand) {
		case "Aastra":
		case "Yealink":
			aastra_toggle_application($extension, "Parking", $status, $alert, $message, $buttons);
			break;
	}
}

function parking_event_parkedcall($event, $data, $server, $port) {
	global $parking_parkedcalls;

	if (!isset($data["ActionID"])) {
		/* No ActionID means it isn't a response. */
		return;
	}

	$parkedcall = array();
	$parkedcall["ParkingLot"] = $data["Parkinglot"];
	$parkedcall["Exten"] = $data["Exten"];
	$parkedcall["CallerIDNum"] = $data["CallerIDNum"];
	$parkedcall["CallerIDName"] = $data["CallerIDName"];
	$parkedcall["Duration"] = $data["Duration"];

	$parking_parkedcalls[] = $parkedcall;
}

function parking_event_parkedcallscomplete($event, $data, $server, $port) {
	global $astman;

	/* HACK: Force a timeout after we get this event, so that the wait_response() returns. */
	stream_set_timeout($astman->socket, 0, 1);
}

function parking_parkpro_get_user_lot($rest, $user) {
	/* Check that Park Pro module is enabled */
	$data = json_decode($rest->execute("parkpro", "GET", NULL));

	if ($data != NULL) {
		return NULL;
	}

	$data = json_decode($rest->execute("parkpro/users/" . $user, "GET", NULL));

	if ($data == NULL) {
		return NULL;
	}

	if (isset($data->error)) {
		return NULL;
	}

	$user_lot = $data[0];

	$data = json_decode($rest->execute("parkpro/lots/" . $user_lot, "GET", NULL));

	if ($data == NULL) {
		return NULL;
	}

	$lot = new stdClass();
	$lot->id = $data->id;
	$lot->parkext = $data->parkext;
	$lot->parkpos = $data->parkpos;
	$lot->numslots = $data->numslots;
	$lot->type = $data->type;

	return $lot;
}

function parking_parkpro_get_lots($rest) {
	/* Check that Park Pro module is enabled */
	$data = json_decode($rest->execute("parkpro", "GET", NULL));

	if ($data != NULL) {
		return NULL;
	}

	$data = json_decode($rest->execute("parkpro/users", "GET", NULL));

	if ($data == NULL) {
		return NULL;
	}

	if (isset($data->error)) {
		switch ($data->error) {
		case 401:
			/* Park Pro is not available for user.  Fallback to Parking. */
			return NULL;
		case 404:
			/* User has no lot.  No big deal. */
			break;
		}
	} else {
		$user_lots = $data[0];
	}

	$data = json_decode($rest->execute("parkpro/lots", "GET", NULL));

	if ($data == NULL) {
		return NULL;
	}

	if (isset($data->error)) {
		return NULL;
	}

	$lots = array();
	$lots['public'] = array();
	$lots['private'] = array();

	foreach($data as $lot) {
		if ($lot->type == "public") {
			$lots['public'][] = $lot;
		} else {
			foreach($user_lots as $userid => $user_lot) {
				if ($user_lot == $lot->id) {
					$lots['private'][$userid] = array($lot);
				}
			}
		}
	}

	return $lots;
}

function parking_user_lots($lots, $user) {
	if (isset($lots['private'][$user])) {
		$user_lots = array_merge($lots['public'], $lots['private'][$user]);
	} else {
		$user_lots = $lots['public'];
	}

	return $user_lots;
}

function parking_parking_get_lot($rest) {
	$data = json_decode($rest->execute("parking", "GET", NULL));

	if ($data == NULL) {
		return NULL;
	}

	if (isset($data->error)) {
		return NULL;
	}

	$lot = new stdClass();
	$lot->id = $data->id;
	$lot->parkext = $data->parkext;
	$lot->parkpos = $data->parkpos;
	$lot->numslots = $data->numslots;
	$lot->type = $data->type;

	return $lot;
}

function parking_main_park($page, $user, $args) {
	$rest = new RESTClient($user, NULL, NULL);

	$lot = parking_parkpro_get_user_lot($rest, $user);
	if ($lot == NULL) {
		$lot = parking_parking_get_lot($rest);

		if ($lot == NULL) {
			$screen = new Screen($page);

			$error = array();

			$error["reason"] = 401;
			$error["display"] = "Parking is not authorized in the API module for extension " . $user . ".";

			$screen->error_add($error);

			return json_encode($screen);
		}
	}

	$fcc = new featurecode('core', 'blindxfer');
	$xfer_code = $fcc->getCodeActive();

	$screen = new Screen($page);
	$screen->type = 'action';

	$screen->entry_add(array(
		"type" => "transfer",
		"value" => $lot->parkext,
		"dtmf" => ($xfer_code ? $xfer_code : NULL)
	));

	return json_encode($screen);
}

function parking_main_display($page, $user, $args) {
	global $astman;
	global $parking_parkedcalls;

	$rest = new RESTClient($user, NULL, NULL);

	$lots = parking_parkpro_get_lots($rest);

	if ($lots == NULL) {
		$lot = parking_parking_get_lot($rest);

		if ($lot == NULL) {
			$screen = new Screen($page);

			$error = array();

			$error["reason"] = 401;
			$error["display"] = "Parking is not authorized in the API module for extension " . $user . ".";

			$screen->error_add($error);

			return json_encode($screen);
		}

		$lots = array();
		$lots['public'] = array();
		$lots['private'] = array();

		$lots['public'][] = $lot;
	}
	$lots = parking_user_lots($lots, $user);

	$astman->events("on");
	$astman->add_event_handler("parkedcall", "parking_event_parkedcall");
	$astman->add_event_handler("parkedcallscomplete", "parking_event_parkedcallscomplete");

	$parking_parkedcalls = array();

	$response = $astman->ParkedCalls('park' . $user);
	if ($response["Response"] == "Success") {
		$astman->wait_response(true);
		stream_set_timeout($astman->socket, 30);
	}

	$screen = new Screen($page);
	$screen->type = "directory";
	$screen->refresh = 5;

	$parkedcalls = 0;
	if (count($parking_parkedcalls) > 0) {
		foreach($parking_parkedcalls as $parkedcall) {
			$match = false;

			if ($parkedcall["ParkingLot"] == "default") {
				$match = true;
			} else {
				foreach($lots as $lot) {
					if (preg_match("/parkinglot_" . $lot->id . "/", $parkedcall["ParkingLot"], $matches)) {
						$match = true;
						break;
					}
				}
			}

			if (!$match) {
				continue;
			}

			$display = $parkedcall["Exten"] . ": ";
			$display.= ($parkedcall["CallerIDName"] ? $parkedcall["CallerIDName"] : $parkedcall["CallerID"]) . " ";
			$display.= "(" . $parkedcall["Duration"] . " sec" . ($parkedcall["Duration"] != "1" ? "s" : "") . ")";

			$screen->entry_add(array(
				"type" => "dial",
				"name" => $parkedcall["ParkingLot"] . $parkedcall["Exten"],
				"display" => $display,
				"value" => $parkedcall["Exten"]
			));

			$parkedcalls++;
		}
	}

	if ($parkedcalls == 0) {
		$screen->type = "display";
		$screen->entry_add(array(
			"type" => "entry",
			"display" => "No calls are parked",
		));
	}

	$screen->action_add(array(
		"name" => "refresh",
		"display" => "Refresh"
	));

	return json_encode($screen);
}

function parking_main_get_cb($page, $user, $args) {
	switch ($args['linestate']) {
	case 'CONNECTED':
		$screen = parking_main_park($page, $user, $args);
		break;
	default:
		$screen = parking_main_display($page, $user, $args);
		break;
	}

	return $screen;
}

?>
