<?php

$application_name = "presencestate";
$application_display = "Presence";

$application = new Application(
	$application_name, $application_display, array(
		new Page("main"),
		new Page("change"),
		new Page("prefs")
	)
);

$daemon_presences_current = NULL;

function presencestate_status_current($user) {
	global $amp_conf;
	global $astman;

	$current = array(
		"type" => "",
		"message" => ""
	);

	if ($amp_conf['AST_FUNC_PRESENCE_STATE']) {
		$ret = $astman->send_request('Getvar', array('Variable' => 'PRESENCE_STATE(CustomPresence:' . $user . ',value)'));
		$current["type"] = $ret['Value'];

		$ret = $astman->send_request('Getvar', array('Variable' => 'PRESENCE_STATE(CustomPresence:' . $user . ',message)'));
		$current["message"] = $ret['Value'];
	}

	return $current;
}

function presencestate_status_types($rest) {
	$data = json_decode($rest->execute("presencestate/types", "GET", NULL), true);

	return $data;
}

function presencestate_toggle_pref($rest, $user, $pref, $status) {
	switch ($pref) {
	case "dnd":
		$data = json_decode($rest->execute("donotdisturb/users/" . $user, "GET", NULL), true);

		$body = $data;
		$body["status"] = ($status ? "YES" : "");

		$data = json_decode($rest->execute("donotdisturb/users/" . $user, "PUT", json_encode($body)));
		return $data;
	case "fm":
		$data = json_decode($rest->execute("findmefollow/users/" . $user, "GET", NULL), true);

		$body = $data;
		$body["ddial"] = ($status ? "" : "CHECKED");

		$data = json_decode($rest->execute("findmefollow/users/" . $user, "PUT", json_encode($body)));
		return $data;
	}

	return NULL;
}

function presencestate_daemon_init() {
	global $astman;
	global $daemon_presences_current;

	$daemon_presences_current = array();

	$astman->add_event_handler("presencestatus", "presencestate_event_presencestatus");

	$response = $astman->send_request('Command', array('Command' => "database show CustomPresence"));
	$presences = explode("\n", $response["data"]);
	foreach ($presences as $presence) {
		if (preg_match("/^\/CustomPresence\/(\d*).*?:\s*(.*?)(,,.*?)?\s*$/", $presence, $matches)) {
			$status = trim($matches[2]);
			if (isset($matches[3])) {
				$message = ltrim($matches[3], ',');
			}

			$daemon_presences_current[$matches[1]] = array(
				"type" => $status,
				"message" => $message
			);
		}
	}
}

function presencestate_event_presencestatus($event, $data, $server, $port) {
	global $rest;
	global $daemon_presences_current;

	$hints = split(',', $data['Hint']);
	$presence = $hints[1];
	if (!$presence) {
		return;
	}
	$user = str_ireplace('CustomPresence:', '', $presence);

	$types = presencestate_status_types($rest);

	$current = array(
		"type" => $data["Status"],
		"message" => $data["Message"]
	);

	$message = $types[$current["type"]] . ($current["message"] ? ' (' . $current["message"] . ')' : '');
	presencestate_sync($user, $current["type"], $message);

	$data = json_decode($rest->execute("presencestate/list", "GET", NULL));

	if ($data == NULL) {
		return NULL;
	}

	if (isset($data->error)) {
		return NULL;
	}

	$presences = $data;

	$data = json_decode($rest->execute("presencestate/prefs/" . $user, "GET", NULL), true);

	if ($data == NULL) {
		return NULL;
	}

	if (isset($data->error)) {
		return NULL;
	}

	$prefs = $data[0];

	$previous = $daemon_presences_current[$user];

	foreach ($presences as $presence) {
		if ($presence->type == $current["type"] && $presence->message == $current["message"]) {
			presencestate_toggle_pref($rest, $user, $prefs[$presence->id], true);
		}

		if ($presence->type == $previous["type"] && $presence->message == $previous["message"]) {
			presencestate_toggle_pref($rest, $user, $prefs[$presence->id], false);
		}
	}

	$daemon_presences_current[$user] = $current;
}

function presencestate_sync_cb($user) {
	global $astman;

	$rest = new RESTClient($user, NULL, NULL);
	$types = presencestate_status_types($rest);

	$current = presencestate_status_current($user);

	$message = $types[$current["type"]] . ($current["message"] ? ' (' . $current["message"] . ')' : '');
	presencestate_sync($user, $current["type"], $message);
}

function presencestate_sync($user, $status, $message) {
	$status = $status != NULL && $status != "available";
	$ep_user = endpoint_api(array('ext' => $user));

	if ($ep_user != NULL) {
		$brand = $ep_user["brand"];
	}

	switch ($brand) {
		case "Aastra":
		case "Yealink":
			aastra_toggle_application($user, "Presence", $status, false, $message);
			break;
	}
}

function presencestate_main_get_cb($page, $user, $args) {
	$rest = new RESTClient($user, NULL, NULL);

	$types = presencestate_status_types($rest);

	if ($types == NULL) {
		return NULL;
	}

	if (isset($types->error)) {
		$screen = new Screen($page);

		$error = array();

		$error["reason"] = $types->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Presence State is not authorized in the API module for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$current = presencestate_status_current($user);

	$screen = new Screen($page);

	$screen->entry_add(array(
		"type" => "link",
		"name" => "current",
		"display" => ($current["type"] ? $types[$current["type"]] : "Unset") . ($current["message"] ? ' (' . $current["message"] . ')' : ''),
		"value" => "presencestate/change",
	));

	$screen->action_add(array(
		"name" => "prefs",
		"link" => "presencestate/prefs",
		"display" => "Prefs"
	));

	return json_encode($screen);
}

function presencestate_change_get_cb($page, $user, $args) {
	global $amp_conf;
	global $astman;

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("presencestate/list", "GET", NULL));

	if ($data == NULL) {
		return NULL;
	}

	if (isset($data->error)) {
		$screen = new Screen($page);

		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Presence State is not authorized in the API module for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$presences = $data;

	if (isset($args['app_presence']) && $amp_conf['AST_FUNC_PRESENCE_STATE']) {
		foreach ($presences as $presence) {
			if ($presence->id == $args['app_presence']) {
				$ret = $astman->set_global($amp_conf['AST_FUNC_PRESENCE_STATE'] . '(CustomPresence:' . $user . ')', '"' . $presence->type . ',,' . $presence->message . '"');
			}
		}
	}

	$types = presencestate_status_types($rest);

	$current = presencestate_status_current($user);

	$screen = new Screen($page);
	$screen->exitPath = "presencestate/main";

	foreach ($presences as $key => $presence) {
		$display = '';
		if ($presence->type == $current["type"] && $presence->message == $current["message"]) {
			$display = '*';
		}
		$display.= $types[$presence->type] . ($presence->message ? ' (' . $presence->message . ')' : '');

		$screen->entry_add(array(
			"type" => "link",
			"name" => $key,
			"display" => $display,
			"value" => "presencestate/change",
			"args" => "app_presence=" . $key
		));
	}

	return json_encode($screen);
}

function presencestate_prefs_get_cb($page, $user, $args) {
	global $amp_conf;
	global $astman;

	$values = array(
		"nothing" => "Nothing",
		"fm" => "Follow Me",
		"dnd" => "DND"
	);

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("presencestate/prefs/" . $user, "GET", NULL), true);

	if ($data == NULL) {
		return NULL;
	}

	if (isset($data->error)) {
		$screen = new Screen($page);

		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Presence State is not authorized in the API module for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$prefs = $data[0];

	$data = json_decode($rest->execute("presencestate/list", "GET", NULL));

	if ($data == NULL) {
		return NULL;
	}

	if (isset($data->error)) {
		$screen = new Screen($page);

		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Presence State is not authorized in the API module for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$presences = $data;

	$types = presencestate_status_types($rest);

	$screen = new Screen($page);
	$screen->title = "Preferences";
	$screen->exitPath = "presencestate/main";

	foreach ($presences as $key => $presence) {
		$display = $types[$presence->type] . ($presence->message ? ' (' . $presence->message . ')' : '');
		if ($prefs[$key] != "" && $prefs[$key] != "nothing") {
			$display.= ' [' . $values[$prefs[$key]] . ']';
		}

		$screen->entry_add(array(
			"type" => "choice",
			"name" => $key,
			"display" => $display,
			"value" => ($prefs[$key] ? $prefs[$key] : "nothing"),
			"values" => $values
		));
	}

	return json_encode($screen);
}

function presencestate_prefs_set_cb($page, $user, $args, $body) {
	$body = json_decode($body, true);

	$keys = array_keys($body);
	$key = $keys[0];
	$prefs = array(
		"id" => $key,
		"pref" => $body[$key]
	);

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("presencestate/prefs/" . $user, "PUT", json_encode($prefs)));

	return NULL;
}
?>
