<?php

$application_name = "queues";
$application_display = "Queues";

$application = new Application(
	$application_name, $application_display, array(
		new Page("main"),
		new Page("details"),
		new Page("callers"),
		new Page("members"),

		new Page("agent", "Queue Agent")
	)
);

$queues_details = array();

function queues_daemon_init() {
	global $astman;

	$astman->add_event_handler("extensionstatus", "queues_event_extensionstatus");
}

function queues_event_extensionstatus($event, $data, $server, $port) {
	if ($data["Context"] != "ext-queues") {
		return;
	}

	$fcc = new featurecode('queues', 'que_callers');
	$qc_code = $fcc->getCodeActive();
	unset($fcc);

	$fcc = new featurecode('queues', 'que_toggle');
	$q_code = $fcc->getCodeActive();
	unset($fcc);

	if ($qc_code == '' && $q_code == '') {
		return;
	}

	if (preg_match('/^' . preg_quote($qc_code . "*") . '(\d*)$/', $data["Exten"], $matches)) {
		$application = "Queues";
	} else if (preg_match('/^' . preg_quote($q_code . "*") . '(\d*)$/', $data["Exten"], $matches)) {
		$application = "Queue Agent";
	} else {
		return;
	}

	$extension = $matches[1];

	dbug($application . ' syncing: ' . $extension . ' (' . $data["Status"] . ')');
	queues_sync($extension, $application, $data["Status"]);
}

function queues_event_queueparams($event, $data, $server, $port) {
	global $queues_details;

	$queue = $data['Queue'];

	$queues_details[$queue] = array();
	$queues_details[$queue]['holdtime'] = $data['Holdtime'];
	$queues_details[$queue]['abandoned'] = $data['Abandoned'];
	$queues_details[$queue]['completed'] = $data['Completed'];
	$queues_details[$queue]['members'] = array();
	$queues_details[$queue]['callers'] = array();
}

function queues_event_queueentry($event, $data, $server, $port) {
	global $queues_details;

	$queue = $data['Queue'];
	$caller = $data['Position'];

	$queues_details[$queue]['callers'][$caller] = array();
	$queues_details[$queue]['callers'][$caller]['channel'] = $data['Channel'];
	$queues_details[$queue]['callers'][$caller]['number'] = $data['CallerIDNum'];
	$queues_details[$queue]['callers'][$caller]['name'] = $data['CallerIDName'];
}

function queues_event_queuemember($event, $data, $server, $port) {
	global $queues_details;

	$queue = $data['Queue'];

	if (preg_match("/^.*?\/([\d]+).*$/", $data['Location'], $matches)) {
		$member = $matches[1];

		$queues_details[$queue]['members'][$member] = array();
		$queues_details[$queue]['members'][$member]['name'] = $data['Name'];
		$queues_details[$queue]['members'][$member]['location'] = $data['Location'];
		$queues_details[$queue]['members'][$member]['dynamic'] = ($data['Membership'] == 'dynamic' ? true : false);
		$queues_details[$queue]['members'][$member]['paused'] = ($data['Paused'] == '1' ? true : false);
	}
}

function queues_event_queuestatuscomplete($event, $data, $server, $port) {
	global $astman;

	/* HACK: Force a timeout after we get this event, so that the wait_response() returns. */
	stream_set_timeout($astman->socket, 0, 1);
}

function queues_sync_cb($extension) {
	global $astman;

	$fcc = new featurecode('queues', 'que_toggle');
	$q_code = $fcc->getCodeActive();

	if ($q_code == '') {
		return;
	}

	$data = $astman->send_request("ExtensionState", array("Exten" => $q_code . '*' . $extension, "Context" => "ext-queues"));

	if (!preg_match("/^Custom:QUEUE(.*)$/", $data["Hint"])) {
		return;
	}

	queues_sync($extension, "Queues", $data["Status"]);
}

function queues_agent_sync_cb($extension) {
	global $astman;

	$fcc = new featurecode('queues', 'que_callers');
	$qc_code = $fcc->getCodeActive();

	if ($qc_code == '') {
		return;
	}

	$data = $astman->send_request("ExtensionState", array("Exten" => $qc_code . '*' . $extension, "Context" => "ext-queues"));

	if (!preg_match("/^Queue:(.*)$/", $data["Hint"])) {
		return;
	}

	queues_sync($extension, "Queue Agent", $data["Status"]);
}

function queues_sync($extension, $application, $status) {
	$ep_user = endpoint_api(array('ext' => $extension));

	if ($ep_user != NULL) {
		$brand = $ep_user["brand"];
	}

	switch ($brand) {
		case "Aastra":
		case "Yealink":
			aastra_toggle_application($extension, $application, $status);
			break;
	}
}

function queues_member_login($queue, $user, $state) {
	global $astman;
	global $amp_conf;

	$interface = "Local/" . $user . "@from-queue/n";

	if ($state) {
		$penalty = $astman->database_get("QPENALTY", $queue . "/agents/" . $user);
		$cidname = $astman->database_get('AMPUSER', $user . '/cidname');

		$ret = $astman->send_request("QueueAdd", array(
			"Queue" => $queue,
			"Interface" => $interface,
			"Penalty" => $penalty,
			"MemberName" => $cidname,
			"StateInterface" => "hint:" . $user . "@ext-local"
		));
	} else {
		$ret = $astman->send_request("QueueRemove", array(
			"Queue" => $queue,
			"Interface" => $interface
		));
	}

	$devices = $astman->database_get("AMPUSER", $user . "/device");

	$device_arr = explode('&', $devices);
	foreach ($device_arr as $device) {
		$ret = $astman->set_global($amp_conf['AST_FUNC_DEVICE_STATE'] . "(Custom:QUEUE" . $device . "*" . $queue . ")", ($state ? 'INUSE' : 'NOT_INUSE'));
	}
}

function queues_member_pause($queue, $user, $state) {
	global $astman;

	$astman->set_global("QUEUE_MEMBER(" . $queue . ",paused,Local/" . $user . "@from-queue/n)", ($state ? 1 : 0));
}

function queues_main_get_cb($page, $user, $args) {
	global $astman;

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("queues/", "GET", NULL));

	if (!$data) {
		return NULL;
	}

	$screen = new Screen($page);
	$screen->refresh = 30;

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Queues is not authorized in the API module for extension " . $user . ".";
			break;
		case 404:
			$error["display"] = "Queues is not enabled for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$queues = $data;

	$queue_count = 0;
	foreach ($queues as $queue) {
		$queue_count++;

		$screen->entry_add(array(
			"type" => "entry",
			"display" => $queue->name . " (" . $queue->extension . ")",
			"value" => $queue->extension,
		));
	}

	if ($queue_count == 0) {
		$screen->type = "display";
		$screen->entry_add(array(
			"type" => "entry",
			"display" => "No queues configured for user " . $user,
		));
	} else {
		$screen->action_add(array(
			"name" => "details",
			"link" => "queues/details",
			"display" => "Details"
		));

		$screen->action_add(array(
			"name" => "members",
			"link" => "queues/members",
			"display" => "Members"
		));

		$screen->action_add(array(
			"name" => "callers",
			"link" => "queues/callers",
			"display" => "Callers"
		));
	}

	return json_encode($screen);
}

function queues_details_get_cb($page, $user, $args) {
	global $astman;
	global $queues_details;

	$action = $args["action"];
	$actiondata = $args["actiondata"];

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("queues/" . $actiondata, "GET", NULL));

	if (!$data) {
		$screen->type = "display";
		$screen->entry_add(array(
			"type" => "entry",
			"display" => "No queues configured for user " . $user,
		));

		return json_encode($screen);
	}

	$screen = new Screen($page);
	$screen->exitPath = "queues/main";

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Queues is not authorized in the API module for extension " . $user . ".";
			break;
		case 404:
			$error["display"] = "Queues is not enabled for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$queue = $data;

	$screen->title = $queue->name . " (" . $queue->extension . ")";

	$astman->events("on");
	$astman->add_event_handler("queueparams", "queues_event_queueparams");
	$astman->add_event_handler("queueentry", "queues_event_queueentry");
	$astman->add_event_handler("queuemember", "queues_event_queuemember");
	$astman->add_event_handler("queuestatuscomplete", "queues_event_queuestatuscomplete");

	$response = $astman->send_request("QueueStatus", array("Member" => $extension));
	if ($response["Response"] == "Success") {
		$astman->wait_response(true);
		stream_set_timeout($astman->socket, 30);
	}

	$queue_detail = $queues_details[$queue->extension];

	$screen->entry_add(array(
		"type" => "entry",
		"display" => "Callers: " . (!$queue_detail ? "unknown" : count($queue_detail['callers'])),
		"value" => $queue->extension
	));

	$screen->entry_add(array(
		"type" => "entry",
		"display" => "Members: " . (!$queue_detail ? "unknown" : count($queue_detail['members'])),
		"value" => $queue->extension
	));

	$screen->entry_add(array(
		"type" => "entry",
		"display" => "Hold Time: " . (!$queue_detail ? "unknown" : $queue_detail['holdtime']),
		"value" => $queue->extension
	));

	$screen->entry_add(array(
		"type" => "entry",
		"display" => "Completed: " . (!$queue_detail ? "unknown" : $queue_detail['completed']),
		"value" => $queue->extension
	));

	$screen->entry_add(array(
		"type" => "entry",
		"display" => "Abandoned: " . (!$queue_detail ? "unknown" : $queue_detail['abandoned']),
		"value" => $queue->extension
	));

	$screen->action_add(array(
		"name" => "callers",
		"link" => "queues/callers",
		"display" => "Callers"
	));

	$screen->action_add(array(
		"name" => "members",
		"link" => "queues/members",
		"display" => "Members"
	));

	return json_encode($screen);
}

function queues_callers_get_cb($page, $user, $args) {
	global $astman;
	global $queues_details;

	$action = $args["action"];
	$actiondata = $args["actiondata"];

	if ($action == "pickup") {
		$channel = $args['app_channel'];

		$res = $astman->send_request('Redirect', array(
			'Channel' => $channel,
			'Exten' => $user,
			'Context' => 'default',
			'Priority' => '1'
		));
	}

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("queues/" . $actiondata, "GET", NULL));

	if (!$data) {
		$screen->type = "display";
		$screen->entry_add(array(
			"type" => "entry",
			"display" => "No queues configured for user " . $user,
		));

		return json_encode($screen);
	}

	$screen = new Screen($page);
	$screen->exitPath = "queues/main";

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Queues is not authorized in the API module for extension " . $user . ".";
			break;
		case 404:
			$error["display"] = "Queues is not enabled for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$queue = $data;

	$screen->title = $queue->name . " (" . $queue->extension . ")";

	$astman->events("on");
	$astman->add_event_handler("queueparams", "queues_event_queueparams");
	$astman->add_event_handler("queueentry", "queues_event_queueentry");
	$astman->add_event_handler("queuestatuscomplete", "queues_event_queuestatuscomplete");

	$response = $astman->send_request("QueueStatus", array("Member" => $extension));
	if ($response["Response"] == "Success") {
		$astman->wait_response(true);
		stream_set_timeout($astman->socket, 30);
	}

	$queue_detail = $queues_details[$queue->extension];

	$caller_count = 0;
	foreach ($queue_detail['callers'] as $caller) {
		$caller_count++;
		$screen->entry_add(array(
			"type" => "entry",
			"display" => $caller['name'] . " (" . $caller['number'] . ")",
			"value" => $queue->extension,
			"args" => "app_channel=" . $caller['channel'],
		));
	}

	if ($caller_count == 0) {
		$screen->type = "display";
		$screen->entry_add(array(
			"type" => "entry",
			"display" => "No callers in queue " . $queue->name . " (" . $queue->extension . ")",
			"value" => $queue->extension
		));
	} else {
		$screen->action_add(array(
			"name" => "pickup",
			"display" => "Pickup",
		));
	}

	$screen->action_add(array(
		"name" => "details",
		"link" => "queues/details",
		"display" => "Details",
	));

	$screen->action_add(array(
		"name" => "members",
		"link" => "queues/members",
		"display" => "Members"
	));

	return json_encode($screen);
}

function queues_members_get_cb($page, $user, $args) {
	global $astman;
	global $queues_details;

	$action = $args["action"];
	$actiondata = $args["actiondata"];

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("queues/" . $actiondata, "GET", NULL));

	if (!$data) {
		$screen->type = "display";
		$screen->entry_add(array(
			"type" => "entry",
			"display" => "No queues configured for user " . $user,
		));

		return json_encode($screen);
	}

	$screen = new Screen($page);
	$screen->exitPath = "queues/main";

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Queues is not authorized in the API module for extension " . $user . ".";
			break;
		case 404:
			$error["display"] = "Queues is not enabled for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$queue = $data;

	$screen->title = $queue->name . " (" . $queue->extension . ")";

	$astman->events("on");
	$astman->add_event_handler("queueparams", "queues_event_queueparams");
	$astman->add_event_handler("queuemember", "queues_event_queuemember");
	$astman->add_event_handler("queuestatuscomplete", "queues_event_queuestatuscomplete");

	$response = $astman->send_request("QueueStatus", array("Member" => $extension));
	if ($response["Response"] == "Success") {
		$astman->wait_response(true);
		stream_set_timeout($astman->socket, 30);
	}

	$queue_detail = $queues_details[$queue->extension];

	$member_count = 0;
	foreach ($queue_detail['members'] as $extension => $member) {
		$member_count++;

		$loggedin = true;
		$static = !$member['dynamic'];
		$paused = $member['paused'];

		if (isset($args['app_member']) && $args['app_member'] == $extension) {
			switch ($action) {
			case "login":
				if ($member['dynamic']) {
					$loggedin = false;
					queues_member_login($queue->extension, $extension, false);
				}
				break;
			case "pause":
				if (!$paused) {
					$paused = true;
					queues_member_pause($queue->extension, $extension, true);
				} else if ($paused) {
					$paused = false;
					queues_member_pause($queue->extension, $extension, false);
				}
				break;
			}
		}

		if ($loggedin) {
			if ($paused) {
				$icon = "paused";
				$iconstring = "(P)";
			} else {
				if ($static) {
					$icon = "static";
					$iconstring = "(X)";
				} else {
					$icon = "loggedin";
					$iconstring = "(+)";
				}
			}

			$cidname = $astman->database_get('AMPUSER', $extension . '/cidname');

			$screen->entry_add(array(
				"type" => "entry",
				"display" => ($cidname ? $cidname . "	(" . $extension . ")" : $extension),
				"value" => $queue->extension,
				"args" => "app_member=". $extension,
				"icon" => $icon,
				"iconstring" => $iconstring
			));
		}
	}

	if ($member_count == 0) {
		$screen->type = "display";
		$screen->entry_add(array(
			"type" => "entry",
			"display" => "No members in queue " . $queue->name . " (" . $queue->extension . ")",
			"value" => $queue->extension
		));
	} else {
		$screen->action_add(array(
			"name" => "login",
			"display" => "Logout"
		));

		$screen->action_add(array(
			"name" => "pause",
			"display" => "(Un)Pause"
		));
	}

	$screen->action_add(array(
		"name" => "details",
		"link" => "queues/details",
		"display" => "Details",
	));

	$screen->action_add(array(
		"name" => "callers",
		"link" => "queues/callers",
		"display" => "Callers"
	));

	return json_encode($screen);
}

function queues_agent_get_cb($page, $user, $args) {
	global $astman;
	global $queues_details;

	$action = $args["action"];
	$actiondata = $args["actiondata"];

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("queues/", "GET", NULL));

	if (!$data) {
		return NULL;
	}

	$screen = new Screen($page);
	$screen->refresh = 30;

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Queues is not authorized in the API module for extension " . $user . ".";
			break;
		case 404:
			$error["display"] = "Queues is not enabled for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$queues = $data;

	$astman->events("on");
	$astman->add_event_handler("queueparams", "queues_event_queueparams");
	$astman->add_event_handler("queuemember", "queues_event_queuemember");
	$astman->add_event_handler("queuestatuscomplete", "queues_event_queuestatuscomplete");

	$response = $astman->send_request("QueueStatus", array("Member" => $extension));
	if ($response["Response"] == "Success") {
		$astman->wait_response(true);
		stream_set_timeout($astman->socket, 30);
	}

	$queue_data = array();
	foreach ($queues as $queue) {
		$data = json_decode($rest->execute("queues/" . $queue->extension, "GET", NULL));

		if (!$data || isset($data->error)) {
			$queue_data[$queue->extension] = NULL;
			continue;
		}

		if (!in_array($user, $data->member) && !in_array($user, $data->dynmembers)) {
			$queue_data[$queue->extension] = NULL;
			continue;
		}

		$queue_data[$queue->extension] = $data;

		if ($actiondata == 'all_queues') {
			$member = NULL;
			if (isset($queues_details[$queue->extension])) {
				$details = $queues_details[$queue->extension];

				if (isset($details['members'][$user])) {
					$member = $details['members'][$user];
				}
			}

			if ($member) {
				/* If logged in to any queues, the overall state is logged in. */
				if ($member['dynamic']) {
					$all_state = true;
				}

				/* If paused in any queues, the overall state is paused. */
				if ($member['paused']) {
					$all_paused = true;
				}
			}
		}
	}

	$queue_count = 0;
	foreach ($queues as $queue) {
		if ($queue_data[$queue->extension] == NULL) {
			continue;
		}

		$queue_count++;

		$member = NULL;
		if (isset($queues_details[$queue->extension])) {
			$details = $queues_details[$queue->extension];

			if (isset($details['members'][$user])) {
				$member = $details['members'][$user];
			}
		}

		$loggedin = ($member ? true : false);
		$static = $member && !$member['dynamic'];
		$paused = $member && $member['paused'];

		switch ($action) {
		case "login":
			if (!$loggedin) {
				if ($actiondata == $queue->extension || ($actiondata == 'all_queues' && !$all_state)) {
					/* Login */
					$loggedin = true;
					queues_member_login($queue->extension, $user, true);
				}
			} else if ($loggedin && !$static) {
				if ($actiondata == $queue->extension || $actiondata == 'all_queues') {
					/* Logout  */
					$loggedin = false;
					queues_member_login($queue->extension, $user, false);
				}
			}
			break;
		case "pause":
			if ($loggedin && !$paused) {
				if ($actiondata == $queue->extension || ($actiondata == 'all_queues' && !$all_paused)) {
					/* Pause */
					$paused = true;
					queues_member_pause($queue->extension, $user, true);
				}
			} else if ($loggedin && $paused) {
				if ($actiondata == $queue->extension || $actiondata == 'all_queues') {
					/* Unpause */
					$paused = false;
					queues_member_pause($queue->extension, $user, false);
				}
			}
			break;
		}

		if ($loggedin) {
			if ($paused) {
				$icon = "paused";
				$iconstring = "(P)";
			} else {
				if ($static) {
					$icon = "static";
					$iconstring = "(X)";
				} else {
					$icon = "loggedin";
					$iconstring = "(+)";
				}
			}
		} else {
			$icon = "loggedout";
			$iconstring = "(-)";
		}

		$screen->entry_add(array(
			"type" => "entry",
			"display" => $queue->name . " (" . $queue->extension . ")",
			"value" => $queue->extension,
			"icon" => $icon,
			"iconstring" => $iconstring
		));
	}

	if ($queue_count == 0) {
		$screen->type = "display";
		$screen->entry_add(array(
			"type" => "entry",
			"display" => "No queues configured for user " . $user,
		));
	} else {
		if ($queue_count > 1) {
			$screen->entry_add(array(
				"type" => "entry",
				"display" => "All Queues",
				"value" => "all_queues"
			));
		}

		$screen->action_add(array(
			"name" => "login",
			"display" => "Login/out"
		));

		$screen->action_add(array(
			"name" => "pause",
			"display" => "(Un)Pause"
		));
	}

	return json_encode($screen);
}

?>
