<?php

$application_name = "timeconditions";
$application_display = "Time Conditions";

$application = new Application(
	$application_name, $application_display, array(
		new Page("main")
	)
);

function timeconditions_daemon_init() {
	global $astman;

	$astman->add_event_handler("extensionstatus", "timeconditions_event_extensionstatus");
}

function timeconditions_event_extensionstatus($event, $data, $server, $port) {
	global $rest;

	$fcc = new featurecode('timeconditions', 'toggle-mode-all');
	$tc_code = $fcc->getCodeActive();

	if ($tc_code == '') {
		return;
	}

	if (!preg_match("/^" . preg_quote($tc_code . "*") . "(.*)$/", $data["Exten"], $matches)) {
		return;
	}

	$extension = $matches[1];

	$timeconditions = json_decode($rest->execute("timeconditions/", "GET", NULL));
	if ($timeconditions && !isset($timeconditions->error)) {
		foreach ($timeconditions as $timecondition) {
			$fcc = new featurecode('timeconditions', 'toggle-mode-' . $timecondition->timeconditions_id);
			$code = $fcc->getCodeActive();

			if ($code == $data["Exten"]) {
				return;
			}
		}
	}

	dbug('Time Conditions syncing: ' . $extension . ' (' . $data["Status"] . ')');
	timeconditions_sync($extension, $data["Status"]);
}

function timeconditions_sync_cb($extension) {
	global $astman;

	$fcc = new featurecode('timeconditions', 'toggle-mode-all');
	$tc_code = $fcc->getCodeActive();

	if ($tc_code == '') {
		return;
	}

	$data = $astman->send_request("ExtensionState", array("Exten" => $tc_code . "*" . $extension, "Context" => "timeconditions-toggles"));

	if (!preg_match("/^Custom:TC(.*)$/", $data["Hint"])) {
		return;
	}

	timeconditions_sync($extension, $data["Status"]);
}

function timeconditions_sync($extension, $status) {
	$ep_user = endpoint_api(array('ext' => $extension));

	if ($ep_user != NULL) {
		$brand = $ep_user["brand"];
	}

	switch ($brand) {
		case "Aastra":
		case "Yealink":
			aastra_toggle_application($extension, "Time Conditions", $status);
			break;
	}
}

function timeconditions_allowed_timeconditions($rest, $user, $timeconditions) {
	$data = json_decode($rest->execute("cos/users/" . $user . "/features", "GET", NULL), true);

	if (!$data || isset($data['error'])) {
		return $timeconditions;
	}

	foreach ($timeconditions as $key => $timecondition) {
		if (!isset($data['toggle-mode-' . $timecondition->timeconditions_id])) {
			unset($timeconditions[$key]);
		}
	}

	return $timeconditions;
}

function timeconditions_main_get_cb($page, $user, $args) {
	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("timeconditions/", "GET", NULL));

	if (!$data) {
		return NULL;
	}

	$screen = new Screen($page);

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Time Conditions is not authorized in the API module for extension " . $user . ".";
			break;
		}

		$screen->error_add($error);

		return json_encode($screen);
	}

	$values = array(
		'auto' => 'Reset Override',
		'true' => 'Temporary Matched',
		'true_sticky' => 'Permanent Matched',
		'false' => 'Temporary Unmatched',
		'false_sticky' => 'Permanent Unmatched'
	);

	$timeconditions = timeconditions_allowed_timeconditions($rest, $user, $data);
	$combined_state = false;
	foreach ($timeconditions as $timecondition) {
		$data = json_decode($rest->execute("timeconditions/" . $timecondition->timeconditions_id, "GET", NULL));

		if ($data && !$data->error) {
			$timecondition->state = ($data->state == NULL ? 'auto' : $data->state);
		} else {
			$timecondition->state = 'auto';
		}

		$fcc = new featurecode('timeconditions', 'toggle-mode-' . $timecondition->timeconditions_id);
		$tc_code = $fcc->getCodeActive();

		$screen->entry_add(array(
			"type" => "choice",
			"name" => "state_" . $timecondition->timeconditions_id,
			"display" => $timecondition->displayname . ($tc_code ? " (" . $tc_code . ")" : "") . ($timecondition->state != "auto" ? ": " . $values[$timecondition->state] : ""),
			"value" => $timecondition->state,
			"values" => $values
		));

		if ($timecondition->state == true) {
			$combined_state = true;
		}
	}

	if (count($timeconditions) > 1) {
		$screen->entry_add(array(
			"type" => "choice",
			"name" => "state_all",
			"display" => "Set All",
			"value" => "auto",
			"values" => $values
		));
	}

	return json_encode($screen);
}

function timeconditions_main_set_cb($page, $user, $args, $body) {
	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("timeconditions/", "GET", NULL));
	$timeconditions = timeconditions_allowed_timeconditions($rest, $user, $data);

	$body = json_decode($body, true);

	foreach ($body as $key => $val) {
		if ($key == 'state_all') {
			if (!$timeconditions) {
				continue;
			}

			$body_data = array(
				'state' => $val
			);

			foreach ($timeconditions as $timecondition) {
				$data = json_decode($rest->execute("timeconditions/" . $timecondition->timeconditions_id, "PUT", json_encode($body_data)));
			}
		} else if (preg_match('/^state_(\d*)$/', $key, $matches)) {
			$body_data = array(
				'state' => $val
			);
			$data = json_decode($rest->execute("timeconditions/" . $matches[1], "PUT", json_encode($body_data)));
		}
	}

	return NULL;
}

?>
