<?php

$application_name = "voicemail";
$application_display = "Voicemail";

$application = new Application(
	$application_name, $application_display, array(
		new Page("main"),
		new Page("play"),
		new Page("detail"),
		new Page("forward"),
		new Page("settings"),
		new Page("password"),

		new Page("transfer", "Transfer VM")
	)
);

$voicemail_calls = array();

function voicemail_daemon_init() {
	global $astman;

	$astman->add_event_handler("varset", "voicemail_event_varset");
	$astman->add_event_handler("hangup", "voicemail_event_hangup");
	$astman->add_event_handler("userevent", "voicemail_event_userevent");
}

function voicemail_event_varset($event, $data, $server, $port) {
	global $voicemail_calls;

	if ($data["Variable"] != "RESTAPPS_VOICEMAIL") {
		return;
	}

	$var = explode('^', $data["Value"]);

	$voicemail_calls[$data["Uniqueid"]] = array(
		"channel" => $data["Channel"],
		"extension" => $var[0],
		"msg_id" => $var[1],
		"endcall_uri" => $var[2]
	);
}

function voicemail_event_hangup($event, $data, $server, $port) {
	global $voicemail_calls;

	if (isset($voicemail_calls[$data["Uniqueid"]])) {
		$call = $voicemail_calls[$data["Uniqueid"]];

		//Show details screen for Aastra phones.
		$ep_user = endpoint_api(array('ext' => $call['extension']));

		if ($ep_user != NULL) {
			$brand = $ep_user["brand"];
		}

		switch ($brand) {
			case "Aastra":
			case "Yealink":
				aastra_exec_uri($call['extension'], $call['endcall_uri']);
				break;
		}

		unset($voicemail_calls[$data["Uniqueid"]]);
	}
}

function voicemail_event_userevent($event, $data, $server, $port) {
	global $voicemail_calls;

	if ($data["UserEvent"] != "voicemail_hangup") {
		return;
	}

	foreach ($voicemail_calls as $unique_id => $call) {
		if ($call["extension"] == $data["extension"] && $call["msg_id"] == $data["msg_id"]) {
			global $astman;

			$astman->send_request("Hangup", array("Channel" => $call["channel"]));
		}
	}
}

function voicemail_messages_path($user, $context, $folder) {
	global $amp_conf;

	$path = $amp_conf['ASTSPOOLDIR'] . '/voicemail/' . $context . '/' . $user . '/';

	/* Bail if the user dir doesn't exist. */
	if (!is_dir($path) || !is_readable($path)) {
		return NULL;
	}

	$path.= $folder . '/';

	if (!is_dir($path)) {
		mkdir($path, 0700);
	}

	return $path;
}

function voicemail_messages_next($user, $context, $folder) {
	$path = voicemail_messages_path($user, $context, $folder);

	for ($i = 0; $i <= 9999; $i++) {
		$msgnum = sprintf('msg%04d', $i);

		if (!file_exists($path . '/' . $msgnum . '.txt')) {
			return $i;
		}
	}

	return 0;
}

function voicemail_messages_reorder($user, $context, $folder) {
	$path = voicemail_messages_path($user, $context, $folder);

	$prev = 0;

	$origfiles = glob($path . '*.txt');
	foreach ($origfiles as $current) {
		$msg = pathinfo($current, PATHINFO_FILENAME);

		$newmsg = sprintf('msg%04d', $prev);
		if (!file_exists($path . $newmsg . '.txt')) {
			$currentfiles = glob($path . $msg . '.*');
			foreach ($currentfiles as $file) {
				$ext = pathinfo($file, PATHINFO_EXTENSION);
				rename($file, $path . $newmsg . '.' . $ext);
			}
		}

		$prev++;
	}
}

function voicemail_message_copy($user_src, $context_src, $folder_src, $user_dst, $context_dst, $folder_dst, $msg_id) {
	$path_src = voicemail_messages_path($user_src, $context_src, $folder_src);
	$path_dst = voicemail_messages_path($user_dst, $context_dst, $folder_dst);

	if (!$path_src || !$path_dst) {
		/* Path(s) not found. */
		return false;
	}

	$messages = voicemail_messages_get($user_src, $context_src);
	if ($messages && isset($messages[$msg_id])) {
		$message = $messages[$msg_id];

		$files = glob($path_src . $message['file'] . '.*');
	}

	if (!$files) {
		/* Message not found. */
		return false;
	}

	$next = voicemail_messages_next($user_dst, $context_dst, $folder_dst);
	$msgnum = sprintf('msg%04d', $next);

	foreach ($files as $file_src) {
		$ext = pathinfo($file_src, PATHINFO_EXTENSION);
		$file_dst = $path_dst . $msgnum . '.' . $ext;

		copy($file_src, $file_dst);
	}

	return true;
}

function voicemail_message_move($user_src, $context_src, $folder_src, $user_dst, $context_dst, $folder_dst, $msg_id) {
	$path_src = voicemail_messages_path($user_src, $context_src, $folder_src);
	$path_dst = voicemail_messages_path($user_dst, $context_dst, $folder_dst);

	if (!$path_src || !$path_dst) {
		/* Path(s) not found. */
		return false;
	}

	$messages = voicemail_messages_get($user_src, $context_src);
	if ($messages && isset($messages[$msg_id])) {
		$message = $messages[$msg_id];

		$files = glob($path_src . $message['file'] . '.*');
	}

	if (!$files) {
		/* Message not found. */
		return false;
	}

	$next = voicemail_messages_next($user_dst, $context_dst, $folder_dst);
	$msgnum = sprintf('msg%04d', $next);

	foreach ($files as $file_src) {
		$ext = pathinfo($file_src, PATHINFO_EXTENSION);
		$file_dst = $path_dst . $msgnum . '.' . $ext;

		rename($file_src, $file_dst);
	}

	voicemail_messages_reorder($user_src, $context_src, $folder_src);

	return true;
}

function voicemail_message_delete($user, $context, $folder, $msg_id) {
	$messages = voicemail_messages_get($user, $context);

	if ($messages && isset($messages[$msg_id])) {
		$message = $messages[$msg_id];

		if ($folder && $folder != $message['folder']) {
			/* Folder does not match. */
			return false;
		}

		$path = voicemail_messages_path($user, $context, $message['folder']);

		foreach (glob($path . $message['file'] . '.*') as $filename) {
			unlink($filename);
		}

		voicemail_messages_reorder($user, $context, $message['folder']);
		return true;
	}

	return false;
}

function voicemail_messages_get($user, $context) {
	$folders = array('INBOX', 'Old');
	foreach ($folders as $folder) {
		$path = voicemail_messages_path($user, $context, $folder);
		if (!$path) {
			continue;
		}

		foreach (glob($path . '/*.txt') as $filename) {
			$vm = pathinfo($filename, PATHINFO_FILENAME);

			$message = NULL;

			$lines = @file($filename, FILE_SKIP_EMPTY_LINES | FILE_IGNORE_NEW_LINES);
			foreach ($lines as $line) {
				if (preg_match("/^([a-z_-]*) ?=(.*)$/", $line, $matches)) {
					$message[$matches[1]] = trim($matches[2]);
				}
			}

			if ($message) {
				$message['folder'] = $folder;
				$message['file'] = $vm;
				$messages[$message['msg_id']] = $message;
			}
		}
	}

	return $messages;
}

function voicemail_format_duration($duration) {
	$sec = $duration % 60;
	$min = ($duration - $sec) / 60;
	return sprintf('%02d:%02d', $min, $sec);
}

function voicemail_main_get_cb($page, $user, $args) {
	$action = $args["action"];
	$msg_id = $args['app_msg'];

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("voicemail/mailboxes/" . $user, "GET", NULL));

	if (!$data) {
		return NULL;
	}

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Voicemail is not authorized in the API module for extension " . $user . ".";
			break;
		case 404:
			$error["display"] = "Voicemail is not enabled for extension " . $user . ".";
			break;
		}

		$screen = new Screen($page);

		$screen->error_add($error);

		return json_encode($screen);
	}

	switch ($action) {
	case "play":
		global $astman;

		$messages = voicemail_messages_get($user, $data->vmcontext);

		$ua = $GLOBALS["user_agent"];

		switch ($ua["brand"]) {
		case "aastra":
		case "yealink":
			$autoanswer = aastra_autoanswer();
			break;
		case "digium":
			$autoanswer = digium_autoanswer();
			break;
		}

		foreach ($autoanswer as $key => $val) {
			$variables['SIPADDHEADER'] = $key . ': ' . $val;
			$variables['PJSIP_HEADER(' . $key . ')'] = $val;
		}

		$uri = str_replace(trim($_SERVER['PATH_INFO'], "/"), "voicemail/detail", $GLOBALS['server_uri']);
		if (strstr($uri, "?")) {
			$uri.= "&amp;";
		} else {
			$uri.= "?";
		}
		$uri.= "app_msg=" . $msg_id;

		$variables['RESTAPPS_VOICEMAIL'] = $user . '^' . $msg_id . '^' . $uri;

		$astman->Originate(array(
			'Channel' => 'SIP/' . $user,
			'CallerID' => 'Voicemail <' . $msg_id . '>',
			'Variable' => $variables,
			'Application' => 'VoiceMailPlayMsg',
			'Data' => $user . ',' . $msg_id
		));

		if (isset($messages[$msg_id]) && $messages[$msg_id]['folder'] == "INBOX") {
			/* Mark the message as read, by moving it to Old folder. */
			voicemail_message_move($user, $data->vmcontext, "INBOX", $user, $data->vmcontext, "Old", $msg_id);
		}

		$screen = new Screen($page);
		$screen->type = 'action';

		$screen->entry_add(array(
			"type" => "redirect",
			"value" => "voicemail/play",
			"args" => "app_msg=" . $msg_id,
			"endcall" => "voicemail/detail",
			"endcall_args" => "app_msg=" . $msg_id,
		));

		return json_encode($screen);
	case "delete":
		voicemail_message_delete($user, $data->vmcontext, NULL, $msg_id);
		break;
	case "forward":
		$screen = new Screen($page);
		$screen->type = 'action';

		$screen->entry_add(array(
			"type" => "redirect",
			"value" => "voicemail/forward",
			"args" => "app_msg=" . $msg_id
		));

		return json_encode($screen);
	case "toggle":
		$messages = voicemail_messages_get($user, $data->vmcontext);

		if (isset($messages[$msg_id])) {
			if ($messages[$msg_id]['folder'] == "INBOX") {
				/* Mark the message as read, by moving it to Old folder. */
				voicemail_message_move($user, $data->vmcontext, "INBOX", $user, $data->vmcontext, "Old", $msg_id);
			} else {
				voicemail_message_move($user, $data->vmcontext, "Old", $user, $data->vmcontext, "INBOX", $msg_id);
			}
		}
		break;
	}

	$screen = new Screen($page);

	$messages = voicemail_messages_get($user, $data->vmcontext);
	if (count($messages) > 0) {
		foreach ($messages as $message) {
			$duration = voicemail_format_duration($message['duration']);
			$callerid = preg_replace('/\"/', '', $message['callerid']);

			$screen->entry_add(array(
				"type" => "link",
				"display" => $callerid . " [" . $duration . "]",
				"value" => "voicemail/detail",
				"args" => "app_msg=" . $message['msg_id'],
				"icon" => ($message['folder'] == "INBOX" ? "unread" : "read"),
				"iconstring" => ($message['folder'] == "INBOX" ? "(+)" : "(-)")
			));
		}

		$screen->action_add(array(
			"name" => "play",
			"display" => "Play"
		));

		$screen->action_add(array(
			"name" => "delete",
			"display" => "Delete",
		));

		$screen->action_add(array(
			"name" => "toggle",
			"display" => "Toggle",
		));

		$screen->action_add(array(
			"name" => "settings",
			"link" => "voicemail/settings",
			"display" => "Settings",
		));
	} else {
		$screen->type = "display";
		$screen->entry_add(array(
			"type" => "entry",
			"display" => "No voicemail messages.",
		));
	}

	return json_encode($screen);
}

function voicemail_transfer_get_cb($page, $user, $args) {
	$screen = new Screen($page);

	switch ($args['linestate']) {
	case 'CONNECTED':
		$screen->entry_add(array(
			"type" => "number",
			"name" => "destination",
			"display" => "Extension",
			"args" => "linestate=CONNECTED",
			"value" => ""
		));
		break;
	default:
		$screen->type = 'action';

		$screen->entry_add(array(
			"type" => "redirect",
			"value" => "voicemail/main",
		));
		break;
	}

	return json_encode($screen);
}

function voicemail_transfer_set_cb($page, $user, $args, $body) {
	$body = json_decode($body, true);

	$fcc = new featurecode('core', 'blindxfer');
	$xfer_code = $fcc->getCodeActive();

	$fcc = new featurecode('voicemail', 'directdialvoicemail');
	$dd_code = $fcc->getCodeActive();

	$screen = new Screen($page);
	$screen->type = 'action';

	$screen->entry_add(array(
		"type" => "transfer",
		"value" => $dd_code . $body['destination'],
		"dtmf" => ($xfer_code ? $xfer_code : NULL)
	));

	return json_encode($screen);
}

function voicemail_play_get_cb($page, $user, $args) {
	global $astman;

	$action = $args["action"];
	$msg_id = $args['app_msg'];

	if ($action) {
		switch ($action) {
		case "rewind":
			$screen = new Screen($page);
			$screen->type = 'action';

			$screen->entry_add(array(
				"type" => "redirect",
				"value" => "voicemail/play",
				"args" => "app_msg=" . $msg_id,
				"dtmf" => "*",
			));
			return json_encode($screen);
		case "ffwd":
			$screen = new Screen($page);
			$screen->type = 'action';

			$screen->entry_add(array(
				"type" => "redirect",
				"value" => "voicemail/play",
				"args" => "app_msg=" . $msg_id,
				"dtmf" => "#",
			));
			return json_encode($screen);
		case "pause":
			$screen = new Screen($page);
			$screen->type = 'action';

			$screen->entry_add(array(
				"type" => "redirect",
				"value" => "voicemail/play",
				"args" => "app_msg=" . $msg_id,
				"dtmf" => "0",
			));
			return json_encode($screen);
		case "stop":
			$astman->send_request("UserEvent", array("UserEvent" => "voicemail_hangup", "extension" => $user, "msg_id" => $msg_id));

			$screen = new Screen($page);
			$screen->type = 'action';

			$screen->entry_add(array(
				"type" => "redirect",
				"value" => "voicemail/detail",
				"args" => "app_msg=" . $msg_id,
			));
			return json_encode($screen);
		}
	}

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("voicemail/mailboxes/" . $user, "GET", NULL));

	if (!$data) {
		return NULL;
	}

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Voicemail is not authorized in the API module for extension " . $user . ".";
			break;
		case 404:
			$error["display"] = "Voicemail is not enabled for extension " . $user . ".";
			break;
		}

		$screen = new Screen($page);

		$screen->error_add($error);

		return json_encode($screen);
	}

	$messages = voicemail_messages_get($user, $data->vmcontext);
	$message = $messages[$msg_id];

	$screen = new Screen($page);
	$screen->exitPath = "voicemail/play";
	$screen->exitArgs = "action=stop&app_msg=" . $msg_id;

	$screen->entry_add(array(
		"type" => "entry",
		"display" => preg_replace('/\"/', '', $message['callerid']),
		"args" => "app_msg=" . $msg_id,
	));

	$duration = voicemail_format_duration($message['duration']);
	$screen->entry_add(array(
		"type" => "entry",
		"display" => $duration,
		"args" => "app_msg=" . $msg_id,
	));

	$screen->entry_add(array(
		"type" => "entry",
		"display" => strftime('%a %b %d %Y', $message['origtime']),
		"args" => "app_msg=" . $msg_id,
	));

	$screen->entry_add(array(
		"type" => "entry",
		"display" => strftime('%r', $message['origtime']),
		"args" => "app_msg=" . $msg_id,
	));

	$screen->action_add(array(
		"name" => "rewind",
		"display" => "Rewind"
	));

	$screen->action_add(array(
		"name" => "ffwd",
		"display" => "FFwd"
	));

	$screen->action_add(array(
		"name" => "pause",
		"display" => "Pause"
	));

	return json_encode($screen);
}

function voicemail_detail_get_cb($page, $user, $args) {
	global $astman;

	$action = $args["action"];
	$msg_id = $args['app_msg'];

	if ($action) {
		switch ($action) {
		case "delete":
			$screen = new Screen($page);
			$screen->type = 'action';

			$screen->entry_add(array(
				"type" => "redirect",
				"value" => "voicemail/main",
				"args" => "action=delete&app_msg=" . $msg_id
			));
			return json_encode($screen);
		case "play":
			$screen = new Screen($page);
			$screen->type = 'action';

			$screen->entry_add(array(
				"type" => "redirect",
				"value" => "voicemail/main",
				"args" => "action=play&app_msg=" . $msg_id
			));
			return json_encode($screen);
		case "forward":
			$screen = new Screen($page);
			$screen->type = 'action';

			$screen->entry_add(array(
				"type" => "redirect",
				"value" => "voicemail/forward",
				"args" => "app_msg=" . $msg_id
			));
			return json_encode($screen);
		}
	}

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("voicemail/mailboxes/" . $user, "GET", NULL));

	if (!$data) {
		return NULL;
	}

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Voicemail is not authorized in the API module for extension " . $user . ".";
			break;
		case 404:
			$error["display"] = "Voicemail is not enabled for extension " . $user . ".";
			break;
		}

		$screen = new Screen($page);

		$screen->error_add($error);

		return json_encode($screen);
	}

	$messages = voicemail_messages_get($user, $data->vmcontext);
	$message = $messages[$msg_id];

	$screen = new Screen($page);
	$screen->exitPath = "voicemail/main";

	$screen->entry_add(array(
		"type" => "entry",
		"display" => preg_replace('/\"/', '', $message['callerid']),
		"args" => "app_msg=" . $msg_id,
	));

	$duration = voicemail_format_duration($message['duration']);
	$screen->entry_add(array(
		"type" => "entry",
		"display" => $duration,
		"args" => "app_msg=" . $msg_id,
	));

	$screen->entry_add(array(
		"type" => "entry",
		"display" => strftime('%a %b %d %Y', $message['origtime']),
		"args" => "app_msg=" . $msg_id,
	));

	$screen->entry_add(array(
		"type" => "entry",
		"display" => strftime('%r', $message['origtime']),
		"args" => "app_msg=" . $msg_id,
	));

	$screen->action_add(array(
		"name" => "play",
		"display" => "Play"
	));

	$screen->action_add(array(
		"name" => "delete",
		"display" => "Delete",
	));

	$screen->action_add(array(
		"name" => "forward",
		"display" => "Forward",
	));

	return json_encode($screen);
}

function voicemail_forward_get_cb($page, $user, $args) {
	$action = $args["action"];
	$msg_id = $args['app_msg'];

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("voicemail/mailboxes/" . $user, "GET", NULL));

	if (!$data) {
		return NULL;
	}

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Voicemail is not authorized in the API module for extension " . $user . ".";
			break;
		case 404:
			$error["display"] = "Voicemail is not enabled for extension " . $user . ".";
			break;
		}

		$screen = new Screen($page);

		$screen->error_add($error);

		return json_encode($screen);
	}

	$messages = voicemail_messages_get($user, $data->vmcontext);
	$message = $messages[$msg_id];

	$screen = new Screen($page);
	$screen->exitPath = "voicemail/main";

	$screen->entry_add(array(
		"type" => "number",
		"name" => $msg_id,
		"display" => "Enter Number",
		"args" => "app_msg=" . $msg_id,
		"value" => ""
	));

	return json_encode($screen);
}

function voicemail_forward_set_cb($page, $user, $args, $body) {
	$body = json_decode($body, true);

	$rest = new RESTClient($user, NULL, NULL);
	$data = json_decode($rest->execute("voicemail/mailboxes/" . $user, "GET", NULL));

	if (!$data) {
		return NULL;
	}

	if (isset($data->error)) {
		$error = array();

		$error["reason"] = $data->error;
		switch ($error["reason"]) {
		case 401:
			$error["display"] = "Voicemail is not authorized in the API module for extension " . $user . ".";
			break;
		case 404:
			$error["display"] = "Voicemail is not enabled for extension " . $user . ".";
			break;
		}

		$screen = new Screen($page);

		$screen->error_add($error);

		return json_encode($screen);
	}

	$screen = new Screen($page);
	$screen->type = 'action';

	$messages = voicemail_messages_get($user, $data->vmcontext);
	foreach ($body as $msg_id => $extension) {
		/* There should only be one. */
		$message = $messages[$msg_id];

		voicemail_message_copy($user, $data->vmcontext, $message['folder'], $extension, $data->vmcontext, 'INBOX', $msg_id);

		$screen->entry_add(array(
			"type" => "redirect",
			"value" => "voicemail/detail",
			"args" => "app_msg=" . $msg_id
		));
	}

	return json_encode($screen);
}

function voicemail_settings_get_cb($page, $user, $args) {
	$screen = new Screen($page);
	$screen->exitPath = "voicemail/main";

	$screen->entry_add(array(
		"type" => "link",
		"display" => "Change Password",
		"value" => "voicemail/password",
	));

	return json_encode($screen);
}

function voicemail_password_get_cb($page, $user, $args) {
	$screen = new Screen($page);
	$screen->exitPath = "voicemail/settings";

	$screen->entry_add(array(
		"type" => "number",
		"name" => "password",
		"display" => "Enter New Password",
		"value" => "",
		"min" => "4"
	));

	return json_encode($screen);
}

function voicemail_password_set_cb($page, $user, $args, $body) {
	$body = json_decode($body, true);

	$rest = new RESTClient($user, NULL, NULL);

	$data = json_decode($rest->execute("voicemail/password/" . $user, "PUT", json_encode($body)));

	return NULL;
}

?>
