<?php
if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed');}

function sysadmin_get_ddns() {
    global $db;
	$ret = array();
	$sql = 'SELECT * FROM sysadmin_options WHERE `key` IN("interval", "staticip")';
	$res = $db->getAll($sql, DB_FETCHMODE_ASSOC);
	if($db->IsError($res)){
		die_freepbx($res->getDebugInfo());
	} 
	foreach($res as $r) {
		$ret[$r['key']] = $r['value'];
	}
	
	return $ret;
}

/*
 * if $discovered = true, we will use ONLY the true external ip address
 * if its set to false, we will use a staic IP IF WE HAVE ONE SAVED,
 * otherwise we'll fall back to the external IP
 *
 */
function sysadmin_get_public_ip($discovered = true) {
    global $db;

    $sql = "SELECT count(*) AS extensions FROM devices WHERE tech != 'custom'";

    $exten_count = $db->getRow($sql, DB_FETCHMODE_ASSOC);
    if($db->IsError($exten_count)) {
        $exten_count = 0;
    }
	
	$lic = sysadmin_get_license();
    // All servers we can update ip with
    $servers = array('ddns1.schmoozecom.net', 'ddns2.schmoozecom.net');

    // Lets randomize which I hit
    shuffle($servers);

    // Get the id to send to the server
    $pbxactid			= isset($lic['deploymentid']) ? $lic['deploymentid'] : '';
    $pbxactversion		= sysadmin_get_pbxversion();
    $pbxactfailsafe		= sysadmin_get_pbxactfailsafe();
    $pbxactnotes		= sysadmin_client_fetch_pbxactnotes();
    $asteriskversion	= sysadmin_client_fetch_asteriskversion();
    $kernelversion 		= sysadmin_client_fetch_kernelversion();

	if ($discovered) {
		$ip_cfg			= '';
	} else {
		$ip_cfg			= sysadmin_get_ddns();
		$ip_cfg			= isset($ip_cfg['staticip']) && $ip_cfg['staticip'] ? $ip_cfg['staticip'] : '';
	}
	

	
	foreach ($servers as $server) {
		$url = 'http://'	. $server . '/ddnsserver.php?'
				. 'server=' . $pbxactid
				. '&ver=' 	. urlencode($pbxactversion)
				. '&fs='	. urlencode($pbxactfailsafe)
				. '&av='	. urlencode($asteriskversion)
				. '&kv='	. urlencode($kernelversion)
				. '&n='		. urlencode($pbxactnotes)
				. '&e='		. urlencode($exten_count['extensions'])
				. '&st='	. urlencode($ip_cfg);

		$ip = file_get_contents($url);
        if (isset($ip) && $ip) {
			return trim($ip);
        }
    }

    return 'An error has occurred, please contact support.';
}

function sysadmin_set_ddns_cron($interval = 15, $staticip = '') {
	global $amp_conf;
	
	if ($interval > 0) { 
		return edit_crontab('ddns_client.php', 
			array(
				'minute' => '*/' . $interval,
				'command' => 'php /var/lib/asterisk/agi-bin/ddns_client.php staticip=' 
								. $staticip . ' 2>&1 >/dev/null')
				);
	} else {
		return edit_crontab('ddns_client.php');
	}
}

function sysadmin_set_ddns($interval = 15, $staticip = '') {
    global $db;
    sql("REPLACE INTO sysadmin_options (`key`,value) VALUES ('ip_update_interval', '$interval')");
	sql("REPLACE INTO sysadmin_options (`key`,value) VALUES ('staticip', '$staticip')");
	sysadmin_set_ddns_cron($interval, $staticip);
}

function sysadmin_client_fetch_pbxactnotes() {
	if (file_exists('/etc/pbxactnotes')) {
		 return base64_encode(trim(@file_get_contents('/etc/pbxactnotes')));
	} else {
		return '';
	}
 
}

function sysadmin_client_fetch_asteriskversion() {
	global $amp_conf;
	//TODO: this should relaly be called via the astmanager
	$cmd = $amp_conf['AMPSBIN'] . '/asterisk -rx "core show version"';
	$version = trim(exec($cmd));
	return $version;
}

function sysadmin_client_fetch_kernelversion() {
	$uname=posix_uname();
	return $uname['release'];
}

?>
