<?php
if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed');}
function sysadmin_put_storage_email($email) {
	global $db;
	
	foreach ($email as $my => $e) {
		$data[] = array($my, $e);
	}
	
	//save to db
	$sql = $db->prepare('REPLACE INTO sysadmin_options (`key`, value) VALUES (?, ?)');
	$ret = $db->executeMultiple($sql, $data);
	if($db->IsError($ret)) {
    		die_freepbx($ret->getDebugInfo()."\n".$ret->getUserInfo()."\n".$db->last_query);
  	}
}

function sysadmin_get_storage_email() {
	global $db;
	//get from db
	$lic = sysadmin_get_license();
	
	$getAllFields = false;
	$fields['limited'] = '"storageemail", "fromemail", "fail2ban_email", "abnormal_email"';
	$fields['all'] = '"storageemail", "fromemail", "fail2ban_email", "ups_changeme", "ups_commfailure", "ups_commok", "ups_offbattery", "ups_onbattery", "abnormal_email"';

	switch ($lic['deploy_type']) {
        	case 'PBXact':
        	case 'PBXtended':
			$getAllFields = true;		
			break;
	 	case 'OSS':
        	case 'FreePBX-Distro':
                	//if they are licensed for sysadmin pro, give them additional options
                	if(str_replace('-','',$lic['sysadmin_exp']) > date('Ymd')) {
                		$getAllFields = true;	
			} else {
				$getAllFields = false;
			}
			break;
        	default:
			$getAllFields = false;
		break;

	}
	
	//Build SQL Statement based on deploy_type
	$sql = 'SELECT * FROM sysadmin_options WHERE `key` IN (';
	$sql .= ($getAllFields === true) ? $fields['all'] : $fields['limited'];
	$sql .=  ')';

	$ret = $db->getAll($sql, DB_FETCHMODE_ASSOC);
	
	if($db->IsError($ret)) {
	    	die_freepbx($ret->getDebugInfo()."\n".$ret->getUserInfo()."\n".$db->last_query);
  	}
	foreach ($ret as $r) {
		$res[$r['key']] = $r['value'];
	}

	$storage_keys = array('storageemail', 'fail2ban_email', 'ups_changeme', 'ups_commfailure', 'ups_commok', 'ups_offbattery', 'ups_onbattery', 'abnormal_email');
	foreach ($storage_keys as $sk) {
		$res[$sk] = (isset($res[$sk]))?$res[$sk]:'';
	}

	return $res;
	
}

function sysadmin_get_onlystorage_email() {
	global $db;
	//get from db
	$ret = $db->getAll('SELECT * FROM sysadmin_options WHERE `key` IN("storageemail")', DB_FETCHMODE_ASSOC);
	if($db->IsError($ret)) {
    	die_freepbx($ret->getDebugInfo()."\n".$ret->getUserInfo()."\n".$db->last_query);
  	}
	
	$res = array();	
	foreach ($ret as $r) {
		$res[$r['key']] = $r['value'];
	}
	
	return $res;
	
}

function sysadmin_get_filesystems() {
	exec('df', $fs_ret);
	
	//unset the header line, we dont need it
	unset($fs_ret[0]);

	foreach ($fs_ret as $f) {
		$thisf = preg_split('/\s+/', $f);
		$fs[$thisf[0]] = array(
						'size'	=> $thisf[1], 
						'used'	=> $thisf[2], 
						'avail'	=> $thisf[3], 
						'use%'	=> $thisf[4], 
						'mount'	=> $thisf[5]
					);
	}
	
	return $fs;
}

function sysadmin_get_mdadm() {
	if (!file_exists('/etc/mdadm.conf')) {
		$mdam = false;
	} else {
		$file = file('/etc/mdadm.conf');
		
		//Count how many times each variable appears
		foreach ($file as $index => $f) {
			//ignore comments and blank lines
			if (substr($f, 0, 1) == '#' || !trim($f)) {
				unset($file[$index]);
				continue; 
			}
			$line = explode(' ', $f, 2);
			
			if (!isset($line_count[$line[0]])) {
				$line_count[$line[0]] = 0;
			}
			$line_count[$line[0]]++; 
		}
		
		//now load the variabels in to an array
		foreach ($file as $f) {
			$line = explode(' ', $f, 2);
			if ($line_count[$line[0]] > 1) {
				$mdam[trim($line[0])][] = isset($line[1]) ? trim($line[1]) : '';
			} else {
				$mdam[trim($line[0])] = isset($line[1]) ? trim($line[1]) : '';
			}
		}
	}
	
	return $mdam;
}

function sysadmin_set_mdadm() {
	file_put_contents('/var/spool/asterisk/sysadmin/mdadm', time());
}

function sysadmin_get_crititcal_storage() {
	$high_rate		= 75;
	$critical_rate	= 90;
	$ret			= array();
	$fs 			= sysadmin_get_filesystems();

	foreach ($fs as $f => $s) {
		//only ide/sata drives are being checked
		if (substr($f, 0, 8) == '/dev/sda' 
			|| substr($f, 0, 8) == '/dev/hda' 
			|| substr($f, 0, 7) == '/dev/md'
			|| substr($f, 0, 10) == '/dev/simfs'
		) {

			//round up to next who number. Rounding up, so that we error on the side of caution
			$used = ceil((1 - (($s['size'] -  $s['used']) / $s['size'])) * 100);
			if ($used >= $high_rate) {
				if ($used >= $critical_rate) {
					$ret['critical'][] = array('dev' => $f, 'used' => $used);
				} else {
					$ret['high'][] = array('dev' => $f, 'used' => $used);
				}
			}
		}
	}
	return $ret;
}

function sysadmin_get_raid_status() {
	exec('/bin/cat /proc/mdstat', $raid);
	//dbug($raid);
	return $raid;
}
/*
function sysadmin_put_mdadm($mdadm) {
	global $db;
	//save to db
	$ret = $db->query('REPLACE INTO sysadmin_options (`key`, value) VALUES (?, ?)', array('mdadm', serialize($mdadm)));
	if($db->IsError($ret)) {
    	die_freepbx($ret->getDebugInfo()."\n".$ret->getUserInfo()."\n".$db->last_query);
  	}
}*/

//function for searching for part of an array
function my_array_search($needle = null, $haystack_array = null, $skip = 0)
{
    if($needle == null || $haystack_array == null)
        die('$needle and $haystack_array are mandatory for functie my_array_search()');
    foreach($haystack_array as $key => $eval)
    {
        if($skip != 0)$eval = substr($eval, $skip);
        if(stristr($eval, $needle) !== false) return true;
    }
    return false;
}


?>
