<?php
if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed');}
global $db, $amp_conf;

//check for sysadmin version and bomb out
$vercmd = 'rpm -q --qf "%{VERSION} %{RELEASE}\n" sysadmin';
$rpmquery = system($vercmd, $ret);
if ($ret == 0) {
    $rpm = explode(" ",$rpmquery,2);
    $rpmver = $rpm[0];
    $rpmrel = explode('_',$rpm[1],2);
    if (version_compare_freepbx($rpmver,'2.6.1', '<=') && version_compare_freepbx($rpmrel[0], '151', '<=')){
        out(_("ERROR: Unable to install sysadmin. The sysadmin RPM you have installed is too old. Please update the sysadmin rpm via the command line, using yum install sysadmin"));
        return false;
    }
} else {
    out(_("Unable to determine sysadmin rpm version...exiting"));
    return false;
}

if (copy($amp_conf['AMPWEBROOT'].'/admin/modules/sysadmin/agi-bin/LoadLicenseIfExists.php', $amp_conf['ASTAGIDIR'].'/LoadLicenseIfExists.php')) {
    out(_("Successfully copied LoadLicenseIfExists.php."));
} else {
    out(_("Unable to copy LoadLicenseIfExists.php"));
}

require_once(dirname(__FILE__) . '/functions.inc.php');

$new = $db->query('SELECT * FROM sysadmin_options');
if ($db->IsError($new)) {
    $new = true;
} else {
    $new = false;
}

$sql[] = "CREATE TABLE IF NOT EXISTS `sysadmin_options` (
    `key` VARCHAR(25),
        `value` TEXT,
        UNIQUE KEY `key` (`key`)
    )";

$sql[] = 'CREATE TABLE IF NOT EXISTS `sysadmin_update_log` (
    `id` int(11) NOT NULL auto_increment,
    `time` bigint(20) default NULL,
    `version` varchar(25) default NULL,
    `status_id` smallint(6) default NULL,
    `status` varchar(255) default NULL,
    PRIMARY KEY  (`id`)
)';


if ($new) {
    $sql[] = "INSERT INTO `sysadmin_options` (`key`, value) VALUES ('fail2ban_whitelist', '127.0.0.1')";
    $sql[] = "INSERT INTO `sysadmin_options` (`key`, value) VALUES ('fail2ban_ban_time', '1800')";
    $sql[] = "INSERT INTO `sysadmin_options` (`key`, value) VALUES ('fail2ban_max_retry', '8')";
    $sql[] = "INSERT INTO `sysadmin_options` (`key`, value) VALUES ('fail2ban_find_time', '600')";
    $sql[] = "INSERT INTO `sysadmin_options` (`key`, value) VALUES ('fail2ban_email', ' ')";
    $sql[] = "INSERT INTO `sysadmin_options` (`key`, value) VALUES ('ftp_server', '0')";
    $sql[] = "INSERT INTO `sysadmin_options` (`key`, value) VALUES ('tftp_server', '1')";
    $sql[] = "INSERT INTO `sysadmin_options` (`key`, value) VALUES ('ups_enabled', '0')";
    $sql[] = "INSERT INTO `sysadmin_options` (`key`, value) VALUES ('interval', '15')";
    $sql[] = "INSERT INTO `sysadmin_options` (`key`, value) VALUES ('staticip', ' ')";
}

foreach ($sql as $q) {
    $result = $db->query($q);
    if($db->IsError($result)){
        die_freepbx($result->getDebugInfo());
    }
}

//need to get rid an old field here that was never used
$domain = $db->getOne("SELECT value FROM `sysadmin_options` WHERE `key` = 'email_setup_domain'");
if(!$db->IsError($domain)) {
    if ($domain) {
        $db->query("DELETE FROM `sysadmin_options` WHERE `key` = 'email_setup_domain'");
    }
}

//add a default cron for system updates if were on pbxact/pbxtebded
$lic = sysadmin_get_license();

//check to see if we need to migrate from the old ddns table
$ip = $db->getOne('SELECT ddns_static_ip FROM ddns_client_config DECS LIMIT 1');
if(!$db->IsError($ip)) {
    if ($ip) {
        sysadmin_set_ddns('59', $ip);
        $db->query('DROP TABLE ddns_client_config');
    }
}

if ($lic['deploy_type'] == 'PBXact' ||  $lic['deploy_type'] == 'PBXtended') {
    //make sure the ddns cron is set
    $ip = sysadmin_get_ddns_install();

    if (isset($ip['interval'], $ip['staticip'])) {
        sysadmin_ddns_set_cron($ip['interval'], $ip['staticip']);
    } else {
        sysadmin_set_ddns_cron(15);
    }

    //set a defualt update cron
    sysadmin_service_set_cron('monthly');

    //set a defualt license check cron
    sysadmin_set_license_cron();

    //migrate the update cron pbxact_service table if its exists
    $update = $db->getAll('SELECT * FROM pbxact_service', DB_FETCHMODE_ASSOC);
    if($db->IsError($update)) {
        //echo 'pbxact_service table doesnt exists, probably migrated already';
    } else {
        foreach ($update as $u) {
            $sql = 'REPLACE INTO sysadmin_options (`key`, value) VALUES (?, ?)';
            $db->query($sql, array('update_' . $u['key'], $u['value']));
            $db->query('DROP TABLE pbxact_service');
        }

    }
}

//set stroage cron
edit_crontab('storage.php', array('minute' => '*/60', 'command' => '/var/lib/asterisk/bin/storage.php'));

//check to see if we need to migrate from postfix file
$email_setup = $db->getOne('SELECT * FROM sysadmin_options WHERE `key` IN("email_setup_server")');
if ($email_setup == null) {
    sysadmin_setup_email();
}

//make sure we dont have a 'broken" license file in place
if (file_exists($amp_conf['ASTAGIDIR'] . '/license-.zl')) {
    unlink($amp_conf['ASTAGIDIR'] . '/license-.zl');
}

$shmzDir = '/etc/schmooze';
if (is_dir($shmzDir)) {
    //make sure we don't have a "broken" license file in place
    if (file_exists($shmzDir . '/license-.zl')) {
        unlink($shmzDir . '/license-.zl');
    }

    //mv files
    $shmzFiles = array('freepbxdistro-version', 'freepbxdistro-failsafe', 'pbx-version', 'pbx-failsafe', 'pbxid');
    foreach ($shmzFiles as $value) {
        if (file_exists($amp_conf['ASTETCDIR'] . '/' . $value)) {
            switch($value) {
            case 'freepbxdistro-version':
                $change_val = 'pbx-version';
                exec('mv ' . $amp_conf['ASTETCDIR'] . '/' . $value . ' ' . $shmzDir . '/' . $change_val);
                break;
            case 'freepbxdistro-failsafe':
                $change_val = 'pbx-failsafe';
                exec('mv ' . $amp_conf['ASTETCDIR'] . '/' . $value . ' ' . $shmzDir . '/' . $change_val);
                break;
            default:
                exec('mv ' . $amp_conf['ASTETCDIR'] . '/' . $value . ' ' . $shmzDir . '/' . $value);
                //rename($amp_conf['ASTETCDIR'] . '/' . $value, $shmzDir . '/' . $value);
                break;
            }
            //rename($amp_conf['ASTETCDIR'] . '/' . $value, $shmzDir . '/' . ($change_val !== FALSE) ? $change_val : $value);

            if ($value !== 'pbxid' && file_exists($shmzDir . '/' . $value)) {
                symlink($shmzDir . '/' . $value, $amp_conf['ASTETCDIR'] . '/' . $value);
            }
        }
    }

    exec("mv " . $amp_conf['ASTAGIDIR'] . "/license-*.zl " . $shmzDir . "/.");
}

//make sure we move the latest LoadLicenseIfExists over to agi-bin on retreive_conf
if (file_exists($amp_conf['ASTAGIDIR'] . '/LoadLicenseIfExists.php')) {
    unlink($amp_conf['ASTAGIDIR'] . '/LoadLicenseIfExists.php');
}


//create incron-monitered files
@mkdir('/var/spool/asterisk/sysadmin/');
touch('/var/spool/asterisk/sysadmin/network');
touch('/var/spool/asterisk/sysadmin/dns');
touch('/var/spool/asterisk/sysadmin/reboot');
touch('/var/spool/asterisk/sysadmin/poweroff');
touch('/var/spool/asterisk/sysadmin/tz');
touch('/var/spool/asterisk/sysadmin/vpnstop');
touch('/var/spool/asterisk/sysadmin/vpnstart');
touch('/var/spool/asterisk/sysadmin/vpnget');
touch('/var/spool/asterisk/sysadmin/phone_configs');
touch('/var/spool/asterisk/sysadmin/mdadm');
touch('/var/spool/asterisk/sysadmin/intrusion_detection_stop');
touch('/var/spool/asterisk/sysadmin/intrusion_detection_start');
touch('/var/spool/asterisk/sysadmin/intrusion_detection_banned');
touch('/var/spool/asterisk/sysadmin/intrusion_detection');
touch('/var/spool/asterisk/sysadmin/email_setup');
touch('/var/spool/asterisk/sysadmin/ftp_setup');
touch('/var/spool/asterisk/sysadmin/ups_setup');
touch('/var/spool/asterisk/sysadmin/update_system');
touch('/var/spool/asterisk/sysadmin/update_system_cron');
touch('/var/spool/asterisk/sysadmin/restart_httpd');
touch('/var/spool/asterisk/sysadmin/restart_xmpp');
touch('/var/spool/asterisk/sysadmin/portmgmt_setup');
touch('/var/spool/asterisk/sysadmin/amportal_restart');
touch('/var/spool/asterisk/sysadmin/dahdi_restart');
touch('/var/spool/asterisk/sysadmin/wanrouter_restart');
touch('/var/spool/asterisk/sysadmin/incron_restart');
touch('/var/spool/asterisk/sysadmin/restapps_restart');
touch('/var/spool/asterisk/sysadmin/update_sysadmin_rpm');
touch('/usr/local/asterisk/ha_trigger');

sysadmin_put_portmgmt_install();

function sysadmin_find_ports($base, $ports = 1, $increment = 1) {
    $lsof_cmd = '/usr/sbin/lsof';
    if (!file_exists($lsof_cmd)) {
        $lsof_cmd = function_exists('fpbx_which') ? fpbx_which('lsof') : 'lsof';
    }
    $cmd_base = $lsof_cmd.' -i :';
    $ret = array();
    while(count($ret) < $ports) {
        $pport = (int)$base + (int)$increment;
        $p = system($cmd_base.$pport, $retval);
        if ($p == '' && $retval == 1) {
            $ret[] = $pport;
        }
        $increment += $increment;
    }
    return $ret;
}

function sysadmin_put_portmgmt_install() {
    global $db;
    $sysadmin_install_portmgmt_change = false;
    //check if web ports are set, and set if not
    if (!file_exists('/etc/httpd/conf.d/schmoozecom.conf')){
        $listen = system("/bin/grep '^Listen.*' /etc/httpd/conf/httpd.conf | head -n 1");
        $acp = substr($listen, 7);
        $acp = ($acp == '')?80:$acp;

        $ports = sysadmin_find_ports($acp,5);

        $ucp = $ports[0];
        $xml = $ports[1];
        $hpro = $ports[2];
        $restapps = $ports[3];
        $restapi = $ports[4];
        //save to db
        $data = array(
            array('xml', $xml),
            array('ucp', $ucp),
            array('acp',  $acp),
            array('hpro', $hpro),
            array('restapps', $restapps),
            array('restapi', $restapi)
        );

        $sql = $db->prepare('REPLACE INTO sysadmin_options (`key`, `value`) VALUES (?, ?)');
        $ret = $db->executeMultiple($sql, $data);
        if($db->IsError($ret)) {
            die_freepbx($ret->getDebugInfo()."\n".$ret->getUserInfo()."\n".$db->last_query);
        }
        $sysadmin_install_portmgmt_change = true;
    } else {
        $sql = "SELECT MAX(`value`) as port FROM sysadmin_options WHERE `key` IN ('xml','ucp','acp','hpro','restapps','restapi')";
        $ret = $db->getAll($sql, DB_FETCHMODE_ASSOC);
        if($db->IsError($ret)) {
            die_freepbx($ret->getDebugInfo()."\n".$ret->getUserInfo()."\n".$db->last_query);
        }
        if (isset($ret[0]['port']) && $ret[0]['port'] != '') {
            $acp = $ret[0]['port'];
        } else {
            $listen = system("/bin/grep '^Listen.*' /etc/httpd/conf/httpd.conf | head -n 1");
            $acp = substr($listen, 7);
            $acp = ($acp == '')?80:$acp;
        }

        $portmgmtPorts = array(
            'xml' => 'xml',
            'ucp' => 'ucp',
            'acp' => 'acp',
            'hpro' => 'hpro',
            'restapps' => 'restapps',
            'restapi' => 'restapi'
        );

        //determine what we do and dont have
        $sql = "SELECT `key` as port_name, value as port FROM sysadmin_options WHERE `key` IN ('xml','ucp','acp','hpro','restapps','restapi')";
        $ret = $db->getAll($sql, DB_FETCHMODE_ASSOC);
        if ($db->IsError($ret)) {
            die_freepbx($ret->getDebugInfo()."\n".$ret->getUserInfo()."\n".$db->last_query);
        }

        //The following logic should prevent us from duplicating the same port on update
        //Ultimately we, unset the port if its not already in use, otherwise we
        //leave it so that we get a new port number
        $used = array();
        foreach ($ret as $value) {
            if (!in_array($value['port'], $used)) {
                unset($portmgmtPorts[$value['port_name']]);
            }
            $used[] = $value['port'];
        }

        if (is_array($portmgmtPorts) && !empty($portmgmtPorts)) {
            $port = null;
            foreach ($portmgmtPorts as $key => $value) {
                switch($key) {
                case 'acp':
                    $port = $acp;
                    break;
                default:
                    $base = ($port === null)?$acp:$port;
                    $fport = sysadmin_find_ports($base,1);
                    $port = $fport[0];
                }
                //insert into
                $data = array($key, $port);
                $sql = $db->prepare('REPLACE INTO sysadmin_options (`key`, `value`) VALUES (?, ?)');
                $ret = $db->execute($sql, $data);
                if($db->IsError($ret)) {
                    die_freepbx($ret->getDebugInfo()."\n".$ret->getUserInfo()."\n".$db->last_query);
                }
                $sysadmin_install_portmgmt_change = true;
            }
        }
    }
    needreload();
    if (isset($sysadmin_install_portmgmt_change) && $sysadmin_install_portmgmt_change === true) {
        file_put_contents('/var/spool/asterisk/sysadmin/portmgmt_setup', time());
    }
}

function sysadmin_get_ddns_install() {
    global $db;
    $sql = 'SELECT * FROM sysadmin_options WHERE `key` IN("ip_update_interval", "staticip")';
    $res = $db->getAll($sql, DB_FETCHMODE_ASSOC);
    $ret = array();
    if($db->IsError($res)){
        die_freepbx($res->getDebugInfo());
    }
    foreach($res as $r) {
        $ret[$r['key']] = $r['value'];
    }

    return $ret;
}

if (! function_exists("out")) {
    function out($text) {
        echo $text."<br />";
    }
}

if (! function_exists("outn")) {
    function outn($text) {
        echo $text;
    }
}

// sysadmin migration
outn(_("checking for ftp_server field.."));
$sql = "SELECT `key` FROM `sysadmin_options` where `key` = 'ftp_server'";
$check = $db->getRow($sql, DB_FETCHMODE_ASSOC);
if($db->IsError($check) || empty($check)) {
    // add new field
    $sql = "INSERT INTO `sysadmin_options` (`key`, value) VALUES ('ftp_server', '0');";
    $result = $db->query($sql);
    if($db->IsError($result)) {
        die_freepbx($result->getDebugInfo());
    }
    out(_("OK"));
} else {
    out(_("already exists"));
}

// sysadmin migration
outn(_("checking for tftp_server field.."));
$sql = "SELECT `key` FROM `sysadmin_options` where `key` = 'tftp_server'";
$check = $db->getRow($sql, DB_FETCHMODE_ASSOC);
if($db->IsError($check) || empty($check)) {
    // add new field
    $sql = "INSERT INTO `sysadmin_options` (`key`, value) VALUES ('tftp_server', '1');";
    $result = $db->query($sql);
    if($db->IsError($result)) {
        die_freepbx($result->getDebugInfo());
    }
    out(_("OK"));
} else {
    out(_("already exists"));
}

outn(_("checking for ups_enabled field.."));
$sql = "SELECT `key` FROM `sysadmin_options` where `key` = 'ups_enabled'";
$check = $db->getRow($sql, DB_FETCHMODE_ASSOC);
if($db->IsError($check) || empty($check)) {
    // add new field
    $sql = "INSERT INTO `sysadmin_options` (`key`, value) VALUES ('ups_enabled', '0');";
    $result = $db->query($sql);
    if($db->IsError($result)) {
        die_freepbx($result->getDebugInfo());
    }
    out(_("OK"));
} else {
    out(_("already exists"));
}

//Abnormalcallvolume migration if table and module exist
//Abnormalcallvolume migration if table and module exist
outn(_("Checking for abnormalcallvolume.."));
$sql    = 'describe abnormalcallvolume';
$fields = $db->getAssoc($sql);
if ($db->IsError($fields)) {
    outn(_("No abnormalcallvolume found.."));
} else {
    $settings=sql('SELECT * FROM abnormalcallvolume', 'getAssoc', 'DB_FETCHMODE_ASSOC');
    if (is_array($settings) && !empty($settings)) {
        outn(_("Abnormalcallvolume found...Starting migration.."));
        foreach($settings as $setting => $value) {
            if (!empty($value['0'])) {
                //we changed this key to abnormal_email so account for that
                if ($setting == 'email') {
                    $setting = 'abnormal_email';
                }
                $set[] = array($setting, $value['0']);
            }
        }
        $sql = $db->prepare('REPLACE INTO sysadmin_options (`key`, `value`) VALUES (?, ?)');
        $ret = $db->executeMultiple($sql, $set);

        if($db->IsError($ret)) {
            outn(_("Migration FAILED. Please resetup abnormalcallvolume in Sysadmin and remove any previous abnormalcallvolume module..."));
        } else {
            outn(_("Migration complete..Attempting to uninstall Abnormalcallvolume module..."));

            if (module_uninstall('abnormalcallvolume')) {
                outn(_("Abnormalcallvolume uninstalled successfully..."));
            } else {
                outn(_("Unable to uninstall abnormalcallvolume..."));
            }
        }
    }
}

//remove and readd abnormal call volume cron
edit_crontab('abnormal.php', rand(0, 59) . ' ' . rand(0, 5) . ' * * * php ' . $amp_conf['ASTAGIDIR'] . "/abnormal.php 2>&1 >/dev/null");

file_put_contents('/var/spool/asterisk/sysadmin/update_sysadmin_rpm', time());
file_put_contents('/var/spool/asterisk/sysadmin/incron_restart', time());
?>
