<?php
// TODO: not sure if we need any routines in here in which case we should not include it to limit the load
//       but it may be needed for zend or some stuff from it put here
//
include('vmnotify-common.php');

/*
 * Variables passed in from voicemail subsystem (parsed in parent script)
 * We are her because $newvmcount or $urgvmcount are not 0
 *
 * $context; $mailbox; $newvmcount; $oldvmcount; $urgvmcount;
 */

# connect to db and see if $mailbox is set up with notification
# populate array with settings for notification 
# if no settings exit
#
$sql = "SELECT mailbox,recipients,outcid,outcidname,cidtype,retrycount,retrydelay,prioritydelay FROM vmnotify WHERE mailbox=? and enabled=1";
$row = $db->getRow($sql, array($mailbox), DB_FETCHMODE_ASSOC);
if ($db->IsError($row)) {
	die_freepbx("Failed to query vmnotify for mailbox $mailbox");
}

if(empty($row)) {
  //echo "No reminder configured for mailbox $mailbox\n";
  exit(0);
}

// At this point user has it enabled and there appear to be
// new messages so get them:
//
$msg_info = vmnotify_find_msg($mailbox, $context, $urgvmcount);
if ($msg_info === false) {
	freepbx_log(FPBX_LOG_ERROR, "vmnotify failed to find message for mailbox $mailbox when messages exist");
	exit(1);
}

// Now query for what to do with it:
//
$recipients = unserialize($row['recipients']);
ksort($recipients, SORT_NUMERIC);
$outcid = $row['outcid'];
$outcidname = $row['outcidname'];
$cidtype = $row['cidtype'];
$retrycount = $row['retrycount'];
$retrydelay = $row['retrydelay'];
$prioritydelay = $row['prioritydelay'];
$outcidstring = '"'.$outcidname.'" <'.$outcid.'>';

if ($cidtype=='fixed'||$cidtype=='forcemb') {
  $cidvar = "__TRUNKCIDOVERRIDE=$outcid";
}else{
  $cidvar = "__REALCALLERIDNUM=$outcid";
}

foreach ($recipients as $ptmp => $rtmp) {
  $priority = (integer) $ptmp;
  $recipients = explode(',',$rtmp);
  break;
}


# create vmnotify_notifications table entry for this notification 
#
$params = array($mailbox, $priority, $msg_info['time'], $msg_info['callerid'], $msg_info['length']);
$qry = "INSERT INTO vmnotify_notifications 
	(`mailbox`, `status`, `priority`, `time`, `callerid`, `length`) 
	VALUES (?,0,?,?,?,?)";
$result = $db->query($qry, $params);
if ($db->IsError($result)) {
	die_freepbx("Failed inserting notifications for mailbox $mailbox");
}

// get id of just inserted row
//
$vmnotifyid = sql("SELECT last_insert_id()", "getOne");

freepbx_log(FPBX_LOG_INFO, "vmnotify: inserted id $vmnotifyid for $mailbox@$context message from callerid: {$msg_info['callerid']}");

// recursively make the directory
//
$vmnotifydir = $amp_conf['ASTVARLIBDIR'] . "/sounds/vmnotify/$mailbox/$vmnotifyid";
if (!mkdir("$vmnotifydir",0775,true)) {
	freepbx_log(FPBX_LOG_ERROR, "failed to create vmnotify dir but attempting to continue: $vmnotifydir");
}

$basedir = $amp_conf['ASTVARLIBDIR'] . "/sounds/vmnotify/$mailbox";
if (!chmod($basedir, 0775)) {
	freepbx_log(FPBX_LOG_ERROR, "chmod failed on path, attempting to continue: $basedir");
}
if (!chmod("$vmnotifydir", 0775)) {
	freepbx_log(FPBX_LOG_ERROR, "chmod failed on path, attempting to continue: $vmnotifydir");
}

// Now we have to move the file to the path we created:
//
$astvmdir = $msg_info['vmpath'];
$msgnum = $msg_info['msgnum'];
exec("mv $astvmdir/msg$msgnum.* $vmnotifydir/.", $outarr, $ret);
if ($ret != 0) {
	// Something went terribly wrong here! Assuming it's not a permission or disk issue, it's
	// possible there was a race condition and someone got to that message. SO ... we're gonna
	// make a desparate last ditch effort to recover if there is another message and if so
	// if successful then we'll update the record with the new message info
	//
	freepbx_log(FPBX_LOG_ERROR, "vmnotify failed to move $astvmdir/msg$msgnum* to $vmnotifydir checking if another message is available");

	$new_msg_info = vmnotify_find_msg($mailbox, $context, $urgvmcount);
	if ($new_msg_info === false) {
		freepbx_log(FPBX_LOG_FATAL, "vmnotify failed to recover by finding another message in mailbox $mailbox so aborting");

		// TODO: Do we need to delete the DB record create before aborting?
		// Bryan thinks YES delete this record
		exit(1);
	}

	// Phew! we may have recovered, let's try the move again and if it works we'll updated the record
	//
	$astvmdir = $new_msg_info['vmpath'];
	$msgnum = $new_msg_info['msgnum'];
	exec("mv $astvmdir/msg$msgnum.* $vmnotifydir/.", $outarr, $ret);
	if ($ret != 0) {
		freepbx_log(FPBX_LOG_FATAL, "vmnotify failed again in it's retry to mv message for mailbox $mailbox so aborting");

		// TODO: Do we need to delete the DB record create before aborting?
		// Bryan thinks YES delete this record
		exit(1);
	}

	// Success! we recovered so let's update things in case the message info was different
	//
	if ($msg_info != $new_msg_info) {
		$msg_info = $new_msg_info;
		$params = array($msg_info['time'], $msg_info['callerid'], $msg_info['length'], $vmnotifyid);
		$qry = "UPDATE `vmnotify_notifications` SET `time` = ?, `callerid` = ?, `length` = ? WHERE `id` = ?";
		$result = $db->query($qry, $params);
		if ($db->IsError($result)) {
			die_freepbx("Failed inserting notifications for mailbox $mailbox");
		}
	}
	freepbx_log(FPBX_LOG_INFO, "vmnotify: updated id $vmnotifyid for $mailbox@$context message from callerid: {$msg_info['callerid']}");
} 
// If the message number isn't 0000 then change it since that's what's expected. Shouldn't happen too often
// but if multiple messages were received at once...
//
if ($msg_info['msgnum'] !== '0000') {
	exec('for file in '.$vmnotifydir.'/msg*; do bname=${file##*/}; mv "$file" "'.$vmnotifydir.'/msg0000${bname##msg????}"; done');
}
freepbx_log(FPBX_LOG_INFO, "vmnotify: moved message for $mailbox@$context to: $vmnotifydir");

// Now let's kick off the notification process
//
$calltime = time() + 5;

# create call files for each number to call
#
foreach($recipients as $rnum) {
  
	$qry = "INSERT INTO vmnotify_events (`notification_id`, `time`, `number`, `priority`, `status`) VALUES (?,?,?,?,0);";
	$params = array($vmnotifyid, $calltime, $rnum, $priority);
  $result = $db->query($qry, $params);
	if ($db->IsError($result)) {
		// TODO: should I die or keep trying for other recipients?
		die_freepbx("Failed inserting into vmnotify_events for mailbox $mailbox recipient $rnum");
	}
	$rid = sql("SELECT last_insert_id()", "getOne");

  $filecont = "
Channel: Local/$rnum@vmnotify-main
CallerID: $outcidstring
WaitTime: 30
Application: AGI
Data: vmnotify-main.php
Set: __id=$vmnotifyid
Set: __rid=$rid
Set: __rnum=$rnum
Set: $cidvar
";
  
  $filename = $amp_conf['ASTSPOOLDIR'] . "/tmp/$mailbox-$vmnotifyid-0-$priority-$rnum.call";
  $fh = fopen($filename,'w');
  fwrite($fh,$filecont);
  fclose($fh);
  touch($filename,$calltime);
  rename($filename, $amp_conf['ASTSPOOLDIR'] . "/outgoing/$mailbox-$vmnotifyid-0-$priority-$rnum.call");
  freepbx_log(FPBX_LOG_INFO, "vmnotify: launched callfile for id $vmnotifyid to priority: $priority callback number: $rnum");
}

#create audit call file

$filecont = "
Channel: Local/1@vmnotify-dummy
Application: AGI
Data: vmnotify-audit.php
Set: __id=$vmnotifyid
";
  
$filename = $amp_conf['ASTSPOOLDIR'] . "/tmp/$mailbox-$vmnotifyid-0-$priority-audit.call";
$fh = fopen($filename,'w');
fwrite($fh,$filecont);
fclose($fh);
$calltime = ((integer) time() + (integer) $prioritydelay + (integer) $msg_info['length']);
touch($filename,$calltime);
rename($filename, $amp_conf['ASTSPOOLDIR'] . "/outgoing/$mailbox-$vmnotifyid-0-$priority-audit.call");
freepbx_log(FPBX_LOG_INFO, "vmnotify: launched audit callfile for id $vmnotifyid priority: $priority");

/** Find the first message in the specified mailbox
 *
 */
function vmnotify_find_msg($mbox, $context, $check_urgent) {
	global $amp_conf;
	$spooldir = $amp_conf['ASTSPOOLDIR'];
	$varlibdir = $amp_conf['ASTVARLIBDIR'];

	if ($check_urgent) {
		$p = "$spooldir/voicemail/$context/$mbox/Urgent";
		if (is_dir($p)) {
			$vmpath[] = $p;
		}
	}
	$p = "$spooldir/voicemail/$context/$mbox/INBOX";
	if (is_dir($p)) {
		$vmpath[] = $p;
	}

	foreach ($vmpath as $path) {
		if ($handle = opendir($path)) {
			while (false !== ($msgtxt = readdir($handle))) { 
				if (preg_match('/msg(?P<msgnum>\d\d\d\d)\.txt/', $msgtxt, $matches)) {
					$msgtxt = basename($msgtxt);
					$msgpath = $path;
					$msgnum = $matches['msgnum'];
					$msginfo = parse_ini_file($msgpath . '/' . $msgtxt);
					if (preg_match('/<(?P<number>\d+)>/',$msginfo['callerid'],$matches)) {
						$callerid = $matches['number'];
					} else {
						$callerid = '';
					}
					closedir($handle); 
					return array(
						'msgnum' => $msgnum,
						'callerid' => $callerid,
						'time' => $msginfo['origtime'],
						'length' => $msginfo['duration'],
						'vmpath' => $msgpath,
					);
				}
			}
		closedir($handle); 
		}
	}
	return false;
}
