<?php
if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed'); }

if (zend_loader_file_encoded()) {
  $lic_info = zend_loader_file_licensed();
  $lic_exp  = isset($lic_info['vqplus_exp'])?preg_replace('/-/','',$lic_info['vqplus_exp']):00000000;
  if ($lic_exp > date('Ymd')) {
    $vqplus = true;
  } else {
    $vqplus = false;
  }
} else {
	  $vqplus = true;
}

// The destinations this module provides
// returns a associative arrays with keys 'destination' and 'description'

/** vqplus_destinations
 * Short drawselect callback funciton to list available destinations
 * Long returns a hash of destinations that this module provides or null
 * if not destinations are configured.
 *
 * @author Philippe Lindheimer
 * @return mixed
 */
function vqplus_destinations() {
	//get the list of all exisiting
	$results = vqplus_vqueue_get();
	
	//return an associative array with destination and description
	if (isset($results)) {
		$cat = _("Virtual Queues");
		$ds_id = 'vqueue';
		foreach($results as $result){
			$extens[] = array(
				'destination' => 'ext-vqueues,'.$result['id'].',1', 
				'description' => $result['name'].' <'.$result['id'].'>', 
				'category' => $cat,
				'id' => $ds_id
			);
		}
	}
	
	if (isset($extens)) 
		return $extens;
	else
		return null;
}

/** vqplus_getdest
 * Short returns the goto destination format for a given id/exten
 *
 * @author Philippe Lindheimer
 * @param int $exten
 * @return array
 */
function vqplus_getdest($exten) {
	return array('ext-vqueues,'.$exten.',1');
}

/** vqplus_getdestinfo
 * Short destination registry callback
 * Long callback that provides the description and URL for a given destination if one exists for
 * this module, or boolean false if the given destination is not for this module
 *
 * @author Philippe Lindheimer
 * @param string
 * @return mixed
 */
function vqplus_getdestinfo($dest) {
	global $active_modules;

	if (substr(trim($dest),0,12) == 'ext-vqueues,') {
		$exten = explode(',',$dest);
		$exten = $exten[1];
		$thisexten = vqplus_vqueue_get($exten);
		if (empty($thisexten)) {
			return array();
		} else {
			return array('description' => sprintf(_("Virtual Queue (%s) : %s"),$exten,$thisexten['name']),
			             'edit_url' => 'config.php?display=vqueue&action=modify&id='.urlencode($exten),
								  );
		}
	} else {
		return false;
	}
}

/** vqplus_recordings_usage
 * Short recordings usage callback by this module
 * Long callback to the recordings module that provides a list of URLs and descriptions
 * of those configurations that use the given recording ID, or an empty array if the ID
 * is not used by this module.
 *
 * @author Philippe Lindheimer
 * @param int
 * @return array
 */
function vqplus_recordings_usage($recording_id) {
	global $active_modules;

	$results = sql("SELECT `id`, `name` FROM `virtual_queue_config` WHERE `callconfirm_id` = '$recording_id' OR `agentannounce_id` = '$recording_id' OR `joinannounce_id` = '$recording_id'","getAll",DB_FETCHMODE_ASSOC);
	if (empty($results)) {
		return array();
	} else {
		foreach ($results as $result) {
			$usage_arr[] = array(
			  'url_query' => 'config.php?display=vqueue&action=modify&id='.urlencode($result['id']),
				'description' => sprintf(_("Virtual Queue (%s): %s"),$result['id'],$result['name']),
			);
		}
		return $usage_arr;
	}
}

/** vqplus_check_destinations
 * Short destination registry callback
 * Long callback that lists all the configurations in this module that use any of the destinations
 * that are passed in which can be one destination or an array of destinations, or if boolean
 * true, it passes back all module instances that use any destination and which ones.
 *
 * @author Philippe Lindheimer
 * @param mixed
 * @return array
 */
function vqplus_check_destinations($dest=true) {
	global $active_modules;

	$destlist = array();
	if (is_array($dest) && empty($dest)) {
		return $destlist;
	}

	/* get virtual queues
	 */
	$sql = "SELECT id, name, dest, cdest, adest, gotodest, full_dest, joinempty_dest, leaveempty_dest, 
		joinunavail_dest, leaveunavail_dest FROM virtual_queue_config";
	if ($dest !== true) {
		$sql .= " WHERE 
			dest IN ('".implode("','",$dest)."') OR 
			cdest IN ('".implode("','",$dest)."') OR 
			adest IN ('".implode("','",$dest)."')  OR 
			gotodest IN ('".implode("','",$dest)."') OR
			full_dest IN ('".implode("','",$dest)."') OR
			joinempty_dest IN ('".implode("','",$dest)."') OR
			leaveempty_dest IN ('".implode("','",$dest)."') OR
			joinunavail_dest IN ('".implode("','",$dest)."') OR
			leaveunavail_dest IN ('".implode("','",$dest)."')
		";
	}
	$results = sql($sql,"getAll",DB_FETCHMODE_ASSOC);

	foreach ($results as $result) {
		$thisid      = $result['id'];
		$description = sprintf(_("Virtual Queue: %s"),$result['name']);
		$thisurl     = 'config.php?display=vqueue&id='.urlencode($thisid);

		// The main dest should not be blank, the others are allowd to be blank
		//
		$thisdest =	$result['dest'];
		if ($dest === true || $dest == $thisdest) {
			$destlist[] = array(
				'dest' => $thisdest,
				'description' => $description,
				'edit_url' => $thisurl,
			);
		}
		$dests = array(
			$result['cdest'], 
			$result['adest'], 
			$result['gotodest'],
			$result['full_dest'],
			$result['joinempty_dest'],
			$result['leaveempty_dest'],
			$result['joinunavail_dest'],
			$result['leaveunavail_dest']
		);
		foreach ($dests as $thisdest) {
			if ($thisdest != '' && ($dest === true || $dest == $thisdest)) {
				$destlist[] = array(
					'dest' => $thisdest,
					'description' => $description,
					'edit_url' => $thisurl,
				);
			}
		}
	}

	/* get hooks into queues module
	 */
	$sql = "SELECT queue_num, b.descr name, cdest, adest, full_dest, joinempty_dest, leaveempty_dest, joinunavail_dest, leaveunavail_dest FROM vqplus_queue_config a JOIN queues_config b ON a.queue_num = b.extension";
	if ($dest !== true) {
		$sql .= " WHERE
			dest IN ('".implode("','",$dest)."') OR 
			cdest IN ('".implode("','",$dest)."') OR 
			adest IN ('".implode("','",$dest)."')  OR 
			full_dest IN ('".implode("','",$dest)."') OR
			joinempty_dest IN ('".implode("','",$dest)."') OR
			leaveempty_dest IN ('".implode("','",$dest)."') OR
			joinunavail_dest IN ('".implode("','",$dest)."') OR
			leaveunavail_dest IN ('".implode("','",$dest)."')
		";
	}
	$results = sql($sql,"getAll",DB_FETCHMODE_ASSOC);

	foreach ($results as $result) {
		$thisid      = $result['queue_num'];
		$description = sprintf(_("Queue %s : %s"), $result['queue_num'], $result['name']);
		$thisurl     = 'config.php?display=queue&extdisplay='.urlencode($thisid);

		$dests = array(
			$result['cdest'], 
			$result['adest'], 
			$result['full_dest'],
			$result['joinempty_dest'],
			$result['leaveempty_dest'],
			$result['joinunavail_dest'],
			$result['leaveunavail_dest']
		);
		foreach ($dests as $thisdest) {
			if ($thisdest != '' && ($dest === true || $dest == $thisdest)) {
				$destlist[] = array(
					'dest' => $thisdest,
					'description' => $description,
					'edit_url' => $thisurl,
				);
			}
		}
	}
	return $destlist;
}

/** vqplus_check_destinations
 * Short callback to update a registration
 * Long callback given two destinations, will replace all instances
 * of the first with the second that are used in this module.
 *
 * @author Philippe Lindheimer
 * @param string
 * @param string
 */
function vqplus_change_destination($old_dest, $new_dest) {
	$fields = array(
		'dest', 
		'cdest', 
		'adest', 
		'gotodest', 
		'full_dest', 
		'joinempty_dest', 
		'leaveempty_dest', 
		'joinunavail_dest', 
		'leaveunavail_dest'
	);
	foreach ($fields as $field) {
		$sql = 'UPDATE virtual_queue_config SET ' . $field . ' = "' . $new_dest . '" WHERE ' . $field . ' = "' . $old_dest . '"';
		sql($sql, "query");
	}

	$fields = array(
		'cdest', 
		'adest', 
		'full_dest', 
		'joinempty_dest', 
		'leaveempty_dest', 
		'joinunavail_dest', 
		'leaveunavail_dest'
	);
	foreach ($fields as $field) {
		$sql = 'UPDATE vqplus_queue_config SET ' . $field . ' = "' . $new_dest . '" WHERE ' . $field . ' = "' . $old_dest . '"';
		sql($sql, "query");
	}
}
?>
