<?php
if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed'); }

if (zend_loader_file_encoded()) {
  $lic_info = zend_loader_file_licensed();
  $lic_exp  = isset($lic_info['vqplus_exp'])?preg_replace('/-/','',$lic_info['vqplus_exp']):00000000;
  if ($lic_exp > date('Ymd')) {
    $vqplus = true;
  } else {
    $vqplus = false;
  }
} else {
	  $vqplus = true;
}

/* 	Generates dialplan for "queues" components (extensions & inbound routing)
	We call this with retrieve_conf
*/
/** vqplus_get_config
 * Short dialplan generation for this module
 * Long dialplan generation as well as population of conf_classes etc
 * that this module is responsible for.
 *
 * @author Philippe Lindheimer
 * @param string
 */
function vqplus_get_config($engine) {
	global $ext, $amp_conf, $version, $vqplus;

	if (!$vqplus) {
		return false;
	}

	switch($engine) {
		case "asterisk":
			global $astman;

			$vqplus_conf = vqplus_conf::create();

			/* virtual queue destinations */

			$context = 'ext-vqueues';
			
			$vqueues = vqplus_vqueue_get();
			foreach($vqueues as $exten => $vals) {
				if ($vals['priority'] !== null) {
					$ext->add($context, $exten, '', new ext_set('_QUEUE_PRIO', $vals['priority']));
				}
				if ($vals['position'] !== null) {
					$ext->add($context, $exten, '', new ext_set('VQ_POSTION', $vals['position']));
				}
				if ($vals['min_penalty'] !== null) {
					$ext->add($context, $exten, '', new ext_set('VQ_MIN_PENALTY', $vals['min_penalty']));
				}
				if ($vals['max_penalty'] !== null) {
					$ext->add($context, $exten, '', new ext_set('VQ_MAX_PENALTY', $vals['max_penalty']));
				}
				if ($vals['rule_id'] !== null) {
					$ext->add($context, $exten, '', new ext_set('VQ_RULE', $vals['rule_id']));
				}
				if ($vals['cidpp'] != '') {
					$ext->add($context, $exten, '', new ext_set('VQ_CIDPP', $vals['cidpp']));
				}
				if ($vals['alertinfo'] != '') {
					$ext->add($context, $exten, '', new ext_set('VQ_AINFO', $vals['alertinfo']));
				}
				if ($vals['language'] != '') {
					$ext->add($context, $exten, '', new ext_set('CHANNEL(language)', $vals['language']));
				}
				if ($vals['maxwait'] != -1) {
					$ext->add($context, $exten, '', new ext_set('VQ_MAXWAIT', $vals['maxwait']));
				}
				if ($vals['music'] != '') {
					$ext->add($context, $exten, '', new ext_set('VQ_MOH', $vals['music']));
				}
				if ($vals['joinannounce_id'] !== null) {
					$joinannounce = $vals['joinannounce_id'] == 0 ? 0 : recordings_get_file($vals['joinannounce_id']);
					$ext->add($context, $exten, '', new ext_set('VQ_JOINMSG', $joinannounce));
				}
				if ($vals['agentannounce_id'] !== null) {
					$agentannounce = $vals['agentannounce_id'] == 0 ? 0 : recordings_get_file($vals['agentannounce_id']);
					$ext->add($context, $exten, '', new ext_set('VQ_AANNOUNCE', $agentannounce));
				}
				if ($vals['callconfirm_id'] !== null) {
					$callconfirm = $vals['callconfirm_id'] == 0 ? 0 : recordings_get_file($vals['callconfirm_id']);
					$ext->add($context, $exten, '', new ext_set('VQ_CONFIRMMSG', $callconfirm));
				}
				if ($vals['dest'] != '') {
					$ext->add($context, $exten, '', new ext_set('VQ_DEST', str_replace(',', '^', $vals['dest'])));
				}
				if ($vals['cdest'] != '') {
					$ext->add($context, $exten, '', new ext_set('VQ_CDEST', str_replace(',', '^', $vals['cdest'])));
				}
				if ($vals['adest'] != '') {
					$ext->add($context, $exten, '', new ext_set('VQ_ADEST', str_replace(',', '^', $vals['adest'])));
				}
				if ($vals['full_dest'] != '') {
					$ext->add($context, $exten, '', new ext_set('VQ_FULL_DEST', str_replace(',', '^', $vals['full_dest'])));
				}
				if ($vals['joinempty_dest'] != '') {
					$ext->add($context, $exten, '', new ext_set('VQ_JOINEMPTY_DEST', str_replace(',', '^', $vals['joinempty_dest'])));
				}
				if ($vals['leaveempty_dest'] != '') {
					$ext->add($context, $exten, '', new ext_set('VQ_LEAVEEMPTY_DEST', str_replace(',', '^', $vals['leaveempty_dest'])));
				}
				if ($vals['joinunavail_dest'] != '') {
					$ext->add($context, $exten, '', new ext_set('VQ_JOINUNAVAIL_DEST', str_replace(',', '^', $vals['joinunavail_dest'])));
				}
				if ($vals['leaveunavail_dest'] != '') {
					$ext->add($context, $exten, '', new ext_set('VQ_LEAVEUNAVAIL_DEST', str_replace(',', '^', $vals['leaveunavail_dest'])));
				}

				$ext->add($context, $exten, '', new ext_goto($vals['gotodest']));
			}
			
			$qrules = vqplus_qrules_list();
			if (!empty($qrules)) foreach ($qrules as $id => $name) {
				$rule_set = vqplus_qrules_get($id, false, true);
				$rule_set['name'] = $name;
				$vqplus_conf->addQueueRules($id, $rule_set);
			}

		break;
	}
}

/** vqplus_hookGet_config
 * Short dialplan generation splicing into other modules
 * Long dialplan generation that needs to splice into other
 * modules and/or be assured that all instances of _get_config()
 * have alreayd executed.
 *
 * @author Philippe Lindheimer
 * @param string
 */
function vqplus_hookGet_config($engine) {
	global $ext, $vqplus;

	$queues_conf = queues_conf::create();

	if (!$vqplus) {
		return false;
	}

	switch($engine) {
		case "asterisk":

			$c = 'ext-queues';

			$generate_upil = false;
			$ql = queues_list(true);
			foreach ($ql as $item) {
				$qnum = $item[0];
				$vars = vqplus_queues_get($qnum);

				$sl = 'qcall';

				$min_penalty = $vars['vqplus_min_penalty'] != null && $vars['vqplus_min_penalty'] !='' ? $vars['vqplus_min_penalty'] : ' ';
				$ext->splice($c, $qnum, $sl, new ext_set('QMIN_PENALTY', '${IF($[${LEN(${VQ_MIN_PENALTY})}>0]?${VQ_MIN_PENALTY}:' . $min_penalty . ')}'));
				$ext->splice($c, $qnum, $sl, new ext_set('VQ_MIN_PENALTY', ''));
				$ext->splice($c, $qnum, $sl, new ext_execif('$[${LEN(${QMIN_PENALTY})}>0]', 'Set', 'QUEUE_MIN_PENALTY=${QMIN_PENALTY}'));

				$max_penalty = $vars['vqplus_max_penalty'] != null && $vars['vqplus_max_penalty'] !='' ? $vars['vqplus_max_penalty'] : ' ';
				$ext->splice($c, $qnum, $sl, new ext_set('QMAX_PENALTY', '${IF($[${LEN(${VQ_MAX_PENALTY})}>0]?${VQ_MAX_PENALTY}:' . $max_penalty . ')}'));
				$ext->splice($c, $qnum, $sl, new ext_set('VQ_MAX_PENALTY', ''));
				$ext->splice($c, $qnum, $sl, new ext_execif('$[${LEN(${QMAX_PENALTY})}>0]', 'Set', 'QUEUE_MAX_PENALTY=${QMAX_PENALTY}'));

				$cdest = $vars['vqplus_cdest'] ? str_replace(',','^',$vars['vqplus_cdest']) : ' ';
				$ext->splice($c, $qnum, $sl, new ext_set('_CALLER_DEST', '${IF($[${LEN(${VQ_CDEST})}>0]?${VQ_CDEST}:' . $cdest . ')}'));
				$ext->splice($c, $qnum, $sl, new ext_set('VQ_CDEST', ''));

				$adest = $vars['vqplus_adest'] ? str_replace(',','^',$vars['vqplus_adest']) : ' ';
				$ext->splice($c, $qnum, $sl, new ext_set('_AGENT_DEST', '${IF($[${LEN(${VQ_ADEST})}>0]?${VQ_ADEST}:' . $adest . ')}'));
				$ext->splice($c, $qnum, $sl, new ext_set('VQ_ADEST', ''));

				if ($vars['vqplus_upil'] == 'yes') {
					$sub_upil = "sub-unpause-if-life";
					$ext->splice($c, $qnum, $sl, new ext_gosub('1', 's', $sub_upil, $qnum));
					$generate_upil = true;
				}


				if ($vars['vqplus_rule_id'] != '') {
					$ext->splice($c, $qnum, 'qrule', new ext_set('QRULE', $vars['vqplus_rule_id']));
				}

				$sl = 'gotodest';

				$ext->splice($c, $qnum, $sl, new ext_noop_trace('QUEUESTATUS: [${QUEUESTATUS}]'));

				$ext->splice($c, $qnum, $sl, new ext_set('_CALLER_DEST', ''));
				$ext->splice($c, $qnum, $sl, new ext_set('_AGENT_DEST', ''));

				$states = array('FULL', 'JOINEMPTY', 'LEAVEEMPTY', 'JOINUNAVAIL', 'LEAVEUNAVAIL');
				foreach ($states as $state) {
					$QDEST = 'Q' . $state . '_DEST';
					$VQ_DEST = 'VQ_' . $state . '_DEST';
					$idx = 'vqplus_' . strtolower($state) . '_dest';
					$dest = $vars[$idx] ? str_replace(',','^',$vars[$idx]) : ' ';
					$ext->splice($c, $qnum, $sl, new ext_set($QDEST, '${IF($[${LEN(${' . $VQ_DEST . '})}">0]?${' . $VQ_DEST . '}:' . $dest . ')}'));
					$ext->splice($c, $qnum, $sl, new ext_set($VQ_DEST, ''));
					$ext->splice($c, $qnum, $sl, new ext_gotoif('$["${QUEUESTATUS}" = "' . $state . '" & ${LEN(${' . $QDEST . '})}>0]','${CUT(' . $QDEST . ',^,1)},${CUT(' . $QDEST . ',^,2)},${CUT(' . $QDEST . ',^,3)}'));
				}
				
				if ($vars['vqplus_lazymembers'] != '') {
					if (method_exists($queues_conf, 'addQueuesAdditional')) {
						$queues_conf->addQueuesAdditional($qnum, 'lazymembers', $vars['vqplus_lazymembers']);
					}
                        	}
			}
			if ($generate_upil) {
				/** 
				 * Check if there are any callers waiting in the queue, if not
				 * then un-pause all the agents so the new caller will attempt
				 * each if available.
				 * 
				 * If there are caller in the queue, then check all the agents
				 * to see if any of them are active. If so, un-pause all agents
				 * in case one of them had been paused but returned, and send
				 * the caller into the queue
				 * 
				 * End result: if there are no callers in the queue, or if there
				 * are callers in the queue and ANY agents show life, we'll
				 * make sure to unpause all agents and send the caller to the queue
				 * 
				 */
				$c = $sub_upil;
				$ex = 's';

				$ext->add($c, $ex, '', new ext_gotoif('$[${QUEUE_WAITING_COUNT(${ARG1})} > 0]','checklife'));
				$ext->add($c, $ex, '', new ext_gosub('1', 'up', $c, '${ARG1}'));
				$ext->add($c, $ex, '', new ext_return(''));

				$ext->add($c, $ex, 'checklife', new ext_set('LOCAL(AGENTS)', '${QUEUE_MEMBER_LIST(${ARG1})}'));
				$ext->add($c, $ex, '', new ext_while('$["${AGENTS}" != ""]'));
				$ext->add($c, $ex, '', new ext_set('LOCAL(QA)', '${CUT(CUT(POP(AGENTS,\,),@,1),/,2)}'));
				$ext->add($c, $ex, '', new ext_set('LOCAL(QS)', '${EXTENSION_STATE(${QA}@ext-local)}'));
				$ext->add($c, $ex, '', new ext_gotoif('$[${QS}=INUSE | ${QS}=BUSY | ${QS}=RINGINUSE | ${QS}=HOLDINUSE | ${QS}=ONHOLD]', 'life', 'endwhile'));
				$ext->add($c, $ex, 'life', new ext_gosub('1', 'up', $c, '${ARG1}'));
				$ext->add($c, $ex, '', new ext_return(''));
				$ext->add($c, $ex, 'endwhile', new ext_endwhile(''));
				$ext->add($c, $ex, '', new ext_return(''));

				$ex = 'up';
				$ext->add($c, $ex, '', new ext_set('LOCAL(AGENTS)', '${QUEUE_MEMBER_LIST(${ARG1})}'));
				$ext->add($c, $ex, '', new ext_while('$["${AGENTS}" != ""]'));
				$ext->add($c, $ex, '', new ext_set('QUEUE_MEMBER(${ARG1},paused,${POP(AGENTS,\,)})', '0'));
				$ext->add($c, $ex, '', new ext_endwhile(''));
				$ext->add($c, $ex, '', new ext_return(''));
			}
		break;
	}
}
