<?php
if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed'); }

/* Virtual Queue APIs
 */

/** vqplus_vqueue_get
 * Short get virtual queue setting or all settings
 * Long pass in boolean false to get all settings, otherwise
 * the id of the desired virtual queue
 *
 * @author Philippe Lindheimer
 * @param mixed $id
 * @return array
 */
function vqplus_vqueue_get($id=false) {
	global $db;

	if ($id === false) {
		$sql = "SELECT * FROM virtual_queue_config";
		$res = sql($sql,'getAll',DB_FETCHMODE_ASSOC);
		foreach($res as $vq) {
			$results[$vq['id']] = $vq;
		}
	} else {
		$id = $db->escapeSimple($id);
		$sql = "SELECT * FROM virtual_queue_config WHERE id = $id";
		$results = sql($sql,'getRow',DB_FETCHMODE_ASSOC);
	}
	return $results;
}

/** vqplus_vqueue_save
 * Short insert or update virtual queue settings
 * Long takes array of settings to update, missing settings will
 * get default values, if id not present it will insert a new row.
 * Returns the id of the current or newly inserted record or
 * boolean false upon a failure.
 *
 * @author Philippe Lindheimer
 * @param array $parms
 * @return mixed
 */
function vqplus_vqueue_save($parms=array()) {
	global $db, $amp_conf;

	if (!empty($parms['id'])) {
		$var['id'] = $db->escapeSimple($parms['id']);
	}
	$var['name'] = "Virtual Queue";
	$var['priority'] = null;
	$var['position'] = null;
	$var['min_penalty'] = null;
	$var['max_penalty'] = null;
	$var['rule_id'] = null;
	$var['joinannounce_id'] = null;
	$var['agentannounce_id'] = null;
	$var['music'] = null;
	$var['maxwait'] = null;
	$var['cidpp'] = '';
	$var['alertinfo'] = '';
	$var['language'] = '';
	$var['callconfirm_id'] = null;
	$var['dest'] = '';
	$var['cdest'] = '';
	$var['adest'] = '';
	$var['gotodest'] = '';
	$var['full_dest'] = '';
	$var['joinempty_dest'] = '';
	$var['leaveempty_dest'] = '';
	$var['joinunavail_dest'] = '';
	$var['leaveunavail_dest'] = '';

	foreach ($var as $k => $v) {
		if (isset($parms[$k])) {
			$var[$k] = $db->escapeSimple($parms[$k]);
		}
	}

	$fields = "name, priority, position, min_penalty, max_penalty, rule_id, joinannounce_id,
		agentannounce_id, music, maxwait, cidpp, alertinfo, language, callconfirm_id, dest, cdest, adest, gotodest,
	  full_dest, joinempty_dest, leaveempty_dest, joinunavail_dest, leaveunavail_dest";
	$holders = "?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?";

	if (empty($var['id'])) {
		$sql = "INSERT INTO virtual_queue_config ($fields) VALUES ($holders)";
	} else {
		$sql = "REPLACE INTO virtual_queue_config (id, $fields) VALUES (?,$holders)";
	}

	$res = $db->query($sql,$var);
	if (DB::IsError($res)) {
		$id = false;
		// TODO log error
	} elseif (empty($var['id'])) {
  	$id = $amp_conf["AMPDBENGINE"] == "sqlite3" ? sqlite_last_insert_rowid($db->connection) : mysql_insert_id($db->connection);
	} else {
		$id = $var['id'];
	}
	return $id;
}

/** vqplus_vqueue_del
 * Short deletes the record of the provided id
 * Long will delete the requested row and will die upon a db error
 * since normally deleting a row that does not exist will not result
 * in a db error
 *
 * @author Philippe Lindheimer
 * @param int $id
 * @return boolean
 */
function vqplus_vqueue_del($id) {
	global $db, $amp_conf;

	$id = $db->escapeSimple($id);
	$sql = "DELETE FROM virtual_queue_config WHERE id = '$id'";
	$result = $db->query($sql);
	if($db->IsError($result)) {
		die_freepbx($result->getMessage().$sql);
	}
	return true;
}

/*  Queue Rule APIs
 */

/** vqplus_qrules_list
 * Short provide a list of qrules
 * Long provides a hash of the qrules where the id is the index and the
 * rule name is the value
 *
 * @author Philippe Lindheimer
 * @return array
 */
function vqplus_qrules_list() {
	global $db;

	$sql = "SELECT id, name FROM vqplus_qrule_config";
	$res = sql($sql,'getAll',DB_FETCHMODE_ASSOC);
	foreach($res as $qr) {
		$results[$qr['id']] = $qr['name'];
	}
	return $results;
}

/** vqplus_qrules_get
 * Short retrieve a qrule set
 * Long retireves a qrule set for the given $id, setting $get_name to true
 * will also retireve the name and setting $raw to true will pass back
 * the raw data as elapsed time vs. the default which is to convert it to
 * duration for each interaval. The returned array will have two field, the optional
 * 'name' and another array under 'qrules' which will include 'duration', 'min_penalty'
 * and 'max_penalty' and 'duration' will be 'elapsed' if $raw is true
 *
 * @author Philippe Lindheimer
 * @param int $id
 * @param boolean $get_name (false)
 * @param boolean $raw (false)
 * @return array
 */
function vqplus_qrules_get($id, $get_name=false, $raw=false) {
	global $db;
	$result = array();

	$id = $db->escapeSimple($id);
	if ($get_name) {
		$result['name'] = sql("SELECT `name` FROM `vqplus_qrule_config` WHERE `id` = $id", 'getOne');
	}

	$sql = "SELECT elapsed, min_penalty, max_penalty FROM vqplus_qrule_detail WHERE rule_id = $id ORDER BY elapsed";
	$res = sql($sql,'getAll',DB_FETCHMODE_ASSOC);
	if ($raw) {
		$result['qrules'] = $res;
	} else {
		$last = 0;
		foreach ($res as $row) {
			$duration = $row['elapsed'] - $last;
			$last = $row['elapsed'];
			$result['qrules'][] = array(
				'duration' => $duration,
				'min_penalty' => $row['min_penalty'],
				'max_penalty' => $row['max_penalty']
			);
		}
	}
	return $result;
}

/** vqplus_qrules_save
 * Short insert or update a rule set
 * Long insert or update a rule name and set of 10 duration, min_penalty and
 * max_penalty rows of rules for that rule set. Returns the $id of the inserted
 * or updated set or false upon an error.
 *
 * @author Philippe Lindheimer
 * @param array $parms
 * @return mixed
 */
function vqplus_qrules_save($parms=array()) {
	global $db, $amp_conf;

	if (!empty($parms['id'])) {
		$var['id'] = $db->escapeSimple($parms['id']);
	}

	$init_arr = array('','','','','','','','','','');
	$var['name'] = "Penalty Rules";
	$var['duration'] = $init_arr;
	$var['min_penalty'] = $init_arr;
	$var['max_penalty'] = $init_arr;

	foreach ($var as $k => $v) {
		if (isset($parms[$k])) {
			$var[$k] = $parms[$k];
			if (!is_array($var[$k])) {
				$var[$k] = $db->escapeSimple($var[$k]);
			} else {
				foreach ($var[$k] as $i => $val) {
					$var[$k][$i] = $db->escapeSimple($val);
				}
			}
		}
	}

	$last = 0;
	foreach ($var['duration'] as $i => $dur) {
		if ($dur == '') {
			continue;
		}
		$last = $dur + $last;

		/* kludge validation, strip out all non-numerics. Then if the first character was
		 * +/- put it back since relative values are allowed. Lazy validation but ...
		 */
		$min = trim($var['min_penalty'][$i]);
		$pmin = ($min[0] == '+' || $min[0] == '-' ? $min[0] : '') . preg_replace("/[^0-9]/","",$min);
		$max = trim($var['max_penalty'][$i]);
		$pmax = ($max[0] == '+' || $max[0] == '-' ? $max[0] : '') . preg_replace("/[^0-9]/","",$max);
		$insert_arr[] = array($last, $pmin, $pmax);
	}

	if (empty($var['id'])) {
		$sql = "INSERT INTO vqplus_qrule_config (name) VALUES (?)";
		$res = $db->query($sql, array($var['name']));
	} else {
		$sql = "UPDATE vqplus_qrule_config SET name = ? WHERE id = ?";
		$res = $db->query($sql, array($var['name'], $var['id']));
	}
	if (DB::IsError($res)) {
		dbug("DB ERROR updating name", $res);
		// TODO log error
		return false;
	} elseif (empty($var['id'])) {
  	$id = $amp_conf["AMPDBENGINE"] == "sqlite3" ? sqlite_last_insert_rowid($db->connection) : mysql_insert_id($db->connection);
	} else {
		$id = $var['id'];
	}
	// Even if this is new, delete just in case some previous bug left things corrupted with this id
	//
	$sql = "DELETE FROM vqplus_qrule_detail WHERE rule_id = $id";
	sql($sql);

	$fields = "rule_id, elapsed, min_penalty, max_penalty";
	$holders = $id . ",?,?,?";
	$sql = "INSERT INTO vqplus_qrule_detail ($fields) VALUES ($holders)";
	$compiled = $db->prepare($sql);
	$res = $db->executeMultiple($compiled, $insert_arr);

	if (DB::IsError($res)) {
		// TODO log error
		dbug("DB ERROR updating details", $res);
		return false;
	}
	return $id;
}

/** vqplus_qrules_del
 * Short delete a rule set
 * Long  delete the rule set for the given $id, dies on a db error since an invalid
 * $id should not result in a failure, otherwise returns true
 *
 * @author Philippe Lindheimer
 * @param int $id
 * @return boolean
 */
function vqplus_qrules_del($id) {
	global $db, $amp_conf;

	$id = $db->escapeSimple($id);
	$sql = "DELETE FROM vqplus_qrule_detail WHERE rule_id = $id";
	sql($sql);
	$sql = "DELETE FROM vqplus_qrule_config WHERE id = $id";
	sql($sql);

	return true;
}

/*  VQplus Queue Extension APIs (for Queue GUI Hooks)
 */

/** vqplus_queues_get
 * Short get the extended queue settings for the given $queue_num
 * Long returns an array of additional queue settings for the given queue
 * number and default settings where not set. Dies on a db eror since
 * non-existent queue or one not setup will simply return default values.
 *
 * @author Philippe Lindheimer
 * @param int $queue_num
 * @return array
 */
function vqplus_queues_get($queue_num) {
	global $db, $amp_conf;

	$vars['vqplus_queue_num'] = null;
	$vars['vqplus_min_penalty'] = null;
	$vars['vqplus_max_penalty'] = null;
	$vars['vqplus_rule_id'] = null;
	$vars['vqplus_cdest'] = '';
	$vars['vqplus_adest'] = '';
	$vars['vqplus_full_dest'] = '';
	$vars['vqplus_joinempty_dest'] = '';
	$vars['vqplus_leaveempty_dest'] = '';
	$vars['vqplus_joinunavail_dest'] = '';
	$vars['vqplus_leaveunavail_dest'] = '';
	$vars['vqplus_upil'] = 'no';
	$vars['vqplus_lazymembers'] = 'no';

	if ($queue_num != '') {
		$qn = $db->escapeSimple($queue_num);
		$sql = "SELECT * FROM vqplus_queue_config WHERE queue_num = $qn";
		$results = sql($sql,'getRow',DB_FETCHMODE_ASSOC);

		foreach ($results as $k => $v) {
			$vars['vqplus_' . $k] = $v;
		}
	}
	unset($vars['vqplus_queue_num']);
	return $vars;
}

/** vqplus_queues_save
 * Short save a set of extended queue settings
 * Long given a set of extended queue settings associated with the queue listed in
 * $parms['account'] they will be saved to the db. Returns boolean true upon success
 * and false upon a db error
 *
 * @author Philippe Lindheimer
 * @param array $parms
 * @return boolean
 */
function vqplus_queues_save($parms=array()) {
	global $db, $amp_conf;

	$var['queue_num'] = $db->escapeSimple($parms['account']);
	$var['min_penalty'] =  isset($parms['vqplus_min_penalty']) ? $db->escapeSimple($parms['vqplus_min_penalty']) : null;
	$var['max_penalty'] = isset($parms['vqplus_max_penalty']) ? $db->escapeSimple($parms['vqplus_max_penalty']) : null;
	$var['rule_id'] = isset($parms['vqplus_rule_id']) ? $db->escapeSimple($parms['vqplus_rule_id']) : null;
	$var['cdest'] = $db->escapeSimple($parms['vqplus_cdest']);
	$var['adest'] = $db->escapeSimple($parms['vqplus_adest']);

	$var['full_dest'] = $db->escapeSimple($parms['vqplus_full_dest']);
	$var['joinempty_dest'] = $db->escapeSimple($parms['vqplus_joinempty_dest']);
	$var['leaveempty_dest'] = $db->escapeSimple($parms['vqplus_leaveempty_dest']);
	$var['joinunavail_dest'] = $db->escapeSimple($parms['vqplus_joinunavail_dest']);
	$var['leaveunavail_dest'] = $db->escapeSimple($parms['vqplus_leaveunavail_dest']);

	$var['upil'] = $db->escapeSimple($parms['vqplus_upil']);
	$var['lazymembers'] = $db->escapeSimple($parms['vqplus_lazymembers']);

	$fields = "queue_num, min_penalty, max_penalty, rule_id, cdest, adest, full_dest,
		joinempty_dest, leaveempty_dest, joinunavail_dest, leaveunavail_dest, upil, lazymembers";
	$holders = "?,?,?,?,?,?,?,?,?,?,?,?,?";

	if ($var['queue_num'] != '') {
		$sql = "REPLACE INTO vqplus_queue_config ($fields) VALUES ($holders)";
	}
	$res = $db->query($sql,$var);
	if (DB::IsError($res)) {
		dbug("FATAL ERROR UPDATING vqplus_queue_config");
		return false;
	}
	return true;
}

/** vqplus_queues_del
 * Short delete the extended settings associated with $queue_num
 * Long delete's the extended queue settings associated with the
 * $queue_num passed, returns true unless a db error occurs and dies
 * in that sceanrio.
 *
 * @author Philippe Lindheimer
 * @param int $queue_num
 * @return boolean
 */
function vqplus_queues_del($queue_num) {
	global $db, $amp_conf;

	$qn = $db->escapeSimple($queue_num);
	$sql = "DELETE FROM vqplus_queue_config WHERE queue_num = '$qn'";
	$result = $db->query($sql);
	if($db->IsError($result)) {
		die_freepbx($result->getMessage().$sql);
	}
	return true;
}

function vqplus_sysadmin_getinfo() {
    return array(
        "name" => "Virtual Queues Plus", // Required. Descriptive name.
        "expires" => true,         // Optional. Defaults to false (never expires)
        "keyname" => "vqplus_exp", // Optional. Only checked if 'expires' is true
    );
}
