<?php
if (!defined('FREEPBX_IS_AUTH')) { die('No direct script access allowed'); }

if (zend_loader_file_encoded()) {
  $lic_info = zend_loader_file_licensed();
  $lic_exp  = isset($lic_info['vqplus_exp'])?preg_replace('/-/','',$lic_info['vqplus_exp']):00000000;
  if ($lic_exp > date('Ymd')) {
    $vqplus = true;
  } else {
    $vqplus = false;
  }
} else {
	  $vqplus = true;
}

/**
 * vqplus_conf class
 * short used to accumulate and then generate queuerules.conf settings
 *
 * @author Philippe Lindheimer
 */
class vqplus_conf {

	var $_queuerules = array();

	private static $obj;

	// FreePBX magic ::create() call
	public static function create() {
		if (!isset(self::$obj))
			self::$obj = new vqplus_conf();

		return self::$obj;
	}

	public function __construct() {
		self::$obj = $this;
	}

	/** get_filename
 	* Short returns array of file names to write out
 	*
 	* @author Philippe Lindheimer
 	* @return array
 	*/
	function get_filename() {
		$files = array(
			'queuerules_additional.conf',
		);
		return $files;
	}
	
	/** generateConf
 	* Short returns array of file names to write out
	* Long given a filename obtained from get_filename()
	* this will return the contents of that file that
	* should be written out.
 	*
 	* @author Philippe Lindheimer
	* @params	string
	* @return string
 	*/
	function generateConf($file) {
		global $version, $vqplus;

		if (!$vqplus) {
			return '';
		}

		switch ($file) {
			case 'queuerules_additional.conf':
				return $this->generate_queuerules_additional($version);
				break;
		}
	}

	/** addQueueRules
 	* Short accumulate a new rule set
	* Long given a unique rule identifier and full rule set this will
	* accumulate the rule information that will eventually be written
	* out to the conf file.
 	*
 	* @author Philippe Lindheimer
	* @params	string
	* @params	array
 	*/
	function addQueueRules($rule_id, $rule_set) {
		$this->_queuerules[] = array('rule_id' => $rule_id, 'rule_set' => $rule_set);
	}

	/** generate_queuerules_additional
 	* Short returns content of queuerules_additional.conf
	* Long called by generateConf to return the content of
	* the queuerules_additional.conf file
 	*
 	* @author Philippe Lindheimer
	* @params	string
	* @return string
 	*/
	function generate_queuerules_additional($ast_version) {
		$output = "";
		foreach ($this->_queuerules as $rule) {
			$output .= "\n; " . $rule['rule_set']['name'] . "\n;\n";
			$output .= "[" . $rule['rule_id'] . "]\n";
			foreach ($rule['rule_set']['qrules'] as $r) {
				$output .= "penaltychange => " . $r['elapsed'] . "," . $r['max_penalty'] . "," . $r['min_penalty'] . "\n";
			}
		}
		return $output;
	}
}
?>
