<?php

//set hooks
function xmpp_configpageinit($pagename) {
	global $CC;
	$get_vars = array(
					'display'		=> '',
					'extdisplay'	=> '',
				);

	foreach ($get_vars as $k => $v) {
		$vars[$k] = isset($_REQUEST[$k]) ? $_REQUEST[$k] : $v;
	}

	//only hook in to extensions or users page when editing
	if (in_array($vars['display'], array('extensions', 'users'))
		&& $vars['extdisplay']
	) {
		$CC->addprocessfunc('xmpp_core_load', 1);
		$CC->addprocessfunc('xmpp_core_process', 1);
	}
}

//getters && setters
//get users
function xmpp_users_get($id = '') {
	global $db;

	$sql = 'SELECT * FROM xmpp_users';
	if ($id) {
		$sql .= ' WHERE user = ?';
		$ret = $db->getAll($sql, array($id), DB_FETCHMODE_ASSOC);
	} else {
		$ret = $db->getAll($sql, DB_FETCHMODE_ASSOC);
	}
	db_e($ret);

	//if a specifc user was requested
	if (ctype_digit($id)) {
		//if it was found, return a single-dimmensional array
		if ($ret) {
            //make sure the username isnt blank as this can cause prosody
			//to hang.
			$ret[0]['username'] = $ret[0]['username']
				? $ret[0]['username'] : $ret[0]['user'];
			return $ret[0];
		} else {
			//otherwise, return a blank set
			return array(
					'user'		=> $id,
					'username'	=> '',
					'password'	=> ''
			);
		}
	}

	return $ret ? $ret : array();
}

//save user
function xmpp_users_put($put) {
	global $db;

	//validate data
	if (
		!ctype_digit($put['user'])
		|| !$put['jabber_user']
		|| !$put['jabber_pass']
	) {
		return false;
	}

	//prosody only saved lowercase names. correct that here so that we have the proper data saved
	$put['jabber_user'] = strtolower($put['jabber_user']);

	//ensure username isnt in use
	$sql = 'SELECT user FROM xmpp_users WHERE username = ?';
	$curuser = $db->getOne($sql, array($put['jabber_user']));

	if ($curuser && $curuser != $put['user']) {
		return array(
					'status'	=> 'error',
					'msg'		=> _('User already exists')
					);
	}

	//only submit if there really are changes
	$user = xmpp_users_get($put['user']);

	if ($user && $user['username'] == $put['jabber_user'] && $user['password'] == $put['jabber_pass']) {
		return true;
	}

	//insert
	$sql = 'REPLACE INTO xmpp_users (user, username, password)'
		. ' VALUES (?, ?, ?)';
	$ret = $db->query($sql, array_values($put));

	//mark as dirty
	needreload();
	xmpp_opts_put(array('dirty', 'true'));

	return db_e($ret);
}

//del user
function xmpp_users_del($id) {
	global $db;
	$sql = 'DELETE FROM xmpp_users WHERE user = ?';
	$ret = $db->query($sql, array($id));

	//mark as dirty
	needreload();
	xmpp_opts_put(array('dirty', 'true'));

	return db_e($ret);
}

//options
function xmpp_opts_get($opt = '') {
	global $db;

	//return one opt if that was requested
	if ($opt) {
		$sql = 'SELECT value FROM xmpp_options WHERE keyword = ?';
		$ret = $db->getOne($sql, array($opt));
		db_e($ret);
		return $ret;
	}


	$sql = 'SELECT * FROM xmpp_options';
	$ret = $db->getAll($sql, DB_FETCHMODE_ASSOC);
	db_e($ret);

	//ensure defaults are set
	$defaults = array(
		//'dbhost'	=> '',
		//'dbname'	=> 'localhost',
		//'dbpass'	=> '',
		//'dbuser'	=> '',
		'domain'	=> '',
		//'port'		=> '5222',
		'dirty'		=> 'false'

	);
	//dbug('$ret', $ret);
	foreach ($ret as $my => $r) {
		$res[$r['keyword']] = $r['value'];
	}
	foreach ($defaults as $k => $v) {
		if (!isset($res[$k])) {
			$res[$k] = $v;
		}
	}

	return $res;

}

//save opts
function xmpp_opts_put($put) {
	global $db;
	static $dirty = false;

	if (!$dirty) {
		//check what this settings is set to that can decide if anything changed yet
		if (xmpp_opts_get($put[0]) != $put[1]) {
			$dirty = true;
			xmpp_opts_put(array('dirty', 'true'));
			needreload();
		}
	}

	//insert
	$sql = 'REPLACE INTO xmpp_options (keyword, value) VALUES (?, ?)';
	$ret = $db->query($sql, array_values($put));


	return db_e($ret);
}

//del opt
function xmpp_opts_del($opt) {
	global $db;
	$sql = 'DELETE FROM xmpp_options WHERE keyword = ?';
	$ret = $db->query($sql, array($id));
	return db_e($ret);
}

//get conf
function xmpp_get_config() {
	global $xmpplic;
	global $amp_conf, $db;
	if (!isset($xmpplic) || !$xmpplic) {
		return false;
	}
	//get opts
	$opts = xmpp_opts_get();

	//unlike ofther fpbx modules, we dont want to re-generate configs if we dont need to
	//hence we track if we really need to regenerate them or not and act accordingly


	$pfd = $amp_conf['ASTETCDIR'] . '/prosody_additional.conf';

	// If it's not dirty make sure the file is really there since
	// we need to know if it's there for the parsing that comes next
	//
	if ($opts['dirty'] == 'false' && !file_exists($pfd)) {
		$opts['dirty'] = 'true';
	}

	// If it's not dirty (and the file exists) let's make sure the DB credentials
	// have not changed since if they have we need to regenerate
	//
	if ($opts['dirty'] == 'false') {
		$str = file_get_contents($pfd);
		$puname = 'username = "' . $amp_conf['AMPDBUSER'] . '"';
		$ppass  = 'password = "' . $amp_conf['AMPDBPASS'] . '"';
		$pdb    = 'database = "' . $amp_conf['AMPDBNAME'] . '"';

		if (strpos($str, $puname) === false || strpos($str, $ppass) === false || strpos($str, $pdb) === false) {
			$opts['dirty'] = 'true';
		}
	}

	if ($opts['dirty'] == 'false') {
		//nothing to do!
		return true;
	}

	require_once(dirname(__FILE__) . '/xmpp_conf.php');

	//get list of users
	$users = xmpp_users_get();

	//write configs and restart xmpp
	//we cant add users to a virtualhost that doesnt exists
	//so first make sure that it exists, then tend to users

	//write out config file
	$configs = xmpp_generate_xmpp_conf($opts);
	file_put_contents($pfd, $configs);

	/*
	 * update the hostname in the database
	 *
	 * This is a risky move as we dont have any controll over
	 * whtat happens in the table or a spec file/change log for changes in schema
	 * However, once the VirtualHost (i.e. what we call domain) is changed, it is
	 * no longer posible to remove user from deleted host. As a result, for every
	 * time we change the domain, we would have a bunch of stale users that we cant
	 * remove. Instead, we convert them in to "usefull" entires using a direct DB
	 * manipulation. Not the most elegent, but hey - it works.
	 *
	 */
	$sql = 'UPDATE prosody SET host = ?';
	$db->query($sql, array($opts['domain']));

	if (function_exists('sysadmin_restart_xmpp')) {
		sysadmin_restart_xmpp();
		sleep(5);
	}


	//add/update users
	if ($users) {
		$grp = '[user]' . PHP_EOL;
		foreach ($users as $u) {
			if (!$u['username']) {
				// Don't act on empty users
				continue;
			}

			$grp .= $u['username'] . '@' . $opts['domain'] .  PHP_EOL;

			$cmd = fpbx_which('prosodyctl')
					. ' register '
					. $u['username'] . ' '
					. $opts['domain'] . ' '
					. $u['password'];
			exec($cmd);
		}
		//write out shared roster
		file_put_contents($amp_conf['ASTETCDIR'] . '/prosody_groups.txt', $grp);

	}

	//remove posibly stale users
	$sql = 'SELECT * FROM prosody WHERE store = "accounts"';
	$listed = $db->getAll($sql, DB_FETCHMODE_ASSOC);
	if (!db_e($listed, '')) {

		foreach ($listed as $l) {
			$l['delete'] = false;

			//run some test to see if we need to delete this user
			if ($l['host'] != $opts['domain']) {
				$l['delete'] = true;
			}

			//no point in running the next test if we already decided were deleting the user, so only run it conditionally
			if ($l['delete'] == false) {
				$l['delete'] = true; //test
				if ($users) {
					foreach ($users as $u) {
						if ($l['user'] == strtolower($u['username'])) {//prosody saves all users as lowercase
							$l['delete'] = false;
							break;
						}
					}
				}
			}

			//now delete if nesesary
			if ($l['delete']) {
				$cmd = fpbx_which('prosodyctl')
					. ' unregister '
					. $l['user'] . ' '
					. $l['host'];
				exec($cmd);
				//dbug($cmd);
			}
		}
	}
	//mark xmpp as clean
	xmpp_opts_put(array('dirty', 'false'));

	return true;
}
